/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package DiningPhilosophers;

/**
 * This is a Panel GUI used to show a philosopher state.
 * This panel show the philosopher name, the philosopher 
 * forks, the philosopher status and the philospher thicks 
 * since last meal.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 *
 * @version 2.0
 */

public class PhilosopherPanel 
    extends javax.swing.JPanel 
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * The progress bar used to show philosopher's thicks 
     * since last meal.
     */
    protected javax.swing.JProgressBar hungryness_;

    /**
     * The check box used to show if philosopher has 
     * his right fork.
     */
    protected javax.swing.JCheckBox right_fork_;

    /**
     * The check box used to show if philosopher has 
     * his left fork.
     */
    protected javax.swing.JCheckBox left_fork_;

    /** The label used to show the philosopher name. */      
    protected javax.swing.JLabel name_;

    /** The label used to show the philosopher status. */      
    protected javax.swing.JLabel state_;

    /** The philosopher possible states as string. */
    protected String[] stateAsStrings_ = {
        "EATING", "THINKING", "HUNGRY", "STARVING", "DEAD"
    };

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Initializes the philosopher panel using a StatusInfo 
     * event.
     *
     * @param status The StatusInfo event used to initialize 
     *               the philosopher panel.
     */ 
    public PhilosopherPanel(StatusInfo status) 
    {
        /* this panel has a border layout */
        super(new java.awt.BorderLayout());


        java.net.URL url;
        javax.swing.JPanel hungrynessPanel = new javax.swing.JPanel(new java.awt.BorderLayout());
        javax.swing.JPanel hungrynessSubPanel = new javax.swing.JPanel(new java.awt.FlowLayout());

        /* Creating the used font */
        java.awt.Font font = getFont();
        font = font.deriveFont((float)16.0);

        /* setting hungryness progress bar in two pannels 
           to center it in the main panel */
        hungryness_ = new javax.swing.JProgressBar(0, 40);
        hungryness_.setSize(new java.awt.Dimension(150, 30));
        hungrynessSubPanel.add(hungryness_);
        hungrynessPanel.add(new javax.swing.JLabel(" "), java.awt.BorderLayout.NORTH);
        hungrynessPanel.add(hungrynessSubPanel, java.awt.BorderLayout.CENTER);

        /* create and configure the name label */
        name_ = new javax.swing.JLabel(status.name);
        name_.setPreferredSize(new java.awt.Dimension(100, name_.getHeight()));
        name_.setFont(font);

        /* create check boxes */
        right_fork_ = new javax.swing.JCheckBox();
        left_fork_ = new javax.swing.JCheckBox();

        /* create state label */
        state_ = new javax.swing.JLabel();
        state_.setPreferredSize(new java.awt.Dimension(100, state_.getHeight()));
        state_.setFont(font);

        /* Setting fork icons (icons must be located in the dinner jar file) */
        url = PhilosopherPanel.class.getResource("/images/myfork.gif");
        right_fork_.setSelectedIcon(new javax.swing.ImageIcon(url));
        left_fork_.setSelectedIcon(new javax.swing.ImageIcon(url));
        url = PhilosopherPanel.class.getResource("/images/nofork.gif");
        right_fork_.setIcon(new javax.swing.ImageIcon(url));
        left_fork_.setIcon(new javax.swing.ImageIcon(url));

        /* add name to left side of the panel */
        add(name_, java.awt.BorderLayout.WEST);

        /* creates a new sub panel for forks and state */
        javax.swing.JPanel forks = new javax.swing.JPanel(new java.awt.BorderLayout());
        forks.add(right_fork_, java.awt.BorderLayout.EAST);
        forks.add(state_, java.awt.BorderLayout.CENTER);
        forks.add(left_fork_, java.awt.BorderLayout.WEST);
        
        /* adding fork's panel in the center of the panel */
        add(forks, java.awt.BorderLayout.CENTER);
        /* adding progress bar panel to the east side of the panel */
        add(hungrynessPanel, java.awt.BorderLayout.EAST);

        /* initializes panel values */
        updatePanel(status);

        /* Configure the panel */
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setVisible(true);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Returns the color to use for the progress bar 
     * using the philosopher's thicks since last meal.
     *
     * @param thicks The philosopher's thicks since last meal.
     */
    protected java.awt.Color getFG(int thicks)
    {
        if (thicks < 3) return java.awt.Color.green;
        if (thicks < 10) return java.awt.Color.yellow;
        if (thicks < 40) return java.awt.Color.orange;
        return java.awt.Color.red;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Update the philosopher's panel using a StatusInfo event.
     *
     * @param event The event used to update the panel.
     */
    public void updatePanel(StatusInfo event)
    {
        /* update the status label */
        state_.setText(stateAsStrings_[event.state.value()]);
        /* update the hungryness progress bar */
        hungryness_.setForeground(getFG(event.ticks_since_last_meal));
        hungryness_.setValue(event.ticks_since_last_meal);
        /* update forks check boxes */
        left_fork_.setSelected(event.has_left_fork);
        right_fork_.setSelected(event.has_right_fork);
    }
}

