/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * Implementation of the OpenCCM::HomeManager interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class HomeManagerImpl
       extends HomeManagerPOA
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     ** To store (cookie, HomeInfo) associations.
     **/
    private org.objectweb.ccm.util.Table by_cookies_;

    /**
     ** To store (home_repid, CCMHome) associations.
     **/
    private org.objectweb.ccm.util.Table by_home_repids_;

    /**
     ** To store (component_repid, CCMHome) associations.
     **/
    private org.objectweb.ccm.util.Table by_component_repids_;

    /**
     ** To store (home_name, CCMHome) associations.
     **/
    private org.objectweb.ccm.util.Table by_home_names_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    HomeManagerImpl()
    {
        by_cookies_ = new org.objectweb.ccm.util.Table();
        by_home_repids_ = new org.objectweb.ccm.util.Table();
        by_component_repids_ = new org.objectweb.ccm.util.Table();
        by_home_names_ = new org.objectweb.ccm.util.Table();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the OpenCCM::Shutdownable interface.
    //
    // ==================================================================

    //
    // IDL:goal.lifl.fr/OpenCCM/Shutdownable/shutdown:1.0
    //
    /**
     **  Shutdown an OpenCCM server.
     **/
    public void
    shutdown()
    {
	org.objectweb.ccm.CORBA.TheORB.shutdown(true);
    }

    // ==================================================================
    //
    // Methods for the Components::HomeFinder interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/HomeFinder/find_home_by_component_type:1.0
    //
    /**
     **  Finds an home by its managed component type.
     **
     **  @param comp_repid The requested component repository ID.
     **
     **  @return The associated home.
     **
     **  @exception org.omg.Components.HomeNotFound
     **             Thrown if there is no home for the requested
     **             component type.
     **/
    public org.omg.Components.CCMHome
    find_home_by_component_type(String comp_repid)
    throws org.omg.Components.HomeNotFound
    {
	// Search by component repository id.
	Object result = by_component_repids_.get(comp_repid);

	// Check that a CCMHome has been found.
	if(result == null)
	{
	    throw new org.omg.Components.HomeNotFound();
	}

	// Return the found CCMHome.
	return (org.omg.Components.CCMHome)result;
    }

    //
    // IDL:omg.org/Components/HomeFinder/find_home_by_home_type:1.0
    //
    /**
     **  Finds an home by its home type.
     **
     **  @param home_repid The requested home repository ID.
     **
     **  @return The associated home.
     **
     **  @exception org.omg.Components.HomeNotFound
     **             Thrown if there is no home for the requested home type.
     **/
    public org.omg.Components.CCMHome
    find_home_by_home_type(String home_repid)
    throws org.omg.Components.HomeNotFound
    {
	// Search by home repository id.
	Object result = by_home_repids_.get(home_repid);

	// Check that a CCMHome has been found.
	if(result == null)
	{
	    throw new org.omg.Components.HomeNotFound();
	}

	// Return the found CCMHome.
	return (org.omg.Components.CCMHome)result;
    }

    //
    // IDL:omg.org/Components/HomeFinder/find_home_by_name:1.0
    //
    /**
     **  Finds an home by its name.
     **
     **  @param home_name The requested home name.
     **
     **  @return The associated home.
     **
     **  @exception org.omg.Components.HomeNotFound
     **             Thrown if there is no home for the requested home name.
     **/
    public org.omg.Components.CCMHome
    find_home_by_name(String home_name)
    throws org.omg.Components.HomeNotFound
    {
	// Search by home name.
	Object result = by_home_names_.get(home_name);

	// Check that a CCMHome has been found.
	if(result == null)
	{
	    throw new org.omg.Components.HomeNotFound();
	}

	// Return the found CCMHome.
	return (org.omg.Components.CCMHome)result;
    }

    // ==================================================================
    //
    // Methods for the OpenCCM::Components::HomeManager interface.
    //
    // ==================================================================

    //
    // IDL:goal.lifl.fr/OpenCCM/Components/HomeManager/register_home:1.0
    //
    /**
     **  Register a CCMHome.
     **
     **  @param home The CCMHome reference.
     **  @param home_repid The CCMHome Repository Id.
     **  @param component_repid The managed component Repository Id.
     **  @param name The INS name associated to the home.
     **
     **  @return The cookie associated to this registration.
     **/
    public org.omg.Components.Cookie
    register_home(org.omg.Components.CCMHome home,
                  String home_repid,
                  String component_repid,
                  String name)
    {
	// Register the home by its repository Id.
	by_home_repids_.put(home_repid, home);

	// Register the home by its managed component repository Id.
	by_component_repids_.put(component_repid, home);

	// Register the home by its name.
	by_home_names_.put(name, home);

	// Generate a cookie.
	org.omg.Components.Cookie cookie = CookieImpl.generate();

	// Register the home information by the generated cookie.
	by_cookies_.put(cookie,
			new HomeInfo(home,
				     home_repid,
				     component_repid,
				     name) );

	// Return the generated cookie.
	return cookie;
    }

    //
    // IDL:goal.lifl.fr/OpenCCM/Components/HomeManager/unregister_home:1.0
    //
    /**
     **  Unregister a CCMHome.
     **
     **  @param The registration cookie.
     **
     **  @exception ::Components::CookieRequired
     **             Thrown when an invalid cookie is used.
     **/
    public void
    unregister_home(org.omg.Components.Cookie ck)
    throws org.omg.Components.CookieRequired
    {
	// Search the home info by its associated cookie.
	HomeInfo home_info = (HomeInfo)by_cookies_.get(ck);

	// Check that a home info is associated to the cookie.
	if(home_info == null)
	{
	    throw new org.omg.Components.CookieRequired();
	}

	// Remove in the home repository id table.
	by_home_repids_.remove(home_info.home_repid);

	// Remove in the component repository id table.
	by_component_repids_.remove(home_info.component_repid);

	// Remove in the home name table.
	by_home_names_.remove(home_info.name);
    }
}
