/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages the constant or union case value.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AnyValue
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The type of the constant.
     **/
    private int type_;

    /**
     **  The value of the constant.
     **/
    private long integer_;
    private String string_;
    private char char_;
    private String fixed_;
    private double floating_;
    private boolean boolean_;
    private org.omg.CORBA.TypeCode tc_;
    private int enumerator_;

    // ==================================================================
    //
    // Kind of type.
    //
    // ==================================================================

    /**
     ** Represent unknown type for the Any value.
     **/
    final public int NONE = 0;

    /**
     ** Represent integer type for the Any value.
     **/
    final public int INTEGER = 1;

    /**
     ** Represent string type for the Any value.
     **/
    final public int STRING = 2;

    /**
     ** Represent wide string type for the Any value.
     **/
    final public int WSTRING = 3;

    /**
     ** Represent character type for the Any value.
     **/
    final public int CHAR = 4;

    /**
     ** Represent wide character type for the Any value.
     **/
    final public int WCHAR = 5;

    /**
     ** Represent fixed type for the Any value.
     **/
    final public int FIXED = 6;

    /**
     ** Represent float type for the Any value.
     **/
    final public int FLOATING = 7;

    /**
     ** Represent boolean type for the Any value.
     **/
    final public int BOOLEAN = 8;

    /**
     ** Represent enumerate type for the Any value.
     **/
    final public int ENUM = 9;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    AnyValue()
    {
        type_ = NONE;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain the any value.
     **
     **  @param tc The TypeCode of the desired any value.
     **            Note that one of the set* method should have been used
     **            to fix the value before using this method.
     **            If the TypeCode and the set* method are not consistent,
     **            an Error is thrown.
     **
     **  @return An Any created from the TypeCode.
     **/
    protected org.omg.CORBA.Any
    computeAny(org.omg.CORBA.TypeCode tc)
    {
        if(type_ == NONE)
        {
            org.omg.CORBA.Any any =
                org.objectweb.ccm.CORBA.TheORB.create_any();
            any.insert_octet((byte)0);
            return any;
        }

        org.omg.CORBA.TypeCode origTC =
            org.objectweb.ccm.CORBA.TypeCodeUtils.getOrigType(tc);
        org.omg.DynamicAny.DynAny dynAny =
            org.objectweb.ccm.CORBA.TheDynamicAnyFactory.
            create_dyn_any_from_type_code(origTC);

        try
        {
            switch(origTC.kind().value())
            {
            case org.omg.CORBA.TCKind._tk_short:
                if(type_ != INTEGER)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_short((short)integer_);
                break;

            case org.omg.CORBA.TCKind._tk_long:
                if(type_ != INTEGER)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_long((int)integer_);
                break;

            case org.omg.CORBA.TCKind._tk_ushort:
                if(type_ != INTEGER)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_ushort((short)integer_);
                break;

            case org.omg.CORBA.TCKind._tk_ulong:
                if(type_ != INTEGER)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_ulong((int)integer_);
                break;

            case org.omg.CORBA.TCKind._tk_float:
                if(type_ != FLOATING)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_float((float)floating_);
                break;

            case org.omg.CORBA.TCKind._tk_double:
                if(type_ != FLOATING)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_double(floating_);
                break;

            case org.omg.CORBA.TCKind._tk_boolean:
                if(type_ != BOOLEAN)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_boolean(boolean_);
                break;

            case org.omg.CORBA.TCKind._tk_char:
                if(type_ != CHAR)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_char(char_);
                break;

            case org.omg.CORBA.TCKind._tk_string:
                if(type_ != STRING)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_string(string_);
                break;

            case org.omg.CORBA.TCKind._tk_longlong:
                if(type_ != INTEGER)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_longlong(integer_);
                break;

            case org.omg.CORBA.TCKind._tk_ulonglong:
                if(type_ != INTEGER)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_ulonglong(integer_);
                break;

            case org.omg.CORBA.TCKind._tk_wchar:
                if(type_ != WCHAR)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_wchar(char_);
                break;

            case org.omg.CORBA.TCKind._tk_wstring:
                if(type_ != WSTRING)
  	            throw new Error("Invalid constant value!");
                dynAny.insert_wstring(string_);
                break;

            case org.omg.CORBA.TCKind._tk_fixed:
                if(type_ != FIXED)
  	            throw new Error("Invalid constant value!");
                // TODO: Must be wrapped for JavaORB.
                org.omg.DynamicAny.DynFixedHelper.narrow(dynAny).set_value(fixed_);
                break;

            case org.omg.CORBA.TCKind._tk_enum:
                if(type_ != ENUM)
  	            throw new Error("Invalid constant value!");
                if(!tc_.equal(origTC))
  	            throw new Error("Invalid constant type!");
                org.omg.DynamicAny.DynEnumHelper.narrow(dynAny).set_as_ulong(enumerator_);
                break;

            default:
	        throw new Error("Invalid constant type!");
            }
            return dynAny.to_any();
        }
        catch(org.omg.DynamicAny.DynAnyPackage.InvalidValue exc)
        {
            // Wrap the DynamicAny::InvalidValue exception.
            throw new org.objectweb.ccm.CORBA.UserExceptionWrapper(exc);
        }
        catch(org.omg.DynamicAny.DynAnyPackage.TypeMismatch exc)
        {
            // Wrap the DynamicAny::TypeMismatch exception.
            throw new org.objectweb.ccm.CORBA.UserExceptionWrapper(exc);
        }
    }

    /**
     **
     **/
    protected void
    loadAny(org.omg.CORBA.Any any)
    {
        org.omg.CORBA.TypeCode tc = any.type();
        try
        {
            switch(tc.kind().value())
            {
            case org.omg.CORBA.TCKind._tk_short:
                setAsInteger(any.extract_short());
                break;
            case org.omg.CORBA.TCKind._tk_long:
                setAsInteger(any.extract_long());
                break;
            case org.omg.CORBA.TCKind._tk_ushort:
                setAsInteger(any.extract_ushort());
                break;
            case org.omg.CORBA.TCKind._tk_ulong:
                setAsInteger(any.extract_ulong());
                break;
            case org.omg.CORBA.TCKind._tk_float:
                setAsFloating(any.extract_float());
                break;
            case org.omg.CORBA.TCKind._tk_double:
                setAsFloating(any.extract_double());
                break;
            case org.omg.CORBA.TCKind._tk_boolean:
                setAsBoolean(any.extract_boolean());
                break;
            case org.omg.CORBA.TCKind._tk_char:
                setAsChar(any.extract_char());
                break;
            case org.omg.CORBA.TCKind._tk_wchar:
                setAsWChar(any.extract_wchar());
                break;
            case org.omg.CORBA.TCKind._tk_string:
                setAsString(any.extract_string());
                break;
            case org.omg.CORBA.TCKind._tk_wstring:
                setAsWString(any.extract_wstring());
                break;
            case org.omg.CORBA.TCKind._tk_longlong:
                setAsInteger(any.extract_longlong());
                break;
            case org.omg.CORBA.TCKind._tk_ulonglong:
                setAsInteger(any.extract_ulonglong());
                break;
            case org.omg.CORBA.TCKind._tk_fixed:
                org.omg.DynamicAny.DynFixed fixed = 
                    org.objectweb.ccm.CORBA.TheDynamicAnyFactory.create_dyn_fixed(any);
                setAsFixed(fixed.get_value());
                break;
            case org.omg.CORBA.TCKind._tk_enum:
                org.omg.DynamicAny.DynEnum enume = 
                    org.objectweb.ccm.CORBA.TheDynamicAnyFactory.create_dyn_enum(any);
                setAsEnum(tc, enume.get_as_ulong(), enume.get_as_string());
                break;
            default:
                break;
            }
        }
        catch(org.omg.CORBA.BAD_OPERATION ex)
        {}
    }

    // ==================================================================
    //
    // Methods for the AnyValue interface.
    //
    // ==================================================================

    /**
     **  Is it an integer?
     **
     **  @return true if it's a long or false otherwise.
     **/
    public boolean
    isInteger()
    {
        return type_ == INTEGER;
    }

    /**
     **  Set with an integer.
     **
     **  @param value The long to set as value.
     **/
    public void
    setAsInteger(long value)
    {
        type_ = INTEGER;
        integer_ = value;
    }

    /**
     **  Get as an integer.
     **
     **  @return The long that must have been previously set.
     **/
    public long
    getAsInteger()
    {
        return integer_;
    }

    /**
     **  Is it an string?
     **
     **  @return true if it's a string or false otherwise.
     **/
    public boolean
    isString()
    {
        return type_ == STRING;
    }

    /**
     **  Set with a string.
     **
     **  @param value The String to set as value.
     **/
    public void
    setAsString(String value)
    {
        type_ = STRING;
        string_ = value;
    }

    /**
     **  Get as a string.
     **
     **  @return The string that must have been previously set.
     **/
    public String
    getAsString()
    {
        return string_;
    }

    /**
     **  Is it an wstring?
     **
     **  @return true if it's a wstring or false otherwise.
     **/
    public boolean
    isWString()
    {
        return type_ == WSTRING;
    }

    /**
     **  Set with a wstring.
     **
     **  @param value The String to set as value.
     **/
    public void
    setAsWString(String value)
    {
        type_ = WSTRING;
        string_ = value;
    }

    /**
     **  Get as a wstring.
     **
     **  @return The wstring that must have been previously set.
     **/
    public String
    getAsWString()
    {
        return string_;
    }

    /**
     **  Is it an char?
     **
     **  @return true if it's a char or false otherwise.
     **/
    public boolean
    isChar()
    {
        return type_ == CHAR;
    }

    /**
     **  Set with a char.
     **
     **  @param value The char to set as value.
     **/
    public void
    setAsChar(char value)
    {
        type_ = CHAR;
        char_ = value;
    }

    /**
     **  Get as a char.
     **
     **  @return The char that must have been previously set.
     **/
    public char
    getAsChar()
    {
        return char_;
    }

    /**
     **  Is it an wchar?
     **
     **  @return true if it's a wchar or false otherwise.
     **/
    public boolean
    isWChar()
    {
        return type_ == WCHAR;
    }

    /**
     **  Set with a wchar.
     **
     **  @param value The char to set as value.
     **/
    public void
    setAsWChar(char value)
    {
        type_ = WCHAR;
        char_ = value;
    }

    /**
     **  Get as a wchar.
     **
     **  @return The wchar that must have been previously set.
     **/
    public char
    getAsWChar()
    {
        return char_;
    }

    /**
     **  Is it an fixed?
     **
     **  @return true if it's a fixed or false otherwise.
     **/
    public boolean
    isFixed()
    {
        return type_ == FIXED;
    }

    /**
     **  Set with a fixed.
     **
     **  @param value The String to set as value.
     **/
    public void
    setAsFixed(String value)
    {
        type_ = FIXED;
        fixed_ = value;
    }

    /**
     **  Get as a fixed.
     **
     **  @return The string that must have been previously set.
     **/
    public String
    getAsFixed()
    {
        return fixed_;
    }

    /**
     **  Is it a floating?
     **
     **  @return true if it's a float or false otherwise.
     **/
    public boolean
    isFloating()
    {
        return type_ == FLOATING;
    }

    /**
     **  Set with a double.
     **
     **  @param value The double to set as value.
     **/
    public void
    setAsFloating(double value)
    {
        type_ = FLOATING;
        floating_ = value;
    }

    /**
     **  Get as a double.
     **
     **  @return The double that must have been previously set.
     **/
    public double
    getAsFloating()
    {
        return floating_;
    }

    /**
     **  Is it a boolean?
     **
     **  @return true if it's a boolean or false otherwise.
     **/
    public boolean
    isBoolean()
    {
        return type_ == BOOLEAN;
    }

    /**
     **  Set with a boolean.
     **
     **  @param value The boolean to set as value.
     **/
    public void
    setAsBoolean(boolean value)
    {
        type_ = BOOLEAN;
        boolean_ = value;
    }

    /**
     **  Get as a boolean.
     **
     **  @return The boolean that must have been previously set.
     **/
    public boolean
    getAsBoolean()
    {
        return boolean_;
    }

    /**
     **  Is it a enum?
     **
     **  @return true if it's a enum or false otherwise.
     **/
    public boolean
    isEnum()
    {
        return type_ == ENUM;
    }

    /**
     **  Set with an enum.
     **
     **  @param value The long to set as value.
     **/
    public void
    setAsEnum(org.omg.CORBA.TypeCode tc,
              int value,
              String name)
    {
        type_ = ENUM;
        tc_ = tc;
        enumerator_ = value;
        string_ = name;
    }

    /**
     **  Get as a enum.
     **
     **  @return The string that must have been previously set.
     **/
    public String
    getAsEnum()
    {
        return string_;
    }

}
