/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages the Attribute declarationn.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AttributeDeclImpl
       extends DeclarationImpl
       implements AttributeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 AttributeDef.
     **/
    private org.omg.CORBA.ExtAttributeDef ext_attribute_def_;

    /**
     **  The attribute mode.
     **/
    private int mode_;

    /**
     **  The attribute type.
     **/
    private TypeRef type_;

    /**
     **  The attribute get exceptions.
     **/
    private ExceptionListImpl get_exceptions_;

    /**
     **  The attribute put exceptions.
     **/
    private ExceptionListImpl set_exceptions_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    AttributeDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_attribute_def_ = null;
        mode_ = AttributeDecl.ATTR_NORMAL;
        type_ = null;
        get_exceptions_ = new ExceptionListImpl();
        set_exceptions_ = new ExceptionListImpl();
        the_declaration_kind_ = DeclarationKind._dk_attribute;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ModuleDef.
     **
     **  @param moduleDef The ModuleDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_attribute_def_ = org.omg.CORBA.ExtAttributeDefHelper.narrow(contained);
        mode_ = ext_attribute_def_.mode().value();
        setType(getRepository().getAsTypeRef(ext_attribute_def_.type_def()));

        org.omg.CORBA.ExceptionDef[] exc = ext_attribute_def_.get_exceptions();
        for (int i=0;i<exc.length;i++)
            get_exceptions_.add((ExceptionRef)getRepository().lookupId(exc[i].id()));

        exc = ext_attribute_def_.set_exceptions();
        for (int i=0;i<exc.length;i++)
            set_exceptions_.add((ExceptionRef)getRepository().lookupId(exc[i].id()));

        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 ModuleDef.
     **
     **  @param moduleDef The ModuleDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_attribute_def_ = org.omg.CORBA.ExtAttributeDefHelper.narrow(contained);
        mode_ = ext_attribute_def_.mode().value();
        setType(getRepository().getAsMappedTypeRef(ext_attribute_def_.type_def()));

        org.omg.CORBA.ExceptionDef[] exc = ext_attribute_def_.get_exceptions();
        for (int i=0;i<exc.length;i++)
            get_exceptions_.add((ExceptionRef)getRepository().lookupMappedId(exc[i].id()));

        exc = ext_attribute_def_.set_exceptions();
        for (int i=0;i<exc.length;i++)
            set_exceptions_.add((ExceptionRef)getRepository().lookupMappedId(exc[i].id()));

        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    //  Public methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the attribute declaration into the IFR.
     **/
    public void
    create()
    {
        ext_attribute_def_ = the_parent_.createExtAttribute(this, type_.getIDLType(),
                                                            org.omg.CORBA.AttributeMode.from_int(mode_),
                                                            get_exceptions_.getExceptionDefs(),
                                                            set_exceptions_.getExceptionDefs());

        super.create();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector attr_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // attribute type
        if (getType().isDeclaration())
            attr_depend.add(getType());

        if (recursion) {
            depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                attr_depend.add(depend[i]);
        }

        // exceptions
        if (isReadonly())
            addDependencies(attr_depend, recursion, getExceptions());
        else
        {
            addDependencies(attr_depend, recursion, getGetExceptions());
            addDependencies(attr_depend, recursion, getSetExceptions());
        }

        dependencies_ = (Declaration[])attr_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    /**
     **
     **/
    protected void
    addDependencies(org.objectweb.ccm.util.Vector attr_depend,
                    boolean recursion,
                    ExceptionDecl[] excs)
    {
        Declaration[] depend = null;
        for (int i=0;i<excs.length;i++)
        {
            attr_depend.add(excs[i]);
            if (recursion)
            {
                depend = excs[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if (attr_depend.indexOf(depend[j])==-1)
                        attr_depend.add(depend[j]);
                }
            }
        }
    }


    // ==================================================================
    //
    // Public methods for the AttributeDecl interface.
    //
    // ==================================================================

    /**
     **  Set mode as normal.
     **/
    public void
    setNormal()
    {
        mode_ = AttributeDecl.ATTR_NORMAL;
    }

    /**
     **  Set mode as readonly.
     **/
    public void
    setReadonly()
    {
        mode_ = AttributeDecl.ATTR_READONLY;
    }

    /**
     **  Set the attribute type.
     **
     **  @param type The TypeRef of the attribute declaration.
     **/
    public void
    setType(TypeRef type)
    {
        if(type != null)
        {
            type_ = type;
            type_.addRef();
        }
    }

    /**
     **  Obtain the get exceptions list.
     **
     **  @return The getRaises ExceptionList of the attribute declaration
     **          or null if no exceptions were declared.
     **/
    public ExceptionList
    getGetExceptionList()
    {
        return get_exceptions_;
    }

    /**
     **  Obtain the put exceptions list.
     **
     **  @return The setRaises (or raises for readonly attributes)
     **          ExceptionList of the attribute declaration 
     **          or null if no exceptions were declared.
     **/
    public ExceptionList
    getSetExceptionList()
    {
        return set_exceptions_;
    }

    /**
     **  Obtain the get exceptions list.
     **
     **  @return The getRaises ExceptionList of the attribute declaration
     **          or null if no exceptions were declared.
     **/
    public ExceptionDecl[]
    getGetExceptions()
    {
        if (mode_==AttributeDecl.ATTR_NORMAL)
            return get_exceptions_.getExceptions();
        return new ExceptionDecl[0];
    }

    /**
     **  Obtain the put exceptions list.
     **
     **  @return The setRaises (or raises for readonly attributes)
     **          ExceptionList of the attribute declaration 
     **          or null if no exceptions were declared.
     **/
    public ExceptionDecl[]
    getSetExceptions()
    {
        if (mode_==AttributeDecl.ATTR_NORMAL)
            return set_exceptions_.getExceptions();
        return new ExceptionDecl[0];
    }

    /**
     **
     **/
    public ExceptionDecl[]
    getExceptions()
    {
        if (mode_==AttributeDecl.ATTR_READONLY)
            return get_exceptions_.getExceptions();
        return new ExceptionDecl[0];
    }

    /**
     **
     **/
    public boolean
    isReadonly()
    {
        return mode_==AttributeDecl.ATTR_READONLY;
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return type_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (type_!=null)
            type_.removeRef();

        get_exceptions_.destroy();
        set_exceptions_.destroy();
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the
     **           attribute declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_attribute_def_;
    }
}
