/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class implements a component declaration.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ComponentDeclImpl
       extends InterfaceDeclImpl
       implements ComponentRef, ComponentDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ComponentDef.
     **/
    private org.omg.CORBA.ComponentIR.ComponentDef component_def_;

    /**
     **  Its base component.
     **/
    private ComponentDeclImpl base_component_;

    /**
     **
     **/
    private org.objectweb.ccm.util.Vector supported_interfaces_;

    /**
     **
     **/
    private InterfaceDeclImpl client_mapping_;

    /**
     **
     **/
    private InterfaceDeclImpl local_context_mapping_;

    /**
     **
     **/
    private InterfaceDeclImpl local_monolithic_mapping_;

    /**
     **
     **/
    private InterfaceDeclImpl local_main_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    ComponentDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the InterfaceDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        component_def_ = null;
        base_component_ = null;
        client_mapping_ = null;
        local_context_mapping_ = null;
        local_monolithic_mapping_ = null;
        local_main_mapping_ = null;
        supported_interfaces_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_component;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef[].
     **
     **  @return An array containing it's supported InterfaceDef.
     **/
    protected org.omg.CORBA.InterfaceDef[]
    getInterfaceDefs()
    {
        org.omg.CORBA.InterfaceDef[] result =
            new org.omg.CORBA.InterfaceDef[supported_interfaces_.size()];

        for(int i=0; i<result.length; i++)
        {
            result[i] = ((InterfaceRef)(supported_interfaces_.get(i))).getExtInterfaceDef();
        }

        return result;
    }

    /**
     **  Obtain the inherited component base.
     **
     **  @return The inherited component base.
     **/ 
    protected org.omg.CORBA.ComponentIR.ComponentDef
    getBaseComponentDef()
    {
        return (base_component_ != null)
	     ? (base_component_.getComponentDef()) : null;
    }

    /**
     **  Loads infos of the CORBA 3.0 ComponentDef.
     **
     **  @param component_def The ComponentDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        component_def_ = org.omg.CORBA.ComponentIR.ComponentDefHelper.narrow(contained);
        if (component_def_.base_component()!=null)
            setBaseComponent(
                (ComponentRef)getRepository().lookupId(component_def_.base_component().id()));

        org.omg.CORBA.InterfaceDef[] supported = component_def_.supported_interfaces();
        for (int i=0;i<supported.length;i++)
            // WARNING: This will break consistency of array items!!!
            addSupportedInterface((InterfaceRef)getRepository().lookupId(supported[i].id()));

        // load it's mapped declarations
        getClientMapping();
        getLocalContextMapping();

        super.load(contained);
    }

    /**
     **  Find a declaration into inherited interfaces.
     **
     **  @param name The name of the searched declaration.
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    protected Declaration
    findInSupportedInterfaces(String name)
    {
        for(int i=0; i<supported_interfaces_.size(); i++)
        {
            Declaration decl = ((InterfaceDecl)(supported_interfaces_.get(i))).find(name);
            // If found then returns it.
            if (decl != null) return decl;
        }

        // Else not found.
        return null;
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Find a declaration.
     **
     **  Note that the declaration is also recursively searched in the
     **  parent scope.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    public Declaration
    find(String name)
    {
        // Finds in the scope.
        Declaration decl = super.find(name);
        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in base component.
        if (base_component_ != null)
        {
            decl = base_component_.find(name);
            // If found then returns it.
            if(decl != null) return decl;
        }
        // If not found then finds in supported interfaces.
        return findInSupportedInterfaces(name);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        org.objectweb.ccm.util.Vector comp_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // base component
        if (getBaseComponent()!=null)
        {
            comp_depend.add(getBaseComponent());
            if (recursion) {
                depend = getBaseComponent().getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if (depend[j]!=this)
                        comp_depend.add(depend[j]);
                }
            }
        }

        // supported interfaces
        InterfaceDecl[] supp = getSupportedInterfaces();
        for (int i=0;i<supp.length;i++)
        {
            comp_depend.add(supp[i]);
            if (recursion) {
                depend = supp[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((depend[j]!=this) &&
                        (comp_depend.indexOf(depend[j])==-1))
                        comp_depend.add(depend[j]);
                }
            }
        }

        // contents
        Declaration[] decls = getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_all);
        for (int i=0;i<decls.length;i++)
        {
            depend = decls[i].getDependencies(recursion);
            for (int j=0;j<depend.length;j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (depend[j]!=this) &&
                    (comp_depend.indexOf(depend[j])==-1))
                    comp_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])comp_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the component declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return component_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_component;
    }

    // ==================================================================
    //
    // Methods for the InterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the component
     **          declaration.
     **/
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return component_def_;
    }

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the component
     **          declaration.
     **/
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return component_def_;
    }

    // ==================================================================
    //
    // Methods for the ComponentRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its ComponentDef reference.
     **
     **  @return The ComponentDef object associated with the component
     **          declaration.
     **/
    public org.omg.CORBA.ComponentIR.ComponentDef
    getComponentDef()
    {
        return component_def_;
    }

    // ==================================================================
    //
    // Methods for the InterfaceDecl interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the ComponentDecl interface.
    //
    // ==================================================================

    /**
     **  Add an interface.
     **
     **  @param itf The supported interface to add.
     **/
    public void
    addSupportedInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            itf.addRef();
            supported_interfaces_.add(itf);
        }
    }

    /**
     **  Set the base component.
     **
     **  @param base_component The base component of the component.
     **         Note that if more then one base component 
     **         is set, only the first one is considered.
     **/
    public void
    setBaseComponent(ComponentRef base_component)
    {
        if (base_component != null)
        {
            base_component_ = (ComponentDeclImpl)base_component;
            base_component_.addRef();
        }
    }

    /**
     **
     **/
    public ComponentDecl
    getBaseComponent()
    {
        return base_component_;
    }

    /**
     **
     **/
    public InterfaceDecl[]
    getSupportedInterfaces()
    {
        InterfaceDecl[] result = new InterfaceDecl[supported_interfaces_.size()];

        for(int i=0; i<result.length; i++)
            result[i] = (InterfaceDecl)supported_interfaces_.get(i);

        return result;
    }

    /**
     **
     **/
    public InterfaceDecl
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // load the client mapping for this component.
        client_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), getId());

        return client_mapping_;
    }

    /**
     **
     **/
    public InterfaceDecl
    getLocalContextMapping()
    {
        if (local_context_mapping_!=null)
            return local_context_mapping_;

        // load the context mapping for this component.
        String parent_base_id = the_parent_.getId();
        int idx = parent_base_id.lastIndexOf(':');
        parent_base_id = parent_base_id.substring(0, idx);
        String id = parent_base_id + "/CCM_"+getName()+"_Context:"+getVersion();

        local_context_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), id);
        return local_context_mapping_;
    }

    /**
     **
     **/
    public InterfaceDecl
    getLocalMonolithicMapping()
    {
        if (local_monolithic_mapping_!=null)
            return local_monolithic_mapping_;

        // load the monolithic mapping for this component.
        String parent_base_id = the_parent_.getId();
        int idx = parent_base_id.lastIndexOf(':');
        parent_base_id = parent_base_id.substring(0, idx);
        String id = parent_base_id + "/CCM_"+getName()+":"+getVersion();

        local_monolithic_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), id);
        return local_monolithic_mapping_;
    }

    /**
     **
     **/
    public InterfaceDecl
    getLocalMainMapping()
    {
        if (local_main_mapping_!=null)
            return local_main_mapping_;

        // load the main mapping for this component.
        String parent_base_id = the_parent_.getId();
        int idx = parent_base_id.lastIndexOf(':');
        parent_base_id = parent_base_id.substring(0, idx);
        String id = parent_base_id + "/CCM_"+getName()+"_Executor:"+getVersion();

        local_main_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), id);
        return local_main_mapping_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (base_component_!=null)
            base_component_.removeRef();

        InterfaceDecl[] itfs = getSupportedInterfaces();
        for (int i=0;i<itfs.length;i++)
            ((InterfaceRef)itfs[i]).removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the component
     **          declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return component_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the component
     **          declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return component_def_;
    }

    /**
     **  To obtain all the contained Declaration objects.
     **
     **  @param exclude_inherited If false return also objects contained in inherited scopes.
     **  @param limited_types A logical combination of DeclarationKind.
     **
     **  @return An array of Declaration objects.
     **/
    public Declaration[]
    getContents(boolean exclude_inherited, int limited_types)
    {
        if (exclude_inherited)
            return super.getContents(exclude_inherited, limited_types);

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = getSupportedInterfaces();
        for (int i=0;i<itfs.length;i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }

        if (base_component_==null)
            return res;

        Declaration[] tmp1 = base_component_.getContents(exclude_inherited, limited_types);
        Declaration[] tmp2 = res;
        res = new Declaration[tmp1.length+tmp2.length];
        System.arraycopy(tmp2, 0, res, 0, tmp2.length);
        System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        return res;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        component_def_ = the_parent_.getComponentContainer().create_component(getId(),
                                                                              getName(),
                                                                              getVersion(),
                                                                              getBaseComponentDef(),
                                                                              getInterfaceDefs());
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        org.omg.CORBA.ComponentIR.ComponentDef base_component = getBaseComponentDef();
        if (base_component != null)
            component_def_.base_component(base_component);

        org.omg.CORBA.InterfaceDef[] supported_interfaces = getInterfaceDefs();
        if (supported_interfaces.length != 0)
            component_def_.supported_interfaces(supported_interfaces);
    }
}
