/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL constant declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ConstantDeclImpl
       extends AnyValueDeclImpl
       implements ConstantDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ConstantDef.
     **/
    private org.omg.CORBA.ConstantDef constant_def_;

    /**
     **  The type of the constant.
     **/
    private TypeRef type_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    ConstantDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the AnyValueDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        constant_def_ = null;
        type_ = null;
        the_declaration_kind_ = DeclarationKind._dk_constant;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ConstantDef.
     **
     **  @param constant_def The ConstantDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        constant_def_ = org.omg.CORBA.ConstantDefHelper.narrow(contained);
        setType(getRepository().getAsTypeRef(constant_def_.type_def()));
        AnyValue any = new AnyValue();
        any.loadAny(constant_def_.value());
        setAnyValue(any);

        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 ConstantDef.
     **
     **  @param constant_def The ConstantDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        constant_def_ = org.omg.CORBA.ConstantDefHelper.narrow(contained);
        setType(getRepository().getAsMappedTypeRef(constant_def_.type_def()));
        AnyValue any = new AnyValue();
        any.loadAny(constant_def_.value());
        setAnyValue(any);

        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the constant declaration into the IFR.
     **/
    public void
    create()
    {
        constant_def_ = the_parent_.getContainer().create_constant(getId(),
                                                                   getName(),
                                                                   getVersion(),
                                                                   type_.getIDLType(),
                                                                   getAnyValue().computeAny(type_.getIDLType().type()));

        super.create();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector constant_depend = new org.objectweb.ccm.util.Vector();

        // constant type
        if (getType().isDeclaration())
            constant_depend.add(getType());

        if (recursion) {
            Declaration[] depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                constant_depend.add(depend[i]);
        }

        dependencies_ = (Declaration[])constant_depend.toArray(new Declaration[0]);
        return dependencies_;
    }


    // ==================================================================
    //
    //  Methods for the ConstantDecl interface.
    //
    // ==================================================================

    /**
     **  Set the type.
     **
     **  @param type The TypeRef of the constant declaration type.
     **/
    public void
    setType(TypeRef type)
    {
        if(type != null)
        {
            type_ = type;
            type_.addRef();
        }
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return type_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (type_!=null)
            type_.removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the constant declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
        return constant_def_;
    }
}
