/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * Base class for IDL3 declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class DeclarationImpl
                implements Declaration
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The prefix.
     **/
    private java.lang.String prefix_;

    /**
     **  The name.
     **/
    private java.lang.String name_;

    /**
     **  The version.
     **/
    private java.lang.String version_;

    /**
     **  The repository ID.
     **/
    private java.lang.String id_;

    /**
     **  To protect imported declaration from being destroyed
     **/
    private boolean is_imported_;

    /**
     **  Number of references on this declaration.
     **/
    private int nb_refs_;

    /**
     **  The DeclarationKind.
     **/
    protected int the_declaration_kind_;

    /**
     **  The IDL3 repository
     **/
    protected Repository the_repository_;

    /**
     **  The parent scope.
     **/
    protected ScopeImpl the_parent_;

    /**
     **  To know if the declaration is a mapping declaration.
     **/
    protected boolean is_mapping_;

    /**
     **
     **/
    protected Declaration[] dependencies_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    DeclarationImpl(Repository rep, ScopeImpl parent)
    {
        // Init internal state.
        the_repository_ = rep;
        the_parent_ = parent;
        prefix_ = null;
        name_ = null;
        version_ = "1.0";
        id_ = null;
        is_imported_ = false;
        is_mapping_ = false;
        the_declaration_kind_ = DeclarationKind._dk_null;
        nb_refs_ = 0;
        dependencies_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  To compute the OMG IDL prefix of the Declaration.
     **
     **  @return the prefix in OMG IDL or "" if there's no prefix.
     **/
    protected java.lang.String
    computePrefix()
    {
        int idx1 = id_.indexOf(':');
        int idx2 = id_.indexOf('/');

        java.lang.String pname = the_parent_.getName();
        if (pname==null)
        // the parent is the repository itself
        {
            if (id_.indexOf(name_)==-1)
            // there was a pragma id => no prefix
                return null;
            else if (idx2!=-1)
            // id = IDL:prefix/name:version
                return id_.substring(idx1+1,idx2);
            else
            // id = IDL:name:version
                return null;
        }
        else
        { 
            if (id_.indexOf(name_)==-1)
            // there was a pragma id => no prefix
                return null;
            else if (id_.indexOf(the_parent_.computeBaseRID())==-1)
            // there was a pragma prefix 
            // perhaps a pragma id in fact ????? but can't always know
            // eg 
            // module M {
            //    #pragma id I "OpenCCM/M/I:1.0" <=> #pragma prefix "OpenCCM"
            //    interface I {};
            // };
            // id = IDL:prefix/par_1/.../par_n/name:version
            // TODO : check all parents'name
                return id_.substring(idx1+1, idx2);
            else
            // same prefix as it's parent
                return null;
        }
    }

    /**
     **  Compute the base repository ID.
     **
     **  @return The RepositoryId of the declaration without the version and the IDL fields.
     **/
    protected java.lang.String
    computeBaseRID()
    {
        java.lang.String result = "";

        if (prefix_ != null)
            result = prefix_;
        else if (the_parent_ != null &&
                 the_parent_.getDeclKind() != DeclarationKind._dk_filescope &&
                 the_parent_.getDeclKind() != DeclarationKind._dk_repository)
            result = the_parent_.computeBaseRID();

        if (name_ != null)
        {
            if (result.length() != 0)
                result = result + '/' + name_;
            else
                result = name_;
        }

        return result; 
    }

    /**
     **  To set the declaration as an imported declaration.
     **  Imported declarations can't be destroyed in the IR3.
     **/
    protected void
    setImported()
    {
        is_imported_ = true;
    }

    /**
     **
     **/
    protected Repository
    getRepository()
    {
        return the_repository_;
    }

    /**
     **  Loads infos of the CORBA 3.0 Contained.
     **
     **  @param contained The Contained to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        version_ = contained.version();
        id_ = contained.id();
        prefix_ = computePrefix();
        the_repository_.addDeclInRep(getId(), this);
        if (the_parent_!=null)
            the_parent_.addDecl(this);
    }

    /**
     **  Loads infos of the CORBA 3.0 Contained.
     **
     **  @param contained The Contained to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        version_ = contained.version();
        id_ = contained.id();
        prefix_ = computePrefix();
        the_repository_.addMappedDeclInRep(getId(), this);
        is_mapping_ = true;
        // only add declaration if the parent is itself a mapped declaration !!!
        if ((the_parent_!=null) &&
            (the_parent_.is_mapping_))
            the_parent_.addDecl(this);
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the declaration.
     **/
    abstract protected org.omg.CORBA.Contained
    getContained();

    // ==================================================================
    //
    // Methods for the Declaration interface (creation view).
    //
    // ==================================================================

    /**
     **  Destroy the declaration in the IR3.
     **/
    public void
    destroy()
    {
        if (nb_refs_!=0)
            throw new Error("Declaration can\'t be destroyed : it\'s still referenced");

        org.omg.CORBA.IRObject obj = (org.omg.CORBA.IRObject)getContained();
        if (obj != null)
        {
            // destroy the IR3 Contained if it was not imported.
            if (!is_imported_)
                obj.destroy();

            the_repository_.removeDeclFromRep(getId());
            if (the_parent_!=null)
                the_parent_.removeDecl(name_);
        }
    }

    /**
     **  Set the prefix. For use.
     **
     **  @param prefix The prefix to set.
     **      Note that setting the prefix should only be done when 
     **      a new prefix (ie different from the declared-in scope prefix)
     **      was explicitly declared.
     **/
    public void
    setPrefix(java.lang.String prefix)
    {
        prefix_ = prefix;

        org.omg.CORBA.Contained contained = getContained();
        if (contained != null)
            contained.id(getId());
    }

    /**
     **  Set the name.
     **
     **  @param name The name to set.
     **/
    public void
    setName(java.lang.String name)
    {
        name_ = name;
    }

    /**
     **  Set the version.
     **
     **  @param version The version to set.
     **/
    public void
    setVersion(java.lang.String version)
    {
        version_ = version;
        org.omg.CORBA.Contained contained = getContained();
        if (contained != null)
            contained.version(version);
    }

    /**
     **  Set the repository ID.
     **
     **  @param id The RepositoryId to set.
     **/
    public void
    setId(java.lang.String id)
    {
        id_ = id;
        the_repository_.addDeclInRep(id, this);
        org.omg.CORBA.Contained contained = getContained();
        if (contained != null)
            contained.id(id_);
    }

    /**
     **  Create it into the IR3.
     **/
    public void
    create()
    {
        if (the_parent_!=null)
            the_parent_.addDecl(this);
    }

    // ==================================================================
    //
    // Methods for the Declaration interface (load view).
    //
    // ==================================================================

    /**
     **  Obtain the prefix.
     **
     **  @return The prefix of the declaration
     **           or null if there's no prefix.
     **/
    public java.lang.String
    getPrefix()
    {
        if (prefix_!=null)
            return prefix_;
        if ((the_parent_!=null) && (the_parent_.getName()!=null))
            return the_parent_.getPrefix();

        return "";
    }

    /**
     **  Obtain the name.
     **
     **  @return The name of the declaration.
     **/
    public java.lang.String
    getName()
    {
        return name_;
    }

    /**
     **  Obtain the absolute name.
     **
     **  @return The absolute name of the declaration.
     **/
    public java.lang.String
    getAbsoluteName()
    {
        java.lang.String result = "";
        if(the_parent_ != null)
            result = the_parent_.getAbsoluteName();

        if(name_ != null)
            result = result + "::" + name_;

        return result;
    }

    /**
     **  Obtain the version.
     **
     **  @return The version of the declaration.
     **  Note that version "1.0" is assumed is no version was set.
     **/
    public java.lang.String
    getVersion()
    {
        return version_;
    }

    /**
     **  Obtain the repository ID.
     **
     **  @return The RepositoryId of the declaration.
     **/
    public java.lang.String
    getId()
    {
        if (id_ != null)
            return id_;

        return "IDL:" + computeBaseRID() + ':' + version_;
    }

    /**
     **  Returns the kind of the Declaration object.
     **
     **  @return The DeclarationKind of the object.
     **/
    public int
    getDeclKind()
    {
        return the_declaration_kind_;
    }

    /**
     **  Obtain the parent scope.
     **
     **  @return The parent scope of the declaration
     **          or null if the declaration has no parent.
     **/
    public Scope
    getParent()
    {
        return the_parent_;
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return new Declaration[0];
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        return getDependencies();
    }

    // ==================================================================
    //
    //  Public methods for TypeRef interface.
    //  Implemented here to do some code factorization.
    //
    // ==================================================================

    /**
     **
     **/
    public boolean
    isDeclaration()
    {
        return true;
    }

    // ==================================================================
    //
    //  Public methods for reference counting management.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    addRef()
    {
        nb_refs_++;
    }

    /**
     **
     **/
    public void
    removeRef()
    {
        nb_refs_--;
    }
}
