/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages enum declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EnumDeclImpl
       extends DeclarationImpl
       implements TypeRef, EnumDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 EnumDef.
     **/
    private org.omg.CORBA.EnumDef enum_def_;

    /**
     **  List of members names.
     **/
    private org.objectweb.ccm.util.Vector members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    EnumDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        enum_def_ = null;
        members_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_enum;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 EnumDef.
     **
     **  @param enum_def The EnumDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        enum_def_ = org.omg.CORBA.EnumDefHelper.narrow(contained);
        String[] members = enum_def_.members();
        org.omg.CORBA.TypeCode enumTC = enum_def_.type();
        for (int i=0;i<members.length;i++)
        {
            addMember(members[i]);
            AnyValueDeclImpl decl = new AnyValueDeclImpl(getRepository(), the_parent_);
            decl.setName(members[i]);
            AnyValue av = new AnyValue();
            av.setAsEnum(enumTC, i, members[i]);
            decl.setAnyValue(av);
            decl.create();
        }
        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 EnumDef.
     **
     **  @param enum_def The EnumDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        enum_def_ = org.omg.CORBA.EnumDefHelper.narrow(contained);
        String[] members = enum_def_.members();
        org.omg.CORBA.TypeCode enumTC = enum_def_.type();
        for (int i=0;i<members.length;i++)
        {
            addMember(members[i]);
            AnyValueDeclImpl decl = new AnyValueDeclImpl(getRepository(), the_parent_);
            decl.setName(members[i]);
            AnyValue av = new AnyValue();
            av.setAsEnum(enumTC, i, members[i]);
            decl.setAnyValue(av);
            decl.create();
        }

        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the enum declaration.
     **/
    public void
    create()
    {
        String[] members = new String[members_.size()];
        for(int i=0; i<members.length; i++)
            members[i] = (String)(members_.get(i));

        enum_def_ = the_parent_.getContainer().create_enum(getId(), getName(),
                                                           getVersion(), members);

        org.omg.CORBA.TypeCode enumTC = enum_def_.type();
        for(int i=0; i<members_.size(); i++)
        {
            String name = (String)members_.get(i);
            AnyValueDeclImpl decl = new AnyValueDeclImpl(getRepository(), the_parent_);
            decl.setName(name);
            AnyValue av = new AnyValue();
            av.setAsEnum(enumTC, i, name);
            decl.setAnyValue(av);
            decl.create();
        }

        super.create();
    }

    /**
     **  Return the index declarator.
     **
     **  @param index The index of the element.
     **
     **  @return The enumeration symbolic name of the element
     **          at the specified index.
     **/
    public String
    declaratorAt(int index)
    {
        return (String)members_.get(index);
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return enum_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_enum;
    }

    // ==================================================================
    //
    // Methods for the EnumDecl interface.
    //
    // ==================================================================

    /**
     **  Add a member name.
     **
     **  @param name The name of the member.
     **/
    public void
    addMember(String name)
    {
        members_.add(name);
    }

    /**
     **
     **/
    public String[]
    getMembers()
    {
        String[] result = new String[members_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (String)members_.get(i);
        }
        return result;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the enum declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return enum_def_;
    }
}
