/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages exception declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExceptionDeclImpl
       extends ScopeImpl
       implements ExceptionRef, ExceptionDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ExceptionDef.
     **/
    private org.omg.CORBA.ExceptionDef exception_def_;

    /**
     **  List of members.
     **/
    private StructMembersImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the exception declaration.
     **/
    public
    ExceptionDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        exception_def_ = null;
        members_ = new StructMembersImpl();
        the_declaration_kind_ = DeclarationKind._dk_exception;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Create the exception declaration.
     **/
    protected void
    doCreation()
    {
        exception_def_ = the_parent_.getContainer().create_exception(getId(), getName(), getVersion(),
                                                                        members_.getMemberDefs());
        super.create();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ExceptionDef.
     **
     **  @param exceptionDef The ExceptionDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        exception_def_ = org.omg.CORBA.ExceptionDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = exception_def_.members();

        // for inlined struct, union or enum declarations
        super.load(contained);

        for (int i=0;i<members.length;i++)
        {
            members_.addMember(members[i].name,
                               getRepository().getAsTypeRef((members[i].type_def)));
        }
    }

    /**
     **  Loads infos of the CORBA 3.0 ExceptionDef.
     **
     **  @param exceptionDef The ExceptionDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        exception_def_ = org.omg.CORBA.ExceptionDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = exception_def_.members();

        // for inlined struct, union or enum declarations
        super.loadAsMapping(contained);

        for (int i=0;i<members.length;i++)
            members_.addMember(members[i].name,
                               getRepository().getAsMappedTypeRef((members[i].type_def)));
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Return the index declarator.
     **
     **  @param index The index of the declarator that should be returned.
     **
     **  @return The name of the declarator at the specified index.
     **/
    public String
    declaratorAt(int index)
    {
        return members_.nameAt(index);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        // members
        org.objectweb.ccm.util.Vector exc_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;
        TypeRef[] member_types = getMembers().getMemberTypes();

        for (int i=0;i<member_types.length;i++)
        {
            if (member_types[i].isDeclaration())
            {
                if ((!containsDecl((Declaration)member_types[i])) &&
                    (exc_depend.indexOf(member_types[i])==-1))
                    exc_depend.add(member_types[i]);
            }

            if (recursion) {
                depend = member_types[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((exc_depend.indexOf(depend[j])==-1) &&
                        (!containsDecl(depend[j])))
                        exc_depend.add(depend[j]);
                }
            }
        }

        dependencies_ = (Declaration[])exc_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the Scope interface.
    //
    // ==================================================================

    /**
     **  Close the exception scope.
     **/
    public void
    create()
    {
        if(exception_def_ == null)
           doCreation();
        else
          exception_def_.members(members_.getMemberDefs());
    }

    // ==================================================================
    //
    // Methods for the ExceptionRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its ExceptionDef reference.
     **
     **  @return The ExceptionDef associated with the exception declaration.
     **/
    public org.omg.CORBA.ExceptionDef
    getExceptionDef()
    {
        return exception_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_exception;
    }

    // ==================================================================
    //
    // Methods for the ExceptionDecl interface.
    //
    // ==================================================================

    /**
     **  Obtain the member list.
     **
     **  @return The StructMembers of the exception declaration.
     **/
    public StructMembers
    getMembers()
    {
        return members_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the exception
     **           declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        if(exception_def_ == null)
           doCreation();

        return exception_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        members_.destroy();
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the exception
     **          declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
        if(exception_def_ == null)
           doCreation();

       return exception_def_;
    }
}
