/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages home declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class HomeDeclImpl
       extends InterfaceDeclImpl
       implements HomeRef, HomeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 HomeDef.
     **/
    private org.omg.CORBA.ComponentIR.HomeDef home_def_;

    /**
     **  Reference to the base home.
     **/
    private HomeDeclImpl base_home_;

    /**
     **  Reference to the managed component.
     **/
    private ComponentDeclImpl managed_component_;

    /**
     **  Reference to the primary key.
     **/
    private ValueDeclImpl primary_key_;

    /**
     **
     **/
    private org.objectweb.ccm.util.Vector supported_interfaces_;

    /**
     **
     **/
    private InterfaceDeclImpl client_mapping_;

    /**
     **
     **/
    private InterfaceDeclImpl local_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the home declaration.
     **/
    protected
    HomeDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the InterfaceDefImpl constructor.
        super(rep, parent);

        // Init internal state.
        home_def_ = null;
        base_home_ = null;
        managed_component_ = null;
        primary_key_ = null;
        client_mapping_ = null;
        local_mapping_ = null;
        supported_interfaces_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_home;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef[].
     **
     **  @return An array containing it's supported InterfaceDef.
     **/
    protected org.omg.CORBA.InterfaceDef[]
    getInterfaceDefs()
    {
        org.omg.CORBA.InterfaceDef[] result =
            new org.omg.CORBA.InterfaceDef[supported_interfaces_.size()];

        for(int i=0; i<result.length; i++)
        {
            result[i] = ((InterfaceRef)(supported_interfaces_.get(i))).getExtInterfaceDef();
        }

        return result;
    }

    /**
     **  Obtain the inherited home base.
     **
     **  @return The inherited home base.
     **/
    protected org.omg.CORBA.ComponentIR.HomeDef
    getBaseHomeDef()
    {
        return (base_home_ != null)
              ?(base_home_.getHomeDef()) : null;
    }

    /**
     **
     **/
    protected org.omg.CORBA.ExtValueDef
    getPrimaryKeyDef()
    {
        return (primary_key_ != null)
              ?(primary_key_.getExtValueDef()) : null;
    }

    /**
     **  Loads infos of the CORBA 3.0 HomeDef.
     **
     **  @param home_def The HomeDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        home_def_ = org.omg.CORBA.ComponentIR.HomeDefHelper.narrow(contained);
        if (home_def_.base_home()!=null)
            setBaseHome((HomeRef)getRepository().lookupId(home_def_.base_home().id()));

        setManagedComponent(
            (ComponentRef)getRepository().lookupId(home_def_.managed_component().id()));

        if (home_def_.primary_key()!=null)
            setPrimaryKey((ValueRef)getRepository().lookupId(home_def_.primary_key().id()));

        org.omg.CORBA.InterfaceDef[] supported = home_def_.supported_interfaces();
        for (int i=0;i<supported.length;i++)
   	        // WARNING: This will break consistency of array items!!!
            addSupportedInterface((InterfaceRef)getRepository().lookupId(supported[i].id()));

        // load it's mapped declarations
        getClientMapping();

        super.load(contained);
    }

    /**
     **  Find a declaration into inherited interfaces.
     **
     **  @param name The name of the searched declaration.
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    protected Declaration
    findInSupportedInterfaces(String name)
    {
        for(int i=0; i<supported_interfaces_.size(); i++)
        {
            Declaration decl = ((InterfaceDecl)(supported_interfaces_.get(i))).find(name);
            // If found then returns it.
            if (decl != null) return decl;
        }

        // Else not found.
        return null;
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Find a declaration.
     **
     **  Note that the declaration is also recursively searched in the
     **  parent scope.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    public Declaration
    find(String name)
    {
        // Finds in the scope.
        Declaration decl = super.find(name);
        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in inherited interfaces.
        if (base_home_ != null)
        {
            decl = base_home_.find(name);
            if(decl != null) return decl;
        }

        return findInSupportedInterfaces(name);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector home_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // base home
        if (getBaseHome()!=null)
        {
            home_depend.add(getBaseHome());
            if (recursion) {
                depend = getBaseHome().getDependencies();
                for (int j=0;j<depend.length;j++)
                    home_depend.add(depend[j]);
            }
        }

        // primary key
        if (getPrimaryKey()!=null)
        {
            home_depend.add(getPrimaryKey());
            if (recursion) {
                depend = getPrimaryKey().getDependencies();
                for (int j=0;j<depend.length;j++)
                    home_depend.add(depend[j]);
            }
        }

        // managed component
        home_depend.add(getManagedComponent());
        if (recursion) {
            depend = getManagedComponent().getDependencies();
            for (int j=0;j<depend.length;j++)
                home_depend.add(depend[j]);
        }

        // supported interfaces
        InterfaceDecl[] supp = getSupportedInterfaces();
        for (int i=0;i<supp.length;i++)
        {
            home_depend.add(supp[i]);
            if (recursion) {
                depend = supp[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if (home_depend.indexOf(depend[j])==-1)
                        home_depend.add(depend[j]);
                }
            }
        }

        // contents
        Declaration[] decls = getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_all);
        for (int i=0;i<decls.length;i++)
        {
            depend = decls[i].getDependencies(recursion);
            for (int j=0;j<depend.length;j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (home_depend.indexOf(depend[j])==-1))
                    home_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])home_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the home declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return home_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_home;
    }

    // ==================================================================
    //
    // Methods for the InterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the home declaration.
     **/
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return home_def_;
    }

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the interface
     **          declaration.
     **/
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return home_def_;
    }

    // ==================================================================
    //
    // Methods for the HomeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its HomeDef reference.
     **
     **  @return The HomeDef object associated with the home declaration.
     **/
    public org.omg.CORBA.ComponentIR.HomeDef
    getHomeDef()
    {
        return home_def_;
    }

    // ==================================================================
    //
    // Methods for the HomeDecl interface.
    //
    // ==================================================================

    /**
     **  Add an interface.
     **
     **  @param itf The supported interface to add.
     **/
    public void
    addSupportedInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            itf.addRef();
            supported_interfaces_.add(itf);
        }
    }

    /**
     **  Set the base home.
     **
     **  @param base_home The base home.
     **/
    public void
    setBaseHome(HomeRef base_home)
    {
        if (base_home != null)
        {
            base_home_ = (HomeDeclImpl)base_home;
            base_home_.addRef();
        }
    }

    /**
     **  Set the managed component.
     **
     **  @param managed_component The managed component.
     **/
    public void
    setManagedComponent(ComponentRef managed_component)
    {
        if (managed_component != null)
        {
            managed_component_ = (ComponentDeclImpl)managed_component;
            managed_component_.addRef();
        }
    }

    /**
     **  Set the primary key.
     **
     **  @param primary_key The primary key.
     **/
    public void
    setPrimaryKey(ValueRef primary_key)
    {
        if (primary_key != null)
        {
            primary_key_ = (ValueDeclImpl)primary_key;
            primary_key_.addRef();
        }
    }

    /**
     **
     **/
    public HomeDecl
    getBaseHome()
    {
        return base_home_;
    }

    /**
     **
     **/
    public ComponentDecl
    getManagedComponent()
    {
        return managed_component_;
    }

    /**
     **
     **/
    public ValueDecl
    getPrimaryKey()
    {
        return primary_key_;
    }

    /**
     **
     **/
    public InterfaceDecl[]
    getSupportedInterfaces()
    {
        InterfaceDecl[] result = new InterfaceDecl[supported_interfaces_.size()];
        for(int i=0; i<result.length; i++)
            result[i] = (InterfaceDecl)supported_interfaces_.get(i);

        return result;
    }

    /**
     **
     **/
    public InterfaceDecl
    getClientExplicitMapping()
    {
        // retrieve client mapping interface.
        InterfaceDecl client = getClientMapping();
        InterfaceDecl[] bases = client.getBaseInterfaces();
        if (bases[0].getName().equals(getName()+"Explicit"))
            return bases[0];
        else
            return bases[1];
    }

    /**
     **
     **/
    public InterfaceDecl
    getClientImplicitMapping()
    {
        // retrieve client mapping interface.
        InterfaceDecl client = getClientMapping();
        InterfaceDecl[] bases = client.getBaseInterfaces();
        if (bases[0].getName().equals(getName()+"Implicit"))
            return bases[0];
        else
            return bases[1];
    }

    /**
     **
     **/
    public InterfaceDecl
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // load the client mapping for this component.
        client_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), getId());

        return client_mapping_;
    }

    /**
     **
     **/
    public InterfaceDecl
    getLocalMapping()
    {
        if (local_mapping_!=null)
            return local_mapping_;

        // load the local mapping for this home.
        String parent_base_id = the_parent_.getId();
        int idx = parent_base_id.lastIndexOf(':');
        parent_base_id = parent_base_id.substring(0, idx);
        String id = parent_base_id + "/CCM_"+getName()+":"+getVersion();

        local_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getParent(), id);
        return local_mapping_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (base_home_!=null)
            base_home_.removeRef();

        if (managed_component_!=null)
            managed_component_.removeRef();

        if (primary_key_!=null)
            primary_key_.removeRef();

        InterfaceDecl[] itfs = getSupportedInterfaces();
        for (int i=0;i<itfs.length;i++)
            ((InterfaceRef)itfs[i]).removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the home declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return home_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        home_def_ = the_parent_.getComponentContainer().create_home(getId(),
                                                                    getName(),
                                                                    getVersion(),
                                                                    getBaseHomeDef(),
                                                                    managed_component_.getComponentDef(),
                                                                    getInterfaceDefs(),
                                                                    getPrimaryKeyDef());
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the home declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return home_def_;
    }

    /**
     **  To obtain all the contained Declaration objects.
     **
     **  @param exclude_inherited If false return also objects contained in inherited scopes.
     **  @param limited_types A logical combination of DeclarationKind.
     **
     **  @return An array of Declaration objects.
     **/
    public Declaration[]
    getContents(boolean exclude_inherited, int limited_types)
    {
        if ((exclude_inherited) || (base_home_==null))
            return super.getContents(exclude_inherited, limited_types);

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = getSupportedInterfaces();
        for (int i=0;i<itfs.length;i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }
        Declaration[] tmp1 = base_home_.getContents(exclude_inherited, limited_types);
        Declaration[] tmp2 = res;
        res = new Declaration[tmp1.length+tmp2.length];
        System.arraycopy(tmp2, 0, res, 0, tmp2.length);
        System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        return res;
    }
}
