/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL interface declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class InterfaceDeclImpl
       extends ForwardScopeImpl
       implements InterfaceRef, InterfaceDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 InterfaceDef.
     **/
    private org.omg.CORBA.ExtInterfaceDef ext_interface_def_;

    /**
     **  List of inherited interfaces.
     **/
    protected org.objectweb.ccm.util.Vector inherited_interfaces_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the interface declaration.
     **/
    protected 
    InterfaceDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ForwardScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_interface_def_ = null;
        inherited_interfaces_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_interface;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef[].
     **
     **  @return An array containing it's inherited InterfaceDef.
     **/
    protected org.omg.CORBA.InterfaceDef[]
    getInterfaceDefs()
    {
        org.omg.CORBA.InterfaceDef[] result =
            new org.omg.CORBA.InterfaceDef[inherited_interfaces_.size()];

        for(int i=0; i<result.length; i++)
        {
            result[i] = ((InterfaceRef)(inherited_interfaces_.get(i))).getExtInterfaceDef();
        }

        return result;
    }

    /**
     **  Find a declaration into inherited interfaces.
     **
     **  @param name The name of the searched declaration.
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    protected Declaration
    findInInheritedInterfaces(String name)
    {
        for(int i=0; i<inherited_interfaces_.size(); i++)
        {
            Declaration decl = ((InterfaceDecl)(inherited_interfaces_.get(i))).find(name);
            // If found then returns it.
            if (decl != null) return decl;
        }

        // Else not found.
        return null;
    }

    /**
     **  Loads infos of the CORBA 3.0 InterfaceDef.
     **
     **  @param interfaceDef The InterfaceDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_interface_def_ = org.omg.CORBA.ExtInterfaceDefHelper.narrow(contained);
        org.omg.CORBA.InterfaceDef[] base_interfaces = ext_interface_def_.base_interfaces();

        for (int i=0;i<base_interfaces.length;i++)
	    // WARNING: This will break consistency of array items!!!
            addInterface((InterfaceRef)getRepository().lookupId(base_interfaces[i].id()));

        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 InterfaceDef.
     **
     **  @param interfaceDef The InterfaceDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_interface_def_ = org.omg.CORBA.ExtInterfaceDefHelper.narrow(contained);
        org.omg.CORBA.InterfaceDef[] base_interfaces = ext_interface_def_.base_interfaces();

        for (int i=0;i<base_interfaces.length;i++)
	    // WARNING: This will break consistency of array items!!!
            addInterface((InterfaceRef)getRepository().lookupMappedId(base_interfaces[i].id()));

        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    // Methods for the Scope interface.
    //
    // ==================================================================

    /**
     **  Find a declaration.
     **
     **  Note that the declaration is also recursively searched
     **  in the parent scope.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    public Declaration
    find(String name)
    {
        // Finds in the scope.
        Declaration decl = super.find(name);

        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in inherited interfaces.
        return findInInheritedInterfaces(name);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        org.objectweb.ccm.util.Vector itf_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // base interfaces
        InterfaceDecl[] bases = getBaseInterfaces();
        for (int i=0;i<bases.length;i++)
        {
            itf_depend.add(bases[i]);
            if (recursion) {
                depend = bases[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((depend[j]!=this) &&
                        (itf_depend.indexOf(depend[j])==-1))
                        itf_depend.add(depend[j]);
                }
            }
        }

        // contents
        Declaration[] decls = getContents(true, org.objectweb.ccm.IDL3.DeclarationKind._dk_all);
        for (int i=0;i<decls.length;i++)
        {
            depend = decls[i].getDependencies(recursion);
            for (int j=0;j<depend.length;j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (depend[j]!=this) &&
                    (itf_depend.indexOf(depend[j])==-1))
                    itf_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])itf_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the interface declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_interface_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_interface;
    }

    // ==================================================================
    //
    // Methods for the InterfaceRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the interface
     **          declaration.
     **/
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return ext_interface_def_;
    }

    /**
     **  Obtain its InterfaceDef reference.
     **
     **  @return The InterfaceDef object associated with the interface
     **          declaration.
     **/
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return ext_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the InterfaceDecl interface (parser view).
    //
    // ==================================================================

    /**
     **  Add an interface.
     **
     **  @param itf The base interface to add.
     **/
    public void
    addInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            itf.addRef();
            inherited_interfaces_.add(itf);
        }
    }

    // ==================================================================
    //
    // Methods for the InterfaceDecl interface (visitor view).
    //
    // ==================================================================

    /**
     **
     **/
    public InterfaceDecl[]
    getBaseInterfaces()
    {
        InterfaceDecl[] result = new InterfaceDecl[inherited_interfaces_.size()];

        for(int i=0; i<result.length; i++)
        {
            result[i] = (InterfaceDecl)inherited_interfaces_.get(i);
        }
        return result;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        for (int i=0;i<inherited_interfaces_.size();i++)
        {
            ((InterfaceRef)inherited_interfaces_.get(i)).removeRef();
        }
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the interface
     **          declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
        return ext_interface_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the interface
     **          declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        return ext_interface_def_;
    }

    /**
     **  Create an attribute definition.
     **
     **  @param attribute The attribute declaration.
     **  @param type The attribute IDLType.
     **  @param mode The attribute mode.
     **  @param get_exceptions An array containing the getRaises
     **                       (or raises for readonly attributes) exceptions.
     **  @param put_exceptions An array containing the setRaises exceptions.
     **
     **  @return The new created AttributeDef.
     **/
    protected org.omg.CORBA.ExtAttributeDef
    createExtAttribute(AttributeDecl attribute,
                       org.omg.CORBA.IDLType type,
                       org.omg.CORBA.AttributeMode mode,
                       org.omg.CORBA.ExceptionDef[] get_exceptions,
                       org.omg.CORBA.ExceptionDef[] set_exceptions)
    {
       // Create an AttributeDef into the IFR.
        org.omg.CORBA.ExtAttributeDef ext_attribute_def =
            getExtInterfaceDef().create_ext_attribute(attribute.getId(),
                                                      attribute.getName(),
                                                      attribute.getVersion(),
                                                      type, mode,
                                                      get_exceptions,
                                                      set_exceptions);
        return ext_attribute_def;
    }

    /**
     **  Create an operation.
     **
     **  @param operation The operation declaration.
     **  @param type The operation return IDLType.
     **  @param mode The mode of the operation.
     **  @param params An array containing the parameters description.
     **  @param exceptions An array containing the exceptions that
     **                    can be raised.
     **  @param contexts An array containing the possible context values.
     **
     **  @return The new created OperationDef.
     **/
    protected org.omg.CORBA.OperationDef
    createOperation(OperationDecl operation,
                    org.omg.CORBA.IDLType type,
                    org.omg.CORBA.OperationMode mode,
                    org.omg.CORBA.ParameterDescription[] params,
                    org.omg.CORBA.ExceptionDef[] exceptions,
                    String[] contexts)
    {
        // Create an OperationDef into the IFR.
        org.omg.CORBA.OperationDef operation_def =
            getExtInterfaceDef().create_operation(operation.getId(),
                                                  operation.getName(),
                                                  operation.getVersion(),
                                                  type, mode,
                                                  params,
                                                  exceptions,
                                                  contexts);
        return operation_def;
    }

    /**
     **  To obtain all the contained Declaration objects.
     **
     **  @param exclude_inherited If false return also objects contained in inherited scopes.
     **  @param limited_types A logical combination of DeclarationKind.
     **
     **  @return An array of Declaration objects.
     **/
    public Declaration[]
    getContents(boolean exclude_inherited, int limited_types)
    {
        if (exclude_inherited)
            return super.getContents(exclude_inherited, limited_types);

        // if it's a mapping scope, then use the OpenCCM repository as an IDL2 repository
        // to retrieve contained objects.
        if (is_mapping_)
            getRepository().useIDL2Repository();

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = getBaseInterfaces();
        for (int i=0;i<itfs.length;i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }

        // get back to an IDL3 repository.
        if (is_mapping_)
            getRepository().useIDL3Repository();

        return res;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        ext_interface_def_ = the_parent_.getContainer().create_ext_interface(getId(),getName(), getVersion(),
                                                                             getInterfaceDefs());
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        org.omg.CORBA.InterfaceDef[] inherited_interfaces = getInterfaceDefs();
        if (inherited_interfaces.length != 0)
            getExtInterfaceDef().base_interfaces(inherited_interfaces);
    }
}
