/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL operation declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class OperationDeclImpl
       extends DeclarationImpl
       implements OperationDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 OperationDef.
     **/
    private org.omg.CORBA.OperationDef operation_def_;

    /**
     **  The operation mode.
     **/
    protected int mode_;

    /**
     **  The result type.
     **/
    protected TypeRef result_;

    /**
     **  List of parameters.
     **/
    protected ParameterListImpl parameters_;

    /**
     **  The operation exceptions.
     **/
    protected ExceptionListImpl exceptions_;

    /**
     **  List of contexts.
     **/
    private org.objectweb.ccm.util.Vector contexts_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the operation declaration.
     **/
    protected
    OperationDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        operation_def_ = null;
        mode_ = OperationDecl.OP_NORMAL;
        result_ = null;
        parameters_ = new ParameterListImpl();
        exceptions_ = new ExceptionListImpl();
        contexts_ = new org.objectweb.ccm.util.Vector();
        the_declaration_kind_ = DeclarationKind._dk_operation;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Add a parameter.
     **
     **  @param name The name of the parameter.
     **  @param type The TypeRef of the parameter.
     **  @param mode The mode of the parameter.
     **/
    protected void
    addParam(String name,
             TypeRef type,
             org.omg.CORBA.ParameterMode mode)
    {
        parameters_.addParam(name, type, mode.value());
    }

    /**
     **  Loads infos of the CORBA 3.0 OperationDef.
     **
     **  @param operation_def The OperationDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        operation_def_ = org.omg.CORBA.OperationDefHelper.narrow(contained);
        mode_ = operation_def_.mode().value();
        setResultType(getRepository().getAsTypeRef(operation_def_.result_def()));
        org.omg.CORBA.ParameterDescription[] params = operation_def_.params();
        for (int i=0;i<params.length;i++)
        {
            addParam(params[i].name,
                     getRepository().getAsTypeRef(params[i].type_def),
                     params[i].mode);
        }
        org.omg.CORBA.ExceptionDef[] exc = operation_def_.exceptions();
        for (int i=0;i<exc.length;i++)
        {
            exceptions_.add((ExceptionRef)getRepository().lookupId(exc[i].id()));
        }
        String[] contexts = operation_def_.contexts();
        for (int i=0;i<contexts.length;i++)
            addContext(contexts[i]);

        super.load(contained);
    }

    /**
     **  Loads infos of the CORBA 3.0 OperationDef.
     **
     **  @param operation_def The OperationDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        operation_def_ = org.omg.CORBA.OperationDefHelper.narrow(contained);
        mode_ = operation_def_.mode().value();
        setResultType(getRepository().getAsMappedTypeRef(operation_def_.result_def()));
        org.omg.CORBA.ParameterDescription[] params = operation_def_.params();
        for (int i=0;i<params.length;i++)
        {
            addParam(params[i].name,
                     getRepository().getAsMappedTypeRef(params[i].type_def),
                     params[i].mode);
        }
        org.omg.CORBA.ExceptionDef[] exc = operation_def_.exceptions();
        for (int i=0;i<exc.length;i++)
        {
            exceptions_.add((ExceptionRef)getRepository().lookupMappedId(exc[i].id()));
        }
        String[] contexts = operation_def_.contexts();
        for (int i=0;i<contexts.length;i++)
            addContext(contexts[i]);

        super.loadAsMapping(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the operation declaration into the IFR.
     **/
    public void
    create()
    {
        String[] contexts = new String[contexts_.size()];
        for(int i=0; i<contexts.length; i++)
            contexts[i] = (String)(contexts_.get(i));

        operation_def_ = the_parent_.createOperation(this,
                                                     result_.getIDLType(),
                                                     org.omg.CORBA.OperationMode.from_int(mode_),
                                                     parameters_.getParameterDescs(),
                                                     exceptions_.getExceptionDefs(),
                                                     contexts);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector op_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // return type
        if (getType().isDeclaration())
            op_depend.add(getType());

        if (recursion) {
            depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                op_depend.add(depend[i]);
        }

        // exceptions
        ExceptionDecl[] excs = getExceptions();
        for (int i=0;i<excs.length;i++)
        {
            op_depend.add(excs[i]);
            if (recursion) {
                depend = excs[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if (op_depend.indexOf(depend[j])==-1)
                        op_depend.add(depend[j]);
                }
            }
        }

        // parameters type
        TypeRef[] param_types = getParameters().getParamTypes();
        for (int i=0;i<param_types.length;i++)
        {
            if (param_types[i].isDeclaration())
                op_depend.add(param_types[i]);

            if (recursion) {
                depend = param_types[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if (op_depend.indexOf(depend[j])==-1)
                        op_depend.add(depend[j]);
                }
            }
        }

        dependencies_ = (Declaration[])op_depend.toArray(new Declaration[0]);
        return dependencies_;
    }



    // ==================================================================
    //
    // Methods for the WithInParameters interface.
    //
    // ==================================================================

    /**
     **  Add a in parameter.
     **
     **  @param name The name of the parameter.
     **  @param type The TypeRef of the parameter.
     **/
    public void
    addInParam(String name,
               TypeRef type)
    {
        addParam(name, type, org.omg.CORBA.ParameterMode.PARAM_IN);
    }

    // ==================================================================
    //
    // Methods for the OperationDecl interface.
    //
    // ==================================================================

    /**
     **  Set mode as normal.
     **/
    public void
    setNormal()
    {
        mode_ = OperationDecl.OP_NORMAL;
    }

    /**
     **  Set mode as oneway.
     **/
    public void
    setOneway()
    {
        mode_ = OperationDecl.OP_ONEWAY;
    }

    /**
     **  Set the result type.
     **
     **  @param type The TypeRef of the returned object.
     **/
    public void
    setResultType(TypeRef type)
    {
        if(type != null)
        {
            result_ = type;
            result_.addRef();
        }
    }

    /**
     **  Add an out parameter.
     **
     **  @param name The name of the parameter.
     **  @param type The TypeRef of the parameter.
     **/
    public void
    addOutParam(String name,
                TypeRef type)
    {
        addParam(name, type, org.omg.CORBA.ParameterMode.PARAM_OUT);
    }

    /**
     **  Add an inout parameter.
     **
     **  @param name The name of the parameter.
     **  @param type The TypeRef of the parameter.
     **/
    public void
    addInOutParam(String name,
                  TypeRef type)
    {
        addParam(name, type, org.omg.CORBA.ParameterMode.PARAM_INOUT);
    }

    /**
     **  Add a context.
     **
     **  @param name The name of the context.
     **/
    public void
    addContext(String name)
    {
        contexts_.add(name);
    }

    /**
     **
     **/
    public boolean
    isOneway()
    {
        return mode_==OperationDecl.OP_ONEWAY;
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return result_;
    }

    /**
     **
     **/
    public ExceptionList
    getExceptionList()
    {
        return exceptions_;
    }

    /**
     **
     **/
    public ExceptionDecl[]
    getExceptions()
    {
        return exceptions_.getExceptions();
    }

    /**
     **
     **/
    public ParameterList
    getParameters()
    {
        return parameters_;
    }

    /**
     **
     **/
    public String[]
    getContexts()
    {
        String[] result = new String[contexts_.size()];
        for (int i=0;i<result.length;i++)
        {
            result[i] = (String)contexts_.get(i);
        }
        return result;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (result_!=null)
            result_.removeRef();

        exceptions_.destroy();
        parameters_.destroy();
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the
     **          operation declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return operation_def_;
    }
}
