/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL provides port declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ProvidesDeclImpl
       extends DeclarationImpl
       implements ProvidesDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ProvidesDef.
     **/
    private org.omg.CORBA.ComponentIR.ProvidesDef provides_def_;

    /**
     **  The port interface type.
     **/
    private InterfaceDeclImpl interface_;

    /**
     **
     **/
    private OperationDeclImpl[] client_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the provides declaration.
     **/
    protected
    ProvidesDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        provides_def_ = null;
        client_mapping_ = null;
        interface_ = null;
        the_declaration_kind_ = DeclarationKind._dk_provides;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ProvidesDef.
     **
     **  @param provides_def The ProvidesDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        provides_def_ = org.omg.CORBA.ComponentIR.ProvidesDefHelper.narrow(contained);
        setInterface((InterfaceRef)getRepository().lookupId(provides_def_.interface_type().id()));
        super.load(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the provides declaration into the IFR.
     **/
    public void
    create()
    {
        provides_def_ = the_parent_.getComponentDef().create_provides(getId(), getName(),
                                                                      getVersion(),
                                                                      interface_.getExtInterfaceDef());

        super.create();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector provides_depend = new org.objectweb.ccm.util.Vector();

        // content
        if (getType().isDeclaration())
            provides_depend.add(getType());

        if (recursion) {
            Declaration[] depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                provides_depend.add(depend[i]);
        }


        dependencies_ = (Declaration[])provides_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    //  Methods for the ProvidesDecl interface.
    //
    // ==================================================================

    /**
     **  Set the interface type.
     **
     **  @param itf The provided interface reference.
     **/
    public void
    setInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            interface_ = (InterfaceDeclImpl)itf;
            interface_.addRef();
        }
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return interface_;
    }

    /**
     **
     **/
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((ComponentDecl)the_parent_).getClientMapping();
        client.getContents(true, DeclarationKind._dk_null);

        client_mapping_ = new OperationDeclImpl[1];
        client_mapping_[0] = (OperationDeclImpl)client.find("provide_"+getName());

        return client_mapping_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (interface_!=null)
            interface_.removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the provides declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return provides_def_;
    }
}
