/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This interface manages definition scopes.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public interface Scope
       extends Declaration
{
    //
    //  Parser view of a scope.
    //

    /**
     * Start a file scope.
     *
     * @param fileName the name of the file represented by this FileScope.
     * @param isIncluded if set to true, the declarations will be stored
     *        in the new FileScope.
     *        Otherwise, they will be stored in this object.
     *
     * @return The new created FileScope.
     */
    public FileScope
    startFileScope(String filename);

    /**
     * Start a constant declaration.
     *
     * @param name The name of the constant declaration.
     *
     * @return The new created ConstantDecl.
     */
    public ConstantDecl
    startConstant(String name);

    /**
     * Declare a struct declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the struct declaration.
     *
     * @return The new created StructDecl.
     */
    public StructDecl
    declareStruct(String name);

    /**
     * Declare an union declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the union declaration.
     *
     * @return The new created UnionDecl.
     */
    public UnionDecl
    declareUnion(String name);

    /**
     * Start an enum declaration.
     *
     * @param name The name of the enum declaration.
     *
     * @return The new created EnumDecl.
     */
    public EnumDecl
    startEnum(String name);

    /**
     * Start an alias declaration.
     *
     * @param name The name of the alias declaration.
     *
     * @return The new created AliasDecl.
     */
    public AliasDecl
    startAlias(String name);

    /**
     * Find a module declaration
     * If it doesn't exist, create it.
     *
     * @param The name of the module declaration.
     *
     * @return The new created ModuleDecl or a already created ModuleDecl.
     */
    public ModuleDecl
    declareModule(String name);

    /**
     * Declare an interface declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the interface declaration.
     *
     * @return The new created InterfaceDecl or
     *         an already created InterfaceDecl.
     */
    public InterfaceDecl
    declareInterface(String name);

    /**
     * Declare an abstract interface declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the abstract interface declaration.
     *
     * @return The new created AbstractInterfaceDecl or
     *         an already created AbstractInterfaceDecl.
     */
    public AbstractInterfaceDecl
    declareAbstractInterface(String name);

    /**
     * Declare a local interface declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the local interface declaration.
     *
     * @return The new created LocalInterfaceDecl or
     *         an already created LocalInterfaceDecl.
     */
    public LocalInterfaceDecl
    declareLocalInterface(String name);

    /**
     * Declare a value type declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the value type declaration.
     *
     * @return The new created ValueDecl or an already created ValueDecl.
     */
    public ValueDecl
    declareValue(String name);

    /**
     * Start an initializer declaration.
     *
     * @param name The name of the initializer declaration.
     *
     * @return The new created Initializer.
     */
    public Initializer
    startInitializer(String name);

    /**
     * Start a value member declaration.
     *
     * @param name The name of the value member declaration.
     *
     * @return The new created ValueMemberDecl.
     */
    public ValueMemberDecl
    startValueMember(String name);

    /**
     * Start a value box declaration.
     *
     * @param name The name of the value box declaration.
     *
     * @return The new created ValueBoxDecl.
     **/
    public ValueBoxDecl
    startValueBox(String name);

    /**
     * Start an exception declaration.
     *
     * @param name The name of the exception declaration.
     *
     * @return The new created ExceptionDecl.
     **/
    public ExceptionDecl
    startException(String name);

    /**
     * Start a native declaration.
     *
     * @param name The name of the native declaration.
     *
     * @return The new created NativeDecl.
     */
    public NativeDecl
    startNative(String name);

    /**
     * Start an attribute declaration.
     *
     * @param name The name of the attribute declaration.
     *
     * @return The new created AttributeDecl.
     */
    public AttributeDecl
    startAttribute(String name);

    /**
     * Start an operation declaration.
     *
     * @param name The name of the operation declaration.
     *
     * @return The new created OperationDecl.
     */
    public OperationDecl
    startOperation(String name);

    /**
     * Declare a component declaration.
     * If it doesn't exist, create it.
     *
     * @param name The name of the component declaration.
     *
     * @return The new created ComponentDecl or
     *         an already created ComponentDecl.
     */
    public ComponentDecl
    declareComponent(String name);

    /**
     * Start a factory declaration.
     *
     * @param name The name of the factory declaration.
     *
     * @return The new created FactoryDecl.
     */
    public FactoryDecl
    startFactory(String name);

    /**
     * Start a finder declaration.
     *
     * @param name The name of the finder declaration.
     *
     * @return The new created FinderDecl.
     */
    public FinderDecl
    startFinder(String name);

    /**
     * Start a provides declaration.
     *
     * @param name The name of the provides declaration.
     *
     * @return The new created ProvidesDecl.
     */
    public ProvidesDecl
    startProvides(String name);

    /**
     * Start a uses declaration.
     *
     * @param name The name of the uses declaration.
     *
     * @return The new created UsesDecl.
     */
    public UsesDecl
    startUses(String name);

    /**
     * Start a consumes declaration.
     *
     * @param name The name of the consumes declaration.
     *
     * @return The new created ConsumesDecl.
     */
    public ConsumesDecl
    startConsumes(String name);

    /**
     * Start an emits declaration.
     *
     * @param name The name of the emits declaration.
     *
     * @return The new created EmitsDecl.
     */
    public EmitsDecl
    startEmits(String name);

    /**
     * Start a publishes declaration.
     *
     * @param name The name of the publishes declaration.
     *
     * @return The new created PublishesDecl.
     */
    public PublishesDecl
    startPublishes(String name);

    /**
     * Start a home declaration.
     *
     * @param name The name of the home declaration.
     *
     * @return The new created HomeDecl.
     */
    public HomeDecl
    startHome(String name);

    /**
     * Declare a event type declaration.
     * If it not exists, create it.
     *
     * @param name The name of the event type declaration.
     *
     * @return The new created EventDecl or an already created EventDecl.
     */
    public EventDecl
    declareEvent(String name);

    /**
     * Find a declaration.
     *
     * Note that the declaration is also recursively
     * searched in the parent scope and finally searched
     * in the IR3.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    public Declaration
    find(String name);

    //
    //  Visitor view of a declaration.
    //

    /**
     * To obtain all the contained Declaration objects.
     *
     * @param exclude_inherited If false return also objects contained in inherited scopes.
     * @param limited_types A logical combination of DeclarationKind.
     *
     * @return An array of Declaration objects.
     */
    public Declaration[]
    getContents(boolean exclude_inherited,
                int limited_types);

    /**
     * To find a contained Declaration according to it's scoped name.
     *
     * @param n The scoped name.
     *
     * @return The found or loaded from the IR3 Declaration or null.
     */
    public Declaration
    lookup(String scoped_name);
}
