/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages definition scopes.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class ScopeImpl
                extends DeclarationImpl
                implements Scope
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  To store Declaration names.
     **  It's used to keep the order of the declarations.
     **/
    protected org.objectweb.ccm.util.Vector contained_decls_;

    /**
     **  To know if the contained objects have been loaded.
     **/
    protected boolean content_loaded_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    ScopeImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        contained_decls_ = new org.objectweb.ccm.util.Vector();
        content_loaded_ = false;
    }

    // ==================================================================
    //
    //  Internal methods.
    //
    // ==================================================================

    /**
     **  Add a contained declaration.
     **
     **  @param decl The declaration.
     **/
    protected void
    addDecl(Declaration decl)
    {
        //        System.err.println("### adding declaration : "+decl.getName()+" in scope : "+getName());
        contained_decls_.add(decl);
    }

    /**
     **  To know if a declaration is a contained declaration.
     **
     **  @return True if the declaration is contained, false otherwise.
     **  @param decl The declaration.
     **/
    protected boolean
    containsDecl(Declaration decl)
    {
        return ( decl.getAbsoluteName().startsWith(getAbsoluteName()+"::") );
    }


    /**
     **  Remove a contained declaration.
     **/
    protected void
    removeDecl(String name)
    {
        Declaration decl = null;
        for (int i=0;i<contained_decls_.size();i++)
        {
            decl = (Declaration)contained_decls_.get(i);
            if (decl.getName().equalsIgnoreCase(name))
                contained_decls_.remove(i);
        }
    }

    /**
     **
     **/
    protected Declaration
    createDeclaration(int kind,
                      java.lang.String name)
    {
        Declaration decl = null;
        switch (kind)
        {
            // IDL2 objects
            case org.omg.CORBA.DefinitionKind._dk_Interface :
                decl = startInterface(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_LocalInterface :
                decl = startLocalInterface(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_AbstractInterface :
                decl = startAbstractInterface(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Module :
                decl = startModule(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Struct :
                decl = startStruct(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Value :
                decl = startValue(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Union :
                decl = startUnion(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Exception :
                decl = startException(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Constant :
                decl = startConstant(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Enum :
                decl = startEnum(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Alias :
                decl = startAlias(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Attribute :
                decl = startAttribute(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Operation :
                decl = startOperation(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_ValueBox :
                decl = startValueBox(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Native :
                decl = startNative(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_ValueMember :
                decl = startValueMember(name);
                break;

            // IDL3 objects
            case org.omg.CORBA.DefinitionKind._dk_Component :
                decl = startComponent(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Home :
                decl = startHome(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Factory :
                decl = startFactory(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Finder :
                decl = startFinder(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Emits :
                decl = startEmits(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Publishes :
                decl = startPublishes(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Consumes :
                decl = startConsumes(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Provides :
                decl = startProvides(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Uses :
                decl = startUses(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Event :
                decl = startEvent(name);
                break;
            default :
                // should not happen
                break;
        }

        return decl;
    }


    /**
     **  To load an Contained object.
     **
     **  @param contained The Contained object.
     **
     **  @return The newly created Declaration object.
     **/
    protected Declaration
    loadContained(org.omg.CORBA.Contained contained)
    {
        DeclarationImpl decl = null;

        decl = (DeclarationImpl)createDeclaration(contained.def_kind().value(), contained.name());
        decl.load(contained);
        decl.setImported();

        return decl;
    }

    /**
     **  To load an Contained object.
     **
     **  @param contained The Contained object.
     **
     **  @return The newly created Declaration object.
     **/
    protected Declaration
    loadContainedAsMapping(org.omg.CORBA.Contained contained)
    {
        DeclarationImpl decl = null;

        decl = (DeclarationImpl)createDeclaration(contained.def_kind().value(), contained.name());
        decl.loadAsMapping(contained);

        return decl;
    }

    /**
     **  Start a struct declaration.
     **
     **  @return The new created StructDecl.
     **/
    protected StructDecl
    startStruct(String name)
    {
        StructDecl decl = new StructDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an union declaration.
     **
     **  @return The new created UnionDecl.
     **/
    protected UnionDecl
    startUnion(String name)
    {
        UnionDecl decl = new UnionDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an interface declaration.
     **
     **  @return The new created InterfaceDecl.
     **/
    protected InterfaceDecl
    startInterface(String name)
    {
        InterfaceDecl decl = new InterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an local interface declaration.
     **
     **  @return The new created LocalInterfaceDecl.
     **/
    protected LocalInterfaceDecl
    startLocalInterface(String name)
    {
        LocalInterfaceDecl decl = new LocalInterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an abstract interface declaration.
     **
     **  @return The new created AbstractInterfaceDecl.
     **/
    protected AbstractInterfaceDecl
    startAbstractInterface(String name)
    {
        AbstractInterfaceDecl decl = new AbstractInterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an value declaration.
     **
     **  @return The new created ValueDecl.
     **/
    protected ValueDecl
    startValue(String name)
    {
        ValueDecl decl = new ValueDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a module declaration.
     **
     **  @return The new created ModuleDecl.
     **/
    protected ModuleDecl
    startModule(String name)
    {
        ModuleDecl module = new ModuleDeclImpl(getRepository(), this);
        module.setName(name);
        return module;
    }

    /**
     **  Start an component declaration.
     **
     **  @return The new created ComponentDecl.
     **/
    protected ComponentDecl
    startComponent(String name)
    {
        ComponentDecl decl = new ComponentDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an event declaration.
     **
     **  @return The new created EventDecl.
     **/
    protected EventDecl
    startEvent(String name)
    {
        EventDecl decl = new EventDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Find a declaration.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    protected Declaration
    findInScope(String name)
    {
        Declaration decl = null;
        for (int i=0;i<contained_decls_.size();i++)
        {
            decl = (Declaration)contained_decls_.get(i);
            if (decl.getName().equalsIgnoreCase(name))
                return decl;
        }

        return null;
    }

    /**
     **  Find a declaration in the IR3.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    protected Declaration
    findInIR3(String name)
    {
        Declaration decl = null;
        org.omg.CORBA.Contained contained = getContainer().lookup(name);
        if (contained == null)
            return null;

        return loadContained(contained);
    }

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the scope declaration.
     **/
    abstract protected org.omg.CORBA.Container
    getContainer();

    /**
     **  To avoid casts
     **/
    protected org.omg.CORBA.ComponentIR.ComponentDef
    getComponentDef()
    {
        return null;
    }

    /**
     **  To avoid casts
     **/
    protected org.omg.CORBA.ExtValueDef
    getExtValueDef()
    {
        return null;
    }

    /**
     **  To avoid casts
     **/
    protected org.omg.CORBA.ComponentIR.HomeDef
    getHomeDef()
    {
        return null;
    }

    /**
     **  To avoid casts
     **
     **  @return The Container object associated with the scope declaration.
     **/
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
        return null;
    }

    /**
     **  Create an attribute definition.
     **
     **  @param attribute The attribute declaration.
     **  @param type The IDLType of the attribute.
     **  @param mode The mode of the attribute ie normal or readonly
     **  @param get_exceptions An array containing the exceptions
     **                        that accessor operations can raise.
     **  @param set_exceptions An array containing the exceptions
     **                        that mutator operations can raise.
     **
     **  @return The new created AttributeDef.
     **/
    protected org.omg.CORBA.ExtAttributeDef
    createExtAttribute(AttributeDecl attribute,
                       org.omg.CORBA.IDLType type,
                       org.omg.CORBA.AttributeMode mode,
                       org.omg.CORBA.ExceptionDef[] get_exceptions,
                       org.omg.CORBA.ExceptionDef[] set_exceptions)
    {
        return null;
    }

    /**
     **  Create an operation.
     **
     **  @param operation The operation declaration.
     **  @param type The IDLType of the object that the operations returns.
     **  @param mode The mode of the operation ie normal or oneway.
     **  @param params An array containing the parameters description.
     **  @param exceptions An array containing the exceptions
     **                    that the operation can raise.
     **  @param contexts An array containing the possible context values.
     **
     **  @return The new created OperationDef.
     **/
    protected org.omg.CORBA.OperationDef
    createOperation(OperationDecl operation,
                    org.omg.CORBA.IDLType type,
                    org.omg.CORBA.OperationMode mode,
                    org.omg.CORBA.ParameterDescription[] params,
                    org.omg.CORBA.ExceptionDef[] exceptions,
                    String[] contexts)
    {
        return null;
    }

    // ==================================================================
    //
    // Methods for the Scope interface (parser view).
    //
    // ==================================================================

    /**
     **  Start a file scope.
     **
     **  @param fileName the name of the file represented by this FileScope.
     **/
    public FileScope
    startFileScope(String filename)
    {
        FileScope decl = new FileScope(getRepository(), this, contained_decls_);
        decl.setName(filename);
        return decl;
    }

    /**
     **  Find a module declaration
     **  If it not exists then create it.
     **
     **  @param The name of the module declaration.
     **
     **  @return The new created ModuleDecl or a already created ModuleDecl.
     **/
    public ModuleDecl
    declareModule(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        // if it is a module then return it
        // because this is a module reopening.
        if (decl != null && decl instanceof ModuleDecl)
            return (ModuleDecl)decl;

        // else create it.
        ModuleDeclImpl module = new ModuleDeclImpl(getRepository(), this);
        module.setName(name);

        // Search its ModuleDef into the IR3.
        try
        {
            org.omg.CORBA.Contained contained = getContainer().lookup(name);
            if (contained!=null)
                module.load(contained);
        }
        catch(org.omg.CORBA.SystemException exc)
	{
            // TODO: Do nothing if narrow fails?
        }

        return module;
    }

    /**
     **  Start a constant declaration.
     **
     **  @return The new created ConstantDecl.
     **/
    public ConstantDecl
    startConstant(String name)
    {
        ConstantDecl decl = new ConstantDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a struct declaration.
     **
     **  @return The new created StructDecl.
     **/
    public StructDecl
    declareStruct(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof StructDecl))
            return (StructDecl)decl;

        // else create it.
        decl = new StructDeclImpl(getRepository(), this);
        decl.setName(name);
        return (StructDecl)decl;
    }

    /**
     **  Start an union declaration.
     **
     **  @return The new created UnionDecl.
     **/
    public UnionDecl
    declareUnion(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof UnionDecl))
            return (UnionDecl)decl;

        // else create it.
        decl = new UnionDeclImpl(getRepository(), this);
        decl.setName(name);
        return (UnionDecl)decl;
    }

    /**
     **  Start an enum declaration.
     **
     **  @return The new created EnumDecl.
     **/
    public EnumDecl
    startEnum(String name)
    {
        EnumDecl decl = new EnumDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an alias declaration.
     **
     **  @return The new created AliasDecl.
     **/
    public AliasDecl
    startAlias(String name)
    {
        AliasDecl decl = new AliasDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Declare an interface declaration.
     **  If it not exists, create it.
     **
     **  @param name The name of the interface declaration.
     **
     **  @return The new created InterfaceDecl
     **          or an already created InterfaceDecl.
     **/
    public InterfaceDecl
    declareInterface(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if (decl != null)
	{
            // if it is an interface then return it.
            if (decl.getClass() == InterfaceDeclImpl.class)
                return (InterfaceDecl)decl;

            // if it is an abstract interface then raise an error.
            if (decl instanceof AbstractInterfaceDeclImpl)
                throw new Error("interface `" + name +
                                "' does not match forward abstract declaration");

            // if it is a local interface then raise an error.
            if (decl instanceof LocalInterfaceDeclImpl)
                throw new Error("interface `" + name +
                                "' does not match forward local declaration");
	}

        // else create it.
        decl = new InterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return (InterfaceDecl)decl;
    }

    /**
     **  Declare an abstract interface declaration.
     **  If it not exits, create it.
     **
     **  @param name The name of the abstract interface declaration.
     **
     **  @return The new created AbstractInterfaceDecl
     **          or an already created AbstractInterfaceDecl.
     **/
    public AbstractInterfaceDecl
    declareAbstractInterface(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if (decl != null)
        {
            // if it is an abstract interface then return it.
            if (decl instanceof AbstractInterfaceDeclImpl)
                return (AbstractInterfaceDecl)decl;

            // if it is a local interface then raise an error.
            if (decl instanceof LocalInterfaceDeclImpl)
                throw new Error("interface `" + name +
                                "' does not match forward local declaration");

            // if it is an interface then raise an error.
            if (decl instanceof InterfaceDeclImpl)
                throw new Error("interface `" + name +
                                "' does not match forward declaration");
        }

        // else create it.
        decl = new AbstractInterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return (AbstractInterfaceDecl)decl;
    }

    /**
     **  Declare a local interface declaration.
     **  If it not exists, create it.
     **
     **  @param name The name of the local interface declaration.
     **
     **  @return The new created LocalInterfaceDecl
     **          or an already created LocalInterfaceDecl.
     **/
    public LocalInterfaceDecl
    declareLocalInterface(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if (decl != null)
        {
            // if it is a local interface then return it.
            if (decl instanceof LocalInterfaceDeclImpl)
                return (LocalInterfaceDecl)decl;

            // if it is an abstract interface then raise an error.
            if (decl instanceof AbstractInterfaceDeclImpl)
                throw new Error("interface `" + name +
                                "' does not match forward abstract declaration");

            // if it is an interface then raise an error.
            if (decl instanceof InterfaceDeclImpl)
                throw new Error("interface `" + name +
                                "' does not match forward declaration");
        }

        // else create it.
        decl = new LocalInterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return (LocalInterfaceDecl)decl;
    }

    /**
     **  Declare a value type declaration.
     **  If it not exists, create it.
     **
     **  @param name The name of the value type declaration.
     **
     **  @return The new created ValueDecl or an already created ValueDecl.
     **/
    public ValueDecl
    declareValue(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof ValueDecl))
            return (ValueDecl)decl;

        // else create it.
        decl = new ValueDeclImpl(getRepository(), this);
        decl.setName(name);
        return (ValueDecl)decl;
    }

    /**
     **  Start an initializer declaration.
     **
     **  @return The new created Initializer.
     **/
    public Initializer
    startInitializer(String name)
    {
        Initializer init = new InitializerImpl();
        init.setName(name);
        return init;
    }

    /**
     **  Declare a event type declaration.
     **  If it not exists, create it.
     **
     **  @param name The name of the event type declaration.
     **
     **  @return The new created EventDecl or an already created EventDecl.
     **/
    public EventDecl
    declareEvent(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof EventDecl))
            return (EventDecl)decl;

        // else create it.
        decl = new EventDeclImpl(getRepository(), this);
        decl.setName(name);
        return (EventDecl)decl;
    }

    /**
     **  Start a value member declaration.
     **
     **  @return The new created ValueMemberDecl.
     **/
    public ValueMemberDecl
    startValueMember(String name)
    {
        ValueMemberDecl decl = new ValueMemberDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a value box declaration.
     **
     **  @return The new created ValueBoxDecl.
     **/
    public ValueBoxDecl
    startValueBox(String name)
    {
        ValueBoxDecl decl = new ValueBoxDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an exception declaration.
     **
     **  @return The new created ExceptionDecl.
     **/
    public ExceptionDecl
    startException(String name)
    {
        ExceptionDecl decl = new ExceptionDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a native declaration.
     **
     **  @return The new created NativeDecl.
     **/
    public NativeDecl
    startNative(String name)
    {
        NativeDecl decl = new NativeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an attribute declaration.
     **
     **  @return The new created AttributeDecl.
     **/
    public AttributeDecl
    startAttribute(String name)
    {
        AttributeDecl decl = new AttributeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start an operation declaration.
     **
     **  @return The new created OperationDecl.
     **/
    public OperationDecl
    startOperation(String name)
    {
        OperationDecl decl = new OperationDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a factory declaration.
     **
     **  @return The new created FactoryDecl.
     **/
    public FactoryDecl
    startFactory(String name)
    {
        FactoryDecl decl = new FactoryDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a finder declaration.
     **
     **  @return The new created FinderDecl.
     **/
    public FinderDecl
    startFinder(String name)
    {
        FinderDecl decl = new FinderDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a provides declaration.
     **
     **  @return The new created ProvidesDecl.
     **/
    public ProvidesDecl
    startProvides(String name)
    {
        ProvidesDecl decl = new ProvidesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a uses declaration.
     **
     **  @return The new created UsesDecl.
     **/
    public UsesDecl
    startUses(String name)
    {
        UsesDecl decl = new UsesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a consumes declaration.
     **
     **  @return The new created ConsumesDecl.
     **/
    public ConsumesDecl
    startConsumes(String name)
    {
        ConsumesDecl decl = new ConsumesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a emits declaration.
     **
     **  @return The new created EmitsDecl.
     **/
    public EmitsDecl
    startEmits(String name)
    {
        EmitsDecl decl = new EmitsDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Start a publishes declaration.
     **
     **  @return The new created PublishesDecl.
     **/
    public PublishesDecl
    startPublishes(String name)
    {
        PublishesDecl decl = new PublishesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Declare a component declaration.
     **  If it not exits, create it.
     **
     **  @param name The component declaration name.
     **
     **  @return The new created ComponentDecl
     **          or an already created ComponentDecl.
     **/
    public ComponentDecl
    declareComponent(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof ComponentDecl))
            return (ComponentDecl)decl;

        // else create it.
        decl = new ComponentDeclImpl(getRepository(), this);
        decl.setName(name);
        return (ComponentDecl)decl;
    }

    /**
     **  Start a home declaration.
     **
     **  @return The new created HomeDecl.
     **/
    public HomeDecl
    startHome(String name)
    {
        HomeDecl decl = new HomeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     **  Find a declaration.
     **
     **  Note that the declaration is also recursively
     **  searched in the parent scope.
     **  Used from ComponentDeclImpl and HomeDeclImpl classes.
     **
     **  @param name The name of the searched declaration.
     **
     **  @return The declaration that was searched
     **          or null if it does not exist.
     **/
    public Declaration
    find(String name)
    {
        Declaration decl = findInScope(name);
        // If found then returns it.
        if (decl != null) return decl;

        // Finds in the parent scope.
        if (the_parent_!=null)
           decl = the_parent_.find(name);

        // If found then returns it.
        if (decl != null) return decl;

        return findInIR3(name);
    }

    // ==================================================================
    //
    // Methods for the Scope interface (visitor view).
    //
    // ==================================================================

    /**
     **  To obtain all the contained Declaration objects.
     **
     **  @param exclude_inherited If false return also objects contained in inherited scopes.
     **  @param limited_types A logical combination of DeclarationKind.
     **
     **  @return An array of Declaration objects.
     **/
    public Declaration[]
    getContents(boolean exclude_inherited, int limited_types)
    {
        if (!content_loaded_)
     	{
            // if it's a mapping scope, then use the OpenCCM repository as an IDL2 repository
            // to retrieve contained objects.
            if (is_mapping_)
                getRepository().useIDL2Repository();

            org.omg.CORBA.Contained[] content = getContainer().contents(org.omg.CORBA.DefinitionKind.dk_all, true);

            // we need to reorder the contained declarations !!!
            // because the could have been loaded one by one without any
            // garanties on the order.
            org.objectweb.ccm.util.Vector ordered_decls = new org.objectweb.ccm.util.Vector();
            Declaration decl = null;
            contained_decls_.clear();

            //            System.err.println(" ");
            //            System.err.println("Container is : "+getName());

            for (int i=0;i<content.length;i++)
            {
                //                System.err.println("    Contained is : "+content[i].name());
                decl = findInScope(content[i].name());
                if (decl==null)
                {
                    if (is_mapping_)
                        decl = loadContainedAsMapping(content[i]);
                    else
                        decl = loadContained(content[i]);
                }

                //                System.err.println("                   loaded");
                ordered_decls.add(decl);
            }

            contained_decls_ = ordered_decls;

            // get back to an IDL3 repository.
            if (is_mapping_)
                getRepository().useIDL3Repository();

            content_loaded_ = true;
        }

        Declaration decl = null;
        org.objectweb.ccm.util.Vector selection = new org.objectweb.ccm.util.Vector();
        for (int i=0;i<contained_decls_.size();i++)
        {
            decl = (Declaration)contained_decls_.get(i);
            if ((decl.getDeclKind()&limited_types)!=0)
                selection.add(decl);
        }
        return (Declaration[])selection.toArray(new Declaration[0]);
    }

    /**
     **  To find a contained Declaration according to it's scoped name.
     **
     **  @param n The scoped name.
     **
     **  @return The found or loaded from the IR3 Declaration or null.
     **/
    public Declaration
    lookup(String scoped_name)
    {
        int idx = scoped_name.indexOf("::");
        String s1 = null;
        String s2 = null;
        if (idx==-1)
        {
            s1 = scoped_name;
            s2 = "";
        }
        else
        {
            s1 = scoped_name.substring(0,idx);
            s2 = scoped_name.substring(idx+2);
        }

        Declaration decl = findInScope(s1);
        if (decl==null)
        {
	    // try to load it from the IR3
            decl = findInIR3(s1);
        }

        if (s2.equals(""))
            return decl;

        if (decl instanceof Scope)
            return ((Scope)decl).lookup(s2);

        return null;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **  Destroy all the contained declarations of the scope in the IR3.
     **/
    public void
    destroy()
    {
        Declaration decl = null;
        for (int i=contained_decls_.size()-1;i>=0;i--)
        {
            decl = (Declaration)contained_decls_.get(i);
            decl.destroy();
        }

        // destroy it as a declaration if there is no content.
        if (contained_decls_.size()==0)
            super.destroy();
    }
}
