/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL struct declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class StructDeclImpl
       extends ForwardScopeImpl
       implements TypeRef, StructDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 StructDef.
     **/
    protected org.omg.CORBA.StructDef struct_def_;

    /**
     **  List of members.
     **/
    protected StructMembersImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    StructDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        struct_def_ = null;
        members_ = new StructMembersImpl();
        the_declaration_kind_ = DeclarationKind._dk_struct;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 StructDef.
     **
     **  @param structDef The StructDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        struct_def_ = org.omg.CORBA.StructDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = struct_def_.members();

        // load as scope now to prevent infinite loop in the case
        // of recursive struct.
        super.load(contained);

        for (int i=0;i<members.length;i++)
            members_.addMember(members[i].name,
                               getRepository().getAsTypeRef(members[i].type_def));
    }

    /**
     **  Loads infos of the CORBA 3.0 StructDef.
     **
     **  @param structDef The StructDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        struct_def_ = org.omg.CORBA.StructDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = struct_def_.members();

        // load as scope now to prevent infinite loop in the case
        // of recursive struct.
        super.loadAsMapping(contained);

        for (int i=0;i<members.length;i++)
            members_.addMember(members[i].name,
                               getRepository().getAsMappedTypeRef(members[i].type_def));
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Return the index declarator.
     **
     **  @param index The index of the declarator that should be returned.
     **
     **  @return The name of the declarator at the specified index.
     **/
    public String
    declaratorAt(int index)
    {
        return members_.nameAt(index);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];

        // members
        org.objectweb.ccm.util.Vector struct_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;
        TypeRef[] member_types = getMembers().getMemberTypes();

        for (int i=0;i<member_types.length;i++)
        {
            if (member_types[i].isDeclaration())
            {
                if ((!containsDecl((Declaration)member_types[i])) &&
                    (struct_depend.indexOf(member_types[i])==-1))
                    struct_depend.add(member_types[i]);
            }

            if (recursion) {
                depend = member_types[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((struct_depend.indexOf(depend[j])==-1) &&
                        (!containsDecl(depend[j])) &&
                        (depend[j]!=this))
                        struct_depend.add(depend[j]);
                }
            }
        }

        dependencies_ = (Declaration[])struct_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the Scope interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the struct declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return struct_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_struct;
    }

    // ==================================================================
    //
    // Methods for the StructDecl interface.
    //
    // ==================================================================

    /**
     **  Obtain the member list.
     **
     **  @return The members of the struct declaration.
     **/
    public StructMembers
    getMembers()
    {
        return members_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the struct declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        return struct_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        members_.destroy();
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the struct declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
        return struct_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        struct_def_ = the_parent_.getContainer().create_struct(getId(), getName(), getVersion(),
                                                               members_.getMemberDefs());
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        struct_def_.members(members_.getMemberDefs());
    }
}
