/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL union declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UnionDeclImpl
       extends ForwardScopeImpl
       implements TypeRef, UnionDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 UnionDef.
     **/
    private org.omg.CORBA.UnionDef union_def_;

    /**
     **
     **/
    private int default_index_;

    /**
     **  List of members.
     **/
    private UnionMembersImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the declaration.
     **/
    protected
    UnionDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        union_def_ = null;
        members_ = new UnionMembersImpl();
        default_index_ = -1;
        the_declaration_kind_ = DeclarationKind._dk_union;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     ** Loads infos of the CORBA 3.0 UnionDef.
     **
     **  @param unionDef The UnionDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        union_def_ = org.omg.CORBA.UnionDefHelper.narrow(contained);
        setDiscriminator(getRepository().getAsTypeRef(union_def_.discriminator_type_def()));
        try
        {
            default_index_ = union_def_.type().default_index();
        }
        catch (org.omg.CORBA.TypeCodePackage.BadKind e) {}

        org.omg.CORBA.UnionMember[] members = union_def_.members();

        // load as scope now to prevent infinite loop in the case
        // of recursive union.
        super.load(contained);

        for (int i=0;i<members.length;i++)
        {
            AnyValue any = new AnyValue();
            any.loadAny(members[i].label);
            addMember(members[i].name,
                      getRepository().getAsTypeRef(members[i].type_def),
                      any);
        }
    }

    /**
     ** Loads infos of the CORBA 3.0 UnionDef.
     **
     **  @param unionDef The UnionDef to load.
     **/
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        union_def_ = org.omg.CORBA.UnionDefHelper.narrow(contained);
        setDiscriminator(getRepository().getAsMappedTypeRef(union_def_.discriminator_type_def()));
        try
        {
            default_index_ = union_def_.type().default_index();
        }
        catch (org.omg.CORBA.TypeCodePackage.BadKind e) {}
        org.omg.CORBA.UnionMember[] members = union_def_.members();

        // load as scope now to prevent infinite loop in the case
        // of recursive struct.
        super.loadAsMapping(contained);

        for (int i=0;i<members.length;i++)
        {
            AnyValue any = new AnyValue();
            any.loadAny(members[i].label);
            addMember(members[i].name,
                      getRepository().getAsMappedTypeRef(members[i].type_def),
                      any);
        }
    }

    // ==================================================================
    //
    // Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        org.objectweb.ccm.util.Vector union_depend = new org.objectweb.ccm.util.Vector();
        Declaration[] depend = null;

        // discriminator
        if (getType().isDeclaration())
            union_depend.add(getType());

        if (recursion) {
            depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                union_depend.add(depend[i]);
        }


        // members
        TypeRef[] member_types = getMembers().getMemberTypes();

        for (int i=0;i<member_types.length;i++)
        {
            if (member_types[i].isDeclaration())
            {
                if ((!containsDecl((Declaration)member_types[i])) &&
                    (union_depend.indexOf(member_types[i])==-1))
                    union_depend.add(member_types[i]);
            }

            if (recursion) {
                depend = member_types[i].getDependencies();
                for (int j=0;j<depend.length;j++)
                {
                    if ((union_depend.indexOf(depend[j])==-1) &&
                        (!containsDecl(depend[j])) &&
                        (depend[j]!=this))
                        union_depend.add(depend[j]);
                }
            }
        }

        dependencies_ = (Declaration[])union_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for the Scope interface.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for the TypeRef interface.
    //
    // ==================================================================

    /**
     **  Obtain its IDLType reference.
     **
     **  @return The IDLType associated with the union declaration.
     **/
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return union_def_;
    }

    /**
     **
     **/
    public int
    getTypeKind()
    {
        return TypeKind._tk_union;
    }

    // ==================================================================
    //
    // Methods for the UnionDecl interface.
    //
    // ==================================================================

    /**
     **  Set the discriminator.
     **
     **  @param type The TypeRef of the discriminator.
     **/
    public void
    setDiscriminator(TypeRef type)
    {
        if(type!=null)
        {
            type.addRef();
            members_.setDiscriminator(type);
        }
    }

    /**
     **  Add a member.
     **
     **  @param name The name of the member.
     **  @param type The TypeRef of the member.
     **  @param value The value of the member.
     **/
    public void
    addMember(String name,
              TypeRef type,
              AnyValue value)
    {
        members_.addMember(name, type, value);
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return members_.getDiscriminator();
    }

    /**
     **
     **/
    public UnionMembers
    getMembers()
    {
        return members_;
    }

    /**
     **
     **/
    public int
    getDefaultIndex()
    {
        return default_index_;
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        members_.destroy();
        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the union declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
        return union_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the union declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
        return union_def_;
    }

    // ==================================================================
    //
    // Methods for the inherited ForwardScopeImpl class.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        // Create an UnionDef into the IFR.
        org.omg.CORBA.IDLType disc = null;
        if (members_.getDiscriminator()!=null)
            disc = members_.getDiscriminator().getIDLType();

        union_def_ = the_parent_.getContainer().create_union(getId(), getName(), getVersion(),
                                                             disc,
                                                             members_.getMemberDefs());
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        union_def_.discriminator_type_def(members_.getDiscriminator().getIDLType());
        union_def_.members(members_.getMemberDefs());
    }
}
