/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL uses port declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UsesDeclImpl
       extends DeclarationImpl
       implements UsesDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 UsesDef.
     **/
    private org.omg.CORBA.ComponentIR.UsesDef uses_def_;

    /**
     **  The port interface type.
     **/
    private InterfaceDeclImpl interface_;

    /**
     **  Is multiple.
     **/
    private boolean is_multiple_;

    /**
     **
     **/
    private OperationDeclImpl[] client_mapping_;

    /**
     **
     **/
    private OperationDeclImpl[] local_context_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the uses declaration.
     **/
    protected
    UsesDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        uses_def_ = null;
        client_mapping_ = null;
        local_context_mapping_ = null;
        interface_ = null;
        is_multiple_ = false;
        the_declaration_kind_ = DeclarationKind._dk_uses;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 UsesDef.
     **
     **  @param uses_def The UsesDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        uses_def_ = org.omg.CORBA.ComponentIR.UsesDefHelper.narrow(contained);
        setInterface((InterfaceRef)getRepository().lookupId(uses_def_.interface_type().id()));
        is_multiple_ = uses_def_.is_multiple();
        super.load(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the uses declaration into the IFR.
     **/
    public void
    create()
    {
        uses_def_ = the_parent_.getComponentDef().create_uses(getId(), getName(),
                                                              getVersion(),
                                                              interface_.getInterfaceDef(),
                                                              is_multiple_);

        super.create();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector uses_depend = new org.objectweb.ccm.util.Vector();

        // content
        if (getType().isDeclaration())
            uses_depend.add(getType());

        if (recursion) {
            Declaration[] depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                uses_depend.add(depend[i]);
        }


        dependencies_ = (Declaration[])uses_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    //   Methods for the ProvidesDecl interface.
    //
    // ==================================================================

    /**
     **  Set as multiple.
     **/
    public void
    setMultiple()
    {
        is_multiple_ = true;
    }

    /**
     **  Set the type.
     **
     **  @param itf The provided interface reference for the uses declaration.
     **/
    public void
    setInterface(InterfaceRef itf)
    {
        if(itf != null)
        {
            interface_ = (InterfaceDeclImpl)itf;
            interface_.addRef();
        }
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return interface_;
    }

    /**
     **
     **/
    public boolean
    isMultiple()
    {
        return is_multiple_;
    }

    /**
     **
     **/
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((ComponentDecl)the_parent_).getClientMapping();
        client.getContents(true, DeclarationKind._dk_null);

        client_mapping_ = new OperationDeclImpl[3];
        client_mapping_[0] = (OperationDeclImpl)client.find("connect_"+getName());
        client_mapping_[1] = (OperationDeclImpl)client.find("disconnect_"+getName());

        if (is_multiple_)
            client_mapping_[2] = (OperationDeclImpl)client.find("get_connections_"+getName());
        else
            client_mapping_[2] = (OperationDeclImpl)client.find("get_connection_"+getName());

        return client_mapping_;
    }

    /**
     **
     **/
    public OperationDecl[]
    getLocalContextMapping()
    {
        if (local_context_mapping_!=null)
            return local_context_mapping_;

        // retrieve the context mapping local interface.
        InterfaceDecl context = ((ComponentDecl)the_parent_).getLocalContextMapping();
        context.getContents(true, DeclarationKind._dk_null);

        local_context_mapping_ = new OperationDeclImpl[1];
        if (is_multiple_)
            local_context_mapping_[0] = (OperationDeclImpl)context.find("get_connections_"+getName());
        else
            local_context_mapping_[0] = (OperationDeclImpl)context.find("get_connection_"+getName());

        return local_context_mapping_;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (interface_!=null)
            interface_.removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the uses declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return uses_def_;
    }
}
