/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.IDL3;

/**
 * This class manages IDL value member declarations.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueMemberDeclImpl
       extends DeclarationImpl
       implements ValueMemberDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  Reference to the CORBA 3.0 ValueMemberDef.
     **/
    private org.omg.CORBA.ValueMemberDef value_member_def_;

    /**
     **  The type of the value member.
     **/
    private TypeRef type_;

    /**
     **  The visibility of the value member.
     **/
    private short access_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor with the parent scope.
     **
     **  @param parent The parent scope of the value member declaration.
     **/
    protected
    ValueMemberDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        value_member_def_ = null;
        type_ = null;
        access_ = (short)0;
        the_declaration_kind_ = DeclarationKind._dk_value_member;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Loads infos of the CORBA 3.0 ValueMemberDef.
     **
     **  @param value_member_def The ValueMemberDef to load.
     **/
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        value_member_def_ = org.omg.CORBA.ValueMemberDefHelper.narrow(contained);
        setType(getRepository().getAsTypeRef(value_member_def_.type_def()));
        access_ = value_member_def_.access();
        super.load(contained);
    }

    // ==================================================================
    //
    //  Methods for the Declaration interface.
    //
    // ==================================================================

    /**
     **  Create the value member declaration into the IFR.
     **/
    public void
    create()
    {
        // Create a ValueMemberDef into the IFR.
        value_member_def_ = the_parent_.getExtValueDef().create_value_member(getId(),
                                                                             getName(),
                                                                             getVersion(),
                                                                             type_.getIDLType(),
                                                                             access_);

        super.create();
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies()
    {
        return getDependencies(true);
    }

    /**
     **  Obtain the declaration external dependencies.
     **  Note: for scopes, contained objects are not considered
     **  as dependencies.
     **
     **  @return The list of dependencies as an array of Declaration.
     **/
    public Declaration[]
    getDependencies(boolean recursion)
    {
        if (dependencies_!=null)
            return dependencies_;

        org.objectweb.ccm.util.Vector vmember_depend = new org.objectweb.ccm.util.Vector();

        // content
        if (getType().isDeclaration())
            vmember_depend.add(getType());

        if (recursion) {
            Declaration[] depend = getType().getDependencies();
            for (int i=0;i<depend.length;i++)
                vmember_depend.add(depend[i]);
        }


        dependencies_ = (Declaration[])vmember_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    //  Methods for the ValueMemberDecl interface.
    //
    // ==================================================================

    /**
     **  Set the type.
     **
     **  @param type The type reference of the value member.
     **/
    public void
    setType(TypeRef type)
    {
        if(type != null)
        {
            type_ = type;
            type_.addRef();
        }
    }

    /**
     **  Set as private.
     **/
    public void
    setPrivate()
    {
        access_ = (short)0;
    }

    /**
     **  Set as public.
     **/
    public void
    setPublic()
    {
        access_ = (short)1;
    }

    /**
     **
     **/
    public TypeRef
    getType()
    {
        return type_;
    }

    /**
     **
     **/
    public boolean
    isPrivate()
    {
        return access_==(short)0;
    }

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    destroy()
    {
        if (type_!=null)
            type_.removeRef();

        super.destroy();
    }

    /**
     **  Obtain its CORBA 3.0 Contained reference.
     **
     **  @return The Contained object associated with the value
     **          member declaration.
     **/
    protected org.omg.CORBA.Contained
    getContained()
    {
       return value_member_def_;
    }
}
