/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of IDL:omg.org/CORBA/Contained:1.0.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class Contained_impl
                extends IRObject_impl
                implements ContainedOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    Contained_impl(IFR ifr,
                   Container_impl container)
    {
        // call the IRObject_impl contructor.
        super(ifr);

        // set the container.
        container_ = container;

        // set the id, name, version to empty strings
        id_ = "";
        name_ = "";
        version_ = "";
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The container of this contained object.
     **/
    protected Container_impl container_;

    /**
        The repository id.
     **/
    protected String id_;

    /**
        The name.
     **/
    protected String name_;

    /**
        The version.
     **/
    protected String version_;

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::Contained object reference.
     **/
    public Contained
    asContained()
    {
        return ContainedHelper.narrow(asObject());
    }

    /**
        Obtain the repository ID of the container.
     **/
    public String
    getContainerID()
    {
      if (container_ == null) return "";
      return container_.id();
    }

     /**
        Obtain the base repository ID of the container.
     **/
    public String
    getBaseContainerID()
    {
      if (container_ == null) return "IDL:";
      return computeBaseID(container_.id());
    }

     /**
        Obtain the base repository ID.
     **/
    public String
    getBaseID()
    {
      return computeBaseID(id());
    }

     /**
        Compute a base repository ID + '/'.
     **/
    public String
    computeBaseID(String id)
    {
      return computeIDWithoutVersion(id) + '/';
    }

     /**
        Compute a base repository ID without version.
     **/
    public String
    computeIDWithoutVersion(String id)
    {
      int idx = id.lastIndexOf(':');
      if(idx == -1) return id;
      return id.substring(0, idx);
    }

    /**
        To set the value field of the CORBA::Contained::Description struct.
        By default, set as a CORBA::TypeDescription.
        Must be implemented into subclasses.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // create the CORBA::TypeDescription.
        TypeDescription description = new TypeDescription();
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.type = type();

        // insert it into the any.
        TypeDescriptionHelper.insert(any, description);
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // remove it from the repository.
        if(id_.length() != 0)
            getIFR().getRepository().removeRID(id_);

        // remove it from the container.
        if (container_ != null)
            container_.removeContained(this);

        // call the IRObject_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::Contained interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/Contained/id:1.0
     **/
    public String
    id()
    {
        return id_;
    }

    public void
    id(String val)
    {
        // if not changed then return immediatelly. 
        if (val.equals(id_)) return;

        // search if val is an already used RID.
        Contained contained = getIFR().getRepository().lookup_id(val);
        // if yes then raise an RID AlreadyDefined exception.
        if (contained != null)
            throw exceptionRidAlreadyDefined(val);

        // remove it from the repository.
        if(id_.length() != 0)
            getIFR().getRepository().removeRID(id_);

        // set the id.
        id_ = val;

        // add it to the repository.
        if(id_.length() != 0)
            getIFR().getRepository().addRID(id_, this);
    }

    /**
        IDL:omg.org/CORBA/Contained/name:1.0
     **/
    public String
    name()
    {
        return name_;
    }

    public void
    name(String val)
    {
        // if not changed then return immediatelly. 
        if (val.equals(name_)) return;

        // check for name redefinition with the container name
        if (val.equalsIgnoreCase(container_.name()))
            throw exceptionNameAlreadyUsedByImmediateScope(val);

        // check if the name already exists in the container.
        // Exclude inherited names.
        container_.checkName(val, true, false);

        name_ = val;
    }

    /**
        IDL:omg.org/CORBA/Contained/version:1.0
     **/
    public String
    version()
    {
        return version_;
    }

    public void
    version(String val)
    {
        version_  = val;
    }

    /**
        IDL:omg.org/CORBA/Contained/defined_in:1.0
     **/
    public Container
    defined_in()
    {
        return container_.asContainer();
    }

    /**
        IDL:omg.org/CORBA/Contained/absolute_name:1.0
     **/
    public String
    absolute_name()
    {
        // If this is an CORBA::Repository then return an empty string.
        if (container_ == null) return "";

        return container_.absolute_name() + "::" + name_;
    }

    /**
        IDL:omg.org/CORBA/Contained/containing_repository:1.0
     **/
    public Repository
    containing_repository()
    {
        return getIFR().getRepository().asRepository();
    }

    /**
        IDL:omg.org/CORBA/Contained/describe:1.0
     **/
    public org.omg.CORBA.ContainedPackage.Description
    describe()
    {
        // obtain the value description field.
        Any any = org.objectweb.ccm.CORBA.TheORB.create_any();
        setDescriptionValue(any);

        // return a CORBA::Contained::Description
        return new org.omg.CORBA.ContainedPackage.Description(def_kind(), any);
    }

    /**
        IDL:omg.org/CORBA/Contained/move:1.0
     **/
    public void
    move(Container new_container,
         String new_name,
         String new_version)
    {
        // Must be a non null container reference.
        if (new_container == null)
            throw exceptionBadParam("IDL:omg.org/CORBA/Contained/move:1.0");

        // Must be a local container.
        Container_impl newContainer = castToLocal(new_container);

        // Remember the old name, version and container.
        String oldName = name_;
        String oldVersion = version_;
        Container_impl oldContainer = container_;

        // Add to the new container.
        // This raises an invalid container exception if the kind of
        // this contained object is not accepted by the new container.
        oldContainer.removeContained(this);
        try
        {
            newContainer.addContained(this);
            container_ = newContainer;
        }
        catch(SystemException ex)
        {
            //
            // Rollback to previous state if there is an exception!
            //
            oldContainer.addContained(this);
            container_ = oldContainer;
            throw ex;
        }

        try
        {
            //
            // Change name and version.
            //
            name(new_name);
            version(new_version);
        }
        catch(SystemException ex)
        {
            //
            // Rollback to previous state if there is an exception!
            //
            oldContainer.addContained(this);
            newContainer.removeContained(this);
            container_ = oldContainer;
            name(oldName);
            version(oldVersion);
            throw ex;
        }
    }
}
