/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::Container interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class Container_impl
                extends Contained_impl
                implements ContainerOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The contained objects list.
     **/
    protected java.util.Vector contained_elements_;

    /**
     **  The contained objects list.
     **/
    protected java.util.Vector contained_with_mappings_;

    /**
     **  To know if the addition of contained objects should be done
     **  in the contained_elements_ vector or in the mappings_elements_ vector.
     **/
    protected boolean mapping_started_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    Container_impl(IFR ifr,
                   Container_impl container)
    {
        // call the Contained_impl contructor.
        super(ifr, container);

        // create the contained vector.
        contained_elements_ = new java.util.Vector();
        contained_with_mappings_ = new java.util.Vector();
        mapping_started_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **
     **/
    protected Contained_impl[]
    getContainedElements()
    {
        if (!getIFR().getRepository().isIDL2Repository())
            return (Contained_impl[])contained_elements_.toArray(new Contained_impl[0]);
        else
            return (Contained_impl[])contained_with_mappings_.toArray(new Contained_impl[0]);
    }


    /**
     **  Add a contained object to the container.
     **/
    protected void
    addContained(Contained_impl contained)
    {
        // check if the kind of this contained object is accepted.
        if (!checkDefinitionKind(contained.def_kind()))
            throw exceptionInvalidContainer();

        if (!mapping_started_)
            contained_elements_.add(contained);

        contained_with_mappings_.add(contained);
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
        Convert a Vector to an CORBA::ContainedSeq.
     **/
    static public Contained[]
    toContainedSeq(java.util.Vector vector)
    {
        Contained[] result = new Contained[vector.size()];
        for(int i=0; i<result.length; i++)
        {
            result[i] = ((Contained_impl)vector.elementAt(i)).asContained();
        }
        return result;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::Container object reference.
     **/
    public Container
    asContainer()
    {
        return ContainerHelper.narrow(asObject());
    }

    /**
         Check if this kind of objects is accepted by the container.
     **/
    public boolean
    checkDefinitionKind(DefinitionKind def_kind)
    {
        // By default, all kinds are accepted!
        return true;
    }

    /**
        Add a contained object to the container.
     **/
    public void
    addContained(Contained_impl contained,
                 String id,
                 String name,
                 String version)
    {
        contained.name(name);
        contained.id(id);
        contained.version(version);
        addContained(contained);
    }

    /**
        Remove a contained object from the container.
     **/
    public void
    removeContained(Contained_impl contained)
    {
        contained_elements_.removeElement(contained);
        contained_with_mappings_.removeElement(contained);
    }

    /**
        Select named contained objects.
     **/
    public java.util.Vector
    selectNamedContained(java.util.Vector selection,
                         String search_name,
                         int levels_to_search,
                         DefinitionKind limit_type,
                         boolean exclude_inherited)
    {
        if(levels_to_search == 0) return selection;
        levels_to_search --;

        // iterate on all contained elements.
        Contained_impl[] containeds = getContainedElements();
        for (int i=0;i<containeds.length;i++)
        {
            Contained_impl contained = containeds[i];

            if (   (limit_type == DefinitionKind.dk_all)
	        || (limit_type == contained.def_kind()) )
            {
                if ((getIFR().getRepository().isIDL2Repository()) && 
                    (contained.def_kind()!=DefinitionKind.dk_Consumes) &&
                    (contained.def_kind()!=DefinitionKind.dk_Emits) &&
                    (contained.def_kind()!=DefinitionKind.dk_Publishes) &&
                    (contained.def_kind()!=DefinitionKind.dk_Provides) &&
                    (contained.def_kind()!=DefinitionKind.dk_Uses) &&
                    (contained.def_kind()!=DefinitionKind.dk_Factory) &&
                    (contained.def_kind()!=DefinitionKind.dk_Finder) &&
                    (contained.def_kind()!=DefinitionKind.dk_Event) &&
                    (contained.def_kind()!=DefinitionKind.dk_Component) &&
                    (contained.def_kind()!=DefinitionKind.dk_Home) &&
                    (search_name.equals(contained.name())))
                    selection.addElement(contained);
                else if ((!getIFR().getRepository().isIDL2Repository()) && 
                         (search_name.equals(contained.name())))
                    selection.addElement(contained);
            }

            if(levels_to_search != 0)
            {
                Container_impl container = (Container_impl)contained;
                if(container != null)
                {
                    container.selectNamedContained(selection,
                                                   search_name,
                                                   levels_to_search,
                                                   limit_type,
                                                   exclude_inherited);
                }
            }
        }

        return selection;
    }

    /**
        Select contained objects.
     **/
    public java.util.Vector
    selectContained(java.util.Vector selection,
                    DefinitionKind limit_type,
                    boolean exclude_inherited)
    {
        // iterate on all contained elements.
        Contained_impl[] containeds = getContainedElements();
        for (int i=0;i<containeds.length;i++)
        {
            Contained_impl contained = containeds[i];

            if ( (limit_type == DefinitionKind.dk_all)
		 || (limit_type == contained.def_kind()) )
            {
                selection.addElement(contained);
            }
        }

        return selection;
    }

    /**
        Compute its CORBA::AttrDescriptionSeq.
     **/
    public AttributeDescription[]
    getAttrDescriptionSeq()
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, DefinitionKind.dk_Attribute, true);

        AttributeDescription[] result =
                  new AttributeDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((AttributeDef_impl)selection.elementAt(i))
                        .getAttributeDescription();

        return result;
    }

    /**
        Compute its CORBA::ExtAttrDescriptionSeq.
     **/
    public ExtAttributeDescription[]
    getExtAttrDescriptionSeq()
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, DefinitionKind.dk_Attribute, true);

        ExtAttributeDescription[] result =
                  new ExtAttributeDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((ExtAttributeDef_impl)selection.elementAt(i))
                        .getExtAttributeDescription();

        return result;
    }

    /**
        Compute its CORBA::OpDescriptionSeq.
     **/
    public OperationDescription[]
    getOpDescriptionSeq()
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, DefinitionKind.dk_Operation, true);

        OperationDescription[] result =
                 new OperationDescription[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((OperationDef_impl)selection.elementAt(i))
                        .getOperationDescription();

        return result;
    }

    /**
        Check a new name according to already contained names.
     **/
    public void
    checkName(String name,
              boolean exclude_inherited,
              boolean in_inherited)
    {
        // iterate on all contained elements.
        Contained_impl[] containeds = getContainedElements();
        for (int i=0;i<containeds.length;i++)
        {
            Contained_impl contained = containeds[i];

            DefinitionKind dk = contained.def_kind();

            if (in_inherited && (   (dk == DefinitionKind.dk_Constant)
                                 || (dk == DefinitionKind.dk_Exception)
                                 || (dk == DefinitionKind.dk_Alias)
                                 || (dk == DefinitionKind.dk_Struct)
                                 || (dk == DefinitionKind.dk_Union)
                                 || (dk == DefinitionKind.dk_Enum)
                                 || (dk == DefinitionKind.dk_Native)
                                ))
                continue;

            boolean found = name.equalsIgnoreCase(contained.name());

            if(!found && dk == DefinitionKind.dk_Enum)
	    {
                String[] members = ((EnumDef_impl)contained).members();
                for(int j=0; j<members.length; j++)
                {
                    if(name.equalsIgnoreCase(members[j]))
                    {
                        found = true;
                        break;
                    }
	        }
	    }

            if(found)
	    {
                if(in_inherited)
                   throw exceptionNameClashInInheritedContext(name);
                else
                   throw exceptionNameAlreadyUsed(name);
	    }
        }
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // destroy all the contained objects and their mappings.
        while(contained_with_mappings_.size() != 0)
	{
            ((Contained_impl)contained_with_mappings_.lastElement()).destroy();
        }


        // call the Contained_impl destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    /**
       IDL:omg.org/CORBA/Container/lookup:1.0
     **/
    public Contained
    lookup(String search_name)
    {
        // if search_name begins with "::" then lookup into the repository.
        if(search_name.startsWith("::"))
            return getIFR().getRepository().lookup(search_name);

        int index = search_name.indexOf("::");
        String s1 = null;
        String s2 = null;

        if (index == -1) {
            s1 = search_name;
        } else {
            s1 = search_name.substring(0,index);
            s2 = search_name.substring(index+2);
        }

        // lookup into all contained elements.
        Contained_impl[] containeds = getContainedElements();
        for (int i=0;i<containeds.length;i++)
        {
            Contained_impl contained = containeds[i];

            if (s1.equals(contained.name())) {
                if (s2 == null) {
                    return contained.asContained();
                } else {
                    Container_impl container = (Container_impl)contained;
                    if (container == null) {
                        return null;
                    } else {
                        return container.lookup(s2);
                    }
                }
            }
        }

        return null;
    }

    /**
        IDL:omg.org/CORBA/Container/contents:1.0
     **/
    public Contained[]
    contents(DefinitionKind limit_type,
             boolean exclude_inherited)
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, limit_type, exclude_inherited);

        return toContainedSeq(selection);
    }

    /**
        IDL:omg.org/CORBA/Container/lookup_name:1.0
     **/
    public Contained[]
    lookup_name(String search_name,
                int levels_to_search,
                DefinitionKind limit_type,
                boolean exclude_inherited)
    {
        java.util.Vector selection = new java.util.Vector();

        selectNamedContained(selection, search_name, levels_to_search,
                             limit_type, exclude_inherited);

        return toContainedSeq(selection);
    }

    /**
        IDL:omg.org/CORBA/Container/describe_contents:1.0
     **/
    public org.omg.CORBA.ContainerPackage.Description[]
    describe_contents(DefinitionKind limit_type,
                      boolean exclude_inherited,
                      int max_returned_objs)
    {
        java.util.Vector selection = new java.util.Vector();
        selectContained(selection, limit_type, exclude_inherited);

        int len = selection.size();
        if(max_returned_objs >= 0 && len > max_returned_objs)
            len = max_returned_objs;

        org.omg.CORBA.ContainerPackage.Description[] result =
                 new org.omg.CORBA.ContainerPackage.Description[len];

        for(int i=0; i<len; i++)
	{
            Contained_impl contained = (Contained_impl)selection.elementAt(i);

            org.omg.CORBA.ContainerPackage.Description tmp =
                        new org.omg.CORBA.ContainerPackage.Description();

            tmp.contained_object = contained.asContained();
            tmp.kind = contained.def_kind();
            tmp.value = org.objectweb.ccm.CORBA.TheORB.create_any();
            contained.setDescriptionValue(tmp.value);

            result[i] = tmp;
	}

        return result;
    }

    /**
        IDL:omg.org/CORBA/Container/create_module:1.0
     **/
    public ModuleDef
    create_module(String id,
                  String name,
                  String version)
    {
        ModuleDef_impl module = new ModuleDef_impl(getIFR(), this);

        try
        {
            addContained(module, id, name, version);
        }
        catch(SystemException exc)
        {
            module.destroy();
            throw exc;
        }

        return module.asModuleDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_constant:1.0
     **/
    public ConstantDef
    create_constant(String id,
                    String name,
                    String version,
                    IDLType type,
                    Any value)
    {
        ConstantDef_impl constant = new ConstantDef_impl(getIFR(), this);

        try
        {
            addContained(constant, id, name, version);
            constant.type_def(type);
            constant.value(value);
        }
        catch(SystemException exc)
        {
            constant.destroy();
            throw exc;
        }

        return constant.asConstantDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_struct:1.0
     **/
    public StructDef
    create_struct(String id,
                  String name,
                  String version,
                  StructMember[] members)
    {
        StructDef_impl struct = new StructDef_impl(getIFR(), this);

        try
        {
            addContained(struct, id, name, version);
            struct.members(members);
        }
        catch(SystemException exc)
        {
            struct.destroy();
            throw exc;
        }

        return struct.asStructDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_union:1.0
     **/
    public UnionDef
    create_union(String id,
                 String name,
                 String version,
                 IDLType discriminator_type,
                 UnionMember[] members)
    {
        UnionDef_impl union = new UnionDef_impl(getIFR(), this);

        try
        {
            addContained(union, id, name, version);
            union.discriminator_type_def(discriminator_type);
            union.members(members);
        }
        catch(SystemException exc)
        {
            union.destroy();
            throw exc;
        }

        return union.asUnionDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_enum:1.0
     **/
    public EnumDef
    create_enum(String id,
                String name,
                String version,
                String[] members)
    {
        EnumDef_impl enume = new EnumDef_impl(getIFR(), this);

        try
        {
            addContained(enume, id, name, version);
            enume.members(members);
        }
        catch(SystemException exc)
        {
            enume.destroy();
            throw exc;
        }

        return enume.asEnumDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_alias:1.0
     **/
    public AliasDef
    create_alias(String id,
                 String name,
                 String version,
                 IDLType original_type)
    {
        AliasDef_impl alias = new AliasDef_impl(getIFR(), this);

        try
        {
            addContained(alias, id, name, version);
            alias.original_type_def(original_type);
        }
        catch(SystemException exc)
        {
            alias.destroy();
            throw exc;
        }

        return alias.asAliasDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_interface:1.0
     **/
    public InterfaceDef
    create_interface(String id,
                     String name,
                     String version,
                     InterfaceDef[] base_interfaces)
    {
        InterfaceDef_impl itf = new InterfaceDef_impl(getIFR(), this);

        try
        {
            addContained(itf, id, name, version);
            itf.base_interfaces(base_interfaces);
        }
        catch(SystemException exc)
        {
            itf.destroy();
            throw exc;
        }

        return itf.asInterfaceDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_abstract_interface:1.0
     **/
    public AbstractInterfaceDef
    create_abstract_interface(String id,
                              String name,
                              String version,
                              AbstractInterfaceDef[] base_interfaces)
    {
        AbstractInterfaceDef_impl itf =
             new AbstractInterfaceDef_impl(getIFR(), this);

        try
        {
            addContained(itf, id, name, version);
            itf.base_interfaces(base_interfaces);
        }
        catch(SystemException exc)
        {
            itf.destroy();
            throw exc;
        }

        return itf.asAbstractInterfaceDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_local_interface:1.0
     **/
    public LocalInterfaceDef
    create_local_interface(String id,
                           String name,
                           String version,
                           InterfaceDef[] base_interfaces)
    {
        LocalInterfaceDef_impl itf =
             new LocalInterfaceDef_impl(getIFR(), this);

        try
        {
            addContained(itf, id, name, version);
            itf.base_interfaces(base_interfaces);
        }
        catch(SystemException exc)
        {
            itf.destroy();
            throw exc;
        }

        return itf.asLocalInterfaceDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_value:1.0
     **/
    public ValueDef
    create_value(String id,
                 String name,
                 String version,
                 boolean is_custom,
                 boolean is_abstract,
                 ValueDef base_value,
                 boolean is_truncatable,
                 ValueDef[] abstract_base_values,
                 InterfaceDef[] supported_interfaces,
                 Initializer[] initializers)
    {
        ValueDef_impl value = new ValueDef_impl(getIFR(), this);

        try
        {
            addContained(value, id, name, version);
            value.is_custom(is_custom);
            value.is_abstract(is_abstract);
            value.base_value(base_value);
            value.is_truncatable(is_truncatable);
            value.abstract_base_values(abstract_base_values);
            value.supported_interfaces(supported_interfaces);
            value.initializers(initializers);
        }
        catch(SystemException exc)
        {
            value.destroy();
            throw exc;
        }

        return value.asValueDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_value_box:1.0
     **/
    public ValueBoxDef
    create_value_box(String id,
                     String name,
                     String version,
                     IDLType original_type_def)
    {
        ValueBoxDef_impl value_box = new ValueBoxDef_impl(getIFR(), this);

        try
        {
            addContained(value_box, id, name, version);
            value_box.original_type_def(original_type_def);
        }
        catch(SystemException exc)
        {
            value_box.destroy();
            throw exc;
        }

        return value_box.asValueBoxDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_exception:1.0
     **/
    public ExceptionDef
    create_exception(String id,
                     String name,
                     String version,
                     StructMember[] members)
    {
        ExceptionDef_impl e = new ExceptionDef_impl(getIFR(), this);

        try
        {
            addContained(e, id, name, version);
            e.members(members);
        }
        catch(SystemException exc)
        {
            e.destroy();
            throw exc;
        }

        return e.asExceptionDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_native:1.0
     **/
    public NativeDef
    create_native(String id,
                  String name,
                  String version)
    {
        NativeDef_impl n = new NativeDef_impl(getIFR(), this);

        try
        {
            addContained(n, id, name, version);
        }
        catch(SystemException exc)
        {
            n.destroy();
            throw exc;
        }

        return n.asNativeDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_ext_value:1.0
     **/
    public ExtValueDef
    create_ext_value(String id,
                     String name,
                     String version,
                     boolean is_custom,
                     boolean is_abstract,
                     ValueDef base_value,
                     boolean is_truncatable,
                     ValueDef[] abstract_base_values,
                     InterfaceDef[] supported_interfaces,
                     ExtInitializer[] initializers)
    {
        ExtValueDef_impl value = new ExtValueDef_impl(getIFR(), this);

        try
        {
            addContained(value, id, name, version);
            value.is_custom(is_custom);
            value.is_abstract(is_abstract);
            value.base_value(base_value);
            value.is_truncatable(is_truncatable);
            value.abstract_base_values(abstract_base_values);
            value.supported_interfaces(supported_interfaces);
            value.ext_initializers(initializers);
        }
        catch(SystemException exc)
        {
            value.destroy();
            throw exc;
        }

        return value.asExtValueDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_ext_interface:1.0
     **/
    public ExtInterfaceDef
    create_ext_interface(String id,
                         String name,
                         String version,
                         InterfaceDef[] base_interfaces)
    {
        ExtInterfaceDef_impl itf = new ExtInterfaceDef_impl(getIFR(), this);

        try
        {
            addContained(itf, id, name, version);
            itf.base_interfaces(base_interfaces);
        }
        catch(SystemException exc)
        {
            itf.destroy();
            throw exc;
        }

        return itf.asExtInterfaceDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_ext_abstract_interface:1.0
     **/
    public ExtAbstractInterfaceDef
    create_ext_abstract_interface(String id,
                                  String name,
                                  String version,
                                  AbstractInterfaceDef[] base_interfaces)
    {
        ExtAbstractInterfaceDef_impl itf =
             new ExtAbstractInterfaceDef_impl(getIFR(), this);

        try
        {
            addContained(itf, id, name, version);
            itf.base_interfaces(base_interfaces);
        }
        catch(SystemException exc)
        {
            itf.destroy();
            throw exc;
        }

        return itf.asExtAbstractInterfaceDef();
    }

    /**
        IDL:omg.org/CORBA/Container/create_ext_local_interface:1.0
     **/
    public ExtLocalInterfaceDef
    create_ext_local_interface(String id,
                               String name,
                               String version,
                               InterfaceDef[] base_interfaces)
    {
        ExtLocalInterfaceDef_impl itf =
             new ExtLocalInterfaceDef_impl(getIFR(), this);

        try
        {
            addContained(itf, id, name, version);
            itf.base_interfaces(base_interfaces);
        }
        catch(SystemException exc)
        {
            itf.destroy();
            throw exc;
        }

        return itf.asExtLocalInterfaceDef();
    }
}
