/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;
import org.omg.CORBA.ValueDefPackage.FullValueDescription;

/**
 * Implementation of the CORBA::ValueDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueDef_impl
       extends Container_impl
       implements ValueDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
        The supported interfaces.
     **/
    protected InterfaceDef_ref[] supported_interfaces_;

    /**
        The initializers.
     **/
    protected Initializer_impl[] initializers_;

    /**
        The base value.
     **/
    protected ValueDef_ref base_value_;

    /**
        The abstract base values.
     **/
    protected ValueDef_ref[] abstract_base_values_;

    /**
        Is abstract.
     **/
    protected boolean is_abstract_;

    /**
        Is custom.
     **/
    protected boolean is_custom_;

    /**
        Is truncatable.
     **/
    protected boolean is_truncatable_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    ValueDef_impl(IFR ifr,
                  Container_impl container)
    {
        // call the Container_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new ValueDefPOATie(this));

	// init internal state.
        supported_interfaces_ = null;
        initializers_ = new Initializer_impl[0];
        base_value_ = new ValueDef_ref();
        abstract_base_values_ = null;
        is_abstract_ = false;
        is_custom_ = false;
        is_truncatable_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  To obtain the list of abstract base values.
     **/
    protected ValueDef_ref[]
    getAbstractBaseValues()
    {
        return abstract_base_values_;
    }

    /**
     **  To obtain the base value.
     **/
    protected ValueDef_ref
    getBaseValue()
    {
        return base_value_;
    }

    // ==================================================================
    //
    // Internal methods inherited from IROject_impl.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        int idx = idSeq.indexOf(id());
        if (idx!=-1)
            return getIFR().getTCF().create_recursive_tc(id());

        idSeq.add(id());

        short mode = (short)0;
        if(is_custom())
            mode = (short)1;
        else if(is_abstract())
            mode = (short)2;
        else if(is_truncatable())
            mode = (short)3;
        else
            mode = (short)0;
 
	// Compute the base TypeCode.
	org.omg.CORBA.TypeCode tc = getBaseValue().recursiveType(idSeq);

	// for JavaORB ?
	if(tc == null)
	     tc = org.objectweb.ccm.CORBA.TheORB.get_null_valuebase_tc();

        ValueMember[] members = getValueMemberSeq(idSeq);
        idSeq.remove(id());

	 // delegate to the TypeCodeFactory.
	 return getIFR().getTCF().
              create_value_tc(id(), name(), mode,
                              tc, members);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::ValueDef object reference.
     **/
    public ValueDef
    asValueDef()
    {
        return ValueDefHelper.narrow(asObject());
    }

    /**
        Compute its CORBA::OpDescriptionSeq.
     **/
    public ValueMember[]
    getValueMemberSeq(org.objectweb.ccm.util.Vector idSeq)
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, DefinitionKind.dk_ValueMember, true);

        ValueMember[] result = new ValueMember[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((ValueMemberDef_impl)selection.elementAt(i))
                        .getValueMember(idSeq);

        return result;
    }

    /**
        Compute its CORBA::OpDescriptionSeq.
     **/
    public ValueMember[]
    getValueMemberSeq()
    {
        java.util.Vector selection = new java.util.Vector();

        selectContained(selection, DefinitionKind.dk_ValueMember, true);

        ValueMember[] result = new ValueMember[selection.size()];

        for(int i=0; i<selection.size(); i++)
            result[i] = ((ValueMemberDef_impl)selection.elementAt(i))
                        .getValueMember();

        return result;
    }

    /**
        Check name redefinition.
     **/
    public void
    checkNameInit(String name,
                  boolean exclude_inherited,
                  boolean in_inherited,
                  boolean check_init)
    {
        // calls the Container_impl::validateName operations.
        super.checkName(name, exclude_inherited, in_inherited);

        //
        // Check name against initializers.
        //
        if(check_init)
        {
            for(int i=0; i<initializers_.length; i++)
            {
                if(name.equalsIgnoreCase(initializers_[i].getName()))
                {
                    if(in_inherited)
                        throw exceptionNameClashInInheritedContext(name);
                    else
                        throw exceptionNameAlreadyUsed(name);
	        }
	    }
	}
 
        if(!exclude_inherited)
        {
	    //
  	    // Check name against base value.
	    //
            if(getBaseValue().getImpl() != null)
                getBaseValue().getImpl().checkNameInit(name, false, true, check_init);

	    //
  	    // Check name against abstract base values.
	    //
            ValueDef_ref[] abs_refs = getAbstractBaseValues();
            for(int i=0; i<abs_refs.length; i++)
                abs_refs[i].getImpl().checkNameInit(name, false,
                                                   true, check_init);

	    //
  	    // Check name against supported interfaces.
	    //
            for(int i=0; i<supported_interfaces_.length; i++)
                supported_interfaces_[i].getImpl().checkName(name, false, true);
        }
    }

    /**
     **  Obtain the CORBA::ComponentIR::ValueDescription of the declaration.
     **/
    public ValueDescription
    getValueDescription()
    {
        // create the CORBA::ValueDescription.
        ValueDescription description = new ValueDescription();
        description.name = name();
        description.id = id();
        description.is_abstract = is_abstract();
        description.is_custom = is_custom();
        description.defined_in = getContainerID();
        description.version = version();
        description.supported_interfaces = InterfaceDef_ref.
                         computeRepositoryIdSeq(supported_interfaces_);
        description.abstract_base_values = ValueDef_ref.
                         computeRepositoryIdSeq(getAbstractBaseValues());
        description.is_truncatable = is_truncatable();
        description.base_value = getBaseValue().getId();

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /**
        To set the value field of the CORBA::Contained::Description struct.
     **/
    public void
    setDescriptionValue(Any any)
    {
        // insert it into the any.
        ValueDescriptionHelper.insert(any, getValueDescription());
    }

    // ==================================================================
    //
    // For Container_impl class.
    //
    // ==================================================================

    /**
         Check if this kind of objects is accepted by the container.
     **/
    public boolean
    checkDefinitionKind(DefinitionKind dk)
    {
        return (dk == DefinitionKind.dk_Attribute)
            || (dk == DefinitionKind.dk_Constant)
            || (dk == DefinitionKind.dk_Exception)
            || (dk == DefinitionKind.dk_Operation)
            || (dk == DefinitionKind.dk_Alias)
            || (dk == DefinitionKind.dk_Struct)
            || (dk == DefinitionKind.dk_Union)
            || (dk == DefinitionKind.dk_Enum)
            || (dk == DefinitionKind.dk_Native)
            || (dk == DefinitionKind.dk_ValueMember);
    }

    /**
        Check a new name according to already contained names.
     **/
    public void
    checkName(String name,
              boolean exclude_inherited,
              boolean in_inherited)
    {
        checkNameInit(name, exclude_inherited, in_inherited, true);
    }

    /**
        Select named contained objects.
     **/
    public java.util.Vector
    selectNamedContained(java.util.Vector selection,
                         String search_name,
                         int levels_to_search,
                         DefinitionKind limit_type,
                         boolean exclude_inherited)
    {
        // call the Container_impl::selectContained operation.
        super.selectNamedContained(selection, search_name, levels_to_search,
                                   limit_type, exclude_inherited);

        if(!exclude_inherited)
        {
            for(int i=0; i<supported_interfaces_.length; i++)
	    {
                supported_interfaces_[i].getImpl().selectNamedContained(selection,
                                              search_name, levels_to_search,
                                              limit_type, false);
	    }

            if(getBaseValue().getImpl() != null)
                getBaseValue().getImpl().selectNamedContained(selection,
                                              search_name, levels_to_search,
                                              limit_type, false);

            ValueDef_ref[] abs_refs = getAbstractBaseValues();
            for(int i=0; i<abs_refs.length; i++)
	    {
                abs_refs[i].getImpl().selectNamedContained(selection,
                                              search_name, levels_to_search,
                                              limit_type, false);
	    }

        }

        return selection;
    }

    /**
        Select contained objects.
     **/
    public java.util.Vector
    selectContained(java.util.Vector selection,
                    DefinitionKind limit_type,
                    boolean exclude_inherited)
    {
        // call the Container_impl::selectContained operation.
        super.selectContained(selection, limit_type, exclude_inherited);

        if(!exclude_inherited)
        {
            for(int i=0; i<supported_interfaces_.length; i++)
	    {
                supported_interfaces_[i].getImpl().selectContained(
                                           selection, limit_type, false);
	    }

            if(getBaseValue().getImpl() != null)
                getBaseValue().getImpl().selectContained(
                                           selection, limit_type, false);

            ValueDef_ref[] abs_refs = getAbstractBaseValues();
            for(int i=0; i<abs_refs.length; i++)
	    {
                abs_refs[i].getImpl().selectContained(
                                           selection, limit_type, false);
	    }

        }

        return selection;
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Value;
    }

    /**
        IDL:omg.org/CORBA/IRObject/destroy:1.0
     **/
    public void
    destroy()
    {
        // cut the dependency to supported interfaces.
        InterfaceDef_ref.cutDependencies(supported_interfaces_);
        supported_interfaces_ = null;

        // cut the dependency to initializers.
        for(int i=0; i<initializers_.length; i++)
            initializers_[i].cutDependencies();
        initializers_ = null;

        // cut the dependency to the base value.
        base_value_.cutDependency();
        base_value_ = null;

        // cut the dependency to abstract base values.
        ValueDef_ref.cutDependencies(abstract_base_values_);
        abstract_base_values_ = null;

        // call the Container_impl::destroy operation.
        super.destroy();
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    // ==================================================================
    //
    // For CORBA::Container interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/Container/lookup:1.0
     **/
    public Contained
    lookup(String search_name)
    {
        // call the Container_impl::lookup operation.
        Contained contained = super.lookup(search_name);

        if (contained == null)
        {
            for(int i=0; i<supported_interfaces_.length; i++)
	    {
                contained = supported_interfaces_[i].getImpl().
                                           lookup(search_name);
                if(contained != null)
                    break;
	    }    
        }

        if (contained == null)
        {
	    if(getBaseValue().getImpl() != null)
                contained = getBaseValue().getImpl().lookup(search_name);
        }

        if (contained == null)
        {
            ValueDef_ref[] abs_refs = getAbstractBaseValues();
            for(int i=0; i<abs_refs.length; i++)
	    {
                Contained contained2 = abs_refs[i].getImpl().lookup(search_name);
                if(contained2 != null)
		{
                    if(contained != null)
	            {
                        String id1 = contained.id();
                        String id2 = contained2.id();
                        if(!id1.equals(id2))
                            throw exceptionAmbiguousLookup(id1, id2);
		    }

                    contained = contained2;
		}
	    }    
        }

        return contained;
    }

    // ==================================================================
    //
    // For CORBA::ValueDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/ValueDef/supported_interfaces:1.0
     **/
    public InterfaceDef[]
    supported_interfaces()
    {
        return InterfaceDef_ref.computeInterfaceDefSeq(supported_interfaces_);
    }

    public void
    supported_interfaces(InterfaceDef[] val)
    {
        InterfaceDef_impl[] itfs = new InterfaceDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            itfs[i] = castToLocal(val[i]);

        // TODO: check inheritance

        InterfaceDef_ref[] refs = new InterfaceDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new InterfaceDef_ref( itfs[i] );

        // cut the previous dependency to base interfaces.
        InterfaceDef_ref.cutDependencies(supported_interfaces_);

        supported_interfaces_ = refs;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/initializers:1.0
     **/
    public Initializer[]
    initializers()
    {
        Initializer[] result = new Initializer[initializers_.length];

        for(int i=0; i<result.length; i++)
            result[i] = initializers_[i].getInitializer();

        return result;
    }

    public void
    initializers(Initializer[] val)
    {
        Initializer_impl[] tmp = new Initializer_impl[val.length];
        for(int i=0; i<tmp.length; i++)
	{
            tmp[i] = new Initializer_impl(this);
            tmp[i].setInitializer(val[i]);
        }
        initializers_ = tmp;
    }

    /**
       IDL:omg.org/CORBA/ValueDef/base_value:1.0
     **/
    public ValueDef
    base_value()
    {
        // delegate to the ValueDef reference.
        return getBaseValue().asValueDef();
    }

    public void
    base_value(ValueDef val)
    {
        if (val == null)
            getBaseValue().setDependency(null);
        else
        {
            // try to cast val to a local object
            // and delegate to the ValueDef reference.
            getBaseValue().setDependency( castToLocal(val) );
        }
    }

    /**
       IDL:omg.org/CORBA/ValueDef/abstract_base_values:1.0
     **/
    public ValueDef[]
    abstract_base_values()
    {
        ValueDef_ref[] abs_refs = getAbstractBaseValues();
        ValueDef[] result = new ValueDef[abs_refs.length];

        for(int i=0; i<abs_refs.length; i++)
             result[i] = abs_refs[i].asValueDef();

        return result;
    }

    public void
    abstract_base_values(ValueDef[] val)
    {
        ValueDef_impl[] values = new ValueDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            values[i] = castToLocal(val[i]);

        // TODO: check inheritance

        ValueDef_ref[] refs = new ValueDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new ValueDef_ref( values[i] );

        // cut the previous dependency to abstract base values.
        ValueDef_ref.cutDependencies(abstract_base_values_);

        abstract_base_values_ = refs;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/is_abstract:1.0
     **/
    public boolean
    is_abstract()
    {
        return is_abstract_;
    }

    public void
    is_abstract(boolean val)
    {
        // TODO: check
        is_abstract_ = val;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/is_custom:1.0
     **/
    public boolean
    is_custom()
    {
        return is_custom_;
    }

    public void
    is_custom(boolean val)
    {
        // TODO: check
        is_custom_ = val;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/is_truncatable:1.0
     **/
    public boolean
    is_truncatable()
    {
        return is_truncatable_;
    }

    public void
    is_truncatable(boolean val)
    {
        // TODO: check
        is_truncatable_ = val;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/is_a:1.0
     **/
    public boolean
    is_a(String the_id)
    {
        if(the_id.equals(id()))
            return true;

        for(int i=0; i<supported_interfaces_.length; i++)
            if (supported_interfaces_[i].getImpl().is_a(the_id))
                return true;

        if(getBaseValue().getImpl() != null
           && getBaseValue().getImpl().is_a(the_id))
            return true;

        ValueDef_ref[] abs_refs = getAbstractBaseValues();
        for(int i=0; i<abs_refs.length; i++)
            if (abs_refs[i].getImpl().is_a(the_id))
                return true;

        return false;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/describe_value:1.0
     **/
    public FullValueDescription
    describe_value()
    {
        FullValueDescription description = new FullValueDescription();
        description.name = name();
        description.id = id();
        description.is_abstract = is_abstract();
        description.is_custom = is_custom();
        description.defined_in = getContainerID();
        description.version = version();
        description.operations = getOpDescriptionSeq();
        description.attributes = getAttrDescriptionSeq();
        description.members = getValueMemberSeq();
        description.initializers = initializers();
        description.supported_interfaces = InterfaceDef_ref.
                         computeRepositoryIdSeq(supported_interfaces_);
        description.abstract_base_values = ValueDef_ref.
                         computeRepositoryIdSeq(getAbstractBaseValues());
        description.is_truncatable = is_truncatable();
        description.base_value = getBaseValue().getId();
        description.type = type();
        return description;
    }

    /**
        IDL:omg.org/CORBA/ValueDef/create_value_member:1.0
     **/
    public ValueMemberDef
    create_value_member(String id,
                        String name,
                        String version,
                        IDLType type,
                        short access)
    {
        ValueMemberDef_impl vm = new ValueMemberDef_impl(getIFR(), this);

        try
        {
            addContained(vm, id, name, version);
            vm.type_def(type);
            vm.access(access);
        }
        catch(SystemException exc)
        {
            vm.destroy();
            throw exc;
        }

        return vm.asValueMemberDef();
    }

    /**
        IDL:omg.org/CORBA/ValueDef/create_attribute:1.0
     **/
    public AttributeDef
    create_attribute(String id,
                     String name,
                     String version,
                     IDLType type,
                     AttributeMode mode)
    {
        AttributeDef_impl attr = new AttributeDef_impl(getIFR(), this);

        try
        {
            addContained(attr, id, name, version);
            attr.type_def(type);
            attr.mode(mode);
        }
        catch(SystemException exc)
        {
            attr.destroy();
            throw exc;
        }

        return attr.asAttributeDef();
    }

    /**
        IDL:omg.org/CORBA/ValueDef/create_operation:1.0
     **/
    public OperationDef
    create_operation(String id,
                     String name,
                     String version,
                     IDLType result,
                     OperationMode mode,
                     ParameterDescription[] params,
                     ExceptionDef[] exceptions,
                     String[] contexts)
    {
        OperationDef_impl op = new OperationDef_impl(getIFR(), this);

        try
        {
            addContained(op, id, name, version);
            op.result_def(result);
            op.mode(mode);
            op.params(params);
            op.exceptions(exceptions);
            op.contexts(contexts);
        }
        catch(SystemException exc)
        {
            op.destroy();
            throw exc;
        }

        return op.asOperationDef();
    }
}
