/*================================================================
  
OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.plugins.transaction.deployment ;

import org.objectweb.ccm.plugins.transaction.deployment.util.*; 
import org.omg.CosTransactions.Coordinator ;

/**
 * This class implements the OMG IDL
 * ::plugins::deployment::transaction::ResourceManager interface.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.3
 */
public class ResourceManagerImpl
extends ResourceManagerPOA
implements ResourceManagerLocal
{
  // ==================================================================
  //
  // Internal state.
  //
  // ==================================================================
  /**
   * number of resources list available
   */
  protected static int resources_count_=3;
  
  /**
   * array of list of Resources available
   *
   * index 0 : home resources
   * index 1 : container resources
   * index 2 : component server resources
   */ 
  protected ResourceVector[] resources_;
  
  /**
   * reference to the transaction manager
   */ 
  protected TransactionManagerLocal OTSManager_;

  /**
   * reference to the transaction coordinator
   */ 
  protected Coordinator coordinator_; 
  
  // ==================================================================
  //
  // Constructor.
  //
  // ==================================================================
  /**
   * Constructor with parameters
   *
   * @param coordinator the transaction coordinator
   * @param manager the transaction manager
   */ 
  public
  ResourceManagerImpl (Coordinator coordinator, 
                       TransactionManagerLocal manager)
  {
    coordinator_ = coordinator; 
    OTSManager_ = manager; 
    resources_ = new ResourceVector[resources_count_];
    for (int i=0 ; i < resources_count_ ; i++) 
      resources_[i] = new ResourceVector();
  }
  
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  /**
   * Apply a specific treatment on each list
   *
   * @param action action to execute
   *
   */
  protected void
  apply(ResourceTreatment action)
    throws Exception
  {
    for (int i=1 ; i<=resources_count_ ; i++)
      resources_[resources_count_-i].apply(action);
  }
  
  /**
   * Apply a specific treatment on each list
   *
   * @param action action to execute
   *
   */
  protected void
  reverse_apply(ResourceTreatment action)
    throws Exception
  {
    for (int i=0 ; i < resources_count_ ; i++)
      resources_[i].apply(action);
  }
    
  /**
   * Desactivate the current resource
   *
   */ 
  protected void
  removeItself()
  {
    try {    
      OTSManager_.remove_manager (this);
    } catch ( org.omg.Components.RemoveFailure ex ){ }
    try {
      _poa().deactivate_object( _poa().servant_to_id( this ) );
    } catch ( org.omg.PortableServer.POAPackage.ObjectNotActive ex ){ 
    } catch ( org.omg.PortableServer.POAPackage.WrongPolicy ex ) { 
    } catch ( org.omg.PortableServer.POAPackage.ServantNotActive ex ) { 
    }
  }  
  
  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  // ==================================================================
  //
  // Methods for the ResourceManagerLocal interface.
  //
  // ==================================================================
  //
  //  JAVA:ResourceManagerLocal/add_component_server
  //
  /**
   * Add a new Component Server Resource in the Resource Manager
   * 
   * @param ref reference of the ComponentServerResource to register
   *
   */
  public void
  add_component_server (ComponentServerResource ref)
  {
    resources_[2].addElement (ref); 
  }
  
  //
  //  JAVA:ResourceManagerLocal/add_container
  //
  /**
   * Add a new Container Resource in the Resource Manager
   * 
   * @param ref reference of the ComponentServerResource to register
   *
   */
  public void
  add_container (ContainerResource ref)
  {
    resources_[1].addElement (ref); 
  }
  
  //
  //  JAVA:ResourceManagerLocal/add_home
  //
  /**
   * Add a new Home Resource in the Resource Manager
   * 
   * @param ref reference of the ComponentServerResource to register
   *
   */
  public void
  add_home (CCMHomeResource ref)
  {
    resources_[0].addElement (ref); 
  }
  
  //
  //  JAVA:ResourceManagerLocal/remove_component_server
  //
  /**
   * Remove a Component Server Resource from the Resource Manager
   * 
   * @param ref reference of the ComponentServerResource to remove
   *
   */
  public void
  remove_component_server(ComponentServerResource ref)
    throws org.omg.Components.RemoveFailure
  {
    resources_[2].removeElement (ref); 
  }
  
  //
  //  JAVA:ResourceManagerLocal/remove_container
  //
  /**
   * Remove a new Container Resource in the Resource Manager
   * 
   * @param ref reference of the ComponentServerResource to register
   *
   */
  public void
  remove_container (ContainerResource ref)
    throws org.omg.Components.RemoveFailure
  {
    resources_[1].removeElement (ref); 
  }
  
  //
  //  JAVA:ResourceManagerLocal/remove_home
  //
  /**
   * Remove a new Home Resource in the Resource Manager
   * 
   * @param ref reference of the ComponentServerResource to register
   *
   */
  public void
  remove_home (CCMHomeResource ref)
    throws org.omg.Components.RemoveFailure
  {
    resources_[0].removeElement (ref); 
  }
  
  // ==================================================================
  //
  // Methods for the CosTransactions::Resource interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/CosTransaction/Resource/prepare:1.0
  //
  /**
   * Vote request preparing the commit phase
   *
   * @return the vote for the resource
   */ 
  public org.omg.CosTransactions.Vote
  prepare()
    throws org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    try {
      ResourcePrepare _vote = new ResourcePrepare();
      apply(_vote);
      return _vote.final_vote(); 
    } catch (Exception ex) {
      return org.omg.CosTransactions.Vote.VoteRollback; 
    }				
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/rollback:1.0
  //
  /**
   * Rollback all operations registered by the resource
   * 
   */ 
  public void
  rollback()
    throws org.omg.CosTransactions.HeuristicCommit,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  {
    try
    {
      reverse_apply(new ResourceRollback());
    } catch (org.omg.CosTransactions.HeuristicCommit ex) {
      throw ex;
    } catch (org.omg.CosTransactions.HeuristicMixed ex) {
      throw ex;
    } catch (org.omg.CosTransactions.HeuristicHazard ex) {
      throw ex;
    } catch (Exception ex) { }
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit:1.0
  //
  /**
   * Commit all operations registered by the resource
   * 
   */ 
  public void
  commit()
    throws org.omg.CosTransactions.NotPrepared,
    org.omg.CosTransactions.HeuristicRollback,
    org.omg.CosTransactions.HeuristicMixed,
    org.omg.CosTransactions.HeuristicHazard
  { 
    try {
      apply(new ResourceCommit());
    } catch (org.omg.CosTransactions.NotPrepared ex) {
      throw ex;
    } catch (org.omg.CosTransactions.HeuristicRollback ex) {
      throw ex;
    } catch (org.omg.CosTransactions.HeuristicMixed ex) {
      throw ex;
    } catch (org.omg.CosTransactions.HeuristicHazard ex) {
      throw ex;
    } catch (Exception ex) { }
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/commit_one_phase:1.0
  //
  /**
   * One phase commit of all operations registered by the resource
   *
   */ 
  public void
  commit_one_phase()
    throws org.omg.CosTransactions.HeuristicHazard
  {
    try {
      apply(new ResourceCommitOnePhase());
    } catch(org.omg.CosTransactions.HeuristicHazard ex) {
      throw ex;
    } catch (Exception ex) { }
    removeItself();
  }
  
  //
  //  IDL:omg.org/CosTransaction/Resource/forget:1.0
  //
  /**
   * Delete the current resource from the resource manager
   *
   */ 
  public void
  forget()
  {
    removeItself();
  }
  
  // ==================================================================
  //
  // Methods for the ::plugins::deployment::transaction::CosCoordinator interface.
  //
  // ==================================================================
  //
  //  IDL:omg.org/plugins/deployment/transaction/ResourceManager/coordinator:1.0
  //
  //
  //  JAVA:ResourceManagerLocal/coordinator
  //
  /**
   * Provide the reference of the transaction coordinator
   *
   * @return reference on the coordinator
   */ 
  public Coordinator
  coordinator()
  {
    return coordinator_;
  }
}

