/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util.io;

/**
 * This class handles an output stream with:
 * - indentation management.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class PrintStream
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The managed stream.
     **/
    private java.io.PrintStream stream_;

    /**
     **  The tabular indentation.
     **/
    private String tabular_;

    /**
     **  The current indentation depth.
     **/
    private int depth_;

    /**
     **  The name of the underlying file.
     **/
    private String name_;

    /**
     **  The manager for this stream.
     **/
    private PrintStreamManager manager_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Constructor.
     **
     **  @param stream A Java PrintStream on which data will be written.
     **  @param tabular The atomic indentation that should contains only space and tabulation
     **                 characters.
     **  @param name The name of the underlying file.
     **/
    public
    PrintStream(java.io.PrintStream stream,
                String tabular,
                String name,
                PrintStreamManager manager)
    {
        // init internal state.
        stream_ = stream;
        tabular_ = tabular;
        depth_ = 0;
        name_ = name;
        manager_ = manager;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    close()
    {
        stream_.close();
    }

    /**
     **  Obtain the name of the underlying file.
     **
     **  @return The file name.
     **/
    public String
    getName()
    {
        return name_;
    }

    /**
     **  Obtain the managed stream.
     **
     **  @return The Java PrintStream on which the data are written.
     **/
    public java.io.PrintStream
    getStream()
    {
        return stream_;
    }

    /**
     **  Obtain the manager of this stream.
     **
     **  @return The PrintStreamManager that manages this stream.
     **/
    public PrintStreamManager
    getManager()
    {
        return manager_;
    }

    /**
     **  Return the current indentation.
     **
     **  @return The current indentation ie depth times the atomic indentation.
     **/
    public String
    getTab()
    {
        StringBuffer ret = new StringBuffer("");
        for(int i=0;i<depth_;i++)
            ret.append(tabular_);
        return ret.toString();
    }

    /**
     **  Increment the depth.
     **/
    public void
    incDepth()
    {
        depth_++;
    }

    /**
     **  Decrement the depth.
     **/
    public void
    decDepth()
    {
        depth_--;
    }

    /**
     **  Print indentation.
     **/
    public void
    printTab()
    {
        for(int i=0; i<depth_; i++)
            stream_.print(tabular_);
    }

    /**
     **  Print a text line with indentation.
     **
     **  @param textLine The text to print.
     **/
    public void
    print(String textLine)
    {
        printTab();
        stream_.print(textLine);
    }

    /**
     **  Print a text line with indentation and new line.
     **
     **  @param textLine The text to print.
     **/
    public void
    println(String textLine)
    {
        printTab();
        stream_.println(textLine);
    }
}
