/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.util.io;

/**
 * To manage many PrintStream objects at the same time ie :
 * - creation and closing
 * - directory setting
 * - streams managment
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class PrintStreamManager
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The directory in which files will be created.
     **/
    private String directory_;

    /**
     **  The list of already created PrintStream objects.
     **/
    private org.objectweb.ccm.util.Table streams_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  Default Constructor.
     **/
    public
    PrintStreamManager()
    {
        directory_ = "";
        streams_ = new org.objectweb.ccm.util.Table();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  Create a new PrintStream connected to a file named name.
     **
     **  @param name The name that will be used for file creation.
     **  @param tabular The atomic indentation.
     **
     **  @return The newly created PrintStream.
     **/
    public PrintStream
    create(String name, String tabular)
    {
        PrintStream ps = null;
        java.io.File file = new java.io.File(directory_);
        try
        {
            file.mkdirs();
            // check if the file name is stdout.
            // in this case, we don't have to create a new file.
            if (name.equals("stdout"))
                ps = new PrintStream(System.out,tabular, "stdout", this);
            else
                ps = new PrintStream(
                         new java.io.PrintStream(new java.io.FileOutputStream(directory_ + name)),
                         tabular, name, this);
        }
        catch (java.io.FileNotFoundException e)
	{
            throw new Error("File " + directory_ + name + " not found.");
        }
        catch (java.lang.SecurityException e)
        {
            throw new Error("Directory " + directory_ + " can\'t be created.");
        }
        streams_.put(name, ps);
        return ps;
    }

    /**
     **  To close an already created PrintStream.
     **
     **  @param name The name of the underlying file.
     **/
    public void
    close(String name)
    {
        PrintStream ps = (PrintStream)streams_.remove(name);
        if (!name.equals("stdout"))
	{
            ps.close();
	}
    }

    /**
     **  Set the directory where the file should be created.
     **  Note that you must set the directory before calling the create method.
     **
     **  @param directory The directory where the files will be create.
     **                   If the directory does not exist, it will be created.
     **/
    public void
    setDirectory(String directory)
    {
        directory_ = directory;
    }

    /**
     **  To obtain a already created PrintStream according to it's underlying file name.
     **
     **  @param name The file name.
     **
     **  @return The PrintStream or null if it was already closed or never created.
     **/
    public PrintStream
    getPrintStream(String name)
    {
        return (PrintStream)streams_.get(name);
    }
}
