/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To use CORBA::InterfaceDef. */
import org.omg.CORBA.InterfaceDef;

/** To use CORBA::ExtInterfaceDef. */
import org.omg.CORBA.ExtInterfaceDef;

/** To use CORBA::AbstractInterfaceDef. */
import org.omg.CORBA.AbstractInterfaceDef;

/** To use CORBA::ExtAbstractInterfaceDef. */
import org.omg.CORBA.ExtAbstractInterfaceDef;
import org.omg.CORBA.ExtAbstractInterfaceDefHelper;

/**
 * AbstractInterfaceDeclImpl is a wrapper class for
 * IDL abstract interface declarations.
 *
 * Inherits from:
 * - InterfaceDeclImpl as they are also IDL interfaces.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AbstractInterfaceDeclImpl
       extends InterfaceDeclImpl
       implements org.objectweb.corba.ast.api.AbstractInterfaceDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 AbstractInterfaceDef. */
    private ExtAbstractInterfaceDef ext_abstract_interface_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    AbstractInterfaceDeclImpl(Repository rep,
                              ScopeImpl parent)
    {
        // Call the InterfaceDefImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_abstract_interface_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 InterfaceDef.
     *
     * @param contained The InterfaceDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_abstract_interface_def_ =
             ExtAbstractInterfaceDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 InterfaceDef.
     *
     * @param contained The InterfaceDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_abstract_interface_def_ =
             ExtAbstractInterfaceDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the abstract
     *         interface declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the abstract
     *         interface declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
       return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        AbstractInterfaceDef[] itfs =
            new AbstractInterfaceDef[inherited_interfaces_.getSize()];

        for(int i=0; i<itfs.length; i++)
            itfs[i] = ((AbstractInterfaceDeclImpl)
                      (inherited_interfaces_.get(i))).
                      getExtAbstractInterfaceDef();

        ext_abstract_interface_def_ = the_parent_.getContainer().
             create_ext_abstract_interface(getId(), getName(),
                                           getVersion(), itfs);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the abstract
     *         interface declaration.
     */
    public ExtInterfaceDef
    getExtInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the abstract
     *         interface declaration.
     */
    public InterfaceDef
    getInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    /**
     * Obtain its AbstractInterfaceDef reference.
     *
     * @return The AbstractInterfaceDef object associated with
     *         the abstract interface declaration.
     */
    public ExtAbstractInterfaceDef
    getExtAbstractInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    /**
     * Obtain its AbstractInterfaceDef reference.
     *
     * @return The AbstractInterfaceDef object associated with
     *         the abstract interface declaration.
     */
    public AbstractInterfaceDef
    getAbstractInterfaceDef()
    {
        return ext_abstract_interface_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_abstract_interface;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_abstract_interface;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfaceDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AbstractInterfaceDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the abstract interface declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_abstract_interface_def_;
    }
}
