/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To use CORBA::AliasDef. */
import org.omg.CORBA.AliasDef;
import org.omg.CORBA.AliasDefHelper;

/**
 * AliasDeclImpl is a wrapper class for IDL typedef declarations.
 *
 * Inherits from:
 * - DeclarationWithTypeRefImpl as typedefs are IDL declarations
 *   which refer to an original type.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AliasDeclImpl
       extends DeclarationWithTypeRefImpl
       implements org.objectweb.corba.ast.api.AliasDecl,
                  IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 AliasDef. */
    private AliasDef alias_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    AliasDeclImpl(Repository rep,
                  ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        alias_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 AliasDef.
     *
     * @param contained The AliasDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        alias_def_ = AliasDefHelper.narrow(contained);
        setType(getRepository().getAsTypeRef(alias_def_.original_type_def()));
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 AliasDef.
     *
     * @param contained The AliasDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        alias_def_ = AliasDefHelper.narrow(contained);
        setType(getRepository().getAsMappedTypeRef(alias_def_.
                                          original_type_def()));
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the typedef declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return alias_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector alias_depend = new java.util.Vector();

        // content
        if (getType().isDeclaration())
            alias_depend.add(getType());

        Declaration[] depend = getType().getDependencies();
        for (int i=0; i<depend.length; i++)
            alias_depend.add(depend[i]);

        dependencies_ = (Declaration[])alias_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_alias;
    }

    /**
     * Create the constant declaration into the IFR.
     */
    public void
    create()
    {
        alias_def_ = the_parent_.getContainer().
                     create_alias(getId(), getName(), getVersion(), super.getIDLType());
        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_alias;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AliasDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the type reference.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return alias_def_;
    }
}
