/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to report errors. */
import org.objectweb.corba.ast.api.ASTError;

/** Used to create CORBA Any. */
import org.objectweb.ccm.CORBA.TheORB;

/** Used to create CORBA DynamicAny::DynAny. */
import org.objectweb.ccm.CORBA.TheDynamicAnyFactory;

/** Used to get original TypeCodes. */
import org.objectweb.ccm.CORBA.TypeCodeUtils;

/** Used to wrap CORBA user exceptions. */
import org.objectweb.ccm.CORBA.UserExceptionWrapper;

/** Used to manipulate CORBA TypeCode. */
import org.omg.CORBA.TypeCode;

/** Used to manipulate CORBA TypeCode kind. */
import org.omg.CORBA.TCKind;

/** Used to manipulate CORBA Any. */
import org.omg.CORBA.Any;

/** Used to manipulate CORBA DynamicAny::DynAny. */
import org.omg.DynamicAny.DynAny;

/** Used to manipulate CORBA DynamicAny::DynFixed. */
import org.omg.DynamicAny.DynFixed;
import org.omg.DynamicAny.DynFixedHelper;

/** Used to manipulate CORBA DynamicAny::DynEnum. */
import org.omg.DynamicAny.DynEnum;
import org.omg.DynamicAny.DynEnumHelper;

/**
 * AnyValueImpl is a wrapper class to manipulate CORBA::Any objects
 * more easily.
 *
 * This provides getting, setting, and testing operations
 * for basic IDL data types.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AnyValueImpl
       extends ObjectBase
       implements org.objectweb.corba.ast.api.AnyValue 
{
    // ==================================================================
    //
    // Kind of type.
    //
    // ==================================================================

    /** Represent unknown type for the Any value. */
    final private int NONE = 0;

    /** Represent integer type for the Any value. */
    final private int INTEGER = 1;

    /** Represent string type for the Any value. */
    final private int STRING = 2;

    /** Represent wide string type for the Any value. */
    final private int WSTRING = 3;

    /** Represent character type for the Any value. */
    final private int CHAR = 4;

    /** Represent wide character type for the Any value. */
    final private int WCHAR = 5;

    /** Represent fixed type for the Any value. */
    final private int FIXED = 6;

    /** Represent float type for the Any value. */
    final private int FLOATING = 7;

    /** Represent boolean type for the Any value. */
    final private int BOOLEAN = 8;

    /** Represent enumerate type for the Any value. */
    final private int ENUM = 9;

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The type of the constant. */
    private int type_;

    /** The value when type_ == INTEGER. */
    private long integer_;

    /** The value when type_ == STRING or WSTRING. */
    private String string_;

    /** The value when type_ == CHAR or WCHAR. */
    private char char_;

    /** The value when type_ == FIXED. */
    private String fixed_;

    /** The value when type_ == FLOATING. */
    private double floating_;

    /** The value when type_ == BOOLEAN. */
    private boolean boolean_;

    /** The TypeCode when type_ == ENUM. */
    private TypeCode tc_;

    /** The enumerator when type_ == ENUM. */
    private int enumerator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    AnyValueImpl()
    {
        // Init as empty value.
        type_ = NONE;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtain the any value as a CORBA::Any.
     *
     * @param tc The TypeCode of the desired any value.
     *           Note that one of the set* method should have been used
     *           to fix the value before using this method.
     *           If the TypeCode and the set* method are not consistent,
     *           an ASTError is thrown.
     *
     * @return An Any created from the TypeCode.
     */
    protected Any
    computeAny(TypeCode tc)
    {
        if(type_ == NONE) {
            Any any = TheORB.create_any();
            any.insert_octet((byte)0);
            return any;
        }

        TypeCode origTC = TypeCodeUtils.getOrigType(tc);

        // TODO: Why using DynAny in all cases?
        // TODO: Why creating DynAny from the unaliased TypeCode?
        DynAny dynAny = TheDynamicAnyFactory.
                        create_dyn_any_from_type_code(origTC);

        try {
            switch(origTC.kind().value()) {
            case TCKind._tk_short:
                if(type_ != INTEGER)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_short((short)integer_);
                break;

            case TCKind._tk_long:
                if(type_ != INTEGER)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_long((int)integer_);
                break;

            case TCKind._tk_ushort:
                if(type_ != INTEGER)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_ushort((short)integer_);
                break;

            case TCKind._tk_ulong:
                if(type_ != INTEGER)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_ulong((int)integer_);
                break;

            case TCKind._tk_float:
                if(type_ != FLOATING)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_float((float)floating_);
                break;

            case TCKind._tk_double:
                if(type_ != FLOATING)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_double(floating_);
                break;

            case TCKind._tk_boolean:
                if(type_ != BOOLEAN)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_boolean(boolean_);
                break;

            case TCKind._tk_char:
                if(type_ != CHAR)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_char(char_);
                break;

            case TCKind._tk_string:
                if(type_ != STRING)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_string(string_);
                break;

            case TCKind._tk_longlong:
                if(type_ != INTEGER)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_longlong(integer_);
                break;

            case TCKind._tk_ulonglong:
                if(type_ != INTEGER)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_ulonglong(integer_);
                break;

            case TCKind._tk_wchar:
                if(type_ != WCHAR)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_wchar(char_);
                break;

            case TCKind._tk_wstring:
                if(type_ != WSTRING)
                    throw new ASTError("Invalid constant value!");
                dynAny.insert_wstring(string_);
                break;

            case TCKind._tk_fixed:
                if(type_ != FIXED)
                    throw new ASTError("Invalid constant value!");
                DynFixedHelper.narrow(dynAny).set_value(fixed_);
                break;

            case TCKind._tk_enum:
                if(type_ != ENUM)
                    throw new ASTError("Invalid constant value!");
                if(!tc_.equal(origTC))
                    throw new ASTError("Invalid constant type!");
                DynEnumHelper.narrow(dynAny).set_as_ulong(enumerator_);
                break;

            default:
                throw new ASTError("Invalid constant type!");
            }
            return dynAny.to_any();
        }
        catch(org.omg.DynamicAny.DynAnyPackage.InvalidValue exc)
        {
            // Wrap the DynamicAny::InvalidValue exception.
            throw new UserExceptionWrapper(exc);
        }
        catch(org.omg.DynamicAny.DynAnyPackage.TypeMismatch exc)
        {
            // Wrap the DynamicAny::TypeMismatch exception.
            throw new UserExceptionWrapper(exc);
        }
    }

    /**
     * Set the Any value from a CORBA::Any.
     *
     * @param any The CORBA::Any.
     */
    protected void
    loadAny(Any any)
    {
        TypeCode tc = any.type();
        try
        {
            switch(tc.kind().value())
            {
            case TCKind._tk_short:
                setAsInteger(any.extract_short());
                break;
            case TCKind._tk_long:
                setAsInteger(any.extract_long());
                break;
            case TCKind._tk_ushort:
                setAsInteger(any.extract_ushort());
                break;
            case TCKind._tk_ulong:
                setAsInteger(any.extract_ulong());
                break;
            case TCKind._tk_float:
                setAsFloating(any.extract_float());
                break;
            case TCKind._tk_double:
                setAsFloating(any.extract_double());
                break;
            case TCKind._tk_boolean:
                setAsBoolean(any.extract_boolean());
                break;
            case TCKind._tk_char:
                setAsChar(any.extract_char());
                break;
            case TCKind._tk_wchar:
                setAsWChar(any.extract_wchar());
                break;
            case TCKind._tk_string:
                setAsString(any.extract_string());
                break;
            case TCKind._tk_wstring:
                setAsWString(any.extract_wstring());
                break;
            case TCKind._tk_longlong:
                setAsInteger(any.extract_longlong());
                break;
            case TCKind._tk_ulonglong:
                setAsInteger(any.extract_ulonglong());
                break;
            case TCKind._tk_fixed:
                DynFixed fixed = TheDynamicAnyFactory.create_dyn_fixed(any);
                setAsFixed(fixed.get_value());
                break;
            case TCKind._tk_enum:
                DynEnum enume = TheDynamicAnyFactory.create_dyn_enum(any);
                setAsEnum(tc, enume.get_as_ulong(), enume.get_as_string());
                break;
            default:
                break;
            }

        } catch(org.omg.CORBA.BAD_OPERATION ex) {
            // Wrap the CORBA::BAD_OPERATION exception.
            throw new ASTError(ex.getMessage());
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AnyValue
    //
    // ==================================================================

    /**
     * Does it contain an integer value?
     *
     * @return True if it's an integer or false otherwise.
     */
    public boolean
    isInteger()
    {
        return type_ == INTEGER;
    }

    /**
     * Set with an integer value.
     *
     * @param value The integer to set as value.
     */
    public void
    setAsInteger(long value)
    {
        type_ = INTEGER;
        integer_ = value;
    }

    /**
     * Get as an integer value.
     *
     * @return The integer that must have been previously set.
     */
    public long
    getAsInteger()
    {
        return integer_;
    }

    /**
     * Does it contain a string value?
     *
     * @return True if it's a string or false otherwise.
     */
    public boolean
    isString()
    {
        return type_ == STRING;
    }

    /**
     * Set with a string value.
     *
     * @param value The string to set as value.
     */
    public void
    setAsString(String value)
    {
        type_ = STRING;
        string_ = value;
    }

    /**
     * Get as a string value.
     *
     * @return The string that must have been previously set.
     */
    public String
    getAsString()
    {
        return string_;
    }

    /**
     * Does it contain a wstring value?
     *
     * @return True if it's a wstring or false otherwise.
     */
    public boolean
    isWString()
    {
        return type_ == WSTRING;
    }

    /**
     * Set with a wstring value.
     *
     * @param value The wstring to set as value.
     */
    public void
    setAsWString(String value)
    {
        type_ = WSTRING;
        string_ = value;
    }

    /**
     * Get as a wstring value.
     *
     * @return The wstring that must have been previously set.
     */
    public String
    getAsWString()
    {
        return string_;
    }

    /**
     * Does it contain a char value?
     *
     * @return True if it's a char or false otherwise.
     */
    public boolean
    isChar()
    {
        return type_ == CHAR;
    }

    /**
     * Set with a char value.
     *
     * @param value The char to set as value.
     */
    public void
    setAsChar(char value)
    {
        type_ = CHAR;
        char_ = value;
    }

    /**
     * Get as a char value.
     *
     * @return The char that must have been previously set.
     */
    public char
    getAsChar()
    {
        return char_;
    }

    /**
     * Does it contain a wchar value?
     *
     * @return True if it's a wchar or false otherwise.
     */
    public boolean
    isWChar()
    {
        return type_ == WCHAR;
    }

    /**
     * Set with a wchar value.
     *
     * @param value The char to set as value.
     */
    public void
    setAsWChar(char value)
    {
        type_ = WCHAR;
        char_ = value;
    }

    /**
     * Get as a wchar value.
     *
     * @return The wchar that must have been previously set.
     */
    public char
    getAsWChar()
    {
        return char_;
    }

    /**
     * Does it contain a fixed value?
     *
     * @return True if it's a fixed or false otherwise.
     */
    public boolean
    isFixed()
    {
        return type_ == FIXED;
    }

    /**
     * Set with a fixed value.
     *
     * @param value The fixed string to set as value.
     */
    public void
    setAsFixed(String value)
    {
        type_ = FIXED;
        fixed_ = value;
    }

    /**
     * Get as a fixed value.
     *
     * @return The fixed string that must have been previously set.
     */
    public String
    getAsFixed()
    {
        return fixed_;
    }

    /**
     * Does it contain a floating value?
     *
     * @return True if it's a float or false otherwise.
     */
    public boolean
    isFloating()
    {
        return type_ == FLOATING;
    }

    /**
     * Set with a floating value.
     *
     * @param value The double to set as value.
     */
    public void
    setAsFloating(double value)
    {
        type_ = FLOATING;
        floating_ = value;
    }

    /**
     * Get as a floating value.
     *
     * @return The double that must have been previously set.
     */
    public double
    getAsFloating()
    {
        return floating_;
    }

    /**
     * Does it contain a boolean value?
     *
     * @return True if it's a boolean or false otherwise.
     */
    public boolean
    isBoolean()
    {
        return type_ == BOOLEAN;
    }

    /**
     * Set with a boolean value.
     *
     * @param value The boolean to set as value.
     */
    public void
    setAsBoolean(boolean value)
    {
        type_ = BOOLEAN;
        boolean_ = value;
    }

    /**
     * Get as a boolean value.
     *
     * @return The boolean that must have been previously set.
     */
    public boolean
    getAsBoolean()
    {
        return boolean_;
    }

    /**
     * Does it contain an enum value?
     *
     * @return True if it's a enum or false otherwise.
     */
    public boolean
    isEnum()
    {
        return type_ == ENUM;
    }

    /**
     * Set with an enum value.
     *
     * @param tc The associated enum TypeCode.
     * @param value The enumerator value.
     * @param name The enumerator name.
     */
    public void
    setAsEnum(TypeCode tc,
              int value,
              String name)
    {
        type_ = ENUM;
        tc_ = tc;
        enumerator_ = value;
        string_ = name;
    }

    /**
     * Get as an enum value.
     *
     * @return The string that must have been previously set.
     */
    public String
    getAsEnum()
    {
        return string_;
    }
}
