/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST ExceptionDecl. */
import org.objectweb.corba.ast.api.ExceptionDecl;

/** To access AST ExceptionList. */
import org.objectweb.corba.ast.api.ExceptionList;

/** To use CORBA::ExtAttributeDef. */
import org.omg.CORBA.ExtAttributeDef;
import org.omg.CORBA.ExtAttributeDefHelper;

/** To use CORBA::AttributeMode. */
import org.omg.CORBA.AttributeMode;

/** To use CORBA::ExceptionDef. */
import org.omg.CORBA.ExceptionDef;

/**
 * AttributeDeclImpl is a wrapper class for IDL attribute declarations.
 *
 * Inherits from:
 * - DeclarationWithTypeRefImpl as attributes are IDL declarations
 *   and have an associated type.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class AttributeDeclImpl
       extends DeclarationWithTypeRefImpl
       implements org.objectweb.corba.ast.api.AttributeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 AttributeDef. */
    private ExtAttributeDef ext_attribute_def_;

    /** The attribute mode. */
    private AttributeMode mode_;

    /** The attribute get exceptions. */
    private ExceptionListImpl get_exceptions_;

    /** The attribute put exceptions. */
    private ExceptionListImpl set_exceptions_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    AttributeDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_attribute_def_ = null;
        mode_ = AttributeMode.ATTR_NORMAL;
        get_exceptions_ = new ExceptionListImpl();
        set_exceptions_ = new ExceptionListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add exception declarations as dependencies.
     *
     * @param attr_depend The dependencies vector. 
     * @param excs The exception declarations to add. 
     */
    protected void
    addDependencies(java.util.Vector attr_depend,
                    ExceptionDecl[] excs)
    {
        Declaration[] depend = null;
        for (int i=0;i<excs.length;i++)
        {
            attr_depend.add(excs[i]);
            depend = excs[i].getDependencies();
            for (int j=0;j<depend.length;j++)
            {
                if (attr_depend.indexOf(depend[j])==-1)
                    attr_depend.add(depend[j]);
            }
        }
    }

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ModuleDef.
     *
     * @param contained The ModuleDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_attribute_def_ = ExtAttributeDefHelper.narrow(contained);
        mode_ = ext_attribute_def_.mode();
        setType(getRepository().getAsTypeRef(ext_attribute_def_.type_def()));

        ExceptionDef[] exc = ext_attribute_def_.get_exceptions();
        for (int i=0; i<exc.length; i++)
            get_exceptions_.add((ExceptionDecl)getRepository().
                                lookupId(exc[i].id()));

        exc = ext_attribute_def_.set_exceptions();
        for (int i=0; i<exc.length; i++)
            set_exceptions_.add((ExceptionDecl)getRepository().
                                lookupId(exc[i].id()));

        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 ModuleDef.
     *
     * @param contained The ModuleDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_attribute_def_ = ExtAttributeDefHelper.narrow(contained);
        mode_ = ext_attribute_def_.mode();
        setType(getRepository().
                getAsMappedTypeRef(ext_attribute_def_.type_def()));

        ExceptionDef[] exc = ext_attribute_def_.get_exceptions();
        for (int i=0; i<exc.length; i++)
            get_exceptions_.add((ExceptionDecl)getRepository().
                                lookupMappedId(exc[i].id()));

        exc = ext_attribute_def_.set_exceptions();
        for (int i=0; i<exc.length; i++)
            set_exceptions_.add((ExceptionDecl)getRepository().
                                lookupMappedId(exc[i].id()));

        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the
     *          attribute declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_attribute_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector attr_depend = new java.util.Vector();
        Declaration[] depend = null;

        // attribute type
        if (getType().isDeclaration())
            attr_depend.add(getType());

        depend = getType().getDependencies();
        for (int i=0; i<depend.length; i++)
            attr_depend.add(depend[i]);

        // exceptions
        if (isReadonly())
            addDependencies(attr_depend, getExceptions());
        else
        {
            addDependencies(attr_depend, getGetExceptions());
            addDependencies(attr_depend, getSetExceptions());
        }

        dependencies_ = (Declaration[])attr_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_attribute;
    }

    /**
     * Create the attribute declaration into the IFR.
     */
    public void
    create()
    {
        ext_attribute_def_ = the_parent_.createExtAttribute(this, 
                                 getIDLType(), mode_,
                                 get_exceptions_.getExceptionDefSeq(),
                                 set_exceptions_.getExceptionDefSeq());
        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AttributeDecl
    //
    // ==================================================================

    /**
     * Set the associated mode as normal.
     */
    public void
    setNormal()
    {
        mode_ = AttributeMode.ATTR_NORMAL;
    }

    /**
     * Is a normal attribute?
     *
     * @return True if normal, else false.
     */
    public boolean
    isNormal()
    {
        return mode_ == AttributeMode.ATTR_NORMAL;
    }

    /**
     * Set the associated mode as readonly.
     */
    public void
    setReadonly()
    {
        mode_ = AttributeMode.ATTR_READONLY;
    }

    /**
     * Is a readonly attribute?
     *
     * @return True if readonly, else false.
     */
    public boolean
    isReadonly()
    {
        return mode_ == AttributeMode.ATTR_READONLY;
    }

    /**
     * Obtain the get exceptions list.
     *
     * @return The getraises ExceptionList of the attribute declaration
     *         or null if no exceptions were declared.
     */
    public ExceptionList
    getGetExceptionList()
    {
        return get_exceptions_;
    }

    /**
     * Obtain the all get exception declarations.
     *
     * @return The getraises ExceptionList of the attribute declaration
     *         or null if no exceptions were declared.
     */
    public ExceptionDecl[]
    getGetExceptions()
    {
        if (isNormal())
            return get_exceptions_.getExceptions();
        return new ExceptionDecl[0];
    }

    /**
     * Obtain the put exceptions list.
     *
     * @return The setraises (or raises for readonly attributes)
     *         ExceptionList of the attribute declaration 
     *         or null if no exceptions were declared.
     */
    public ExceptionList
    getSetExceptionList()
    {
        return set_exceptions_;
    }

    /**
     * Obtain the all put exception declarations.
     *
     * @return The setRaises (or raises for readonly attributes)
     *         ExceptionList of  the attribute declaration or
     *         null if no exceptions were declared.
     */
    public ExceptionDecl[]
    getSetExceptions()
    {
        if (isNormal())
            return set_exceptions_.getExceptions();
        return new ExceptionDecl[0];
    }

    /**
     * Obtain all exception declarations for a read only attribute.
     *
     * @return The getraises ExceptionList of the attribute declaration
     *         or null if no exceptions were declared or if the attribute.
     *         is not read only.
     */
    public ExceptionList
    getExceptionList()
    {
        if (isReadonly())
            return get_exceptions_;
        else
            return new ExceptionListImpl();
    }

    /**
     * Obtain all exception declarations for a read only attribute.
     *
     * @return The getraises ExceptionList of the attribute declaration
     *         or null if no exceptions were declared or if the attribute.
     *         is not read only.
     */
    public ExceptionDecl[]
    getExceptions()
    {
        if (isReadonly())
            return get_exceptions_.getExceptions();
        else
            return new ExceptionDecl[0];
    }
}
