/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST ComponentDecl. */
import org.objectweb.corba.ast.api.ComponentDecl;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** To use CORBA::ComponentIR::ComponentDef. */
import org.omg.CORBA.ComponentIR.ComponentDef;
import org.omg.CORBA.ComponentIR.ComponentDefHelper;

/**
 * ComponentDeclImpl is a wrapper class for IDL component declarations.
 *
 *
 * Inherits from:
 *
 * - ComponentBaseImpl as components are also interfaces and
 *   have an equivalent client interface mapping.
 *
 * - ComponentDecl: OMG IDL for component declarations.
 *
 *
 * Provides:
 *
 * - The 'BaseComponent' property,
 *   i.e. the setBaseComponent and getBaseComponent operations.
 *
 * - The local context interface mapping,
 *   i.e. the getLocalContextMapping operation.
 *
 * - The local monolithic interface mapping,
 *   i.e. the getLocalMonolithicMapping operation.
 *
 * - The local main interface mapping,
 *   i.e. the getLocalMainMapping operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ComponentDeclImpl
     extends ComponentBaseImpl
  implements ComponentDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ComponentDef. */
    private ComponentDef component_def_;

    /** Its base component. */
    private ComponentDeclImpl base_component_;

    /** The local context interface mapping. */
    private InterfaceDeclImpl local_context_mapping_;

    /** The local monolithic interface mapping. */
    private InterfaceDeclImpl local_monolithic_mapping_;

    /** The local main interface mapping. */
    private InterfaceDeclImpl local_main_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    ComponentDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the ComponentBaselImpl constructor.
        super(rep, parent);

        // Init internal state.
        component_def_ = null;
        base_component_ = null;
        local_context_mapping_ = null;
        local_monolithic_mapping_ = null;
        local_main_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ComponentDef.
     *
     * @param contained The ComponentDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        component_def_ = ComponentDefHelper.narrow(contained);

        ComponentDef baseComponent = component_def_.base_component();
        if (baseComponent!=null)
            setBaseComponent(
                (ComponentDecl)getRepository().
                lookupId(baseComponent.id()));

        addSupportedInterfaces(component_def_.supported_interfaces());

        // load it's mapped declarations
        getClientMapping();
        getLocalContextMapping();

        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the component
     *          declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return component_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the component
     *         declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
       return component_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
      // System.out.println("create_component on " + the_parent_);
      // System.out.println(getId());

        component_def_ = the_parent_.getComponentContainer().
                         create_component(getId(),
                                          getName(),
                                          getVersion(),
                                          getBaseComponentDef(),
                             supported_interfaces_.getInterfaceDefSeq());
    }

    /**
     * Complete the container object.
     */
    protected void
    completeContainer()
    {
        ComponentDef baseComponent = getBaseComponentDef();
        if (baseComponent != null)
            component_def_.base_component(baseComponent);

        if (supported_interfaces_.getSize() != 0)
            component_def_.supported_interfaces(
                supported_interfaces_.getInterfaceDefSeq());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its ComponentDef reference.
     *
     * @return The ComponentDef object associated with the component
     *         declaration.
     */
    public ComponentDef
    getComponentDef()
    {
        return component_def_;
    }

    /**
     * Obtain the inherited component base.
     *
     * @return The inherited component base.
     */ 
    public ComponentDef
    getBaseComponentDef()
    {
        return (base_component_ != null)
	     ? (base_component_.getComponentDef()) : null;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the component
     *         declaration.
     */
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return component_def_;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the component
     *         declaration.
     */
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return component_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        java.util.Vector comp_depend = new java.util.Vector();
        Declaration[] depend = null;

        // base component
        if (getBaseComponent()!=null)
        {
            comp_depend.add(getBaseComponent());
            depend = getBaseComponent().getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if (depend[j]!=this)
                    comp_depend.add(depend[j]);
            }
        }

        // supported interfaces
        InterfaceDecl[] supp = supported_interfaces_.getInterfaces();
        for (int i=0; i<supp.length; i++)
        {
            comp_depend.add(supp[i]);
            depend = supp[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((depend[j]!=this) &&
                    (comp_depend.indexOf(depend[j])==-1))
                    comp_depend.add(depend[j]);
            }
        }

        // contents
        Declaration[] decls = getContents(true, DeclarationKind.dk_all);
        for (int i=0; i<decls.length; i++)
        {
            depend = decls[i].getDependencies();
           for (int j=0; j<depend.length; j++)
            {
             boolean tmp =false;
             if(depend[j]!=this) tmp=true;
               if ( (!containsDecl(depend[j])) &&
                     (depend[j]!=this) &&
                     (comp_depend.indexOf(depend[j])==-1) )
               {
                    comp_depend.add(depend[j]);
               }
            }
        }

        dependencies_ = (Declaration[])comp_depend.toArray(new Declaration[0]);
/*            System.err.println("dependances du ComponentDecl : "+getAbsoluteName());
            for(int i=0; i<dependencies_.length; i++)
                System.err.println("    |"+dependencies_[i].getAbsoluteName());
*/        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_component;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    /**
     * Find a declaration.
     *
     * Note that the declaration is also recursively searched in the
     * parent scope.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    public Declaration
    find(String name)
    {
        // Finds in the scope.
        Declaration decl = super.find(name);

        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in base component.
        if (base_component_ != null)
        {
            decl = base_component_.find(name);

            // If found then returns it.
            if(decl != null) return decl;
        }

        // If not found then finds in supported interfaces.
        return supported_interfaces_.find(name);
    }

    /**
     * To obtain all the contained Declaration objects.
     *
     * @param exclude_inherited If false return also objects contained
     * in inherited scopes.
     * @param limited_types A logical combination of DeclarationKind.
     *
     * @return An array of Declaration objects.
     */
    public Declaration[]
    getContents(boolean exclude_inherited,
                long limited_types)
    {
        if (exclude_inherited)
            return super.getContents(exclude_inherited, limited_types);

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = supported_interfaces_.getInterfaces();
        for (int i=0; i<itfs.length; i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }

        if (base_component_==null)
            return res;

        Declaration[] tmp1 = base_component_.getContents(exclude_inherited, limited_types);
        Declaration[] tmp2 = res;
        res = new Declaration[tmp1.length+tmp2.length];
        System.arraycopy(tmp2, 0, res, 0, tmp2.length);
        System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        return res;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_component;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfaceDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the component declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return component_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ComponentBase
    //
    // ==================================================================

    /**
     * Obtain the client interface mapping.
     *
     * @return The client interface mapping.
     */
    public InterfaceDecl
    getClientMapping()
    {
        if (client_mapping_ == null)
        {
            // load the client mapping for this component.
            client_mapping_ = (InterfaceDeclImpl)getRepository().
                              loadMapping(getParent(), getId());
        }
        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ComponentDecl
    //
    // ==================================================================

    /**
     * Set the base component.
     *
     * @param base_component The base component of the component.
     *        Note that if more then one base component 
     *        is set, only the first one is considered.
     */
    public void
    setBaseComponent(ComponentDecl base_component)
    {
        if (base_component != null)
        {
            base_component_ = (ComponentDeclImpl)base_component;
        }
    }

    /**
     * Obtain the base component.
     *
     * @return The base component.
     */
    public ComponentDecl
    getBaseComponent()
    {
        return base_component_;
    }

    /**
     * Obtain the local context interface mapping.
     *
     * @return The local context interface mapping.
     */
    public InterfaceDecl
    getLocalContextMapping()
    {
        if (local_context_mapping_ == null)
        {
            // load the context mapping for this component.
            String parent_base_id = the_parent_.getId();
            int idx = parent_base_id.lastIndexOf(':');
            parent_base_id = parent_base_id.substring(0, idx);
            String id = parent_base_id + "/CCM_" + getName() +
                        "_Context:" + getVersion();
            local_context_mapping_ = (InterfaceDeclImpl)getRepository().
                                     loadMapping(getParent(), id);
        }
        return local_context_mapping_;
    }

    /**
     * Obtain the local monolithic interface mapping.
     *
     * @return The local monolithic interface mapping.
     */
    public InterfaceDecl
    getLocalMonolithicMapping()
    {
        if (local_monolithic_mapping_ == null)
        {
            // load the monolithic mapping for this component.
            String parent_base_id = the_parent_.getId();
            int idx = parent_base_id.lastIndexOf(':');
            parent_base_id = parent_base_id.substring(0, idx);
            String id = parent_base_id + "/CCM_" + getName() +
                        ':' + getVersion();

            local_monolithic_mapping_ = (InterfaceDeclImpl)getRepository().
                                      loadMapping(getParent(), id);
        }
        return local_monolithic_mapping_;
    }

    /**
     * Obtain the local main interface mapping.
     *
     * @return The local main interface mapping.
     */
    public InterfaceDecl
    getLocalMainMapping()
    {
        if (local_main_mapping_ == null)
        {
            // load the main mapping for this component.
            String parent_base_id = the_parent_.getId();
            int idx = parent_base_id.lastIndexOf(':');
            parent_base_id = parent_base_id.substring(0, idx);
            String id = parent_base_id + "/CCM_" + getName() +
                        "_Executor:" + getVersion();

            local_main_mapping_ = (InterfaceDeclImpl)getRepository().
                                  loadMapping(getParent(), id);
        }
        return local_main_mapping_;
    }
}
