/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST AnyValue. */
import org.objectweb.corba.ast.api.AnyValue;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To use CORBA::ConstantDef. */
import org.omg.CORBA.ConstantDef;
import org.omg.CORBA.ConstantDefHelper;

/**
 * ConstantDeclImpl is a wrapper class for IDL constant declarations.
 *
 * Inherits from:
 * - DeclarationWithTypeRefImpl as constants are IDL declarations
 *   and have an associated type,
 * - WithAnyValue as constants have an associated any value.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ConstantDeclImpl
       extends DeclarationWithTypeRefImpl
       implements org.objectweb.corba.ast.api.ConstantDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ConstantDef. */
    private ConstantDef constant_def_;

    /** Reference to the AnyValue constant. */
    private AnyValueImpl any_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    ConstantDeclImpl(Repository rep,
                     ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        constant_def_ = null;
        any_ = new AnyValueImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ConstantDef.
     *
     * @param contained The ConstantDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        constant_def_ = ConstantDefHelper.narrow(contained);
        setType(getRepository().getAsTypeRef(constant_def_.type_def()));
        any_ = new AnyValueImpl();
        any_.loadAny(constant_def_.value());

        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 ConstantDef.
     *
     * @param contained The ConstantDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        constant_def_ = ConstantDefHelper.narrow(contained);
        setType(getRepository().getAsMappedTypeRef(constant_def_.type_def()));
        any_ = new AnyValueImpl();
        any_.loadAny(constant_def_.value());

        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the constant declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        return constant_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector constant_depend = new java.util.Vector();

        // constant type
        if (getType().isDeclaration())
            constant_depend.add(getType());

        Declaration[] depend = getType().getDependencies();
        for (int i=0;i<depend.length;i++)
            constant_depend.add(depend[i]);

        dependencies_ = (Declaration[])constant_depend.toArray(
                                                     new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_constant;
    }

    /**
     * Create the constant declaration into the IFR.
     */
    public void
    create()
    {
        constant_def_ = the_parent_.getContainer().
                        create_constant(getId(), getName(), getVersion(),
                                        getIDLType(),
                                        any_.computeAny(getIDLType().type()));

        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithAnyValue
    //
    // ==================================================================

    /**
     * Set the associated AnyValue.
     *
     * @param value The AnyValue to set.
     */
    public void
    setAnyValue(AnyValue value)
    {
      any_ = (AnyValueImpl)value;
    }

    /**
     * Obtain the associated AnyValue.
     *
     * @return The associated AnyValue.
     */
    public AnyValue
    getAnyValue()
    {
      return any_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ConstantDecl
    //
    // ==================================================================

}
