/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST DeclarationCategory. */
import org.objectweb.corba.ast.api.DeclarationCategory;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/**
 * DeclarationImpl is a wrapper class for all IDL declarations.
 *
 * Inherits from:
 * - WithName as IDL declarations have an associated name.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class DeclarationImpl
                extends WithNameImpl
                implements Declaration
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The prefix. */
    protected java.lang.String prefix_;

    /** The version. */
    protected java.lang.String version_;

    /** The repository ID. */
    protected java.lang.String id_;

    /** To protect imported declaration from being destroyed. */
    protected boolean is_imported_;

    /** The IDL3 repository. */
    protected Repository the_repository_;

    /** The parent scope. */
    protected ScopeImpl the_parent_;

    /** To know if the declaration is a mapping declaration. */
    protected boolean is_mapping_;

    /** Associated dependencies. */
    protected Declaration[] dependencies_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    DeclarationImpl(Repository rep,
                    ScopeImpl parent)
    {
        // Call the default WithNameImpl constructor.
        super();

        // Init internal state.
        the_repository_ = rep;
        the_parent_ = parent;
        prefix_ = null;
        version_ = "1.0";
        id_ = null;
        is_imported_ = false;
        is_mapping_ = false;
        dependencies_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * To compute the OMG IDL prefix of the Declaration.
     *
     * @return the prefix in OMG IDL or "" if there's no prefix.
     */
    protected java.lang.String
    computePrefix()
    {
        int idx1 = id_.indexOf(':');
        int idx2 = id_.indexOf('/');

        java.lang.String pname = the_parent_.getName();
        if ((pname==null) || (pname.equals("")))
        // the parent is the repository (null) or the file scope ("") itself
        {
            // System.err.println("Computing prefix for: "+getId()+" with Repository as parent");
            if (id_.indexOf(name_)==-1)
            // there was a pragma id => no prefix
                return null;
            else if (idx2!=-1)
            // id = IDL:prefix/name:version
                return id_.substring(idx1+1,idx2);
            else
            // id = IDL:name:version
                return null;
        }
        else
        {
            // System.err.println("Computing prefix for: "+getId()+" with: "+pname+" as parent");
            if (id_.indexOf(name_)==-1)
            // there was a pragma id => no prefix
                return null;
            else if (id_.indexOf(the_parent_.computeBaseRID())==-1)
            // there was a pragma prefix
            // perhaps a pragma id in fact ????? but can't always know
            // eg
            // module M {
            //    #pragma id I "OpenCCM/M/I:1.0" <=> #pragma prefix "OpenCCM"
            //    interface I {};
            // };
            // id = IDL:prefix/par_1/.../par_n/name:version
            // TODO : check all parents'name
                return id_.substring(idx1+1, idx2);
            else
            // same prefix as it's parent
                return null;
        }
    }

    /**
     * Compute the base repository ID.
     *
     * @return The RepositoryId of the declaration without the version
     *         and the IDL fields.
     */
    protected java.lang.String
    computeBaseRID()
    {
        java.lang.String result = "";

        if (prefix_ != null)
            result = prefix_;
        if (the_parent_ != null)
            result = the_parent_.computeBaseRID();

        if (name_ != null)
        {
            if (result.length() != 0)
                result = result + '/' + name_;
            else
                result = name_;
        }

        return result;
    }

    /**
     * To set the declaration as an imported declaration.
     * Imported declarations can't be destroyed in the IR3.
     */
    protected void
    setImported()
    {
        is_imported_ = true;
    }

    /**
     * Get the associated Repository.
     */
    protected Repository
    getRepository()
    {
        return the_repository_;
    }

    /**
     * Loads infos of the CORBA 3.0 Contained.
     *
     * @param contained The Contained to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        version_ = contained.version();
        id_ = contained.id();
        prefix_ = computePrefix();
        the_repository_.addDeclInRep(getId(), this);
        if (the_parent_!=null)
            the_parent_.addDecl(this);
    }

    /**
     * Loads infos of the CORBA 3.0 Contained.
     *
     * @param contained The Contained to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
      //      System.out.println("loadAsMapping IS MAPPING TRUE " + this);

        version_ = contained.version();
        id_ = contained.id();
        prefix_ = computePrefix();
        the_repository_.addMappedDeclInRep(getId(), this);
        is_mapping_ = true;
        // only add declaration if the parent is itself a mapped declaration !!!
        if ((the_parent_!=null) &&
            (the_parent_.is_mapping_))
            the_parent_.addDecl(this);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the declaration.
     */
    abstract protected org.omg.CORBA.Contained
    getContained();

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the ID prefix, i.e. "IDL", "PSDL", or "CIDL".
     *
     * @return The ID prefix.
     */
    public String
    getIdPrefix()
    {
      return "IDL";
    }

    /**
     * isDeclaration is implemented here to do some code factorization.
     */
    public boolean
    isDeclaration()
    {
        return true;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the external Declaration dependencies.
     * Note: For scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of Declaration dependencies.
     */
    public Declaration[]
    getDependencies()
    {
        return new Declaration[0];
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    abstract public long
    getDeclKind();
//    {
//        return DeclarationKind.dk_null;
//    }

    /**
     * Obtain its absolute name.
     *
     * @return Its absolute name.
     */
    public java.lang.String
    getAbsoluteName()
    {
        java.lang.String result = "";
        if(the_parent_ != null)
            result = the_parent_.getAbsoluteName();

        if(name_ != null)
            result = result + "::" + name_;

        return result;
    }

    /**
     * Set its repository ID.
     *
     * @param id The RepositoryId to set.
     */
    public void
    setId(java.lang.String id)
    {
        id_ = id;
        the_repository_.addDeclInRep(id, this);
        org.omg.CORBA.Contained contained = getContained();
        if (contained != null)
            contained.id(id_);
    }

   /**
    * Obtain its repository ID.
    *
    * @return Its RepositoryId.
    */
    public java.lang.String
    getId()
    {
        if (id_ != null)
            return id_;

        return getIdPrefix() + ':' + computeBaseRID() + ':' + version_;
    }

    /**
     * Set its prefix.
     *
     * @param prefix The prefix to set.
     *   Note that setting the prefix should only be done
     *   when a new prefix (ie different from the
     *   declared-in scope prefix) was explicitly declared.
     */
    public void
    setPrefix(java.lang.String prefix)
    {
        prefix_ = prefix;

        org.omg.CORBA.Contained contained = getContained();
        if (contained != null)
            contained.id(getId());
    }

    /**
     * Obtain its prefix.
     *
     * @return The prefix of the declaration or
     *         null if there's no prefix.
     */
    public java.lang.String
    getPrefix()
    {
        if (prefix_!=null)
            return prefix_;
        if ((the_parent_!=null) && (the_parent_.getName()!=null))
            return the_parent_.getPrefix();

        return "";
    }

    /**
     * Set its version.
     *
     * @param version The version to set.
     */
    public void
    setVersion(java.lang.String version)
    {
        version_ = version;
        org.omg.CORBA.Contained contained = getContained();
        if (contained != null)
            contained.version(version);
    }

    /**
     * Obtain its version.
     *
     * @return The version of the declaration.
     *         Note that version "1.0" is assumed as default.
     */
    public java.lang.String
    getVersion()
    {
        return version_;
    }

    /**
     * Obtain its parent Scope.
     *
     * @return The parent Scope of the declaration
     *         or null if the declaration has no parent.
     */
    public Scope
    getParent()
    {
        return the_parent_;
    }

    /**
     * Create this Declaration into the Interface Repository.
     */
    public void
    create()
    {
        if (the_parent_!=null)
            the_parent_.addDecl(this);
    }

    /**
     * Destroy this Declaration from the Interface Repository.
     */
    public void
    destroy()
    {
        org.omg.CORBA.IRObject obj = (org.omg.CORBA.IRObject)getContained();
        if (obj != null)
        {
            // destroy the IR3 Contained if it was not imported.
            if (!is_imported_)
                obj.destroy();

            the_repository_.removeDeclFromRep(getId());
            if (the_parent_!=null)
                the_parent_.removeDecl(name_);
        }
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public DeclarationCategory
    getCategory()
    {
        return DeclarationCategory.dc_idl;
    }

    /**
     * Check if the Declaration is correct.
     */
    public void
    check()
    {
        // To be overrided in sub-classes
    }

    /**
     * Print the Declaration.
     */
    public void
    print()
    {
        System.out.println(getAbsoluteName());
    }
}
