/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST AbstractStorageHomeDecl. */
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;

/** To access AST StringList. */
import org.objectweb.corba.ast.api.StringList;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/**
 * DeclarationWithMemberListImpl is a wrapper class for
 * PSDL declarations with a list of member names.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - DeclarationWithMemberList: OMG IDL for PSDL declarations 
 *   with a list of member names.
 *
 *
 * Provides:
 *
 * - The list of member names,
 *   i.e. the getMemberList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class DeclarationWithMemberListImpl
     extends DeclarationImpl
  implements org.objectweb.corba.ast.api.DeclarationWithMemberList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The list of member names. */
    protected StringListImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    DeclarationWithMemberListImpl(Repository rep,
                                  ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        members_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the any declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
      return DeclarationKind.dk_null;
    }

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        // Call the DeclarationImpl "create" method.
        super.create();

        // Check if a member is not specify more that once
        int index = members_.checkSameItem();
        if( index != -1 )
        {
            throw new ASTError( (String)members_.get(index) + " must be specified once and once only.");
        }
    }

    /**
     * Check if state members of the list are valid.
     */
    protected void
    check(StorageTypeBaseImpl st)
    {
        int size = members_.getSize();
        for(int i=0; i<size; i++)
        {
             String state = (String)members_.get(i);
             if( st.isValidState(state) == false)
             {
                 throw new ASTError(getName() + " => " + state + " is not a valid state member." );
             }
        }
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.DeclarationWithMemberList
    //
    // ==================================================================

    /**
     * Obtain the list of member names.
     *
     * @return The list of member names.
     */
    public StringList
    getMemberList()
    {
        return members_;
    }
}
