/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** Used to access AST ComponentDecl. */
import org.objectweb.corba.ast.api.ComponentDecl;

/** To use CORBA::ComponentIR::EmitsDef. */
import org.omg.CORBA.ComponentIR.EmitsDef;
import org.omg.CORBA.ComponentIR.EmitsDefHelper;

/**
 * EmitsDeclImpl is a wrapper class for IDL emits port declarations.
 *
 * Inherits from:
 * - EventPortDeclImpl as consumes are event ports,
 * - LocalContextOperationMapping 
 *   to obtain the local context operation mapping.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EmitsDeclImpl
       extends EventPortDeclImpl
       implements org.objectweb.corba.ast.api.EmitsDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 EmitsDef. */
    private EmitsDef emits_def_;

    /** Local context operation mapping. */
    private OperationDecl[] local_context_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    EmitsDeclImpl(Repository rep,
                  ScopeImpl parent)
    {
        // Call the EventPortDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        emits_def_ = null;
        local_context_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 EmitsDef.
     *
     * @param contained The EmitsDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        emits_def_ = EmitsDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the emits declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return emits_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_emits;
    }

    /**
     * Create the emits declaration into the IFR.
     */
    public void
    create()
    {
        emits_def_ = the_parent_.getComponentDef().
                     create_emits(getId(), getName(), getVersion(),
                                  event_.getEventDef());
        super.create();
   }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ClientOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the client operation mapping.
     *
     * @return The client operation mapping.
     */
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((ComponentDecl)the_parent_).getClientMapping();
        client.getContents(true, DeclarationKind.dk_null);

        client_mapping_ = new OperationDecl[2];
        client_mapping_[0] = (OperationDecl)
                             client.find("connect_"+getName());
        client_mapping_[1] = (OperationDecl)
                             client.find("disconnect_"+getName());
        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.EventPortDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.LocalContextOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the local context operation mapping.
     *
     * @return The local context operation mapping.
     */
    public OperationDecl[]
    getLocalContextMapping()
    {
        if (local_context_mapping_!=null)
            return local_context_mapping_;

        // retrieve the context mapping local interface.
        InterfaceDecl context = ((ComponentDecl)the_parent_).
                                getLocalContextMapping();
        context.getContents(true, DeclarationKind.dk_null);

        local_context_mapping_ = new OperationDecl[1];
        local_context_mapping_[0] = (OperationDecl)
                                    context.find("push_"+getName());
        return local_context_mapping_;
    }
}
