/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST AnyValue. */
import org.objectweb.corba.ast.api.AnyValue;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST StringList. */
import org.objectweb.corba.ast.api.StringList;

/** To use CORBA::EnumDef. */
import org.omg.CORBA.EnumDef;
import org.omg.CORBA.EnumDefHelper;

/**
 * EnumDeclImpl is a wrapper class for IDL enum declarations.
 *
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as enums are IDL declarations,
 *
 * - EnumDecl: OMG IDL for enum declarations.
 *
 * - IDLTypeWrapper as enums are IDL types.
 *
 *
 * Provides:
 *
 * - The list of enum members,
 *   i.e. the getMemberList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EnumDeclImpl
       extends DeclarationImpl
       implements org.objectweb.corba.ast.api.EnumDecl,
                  IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 EnumDef. */
    private EnumDef enum_def_;

    /** List of members names. */
    private StringListImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    EnumDeclImpl(Repository rep,
                 ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        enum_def_ = null;
        members_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 EnumDef.
     *
     * @param contained The EnumDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        enum_def_ = EnumDefHelper.narrow(contained);
        String[] members = enum_def_.members();
        org.omg.CORBA.TypeCode enumTC = enum_def_.type();
        for (int i=0; i<members.length; i++)
        {
            members_.add(members[i]);
            DeclarationWithAnyValueImpl decl = new DeclarationWithAnyValueImpl(getRepository(), the_parent_);
            decl.setName(members[i]);
            AnyValue av = new AnyValueImpl();
            av.setAsEnum(enumTC, i, members[i]);
            decl.setAnyValue(av);
            decl.create();
        }
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 EnumDef.
     *
     * @param contained The EnumDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        enum_def_ = EnumDefHelper.narrow(contained);
        String[] members = enum_def_.members();
        org.omg.CORBA.TypeCode enumTC = enum_def_.type();
        for (int i=0;i<members.length;i++)
        {
            members_.add(members[i]);
            DeclarationWithAnyValueImpl decl = new DeclarationWithAnyValueImpl(getRepository(), the_parent_);
            decl.setName(members[i]);
            AnyValue av = new AnyValueImpl();
            av.setAsEnum(enumTC, i, members[i]);
            decl.setAnyValue(av);
            decl.create();
        }

        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the enum declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return enum_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDeclarators
    //
    // ==================================================================

    /**
     * Find the declarator associated to an index.
     *
     * @param index The index of the element.
     *
     * @return The enumeration symbolic name of the element
     *         at the specified index.
     */
    public String
    declaratorAt(int index)
    {
        return (String)members_.get(index);
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_enum;
    }

    /**
     * Create the enum declaration.
     */
    public void
    create()
    {
        String[] members = members_.getStrings();

        enum_def_ = the_parent_.getContainer().
                    create_enum(getId(), getName(), getVersion(), members);

        org.omg.CORBA.TypeCode enumTC = enum_def_.type();
        for(int i=0; i<members_.getSize(); i++)
        {
            String name = (String)members_.get(i);
            DeclarationWithAnyValueImpl decl = new DeclarationWithAnyValueImpl(getRepository(), the_parent_);
            decl.setName(name);
            AnyValue av = new AnyValueImpl();
            av.setAsEnum(enumTC, i, name);
            decl.setAnyValue(av);
            decl.create();
        }

        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_enum;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.EnumDecl
    //
    // ==================================================================

    /**
     * Obtain the list of members.
     *
     * @return The list of members.
     */
    public StringList
    getMemberList()
    {
        return members_;
    }

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the type reference.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return enum_def_;
    }
}
