/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** To use CORBA::ComponentIR::EventDef. */
import org.omg.CORBA.ComponentIR.EventDef;
import org.omg.CORBA.ComponentIR.EventDefHelper;

/**
 * EventDeclImpl is a wrapper class for IDL eventtype declarations.
 *
 *
 * Inherits from:
 *
 * - ValueDeclImpl as eventtypes are also valuetypes,
 *
 * - ClientInterfaceMapping
 *   to obtain the equivalent client interface mapping.
 *
 *
 * Provides:
 *
 * - The local consumer interface mapping,
 *   i.e. the getLocalConsumerMapping interface operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class EventDeclImpl
       extends ValueDeclImpl
       implements org.objectweb.corba.ast.api.EventDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ValueDef. */
    private EventDef event_def_;

    /** The client interface mapping. **/
    private InterfaceDeclImpl client_mapping_;

    /** The local consumer interface mapping. **/
    private InterfaceDeclImpl local_consumer_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the value declaration.
     */
    public
    EventDeclImpl(Repository rep,
                  ScopeImpl parent)
    {
        // Call the ValueDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        event_def_ = null;
        client_mapping_ = null;
        local_consumer_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ValueDef.
     *
     * @param contained The ValueDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        event_def_ = EventDefHelper.narrow(contained);

        // load it's mapped declarations
        getClientMapping();
        getLocalConsumerMapping();

        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the value declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return event_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the value declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return event_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        // Create an EventDef into the IFR.
        event_def_ = the_parent_.getComponentContainer().
                     create_event(getId(), getName(), getVersion(),
                         is_custom_, is_abstract_,
                         getBaseValueDef(),
                         is_truncatable_,
                         abstract_base_values_.getValueDefSeq(),
                         supported_interfaces_.getInterfaceDefSeq(),
                         new org.omg.CORBA.ExtInitializer[0]);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its ValueDef reference.
     *
     * @return The ValueDef associated with the value declaration.
     */
    public org.omg.CORBA.ExtValueDef
    getExtValueDef()
    {
        return event_def_;
    }

    /**
     * Obtain its ValueDef reference.
     *
     * @return The ValueDef associated with the value declaration.
     */
    public org.omg.CORBA.ValueDef
    getValueDef()
    {
        return event_def_;
    }

    /**
     * Obtain its EventDef reference.
     *
     * @return The ValueDef associated with the value declaration.
     */
    public EventDef
    getEventDef()
    {
        return event_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_event;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_event;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ValueDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the value declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return event_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ClientInterfaceMapping
    //
    // ==================================================================

    /**
     * Obtain the client interface mapping.
     *
     * @return The client interface mapping.
     */
    public InterfaceDecl
    getClientMapping()
    {
        if (client_mapping_==null)
        {
            // load the context mapping for this component.
            String base_id = getId();
            int idx = base_id.lastIndexOf(':');
            base_id = base_id.substring(0, idx);
            String id = base_id + "Consumer:" + getVersion();

            client_mapping_ = (InterfaceDeclImpl)getRepository().
                              loadMapping(getParent(), id);
        }
        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.EventDecl
    //
    // ==================================================================

    /**
     * Obtain the local consumer mapping.
     *
     * @return The local consumer mapping.
     */
    public InterfaceDecl
    getLocalConsumerMapping()
    {
        if (local_consumer_mapping_==null)
        {
            // load the local consumer mapping for this event.
            String parent_base_id = the_parent_.getId();
            int idx = parent_base_id.lastIndexOf(':');
            parent_base_id = parent_base_id.substring(0, idx);
            String id = parent_base_id + "/CCM_" + getName() + 
                    "Consumer:" + getVersion();

            local_consumer_mapping_ = (InterfaceDeclImpl)getRepository().
                                      loadMapping(getParent(), id);
        }
        return local_consumer_mapping_;
    }
}
