/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** Used to access AST EventDecl. */
import org.objectweb.corba.ast.api.EventDecl;

/**
 * EventPortDeclImpl is a wrapper class for IDL event port declarations.
 *
 * Inherits from:
 * - DeclarationImpl as these are declarations,
 * - ClientOperationMapping
 *   to obtain the equivalent client operation mapping.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class EventPortDeclImpl
                extends DeclarationImpl
                implements org.objectweb.corba.ast.api.EventPortDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The port event type. */
    protected EventDeclImpl event_;

    /** Client operation mapping. */
    protected OperationDecl[] client_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the event declaration.
     */
    protected
    EventPortDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        event_ = null;
        client_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 EventDef.
     *
     * @param contained The EventDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        org.omg.CORBA.ComponentIR.EventPortDef event_port =
            org.omg.CORBA.ComponentIR.EventPortDefHelper.narrow(contained);
        setEvent((EventDecl)getRepository().lookupId(event_port.event().id()));
        super.load(contained);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector events_depend = new java.util.Vector();

        // content
        // TODO: The next test is not needed
        // as EventDecl is a Declaration, isn't it? 
        if (getEvent().isDeclaration())
            events_depend.add(getEvent());

        Declaration[] depend = getEvent().getDependencies();
        for (int i=0;i<depend.length;i++)
            events_depend.add(depend[i]);

        dependencies_ = (Declaration[])events_depend.toArray(
                                          new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.EventPortDecl
    //
    // ==================================================================

    /**
     * Set the event value type.
     *
     * @param event The EventDecl of the event.
     */
    public void
    setEvent(EventDecl event)
    {
        if (event != null)
        {
            event_ = (EventDeclImpl)event;
        }
    }

    /**
     * Obtain the event value type.
     *
     * @return The EventDecl of the event.
     */
    public EventDecl
    getEvent()
    {
        return event_;
    }
}
