/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST StructMember. */
import org.objectweb.corba.ast.api.StructMember;

/** To access AST StructMemberList. */
import org.objectweb.corba.ast.api.StructMemberList;

/** To use CORBA::ExceptionDef. */
import org.omg.CORBA.ExceptionDef;
import org.omg.CORBA.ExceptionDefHelper;

/**
 * ExceptionDeclImpl is a wrapper class for IDL exception declarations.
 *
 * Inherits from:
 * - ScopeImpl as exceptions are also IDL scopes.
 * - WithDeclarators to access exception members.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ExceptionDeclImpl
       extends ScopeImpl
       implements org.objectweb.corba.ast.api.ExceptionDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ExceptionDef. */
    private ExceptionDef exception_def_;

    /** List of members. */
    private StructMemberListImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the exception declaration.
     */
    public
    ExceptionDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        exception_def_ = null;
        members_ = new StructMemberListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Create the exception declaration.
     */
    protected void
    doCreation()
    {
        exception_def_ = the_parent_.getContainer().
                         create_exception(getId(), getName(), getVersion(),
                                          members_.getStructMemberSeq());
        super.create();
    }

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ExceptionDef.
     *
     * @param contained The ExceptionDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        exception_def_ = ExceptionDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = exception_def_.members();

        // for inlined struct, union or enum declarations
        super.load(contained);

        for (int i=0; i<members.length; i++)
        {
            members_.addMember(members[i].name,
                               getRepository().
                                   getAsTypeRef((members[i].type_def)));
        }
    }

    /**
     * Loads infos of the CORBA 3.0 ExceptionDef.
     *
     * @param contained The ExceptionDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        exception_def_ = ExceptionDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = exception_def_.members();

        // for inlined struct, union or enum declarations
        super.loadAsMapping(contained);

        for (int i=0; i<members.length; i++)
            members_.addMember(members[i].name,
                               getRepository().
                                 getAsMappedTypeRef((members[i].type_def)));
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the exception
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        if(exception_def_ == null)
           doCreation();

       return exception_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the exception
     *          declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        if(exception_def_ == null)
           doCreation();

        return exception_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its ExceptionDef reference.
     *
     * @return The ExceptionDef associated with the exception declaration.
     */
    public ExceptionDef
    getExceptionDef()
    {
        return exception_def_;
    }

    /**
     * Obtain its associated CORBA::ExceptionDescription.
     *
     * @return Its associated CORBA::ExceptionDescription.
     */
    public org.omg.CORBA.ExceptionDescription
    getExceptionDescription()
    {
        return new org.omg.CORBA.ExceptionDescription(getName(),
                                                      getId(),
                                                      the_parent_.getId(),
                                                      getVersion(),
                                                      getExceptionDef().type());
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        // members
        java.util.Vector exc_depend = new java.util.Vector();
        Declaration[] depend = null;

        StructMember[] members = members_.getStructMembers();
        for (int i=0; i<members.length; i++)
        {
            TypeRef member_type = members[i].getType();
            if (member_type.isDeclaration())
            {
                if ((!containsDecl((Declaration)member_type)) &&
                    (exc_depend.indexOf(member_type)==-1))
                    exc_depend.add(member_type);
            }

            depend = member_type.getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((exc_depend.indexOf(depend[j])==-1) &&
                    (!containsDecl(depend[j])))
                    exc_depend.add(depend[j]);
            }
        }
        dependencies_ = (Declaration[])exc_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_exception;
    }

    /**
     * Close the exception scope.
     */
    public void
    create()
    {
        if(exception_def_ == null)
           doCreation();
        else
          exception_def_.members(members_.getStructMemberSeq());
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDeclarators
    //
    // ==================================================================

    /**
     * Find the declarator associated to an index.
     *
     * @param index The index.
     *
     * @return The declarator associated to the index.
     */
    public String
    declaratorAt(int index)
    {
        return members_.nameAt(index);
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ExceptionDecl
    //
    // ==================================================================

    /**
     * Obtain the exception member list.
     *
     * @return The members of the exception declaration.
     */
    public StructMemberList
    getMemberList()
    {
        return members_;
    }
}
