/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST FileScope. */
import org.objectweb.corba.ast.api.FileScope;

/**
 * FileScopeImpl is a wrapper class for included IDL files.
 *
 * Inherits from:
 * - ScopeImpl as file scopes are IDL scopes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class FileScopeImpl
       extends ScopeImpl
       implements FileScope
{
    // ==================================================================
    //
    //  Internal state.
    //
    // ==================================================================

    /** To store included files names. */
    protected java.util.Vector included_files_;

    // ==================================================================
    //
    //  Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope and declaration tables.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     * @param decls The vector of contained declarations.
     */
    public
    FileScopeImpl(Repository rep,
                  ScopeImpl parent,
                  java.util.Vector decls)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        contained_decls_ = decls;
        included_files_ = new java.util.Vector();
    }

    // ==================================================================
    //
    //  Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return null because no Contained object is associated
     *         with the FileScope.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        return null;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the
     *         FileScope declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return the_parent_.getContainer();
    }

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the module declaration.
     */
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
       return the_parent_.getComponentContainer();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_null;
    }

    /**
     * Obtain its absolute name.
     *
     * It is redefined to avoid to add the name of the FileScope.
     *
     * @return Its absolute name.
     */
    public java.lang.String
    getAbsoluteName()
    {
        java.lang.String result = "";
        if(the_parent_ != null)
            result = the_parent_.getAbsoluteName();
        return result;
    }

    /**
     * Destroy all the contained declarations of the file scope in the IR3.
     */
    public void
    destroy()
    {
        // if this FileScope is the root FileScope then destroy it as a Scope.
        if (getName().equals(""))
            super.destroy();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.FileScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for included files.
    //
    // ==================================================================

    /**
     * Add an included file name to the current list.
     *
     * @param fs The parent FileScope.
     */
    public void
    addIncluded(FileScope fs)
    {
        included_files_.add(fs);
    }

    /**
     * Obtain the included file names in order.
     *
     * @return An array containing the included file names.
     */
    public String[]
    getIncluded()
    {
        String[] filenames = new String[included_files_.size()];
        for (int i=0;i<filenames.length;i++)
            filenames[i] = ((Declaration)included_files_.get(i)).getName();

        return filenames;
    }
}
