/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/**
 * ForwardScopeImpl is a wrapper class for Scopes
 * which can be defined as forward.
 *
 * Inherits from:
 * - ScopeImpl as forward scopes are also IDL scopes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public class ForwardScopeImpl
              extends ScopeImpl
           implements org.objectweb.corba.ast.api.ForwardScope
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Is it forward declared scope? */
    private boolean forward_;

    /** Is it completed? */
    private boolean completed_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the forward declaration.
     */
    protected
    ForwardScopeImpl(Repository rep,
                     ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        forward_ = false;
        completed_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tests if this forward declaration is already created.
     *
     * @return True if it is already created else false.
     */
    abstract public boolean
    isForwardCreated();
 
    /**
     * Create this forward declaration.
     */
    abstract public void
    createForward();

    /**
     * Complete this forward declaration.
     */
    abstract public void
    completeForward();

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        if (!isForwardCreated())
        {
            if (the_parent_!=null)
                the_parent_.addDecl(this);

            completed_ = false;

            createForward();

            // Why this?
            // completed_ = !forward_;
        }
        else
        {
            if(forward_)
                return;

            if(completed_)
            {
                throw new org.omg.CORBA.BAD_PARAM(
                         org.objectweb.ccm.IR3.SystemExceptionMinorValues.NameAlreadyUsed,
                         org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            else
            {
                completeForward();
                completed_ = true;
            }
        }
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    /**
     * Is it a forward declaration?
     *
     * @return True if it's a forward declaration, false otherwise.
     */
    public boolean
    isForward()
    {
        return forward_;
    }

    /**
     * Set as forward.
     *
     * @param f True to set the declaration as a forward declaration,
     *          false otherwise.
     */
    public void
    setForward(boolean f)
    {
        forward_ = f;
    }

    /**
     * Is it a completed declaration?
     *
     * @return True if the declaration is complete, false otherwise.
     */
    public boolean
    isCompleted()
    {
        return completed_;
    }
}
