/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST ComponentDecl. */
import org.objectweb.corba.ast.api.ComponentDecl;

/** Used to access AST HomeDecl. */
import org.objectweb.corba.ast.api.HomeDecl;

/** Used to access AST ValueDecl. */
import org.objectweb.corba.ast.api.ValueDecl;

/** To use CORBA::ComponentIR::HomeDef. */
import org.omg.CORBA.ComponentIR.HomeDef;
import org.omg.CORBA.ComponentIR.HomeDefHelper;

/** To use CORBA::ValueDef. */
import org.omg.CORBA.ValueDef;

/**
 * HomeDeclImpl is a wrapper class for IDL home declarations.
 *
 *
 * Inherits from:
 *
 * - ComponentBaseImpl as homes are also interfaces and
 *   have an equivalent client interface mapping.
 *
 * - HomeDecl: OMG IDL for home declarations.
 *
 *
 * Provides:
 *
 * - The inherited base home,
 *   i.e. the setBaseHome and getBaseHome operations.
 *
 * - The managed component,
 *   i.e. the setManagedComponent and getManagedComponent operations.
 *
 * - The primary key,
 *   i.e. the setPrimaryKey and getPrimaryKey operations.
 *
 * - The client explicit interface mapping,
 *   i.e. the getClientExplicitMapping operation.
 *
 * - The client implicit interface mapping,
 *   i.e. the getClientImplicitMapping operation.
 *
 * - The local interface mapping,
 *   i.e. the getLocalMapping operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class HomeDeclImpl
       extends ComponentBaseImpl
       implements HomeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 HomeDef. */
    private HomeDef home_def_;

    /** Reference to the base home. */
    private HomeDeclImpl base_home_;

    /** Reference to the managed component. */
    private ComponentDeclImpl managed_component_;

    /** Reference to the primary key. */
    private ValueDeclImpl primary_key_;

    /** The local interface mapping. */
    private InterfaceDeclImpl local_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the home declaration.
     */
    protected
    HomeDeclImpl(Repository rep,
                 ScopeImpl parent)
    {
        // Call the ComponentBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
        home_def_ = null;
        base_home_ = null;
        managed_component_ = null;
        primary_key_ = null;
        local_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 HomeDef.
     *
     * @param contained The HomeDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        home_def_ = HomeDefHelper.narrow(contained);

        HomeDef baseHome = home_def_.base_home();
        if (baseHome!=null)
            setBaseHome((HomeDecl)getRepository().
                        lookupId(baseHome.id()));

        setManagedComponent(
            (ComponentDecl)getRepository().
            lookupId(home_def_.managed_component().id()));

        ValueDef primaryKey = home_def_.primary_key();
        if (primaryKey!=null)
            setPrimaryKey((ValueDecl)getRepository().
                          lookupId(primaryKey.id()));

        addSupportedInterfaces(home_def_.supported_interfaces());

        // load it's mapped declarations
        getClientMapping();

        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the home declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return home_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the home declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
       return home_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
      // System.out.println("create_home on " + the_parent_);
      // System.out.println(getId());

        home_def_ = the_parent_.getComponentContainer().
                    create_home(getId(), getName(), getVersion(),
                                getBaseHomeDef(),
                                managed_component_.getComponentDef(),
                                supported_interfaces_.getInterfaceDefSeq(),
                                getPrimaryKeyDef());
    }

    /**
     * Complete the container object.
     */
    protected void
    completeContainer()
    {
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the inherited home base.
     *
     * @return The inherited home base.
     */
    public HomeDef
    getBaseHomeDef()
    {
        return (base_home_ != null)
              ?(base_home_.getHomeDef()) : null;
    }

    /**
     * Obtain the primary key value.
     *
     * @return The primary key value.
     */
    public org.omg.CORBA.ExtValueDef
    getPrimaryKeyDef()
    {
        return (primary_key_ != null)
              ?(primary_key_.getExtValueDef()) : null;
    }

    /**
     * Obtain its HomeDef reference.
     *
     * @return The HomeDef object associated with the home declaration.
     */
    public HomeDef
    getHomeDef()
    {
        return home_def_;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the home declaration.
     */
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return home_def_;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the interface
     *         declaration.
     */
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return home_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector home_depend = new java.util.Vector();
        Declaration[] depend = null;

        // base home
        if (getBaseHome()!=null)
        {
            home_depend.add(getBaseHome());
            depend = getBaseHome().getDependencies();
            for (int j=0; j<depend.length; j++)
                home_depend.add(depend[j]);
        }

        // primary key
        if (getPrimaryKey()!=null)
        {
            home_depend.add(getPrimaryKey());
            depend = getPrimaryKey().getDependencies();
            for (int j=0; j<depend.length; j++)
                home_depend.add(depend[j]);
        }

        // managed component
        home_depend.add(getManagedComponent());
        depend = getManagedComponent().getDependencies();
        for (int j=0; j<depend.length; j++)
            home_depend.add(depend[j]);

        // supported interfaces
        InterfaceDecl[] supp = supported_interfaces_.getInterfaces();
        for (int i=0; i<supp.length; i++)
        {
            home_depend.add(supp[i]);
            depend = supp[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if (home_depend.indexOf(depend[j])==-1)
                    home_depend.add(depend[j]);
            }
        }

        // contents
        Declaration[] decls = getContents(true, DeclarationKind.dk_all);
        for (int i=0; i<decls.length; i++)
        {
            depend = decls[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (home_depend.indexOf(depend[j])==-1))
                    home_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])home_depend.toArray(new Declaration[0]);
/*            System.err.println("dependances de la homeDecl : "+getAbsoluteName());
            for(int i=0; i<dependencies_.length; i++)
                System.err.println("    |"+dependencies_[i].getName());
*/        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_home;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    /**
     * Find a declaration.
     *
     * Note that the declaration is also recursively searched in the
     * parent scope.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    public Declaration
    find(String name)
    {
        // Finds in the scope.
        Declaration decl = super.find(name);

        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in inherited interfaces.
        if (base_home_ != null)
        {
            decl = base_home_.find(name);
            if(decl != null) return decl;
        }

        // If not found then finds in supported interfaces.
        return supported_interfaces_.find(name);
    }

    /**
     * To obtain all the contained Declaration objects.
     *
     * @param exclude_inherited If false return also objects contained
     *        in inherited scopes.
     * @param limited_types A logical combination of DeclarationKind.
     *
     * @return An array of Declaration objects.
     */
    public Declaration[]
    getContents(boolean exclude_inherited,
                long limited_types)
    {
        if ((exclude_inherited) || (base_home_==null))
            return super.getContents(exclude_inherited, limited_types);

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = supported_interfaces_.getInterfaces();
        for (int i=0; i<itfs.length; i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }
        Declaration[] tmp1 = base_home_.getContents(exclude_inherited, limited_types);
        Declaration[] tmp2 = res;
        res = new Declaration[tmp1.length+tmp2.length];
        System.arraycopy(tmp2, 0, res, 0, tmp2.length);
        System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        return res;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_home;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfaceDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the home declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return home_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ComponentBase
    //
    // ==================================================================

    /**
     * Obtain the client interface mapping.
     *
     * @return The client interface mapping.
     */
    public InterfaceDecl
    getClientMapping()
    {
        if (client_mapping_ == null)
        {
            // load the client mapping for this component.
            client_mapping_ = (InterfaceDeclImpl)getRepository().
                              loadMapping(getParent(), getId());
        }
        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.HomeDecl
    //
    // ==================================================================

    /**
     * Set the base home.
     *
     * @param base_home The base home.
     */
    public void
    setBaseHome(HomeDecl base_home)
    {
        if (base_home != null)
        {
            base_home_ = (HomeDeclImpl)base_home;
        }
    }

    /**
     * Obtain the base home.
     *
     * @return The base home.
     */
    public HomeDecl
    getBaseHome()
    {
        return base_home_;
    }

    /**
     * Set the managed component.
     *
     * @param managed_component The managed component.
     */
    public void
    setManagedComponent(ComponentDecl managed_component)
    {
        if (managed_component != null)
        {
            managed_component_ = (ComponentDeclImpl)managed_component;
        }
    }

    /**
     * Obtain the managed component.
     *
     * @return The managed component.
     */
    public ComponentDecl
    getManagedComponent()
    {
        return managed_component_;
    }

    /**
     * Set the primary key.
     *
     * @param primary_key The primary key.
     */
    public void
    setPrimaryKey(ValueDecl primary_key)
    {
        if (primary_key != null)
        {
            primary_key_ = (ValueDeclImpl)primary_key;
        }
    }

    /**
     * Obtain the primary key.
     *
     * @return The primary key.
     */
    public ValueDecl
    getPrimaryKey()
    {
        return primary_key_;
    }

    /**
     * Obtain the client explicit interface mapping.
     *
     * @return The client explicit interface mapping.
     */
    public InterfaceDecl
    getClientExplicitMapping()
    {
        // retrieve client mapping interface.
        InterfaceDecl client = getClientMapping();
        InterfaceDecl[] bases = client.getInheritedInterfaceList()
                                      .getInterfaces();
        if (bases[0].getName().equals(getName()+"Explicit"))
            return bases[0];
        else
            return bases[1];
    }

    /**
     * Obtain the client implicit interface mapping.managed_component_
     *
     * @return The client implicit interface mapping.
     */
    public InterfaceDecl
    getClientImplicitMapping()
    {
        // retrieve client mapping interface.
        InterfaceDecl client = getClientMapping();
        InterfaceDecl[] bases = client.getInheritedInterfaceList()
                                      .getInterfaces();
        if (bases[0].getName().equals(getName()+"Implicit"))
            return bases[0];
        else
            return bases[1];
    }

    /**
     * Obtain the local interface mapping.
     *
     * @return The local interface mapping.
     */
    public InterfaceDecl
    getLocalMapping()
    {
        if (local_mapping_ == null)
        {
            // load the local mapping for this home.
            String parent_base_id = the_parent_.getId();
            int idx = parent_base_id.lastIndexOf(':');
            parent_base_id = parent_base_id.substring(0, idx);
            String id = parent_base_id + "/CCM_" + getName() +
                        ':' + getVersion();

            local_mapping_ = (InterfaceDeclImpl)getRepository().
                             loadMapping(getParent(), id);
        }
        return local_mapping_;
    }
}
