/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DelegationList. */
import org.objectweb.corba.ast.api.DelegationList;

/** To access AST StringList. */
import org.objectweb.corba.ast.api.StringList;

/** To access AST HomeDecl. */
import org.objectweb.corba.ast.api.HomeDecl;

/** To access AST AbstractStorageHomeDecl. */
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;

/** To access AST ExecutorDecl. */
import org.objectweb.corba.ast.api.ExecutorDecl;

/**
 * HomeExecutorDeclImpl is a wrapper class for CIDL home executor declarations.
 *
 *
 * Inherits from:
 * - ScopeImpl as executors are also IDL scopes.
 * - HomeExecutorDecl as OMG IDL for CIDL home executor declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class HomeExecutorDeclImpl
     extends ScopeImpl
  implements org.objectweb.corba.ast.api.HomeExecutorDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated home declaration. */
    private HomeDeclImpl home_;

    /** The associated abstract storagehome binding. */
    private AbstractStorageHomeDeclImpl ashBinding_;

    /** The associated abstract storagehome persistence. */
    private AbstractStorageHomeDeclImpl ashPersistence_;

    /** List of abstract storagehome delegations. */
    private DelegationListImpl ash_delegation_;

    /** List of executor delegations. */
    private DelegationListImpl executor_delegation_;

    /** List of abstract operations. */
    private StringListImpl operations_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the exception declaration.
     */
    public
    HomeExecutorDeclImpl(Repository rep,
                         ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        home_ = null;
        ashBinding_ = null;
        ashPersistence_ = null;
        ash_delegation_ = new DelegationListImpl();
        executor_delegation_ = new DelegationListImpl();
        operations_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the exception
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the exception
     *          declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        // TODO
        return null; 
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_home_executor;
    }

    /**
     * Close the exception scope.
     */
    public void
    create()
    {
      // TODO check if the home executor declaration is correct
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.HomeExecutorDecl
    //
    // ==================================================================

    /**
     * Set the associated home declaration.
     *
     * @param ash The associated home declaration.
     */
    public void
    setHomeType(HomeDecl h)
    {
        home_ = (HomeDeclImpl)h;
    }

    /**
     * Obtain the associated home declaration.
     *
     * @return The associated home declaration.
     */
    public HomeDecl
    getHomeType()
    {
        return home_;
    }

    /**
     * Set the associated binding abstract storagehome.
     *
     * @param ash The associated binding abstract storagehome.
     */
    public void
    setAbstractStorageHomeBinding(AbstractStorageHomeDecl ash)
    {
        ashBinding_ = (AbstractStorageHomeDeclImpl)ash;
    }

    /**
     * Obtain the associated binding abstract storagehome.
     *
     * @return The associated binding abstract storagehome.
     */
    public AbstractStorageHomeDecl
    getAbstractStorageHomeBinding()
    {
        return ashBinding_;
    }

    /**
     * Set the associated persistence abstract storagehome.
     *
     * @param ash The associated persistence abstract storagehome.
     */
    public void
    setHomePersistence(AbstractStorageHomeDecl ash)
    {
        ashPersistence_ = (AbstractStorageHomeDeclImpl)ash;
    }

    /**
     * Obtain the associated persistence abstract storagehome.
     *
     * @return The associated persistence abstract storagehome.
     */
    public AbstractStorageHomeDecl
    getHomePersistence()
    {
        return ashPersistence_;
    }

    /**
     * Obtain the list of abstract storagehome delegations.
     *
     * @return The list of abstract storagehome delegations.
     */
    public DelegationList
    getAbstractStorageHomeDelegationList()
    {
        return ash_delegation_;
    }

    /**
     * Obtain the list of executor delegations.
     *
     * @return The list of executor delegations.
     */
    public DelegationList
    getExecutorDelegationList()
    {
        return executor_delegation_;
    }

    /**
     * Obtain the list of abstract operations.
     *
     * @return The list of abstract operations.
     */
    public StringList
    getAbstractOperationList()
    {
        return operations_;
    }

    /**
     * Start a new CIDL executor declaration.
     *
     * @param name The name of the CIDL executor declaration.
     *
     * @return The new created ExecutorDecl.
     */
    public ExecutorDecl
    startExecutor(String name)
    {
        ExecutorDecl decl = new ExecutorDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }
}
