/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST InterfaceList. */
import org.objectweb.corba.ast.api.InterfaceList;

/** Used to access AST AttributeDecl. */
import org.objectweb.corba.ast.api.AttributeDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** To use CORBA::InterfaceDef. */
import org.omg.CORBA.InterfaceDef;

/** To use CORBA::ExtInterfaceDef. */
import org.omg.CORBA.ExtInterfaceDef;
import org.omg.CORBA.ExtInterfaceDefHelper;

/**
 * InterfaceDeclImpl is a wrapper class for IDL interface declarations.
 *
 * Inherits from:
 * - ForwardScope as interfaces are also IDL forward scopes.
 * - TypeRef as interfaces are IDL types.
 *
 * Provides:
 * - The list of inherited interfaces
 *   i.e. the getInheritedInterfaceList operation
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class InterfaceDeclImpl
     extends ForwardScopeIDLImpl
  implements InterfaceDecl,
             IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 InterfaceDef. */
    private ExtInterfaceDef ext_interface_def_;

    /** List of inherited interfaces. */
    protected InterfaceListImpl inherited_interfaces_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the interface declaration.
     */
    protected 
    InterfaceDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the ForwardScopeIDLImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_interface_def_ = null;
        inherited_interfaces_ = new InterfaceListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 InterfaceDef.
     *
     * @param contained The InterfaceDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_interface_def_ = ExtInterfaceDefHelper.narrow(contained);
        InterfaceDef[] base_interfaces = ext_interface_def_.base_interfaces();

        for (int i=0; i<base_interfaces.length; i++)
	    // WARNING: This will break consistency of array items!!!
            inherited_interfaces_.add((InterfaceDecl)
                       getRepository().lookupId(base_interfaces[i].id()));

        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 InterfaceDef.
     *
     * @param contained The InterfaceDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_interface_def_ = ExtInterfaceDefHelper.narrow(contained);
        InterfaceDef[] base_interfaces = ext_interface_def_.base_interfaces();

        for (int i=0; i<base_interfaces.length; i++)
	    // WARNING: This will break consistency of array items!!!
            inherited_interfaces_.add((InterfaceDecl)
                getRepository().lookupMappedId(base_interfaces[i].id()));

        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the interface
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        return ext_interface_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Create an attribute definition.
     *
     * @param attribute The attribute declaration.
     * @param type The attribute IDLType.
     * @param mode The attribute mode.
     * @param get_exceptions An array containing the getRaises
     *                      (or raises for readonly attributes) exceptions.
     * @param put_exceptions An array containing the setRaises exceptions.
     *
     * @return The new created AttributeDef.
     */
    protected org.omg.CORBA.ExtAttributeDef
    createExtAttribute(AttributeDecl attribute,
                       org.omg.CORBA.IDLType type,
                       org.omg.CORBA.AttributeMode mode,
                       org.omg.CORBA.ExceptionDef[] get_exceptions,
                       org.omg.CORBA.ExceptionDef[] set_exceptions)
    {
       // Create an AttributeDef into the IFR.
        org.omg.CORBA.ExtAttributeDef ext_attribute_def =
            getExtInterfaceDef().create_ext_attribute(attribute.getId(),
                                                      attribute.getName(),
                                                      attribute.getVersion(),
                                                      type, mode,
                                                      get_exceptions,
                                                      set_exceptions);
        return ext_attribute_def;
    }

    /**
     * Create an operation.
     *
     * @param operation The operation declaration.
     * @param type The operation return IDLType.
     * @param mode The mode of the operation.
     * @param params An array containing the parameters description.
     * @param exceptions An array containing the exceptions that
     *                   can be raised.
     * @param contexts An array containing the possible context values.
     *
     * @return The new created OperationDef.
     */
    protected org.omg.CORBA.OperationDef
    createOperation(OperationDecl operation,
                    org.omg.CORBA.IDLType type,
                    org.omg.CORBA.OperationMode mode,
                    org.omg.CORBA.ParameterDescription[] params,
                    org.omg.CORBA.ExceptionDef[] exceptions,
                    String[] contexts)
    {
        // Create an OperationDef into the IFR.
        org.omg.CORBA.OperationDef operation_def =
            getExtInterfaceDef().create_operation(operation.getId(),
                                                  operation.getName(),
                                                  operation.getVersion(),
                                                  type, mode,
                                                  params,
                                                  exceptions,
                                                  contexts);
        return operation_def;
    }

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the interface
     *         declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return ext_interface_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        ext_interface_def_ = the_parent_.getContainer().
                    create_ext_interface(getId(), getName(), getVersion(),
                               inherited_interfaces_.getInterfaceDefSeq());
    }

    /**
     **  Complete the container object.
     **/
    protected void
    completeContainer()
    {
        InterfaceDef[] inherited_interfaces = inherited_interfaces_.getInterfaceDefSeq();
        if (inherited_interfaces.length != 0)
            getExtInterfaceDef().base_interfaces(inherited_interfaces);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the interface
     *         declaration.
     */
    public ExtInterfaceDef
    getExtInterfaceDef()
    {
        return ext_interface_def_;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef object associated with the interface
     *         declaration.
     */
    public InterfaceDef
    getInterfaceDef()
    {
        return ext_interface_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        java.util.Vector itf_depend = new java.util.Vector();
        Declaration[] depend = null;

        // base interfaces
        InterfaceDecl[] bases = inherited_interfaces_.getInterfaces();
        for (int i=0; i<bases.length; i++)
        {
            itf_depend.add(bases[i]);
            depend = bases[i].getDependencies();
            for (int j=0;j<depend.length;j++)
            {
                if ((depend[j]!=this) &&
                    (itf_depend.indexOf(depend[j])==-1))
                    itf_depend.add(depend[j]);
            }
        }

        // contents
        Declaration[] decls = getContents(true, DeclarationKind.dk_all);
        for (int i=0; i<decls.length; i++)
        {
            depend = decls[i].getDependencies();
            for (int j=0;j<depend.length;j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (depend[j]!=this) &&
                    (itf_depend.indexOf(depend[j])==-1))
                    itf_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])itf_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_interface;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    /**
     * Find a declaration.
     *
     * Note that the declaration is also recursively searched
     *      in the parent scope.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *          or null if it does not exist.
     */
    public Declaration
    find(String name)
    {
        // Finds in the scope.
        Declaration decl = super.find(name);

        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in inherited interfaces.
        return inherited_interfaces_.find(name);
    }

    /**
     * To obtain all the contained Declaration objects.
     *
     * @param exclude_inherited If false return also objects
     *                          contained in inherited scopes.
     * @param limited_types A logical combination of DeclarationKind.
     *
     * @return An array of Declaration objects.
     */
    public Declaration[]
    getContents(boolean exclude_inherited,
                long limited_types)
    {
        if (exclude_inherited)
            return super.getContents(exclude_inherited, limited_types);

        // if it's a mapping scope, then use the OpenCCM repository
        // as an IDL2 repository to retrieve contained objects.
        if (is_mapping_)
            getRepository().useIDL2Repository();

        Declaration[] res = super.getContents(exclude_inherited, limited_types);
        InterfaceDecl[] itfs = inherited_interfaces_.getInterfaces();
        for (int i=0; i<itfs.length; i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }

        // get back to an IDL3 repository.
        if (is_mapping_)
            getRepository().useIDL3Repository();

        return res;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_interface;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfaceDecl
    //
    // ==================================================================

    /**
     * Obtain the inherited interface list.
     *
     * @return The inherited interface list.
     */
    public InterfaceList 
    getInheritedInterfaceList()
    {
        return inherited_interfaces_;
    }

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the interface declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_interface_def_;
    }
}
