/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To use CORBA::LocalInterfaceDef. */
import org.omg.CORBA.LocalInterfaceDef;

/** To use CORBA::ExtLocalInterfaceDef. */
import org.omg.CORBA.ExtLocalInterfaceDef;
import org.omg.CORBA.ExtLocalInterfaceDefHelper;

/**
 * LocalInterfaceDeclImpl is a wrapper class for
 * IDL local interface declarations.
 *
 * Inherits from:
 * - InterfaceDeclImpl as they are also IDL interfaces.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class LocalInterfaceDeclImpl
       extends InterfaceDeclImpl
       implements org.objectweb.corba.ast.api.LocalInterfaceDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 LocalInterfaceDef. */
    private org.omg.CORBA.ExtLocalInterfaceDef ext_local_interface_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the local interface declaration.
     */
    protected
    LocalInterfaceDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the InterfaceDefImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_local_interface_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 InterfaceDef.
     *
     * @param contained The InterfaceDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_local_interface_def_ = ExtLocalInterfaceDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 InterfaceDef.
     *
     * @param contained The InterfaceDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_local_interface_def_ = ExtLocalInterfaceDefHelper.narrow(contained);
        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the local
     *         interface declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the local
     *         interface declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
       return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    /**
     **  Create the container object.
     **/
    protected void
    createContainer()
    {
        ext_local_interface_def_ = the_parent_.getContainer().
             create_ext_local_interface(getId(), getName(), getVersion(), 
                              inherited_interfaces_.getInterfaceDefSeq());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef associated with
     *         the local interface declaration.
     */
    public org.omg.CORBA.ExtInterfaceDef
    getExtInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    /**
     * Obtain its InterfaceDef reference.
     *
     * @return The InterfaceDef associated with
     *         the local interface declaration.
     */
    public org.omg.CORBA.InterfaceDef
    getInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    /**
     * Obtain its LocalInterfaceDef reference.
     *
     * @return The LocalInterfaceDef associated with the local
     *         interface declaration.
     */
    public org.omg.CORBA.ExtLocalInterfaceDef
    getExtLocalInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    /**
     * Obtain its LocalInterfaceDef reference.
     *
     * @return The LocalInterfaceDef associated with the local
     *         interface declaration.
     */
    public org.omg.CORBA.LocalInterfaceDef
    getLocalInterfaceDef()
    {
        return ext_local_interface_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_local_interface;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_local_interface;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfaceDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.LocalInterfaceDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the local interface declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_local_interface_def_;
    }
}
