/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To use CORBA::NativeDef. */
import org.omg.CORBA.NativeDef;
import org.omg.CORBA.NativeDefHelper;

/**
 * NativeDeclImpl is a wrapper class for IDL native declarations.
 *
 * Inherits from:
 * - DeclarationImpl as natives are IDL declarations.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class NativeDeclImpl
       extends DeclarationImpl
       implements org.objectweb.corba.ast.api.NativeDecl,
                  IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 NativeDef. */
    private NativeDef native_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the native declaration.
     */
    protected
    NativeDeclImpl(Repository rep,
                   ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        native_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 NativeDef.
     *
     * @param contained The NativeDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        native_def_ = NativeDefHelper.narrow(contained);
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 NativeDef.
     *
     * @param contained The NativeDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        native_def_ = NativeDefHelper.narrow(contained);
        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the native declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return native_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_native;
    }

    /**
     * Create the native declaration into the IFR.
     */
    public void
    create()
    {
        // Create a NativeDef into the IFR.
        native_def_ = the_parent_.getContainer().
                      create_native(getId(), getName(), getVersion());

        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_native;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.NativeDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the type reference.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return native_def_;
    }
}
