/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST ExceptionDecl. */
import org.objectweb.corba.ast.api.ExceptionDecl;

/** Used to access AST StringList. */
import org.objectweb.corba.ast.api.StringList;

/** To access AST Parameter. */
import org.objectweb.corba.ast.api.Parameter;

/** To use CORBA::OperationDef. */
import org.omg.CORBA.OperationDef;
import org.omg.CORBA.OperationDefHelper;

/** To use CORBA::OperationMode. */
import org.omg.CORBA.OperationMode;

/** To use CORBA::ExceptionDef. */
import org.omg.CORBA.ExceptionDef;

/** To use CORBA::ParameterDescription. */
import org.omg.CORBA.ParameterDescription;

/**
 * OperationDeclImpl is a wrapper class for IDL operation declarations.
 *
 *
 * Inherits from:
 *
 * - OperationBaseImpl.
 *
 * - OperationDecl: OMG IDL for operation declarations.
 *
 *
 * Provides:
 *
 * - the 'Oneway' or 'Normal' mode,
 *   i.e. the setOneway, isOneway, setNormal, and isNormal operations.
 *
 * - the list of contexts,
 *   i.e. the getContextList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class OperationDeclImpl
       extends OperationBaseImpl
       implements org.objectweb.corba.ast.api.OperationDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 OperationDef. */
    private OperationDef operation_def_;

    /**
     **  The operation mode.
     **/
    protected OperationMode mode_;

    /** List of contexts. */
    private StringListImpl contexts_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the operation declaration.
     */
    protected
    OperationDeclImpl(Repository rep,
                      ScopeImpl parent)
    {
        // Call the OperationBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
        operation_def_ = null;
        mode_ = OperationMode.OP_NORMAL;
        contexts_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 OperationDef.
     *
     * @param contained The OperationDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        operation_def_ = OperationDefHelper.narrow(contained);
        mode_ = operation_def_.mode();
        setType(getRepository().
                getAsTypeRef(operation_def_.result_def()));
        ParameterDescription[] params = operation_def_.params();
        for (int i=0; i<params.length; i++)
        {
            parameters_.addParam(params[i].name,
                     getRepository().getAsTypeRef(params[i].type_def),
                     params[i].mode);
        }
        ExceptionDef[] exc = operation_def_.exceptions();
        for (int i=0; i<exc.length; i++)
        {
            exceptions_.add((ExceptionDecl)getRepository().
                            lookupId(exc[i].id()));
        }
        String[] contexts = operation_def_.contexts();
        for (int i=0; i<contexts.length; i++)
            contexts_.add(contexts[i]);

        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 OperationDef.
     *
     * @param operation_def The OperationDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        operation_def_ = OperationDefHelper.narrow(contained);
        mode_ = operation_def_.mode();
        setType(getRepository().
                getAsMappedTypeRef(operation_def_.result_def()));

        ParameterDescription[] params = operation_def_.params();
        for (int i=0; i<params.length; i++)
        {
            parameters_.addParam(params[i].name,
                     getRepository().getAsMappedTypeRef(params[i].type_def),
                     params[i].mode);
        }

        ExceptionDef[] exc = operation_def_.exceptions();
        for (int i=0; i<exc.length; i++)
        {
            exceptions_.add((ExceptionDecl)getRepository().
                            lookupMappedId(exc[i].id()));
        }

        String[] contexts = operation_def_.contexts();
        for (int i=0; i<contexts.length; i++)
            contexts_.add(contexts[i]);

        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the
     *         operation declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return operation_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector op_depend = new java.util.Vector();
        Declaration[] depend = null;

        // return type
        if (getType().isDeclaration())
            op_depend.add(getType());

        depend = getType().getDependencies();
        for (int i=0; i<depend.length; i++)
            op_depend.add(depend[i]);

        // exceptions
        ExceptionDecl[] excs = exceptions_.getExceptions();
        for (int i=0; i<excs.length; i++)
        {
            op_depend.add(excs[i]);
            depend = excs[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if (op_depend.indexOf(depend[j])==-1)
                    op_depend.add(depend[j]);
            }
        }

        // parameters type
        Parameter[] parameters = parameters_.getParameters();
        for (int i=0;i<parameters.length;i++)
        {
            TypeRef param_type = parameters[i].getType();
            if (param_type.isDeclaration())
                op_depend.add(param_type);

            depend = param_type.getDependencies();
            for (int j=0;j<depend.length;j++)
            {
                if (op_depend.indexOf(depend[j])==-1)
                    op_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])op_depend.toArray(new Declaration[0]);
/*            System.err.println("dependances de l'operation : "+getAbsoluteName());
            for(int i=0; i<dependencies_.length; i++)
                System.err.println("    |"+dependencies_[i].getAbsoluteName());
*/        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_operation;
    }

    /**
     * Create the operation declaration into the IFR.
     */
    public void
    create()
    {
        operation_def_ = the_parent_.
                         createOperation(this,
                                         getIDLType(),
                                         mode_,
                                 parameters_.getParameterDescriptionSeq(),
                                 exceptions_.getExceptionDefSeq(),
                                 contexts_.getStrings());
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.OperationBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.OperationDecl
    //
    // ==================================================================

    /**
     * Set the associated mode as oneway.
     */
    public void
    setOneway()
    {
        mode_ = OperationMode.OP_ONEWAY;
    }

    /**
     * Is it an oneway operation?
     *
     * @return True if oneway operation, or false otherwise.
     */
    public boolean
    isOneway()
    {
        return mode_ == OperationMode.OP_ONEWAY;
    }

    /**
     * Set the associated mode as normal.
     */
    public void
    setNormal()
    {
        mode_ = OperationMode.OP_NORMAL;
    }

    /**
     * Is it a normal operation?
     *
     * @return True if a normal operation, or false otherwise.
     */
    public boolean
    isNormal()
    {
        return mode_ == OperationMode.OP_NORMAL;
    }

    /**
     * Obtain the list of contexts.
     *
     * @return The list of contexts.
     */
    public StringList
    getContextList()
    {
        return contexts_;
    }
}
