/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To use AST Parameter. */
import org.objectweb.corba.ast.api.Parameter;

/** To use CORBA parameter mode. */
import org.omg.CORBA.ParameterMode;

/** To use CORBA::ParameterDescription. */
import org.omg.CORBA.ParameterDescription;

/** To use CORBA::StructMember. */
import org.omg.CORBA.StructMember;

/**
 * ParameterListImpl is a wrapper class for IDL operation parameter lists.
 *
 *
 * Inherits from:
 *
 * - ListBaseImpl: The base class for all lists.
 *
 * - ParameterList: OMG IDL for operation parameter lists.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ParameterListImpl
       extends ListBaseImpl
       implements org.objectweb.corba.ast.api.ParameterList
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public ParameterListImpl()
    {
        // Call the ListBaseImpl constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add an in parameter.
     *
     * @param name The name of the in parameter.
     * @param type The type reference of the in parameter.
     */
    protected void
    addParam(String name,
             TypeRef type,
             ParameterMode mode)
    {
        if (type!=null)
        {
            super.addObject( new ParameterImpl(name, type, mode) );
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its associated CORBA::ParameterDescriptionSeq.
     *
     * @return Its associated CORBA::ParameterDescriptionSeq.
     */
    public ParameterDescription[]
    getParameterDescriptionSeq()
    {
        // Create an array with the same length as the list. 
        ParameterDescription[] result = new ParameterDescription[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            ParameterImpl param = (ParameterImpl)it.next();

            // Obtain its associated CORBA::ParameterDescription.
            result[i] = param.getParameterDescription();       
        }

        // Return the array.
        return result;
    }

    /**
     * Obtain its associated CORBA::StructMemberSeq.
     *
     * @return Its associated CORBA::StructMemberSeq.
     */
    public StructMember[]
    getStructMemberSeq()
    {
        // Create an array with the same length as the list. 
        StructMember[] result = new StructMember[getSize()];

        // Iterate on all elements of the list.
        java.util.Iterator it = iterator();
        for(int i=0; it.hasNext(); i++)
        {
            // Obtain the current element.
            ParameterImpl param = (ParameterImpl)it.next();

            // Obtain its associated CORBA::StructMember.
            result[i] = param.getStructMember();       
        }

        // Return the array.
        return result;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ParameterList
    //
    // ==================================================================

    /**
     * Add an in parameter.
     *
     * @param name The name of the in parameter.
     * @param type The type reference of the in parameter.
     */
    public void
    addInParam(String name,
               TypeRef type)
    {
        addParam(name, type, ParameterMode.PARAM_IN);
    }

    /**
     * Add an out parameter.
     *
     * @param name The name of the parameter.
     * @param type The TypeRef of the parameter.
     */
    public void
    addOutParam(String name,
                TypeRef type)
    {
        addParam(name, type, ParameterMode.PARAM_OUT);
    }

    /**
     * Add an inout parameter.
     *
     * @param name The name of the parameter.
     * @param type The TypeRef of the parameter.
     */
    public void
    addInOutParam(String name,
                  TypeRef type)
    {
        addParam(name, type, ParameterMode.PARAM_INOUT);
    }

    /**
     * Obtain all the parameters.
     *
     * @return All the parameters.
     */
    public Parameter[]
    getParameters()
    {
       return (Parameter[])super.toArray(new Parameter[0]);
    }
}
