/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** Used to access AST ComponentDecl. */
import org.objectweb.corba.ast.api.ComponentDecl;

/** To use CORBA::ComponentIR::ProvidesDef. */
import org.omg.CORBA.ComponentIR.ProvidesDef;
import org.omg.CORBA.ComponentIR.ProvidesDefHelper;

/**
 * ProvidesDeclImpl is a wrapper class for IDL provides port declarations.
 *
 * - InterfacePortDeclImpl as they are ports for an interface,
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ProvidesDeclImpl
       extends InterfacePortDeclImpl
       implements org.objectweb.corba.ast.api.ProvidesDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ProvidesDef. */
    private ProvidesDef provides_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the provides declaration.
     */
    protected
    ProvidesDeclImpl(Repository rep,
                     ScopeImpl parent)
    {
        // Call the InterfacePortDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        provides_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ProvidesDef.
     *
     * @param contained The ProvidesDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        provides_def_ = ProvidesDefHelper.narrow(contained);
        setInterface((InterfaceDecl)getRepository().
                     lookupId(provides_def_.interface_type().id()));
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the provides declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return provides_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector provides_depend = new java.util.Vector();

        // content
        // TODO: The next test is not needed
        // as InterfaceDecl is a Declaration, isn't it? 
        if (getInterface().isDeclaration())
            provides_depend.add(getInterface());

        Declaration[] depend = getInterface().getDependencies();
        for (int i=0;i<depend.length;i++)
            provides_depend.add(depend[i]);

        dependencies_ = (Declaration[])provides_depend.toArray(
                                                     new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_provides;
    }

    /**
     * Create the provides declaration into the IFR.
     */
    public void
    create()
    {
        provides_def_ = the_parent_.getComponentDef().
                        create_provides(getId(), getName(), getVersion(),
                                        interface_.getExtInterfaceDef());
        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ClientOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the client operation mapping.
     *
     * @return The client operation mapping.
     */
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((ComponentDecl)the_parent_).getClientMapping();
        client.getContents(true, DeclarationKind.dk_null);

        client_mapping_ = new OperationDecl[1];
        client_mapping_[0] = (OperationDecl)client.find("provide_"+getName());
        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfacePortDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ProvidesDecl
    //
    // ==================================================================
}
