/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To access AST PsdlTypeRef. */
import org.objectweb.corba.ast.api.PsdlTypeRef;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/** To access AST FileScope. */
import org.objectweb.corba.ast.api.FileScope;

/** To access AST AnyValue. */
import org.objectweb.corba.ast.api.AnyValue;

/** To manipulate CORBA::PrimitiveKind. */
import org.omg.CORBA.PrimitiveKind;

/**
 * Main class of the OMG IDL / PSDL / CIDL Abstract Syntax Tree.
 *
 * This provides the factory of any values and primitive TypeRefs,
 * and some parser helper functions like startFileScope and
 * import ones.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class Repository
       extends ScopeImpl
       implements org.objectweb.corba.ast.api.AST
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the PrimitiveDef for the null type. */
    private TypeRefImpl null_ = null;

    /** Reference to the PrimitiveDef for the void type. */
    private TypeRefImpl void_ = null;

    /** Reference to the PrimitiveDef for the short type. */
    private TypeRefImpl short_ = null;

    /** Reference to the PrimitiveDef for the long type. */
    private TypeRefImpl long_ = null;

    /** Reference to the PrimitiveDef for the unsigned short type. */
    private TypeRefImpl unsigned_short_ = null;

    /** Reference to the PrimitiveDef for the unsigned long type. */
    private TypeRefImpl unsigned_long_ = null;

    /** Reference to the PrimitiveDef for the float type. */
    private TypeRefImpl float_ = null;

    /** Reference to the PrimitiveDef for the double type. */
    private TypeRefImpl double_ = null;

    /** Reference to the PrimitiveDef for the boolean type. */
    private TypeRefImpl boolean_ = null;

    /** Reference to the PrimitiveDef for the char type. */
    private TypeRefImpl char_ = null;

    /** Reference to the PrimitiveDef for the octet type. */
    private TypeRefImpl octet_ = null;

    /** Reference to the PrimitiveDef for the any type. */
    private TypeRefImpl any_ = null;

    /** Reference to the PrimitiveDef for the TypeCode type. */
    private TypeRefImpl TypeCode_ = null;

    /** Reference to the PrimitiveDef for the Principal type. */
    private TypeRefImpl Principal_ = null;

    /** Reference to the PrimitiveDef for the string type. */
    private TypeRefImpl string_ = null;

    /** Reference to the PrimitiveDef for the Object type. */
    private TypeRefImpl object_ = null;

    /** Reference to the PrimitiveDef for the long long type. */
    private TypeRefImpl long_long_ = null;

    /** Reference to the PrimitiveDef for the unsigned long long type. */
    private TypeRefImpl unsigned_long_long_ = null;

    /** Reference to the PrimitiveDef for the long double type. */
    private TypeRefImpl long_double_ = null;

    /** Reference to the PrimitiveDef for the wchar type. */
    private TypeRefImpl wchar_ = null;

    /** Reference to the PrimitiveDef for the wstring type. */
    private TypeRefImpl wstring_ = null;

    /** Reference to the PrimitiveDef for the value base type. */
    private TypeRefImpl value_base_ = null;

    /** Table of registered declarations. */
    private java.util.Hashtable declarations_;

    /** Table of registered mapping declarations. */
    private java.util.Hashtable mapped_declarations_;

    /** Reference to the CORBA 3.0 Interface Repository. */
    private org.omg.CORBA.Repository repository_;

    /** Reference to the OpenCCM Interface Repository. */
    private org.objectweb.ccm.ComponentRepository openccm_repository_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the CORBA Interface Repository.
     *
     * @param repository The CORBA Repository that should be used.
     */
    public
    Repository(org.omg.CORBA.Repository repository)
    {
        // Call the ScopeImpl constructor.
        super(null, null);

        // Init DeclarationImpl internal state.
        the_repository_ = this;

        // Init internal state.
        declarations_ = new java.util.Hashtable();
        mapped_declarations_ = new java.util.Hashtable();
        repository_ = repository;
        openccm_repository_ = org.objectweb.ccm.ComponentRepositoryHelper.
                              narrow(repository);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add a declaration to the Repository.
     *
     * @param rep_id The associated repository ID.
     * @param decl The declaration to add.
     */
    protected void
    addDeclInRep(String rep_id,
                 Declaration decl)
    {
        declarations_.put(rep_id, decl);
    }

    /**
     * Remove a declaration from the Repository.
     *
     * @param rep_id The associated repository ID.
     */
    protected void
    removeDeclFromRep(String rep_id)
    {
        declarations_.remove(rep_id);
    }

    /**
     * Add a mapped declaration to the Repository.
     *
     * @param rep_id The associated repository ID.
     * @param decl The mapped declaration to add.
     */
    protected void
    addMappedDeclInRep(String rep_id,
                       Declaration decl)
    {
        mapped_declarations_.put(rep_id, decl);
    }

    /**
     * Remove a mapped declaration from the Repository.
     *
     * @param rep_id The associated repository ID.
     */
    protected void
    removeMappedDeclFromRep(String rep_id)
    {
        mapped_declarations_.remove(rep_id);
    }

    /**
     * Obtain a newly created Declaration of a IR3 Contained.
     * Note: only the following Contained can be loaded :
     *    dk_Module, dk_Interface, dk_LocalInterface, dk_AbstractInterface,
     *    dk_Value, dk_Struct, dk_Union, dk_Exception, dk_Component, dk_Home,
     *    dk_Event.
     *
     * @param contained The Contained to use.
     * @param scope The Scope in which the Contained will be loaded.
     *
     * @return The associated Declaration or null if it not exists or if
     *         the DefinitionKind is not valid.
     */
    protected Declaration
    loadContained(Scope scope, org.omg.CORBA.Contained contained)
    {
        ScopeImpl si = (ScopeImpl)scope;
        org.omg.CORBA.DefinitionKind kind = contained.def_kind();

        if ((kind!=org.omg.CORBA.DefinitionKind.dk_Module) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Interface) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_LocalInterface) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_AbstractInterface) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Value) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Struct) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Union) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Exception) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Component) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Home) &&
            (kind!=org.omg.CORBA.DefinitionKind.dk_Event))
            return null;

        return si.loadContained(contained);
    }

    /**
     * Obtain the TypeRef associated to a CORBA::IDLType.
     *
     * @param type The CORBA::IDLType used to search.
     * @param is_mapped True if it is a mapped type else false.
     *
     * @return The associated TypeRef found.
     */
    protected TypeRef
    getAsTypeRef(org.omg.CORBA.IDLType type,
                 boolean is_mapped)
    {
        org.omg.CORBA.TypeCode tc = type.type();
        try
        {
            switch(tc.kind().value())
            {
            case org.omg.CORBA.TCKind._tk_null:
                return getNullType();
            case org.omg.CORBA.TCKind._tk_void:
                return getVoidType();
            case org.omg.CORBA.TCKind._tk_short:
                return getShortType();
            case org.omg.CORBA.TCKind._tk_long:
                return getLongType();
            case org.omg.CORBA.TCKind._tk_ushort:
                return getUShortType();
            case org.omg.CORBA.TCKind._tk_ulong:
                return getULongType();
            case org.omg.CORBA.TCKind._tk_float:
                return getFloatType();
            case org.omg.CORBA.TCKind._tk_double:
                return getDoubleType();
            case org.omg.CORBA.TCKind._tk_boolean:
                return getBooleanType();
            case org.omg.CORBA.TCKind._tk_char:
                return getCharType();
            case org.omg.CORBA.TCKind._tk_octet:
                return getOctetType();
            case org.omg.CORBA.TCKind._tk_any:
                return getAnyType();
            case org.omg.CORBA.TCKind._tk_TypeCode:
                return getTypeCodeType();
            case org.omg.CORBA.TCKind._tk_Principal:
                return getPrincipalType();
            case org.omg.CORBA.TCKind._tk_longlong:
                return getLongLongType();
            case org.omg.CORBA.TCKind._tk_ulonglong:
                return getULongLongType();
            case org.omg.CORBA.TCKind._tk_longdouble:
                return getLongDoubleType();
            case org.omg.CORBA.TCKind._tk_wchar:
                return getWCharType();

            case org.omg.CORBA.TCKind._tk_string:
                return new TypeRefWithLengthImpl(type,
                                                 TypeKind.tk_string,
                                                 tc.length());
            case org.omg.CORBA.TCKind._tk_sequence:
                org.omg.CORBA.SequenceDef seq = org.omg.CORBA.SequenceDefHelper.
                                                narrow(type);
                return new TypeRefWithContentImpl(type,
                                                 TypeKind.tk_sequence,
                                                 tc.length(),
                                                 getAsTypeRef(seq.element_type_def()));
            case org.omg.CORBA.TCKind._tk_array:
                org.omg.CORBA.ArrayDef arr = org.omg.CORBA.ArrayDefHelper.narrow(type);
                return new TypeRefWithContentImpl(type,
                                                 TypeKind.tk_array,
                                                 tc.length(),
                                                 getAsTypeRef(arr.element_type_def()));
            case org.omg.CORBA.TCKind._tk_wstring:
                return new TypeRefWithLengthImpl(type,
                                                 TypeKind.tk_wstring,
                                                 tc.length());
            case org.omg.CORBA.TCKind._tk_fixed:
                return new FixedTypeRefImpl(type,
                                           tc.fixed_digits(),
                                           tc.fixed_scale());
            case org.omg.CORBA.TCKind._tk_objref:
              if (tc.id().equals("IDL:omg.org/CORBA/Object:1.0"))
                return getObjectType();

            case org.omg.CORBA.TCKind._tk_struct:
            case org.omg.CORBA.TCKind._tk_union:
            case org.omg.CORBA.TCKind._tk_enum:
            case org.omg.CORBA.TCKind._tk_alias:
            case org.omg.CORBA.TCKind._tk_value:
            case org.omg.CORBA.TCKind._tk_value_box:
            case org.omg.CORBA.TCKind._tk_native:
            case org.omg.CORBA.TCKind._tk_abstract_interface:
/*
 * not defined !
            case org.omg.CORBA.TCKind._tk_local_interface:
            case org.omg.CORBA.TCKind_.tk_event:
            case org.omg.CORBA.TCKind._tk_home:
            case org.omg.CORBA.TCKind._tk_component:
 */
                if (is_mapped)
                    return (TypeRef)lookupMappedId(tc.id());
                else
                    return (TypeRef)lookupId(tc.id());

            //
            // PB: an exception is not an IDLType
            // so it can't have an associated TypeRef
            //
            case org.omg.CORBA.TCKind._tk_except:
            default:
                return getNullType();
            }
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            throw new org.objectweb.ccm.CORBA.UserExceptionWrapper(ex);
        }
    }

    /**
     * Create a TypeRefImpl.
     *
     * @param pk The associated CORBA::PrimitiveKind.
     * @param tk The associated AST TypeKind.
     *
     * @return The create TypeRefImpl.
     */
    protected TypeRefImpl
    createTypeRefImpl(PrimitiveKind pk,
                      TypeKind tk)
    {
      return new TypeRefImpl(repository_.get_primitive(pk), tk);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * To use the OpenCCM Interface Repository as OMG IDL 3.x one.
     */
    public void
    useIDL3Repository()
    {
        // if we're not using the OpenCCM repository, mappings are not available.
        if (openccm_repository_!=null)
            openccm_repository_.as_IDL3_repository();
    }

    /**
     * To use the OpenCCM Interface Repository as OMG IDL 2.x one.
     */
    public void
    useIDL2Repository()
    {
        // if we're not using the OpenCCM repository, mappings are not available.
        if (openccm_repository_!=null)
            openccm_repository_.as_IDL2_repository();
    }

    /**
     * TODO: Should be commented!
     */
    public Declaration
    loadMapping(Scope scope, String rep_id)
    {
        // if we're not using the OpenCCM repository, mappings are not available.
        if (openccm_repository_==null)
            return null;

        // use the OpenCCM repository as an IDL2 repository to extract the mapping !!!
        useIDL2Repository();

        // check if it was not already loaded.
        Declaration result = (Declaration)mapped_declarations_.get(rep_id);
        if (result==null)
        {
            org.omg.CORBA.Contained decl = openccm_repository_.lookup_mapped_id(rep_id);
            if (decl!=null)
            {
                // load the contained.
                ScopeImpl si = (ScopeImpl)scope;
                result = si.loadContainedAsMapping(decl);
            }
        }

        // get back to an IDL3 repository.
        useIDL3Repository();

        return result;
    }

    /**
     * To obtain the Declaration object with it's absolute name.
     *
     * @param abs_name The absolute name of the searched Declaration object.
     *
     * @return The Declaration object or null if the absolute
     *         name was not found.
     */
    public Declaration
    lookup(String scoped_name)
    {
        if (scoped_name.startsWith("::"))
            return super.lookup(scoped_name.substring(2));
        else
            return super.lookup(scoped_name);
    }

    /**
     * TODO: Should be commented!
     */
    public Declaration
    lookupMappedId(String rep_id)
    {
        Object obj = mapped_declarations_.get(rep_id);
        if (obj!=null)
            return (Declaration)obj;

        // if not found then load it from the IR3
        org.omg.CORBA.Contained decl = openccm_repository_.lookup_mapped_id(rep_id);

        // if not found, it's a "normal" id
        if (decl==null)
            return lookupId(rep_id);

        // else load it as a mapped id
        org.omg.CORBA.Contained decl2 = null;
        String abs_name = decl.absolute_name();
        String name = "";
        if (abs_name.startsWith("::"))
            abs_name = abs_name.substring(2, abs_name.length());

        int idx = 0;
        ScopeImpl scope = null;
        obj = this;
        while (true)
        {
            scope = (ScopeImpl)obj;
            idx = abs_name.indexOf(':');
            if (idx==-1)
                return scope.loadContainedAsMapping(decl);
            name = abs_name.substring(0, idx);
            abs_name = abs_name.substring(idx+2, abs_name.length());
            obj = scope.findInScope(name);
            if (obj==null)
            {
                decl2 = scope.getContainer().lookup(name);
                obj = scope.loadContainedAsMapping(decl2);
            }
        }
    }

    /**
     * TODO: Should be commented!
     */
    public Declaration
    lookupId(String rep_id)
    {
        Object obj = declarations_.get(rep_id);

        if (obj!=null)
            return (Declaration)obj;

        // if not found then load it from the IR3
        org.omg.CORBA.Contained decl = repository_.lookup_id(rep_id);
        org.omg.CORBA.Contained decl2 = null;
        String abs_name = decl.absolute_name();
        String name = "";
        if (abs_name.startsWith("::"))
            abs_name = abs_name.substring(2, abs_name.length());

        int idx = 0;
        ScopeImpl scope = null;
        obj = this;
        while (true)
        {
            scope = (ScopeImpl)obj;
            idx = abs_name.indexOf(':');
            if (idx==-1)
                return scope.loadContained(decl);
            name = abs_name.substring(0, idx);
            abs_name = abs_name.substring(idx+2, abs_name.length());
            obj = scope.findInScope(name);
            if (obj==null)
            {
                decl2 = scope.getContainer().lookup(name);
                obj = scope.loadContained(decl2);
            }
        }
    }

    /**
     * TODO: Should be commented!
     */
    public TypeRef
    getAsMappedTypeRef(org.omg.CORBA.IDLType type)
    {
        return getAsTypeRef(type, true);
    }

    /**
     * TODO: Should be commented!
     */
    public TypeRef
    getAsTypeRef(org.omg.CORBA.IDLType type)
    {
        return getAsTypeRef(type, false);
    }

    // ==================================================================
    //
    // Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return null because a Repository does not support the
     *         Contained interface.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_repository;
    }

    // ==================================================================
    //
    // Methods for the inherited ScopeImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Repository.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
       return repository_;
    }

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the module declaration.
     */
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
        return openccm_repository_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.AST
    //
    // ==================================================================

    /**
     * Import an IDL Declaration by its full name
     * from the Interface Repository.
     *
     * @param sc The Scope in which the IDL Declaration
     *           will be imported.
     * @param name The name of the IDL Declaration.
     *
     * @return The newly created Declaration associated
     *         with the IDL Declaration.
     */
    public Declaration
    importByName(Scope sc, String name)
    {
        // Check if the declaration is present in the syntax tree
        Declaration d = ((ScopeImpl)sc).findInScope(name);
        if (d != null)
            return d;

        // Else retrieve the declaration from the repository
        org.omg.CORBA.Contained decl = repository_.lookup(name);
        if (decl==null)
            return null;

        return loadContained(sc, decl);
    }

    /**
     * Import an IDL Declaration by its repository id
     * from the Interface Repository.
     *
     * @param sc The Scope in which the IDL Declaration
     *           will be imported.
     * @param id The RepositoryId of the IDL Declaration.
     *
     * @return The newly created Declaration associated
     *         with the IDL Declaration.
     */
    public Declaration
    importById(Scope sc, String id)
    {
        // Check if the declaration is present in the syntax tree
        Declaration d = (Declaration)declarations_.get(id);
        if (d != null)
            return d;

        // Else retrieve the declaration from the repository
        org.omg.CORBA.Contained decl = repository_.lookup_id(id);
        if (decl==null)
            return null;

        return loadContained(sc, decl);
    }

    /**
     * Create a new any value.
     *
     * @return The new AnyValue.
     */
    public AnyValue
    createAnyValue()
    {
        return new AnyValueImpl();
    }


    /**
     * Obtain the null type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getNullType()
    {
        if (null_ == null)
        {
            null_ = createTypeRefImpl(PrimitiveKind.pk_null,
                                      TypeKind.tk_null);
        }
        return null_;
    }

    /**
     * Obtain the void type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getVoidType()
    {
        if (void_ == null)
        {
            void_ = createTypeRefImpl(PrimitiveKind.pk_void,
                                      TypeKind.tk_void);
        }
        return void_;
    }

    /**
     * Obtain the short type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getShortType()
    {
        if (short_ == null)
        {
            short_ = createTypeRefImpl(PrimitiveKind.pk_short,
                                       TypeKind.tk_short);
        }
        return short_;
    }

    /**
     * Obtain the long type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getLongType()
    {
        if (long_ == null)
        {
            long_ = createTypeRefImpl(PrimitiveKind.pk_long,
                                      TypeKind.tk_long);
        }
        return long_;
    }

    /**
     * Obtain the unsigned short type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getUShortType()
    {
        if (unsigned_short_ == null)
        {
            unsigned_short_ = createTypeRefImpl(PrimitiveKind.pk_ushort, 
                                                TypeKind.tk_ushort);
        }
        return unsigned_short_;
    }

    /**
     * Obtain the unsigned long type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getULongType()
    {
        if (unsigned_long_ == null)
        {
            unsigned_long_ = createTypeRefImpl(PrimitiveKind.pk_ulong, 
                                               TypeKind.tk_ulong);
        }
        return unsigned_long_;
    }

    /**
     * Obtain the float type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getFloatType()
    {
        if (float_ == null)
        {
            float_ = createTypeRefImpl(PrimitiveKind.pk_float,
                                       TypeKind.tk_float);
        }
        return float_;
    }

    /**
     * Obtain the double type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getDoubleType()
    {
        if (double_ == null)
        {
            double_ = createTypeRefImpl(PrimitiveKind.pk_double,
                                        TypeKind.tk_double);
        }
        return double_;
    }

    /**
     * Obtain the boolean type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getBooleanType()
    {
        if (boolean_ == null)
        {
            boolean_ = createTypeRefImpl(PrimitiveKind.pk_boolean,
                                         TypeKind.tk_boolean);
        }
        return boolean_;
    }

    /**
     * Obtain the char type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getCharType()
    {
        if (char_ == null)
        {
            char_ = createTypeRefImpl(PrimitiveKind.pk_char,
                                      TypeKind.tk_char);
        }
        return char_;
    }

    /**
     * Obtain the octet type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getOctetType()
    {
        if (octet_ == null)
        {
            octet_ = createTypeRefImpl(PrimitiveKind.pk_octet,
                                       TypeKind.tk_octet);
        }
        return octet_;
    }

    /**
     * Obtain the any type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getAnyType()
    {
        if (any_ == null)
        {
            any_ = createTypeRefImpl(PrimitiveKind.pk_any,
                                     TypeKind.tk_any);
        }
        return any_;
    }

    /**
     * Obtain the TypeCode type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getTypeCodeType()
    {
        if (TypeCode_ == null)
        {
            TypeCode_ = createTypeRefImpl(PrimitiveKind.pk_TypeCode,
                                          TypeKind.tk_TypeCode);
        }
        return TypeCode_;
    }

    /**
     * Obtain the Principal type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getPrincipalType()
    {
        if (Principal_ == null)
        {
            Principal_ = createTypeRefImpl(PrimitiveKind.pk_Principal,
                                           TypeKind.tk_Principal);
        }
        return Principal_;
    }

    /**
     * Obtain the string type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getStringType()
    {
        if (string_ == null)
        {
	     string_ = new TypeRefWithLengthImpl(
                               repository_.get_primitive(PrimitiveKind.pk_string),
                                TypeKind.tk_string, 0);
        }
        return string_;
    }

    /**
     * Obtain the Object type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getObjectType()
    {
        if (object_ == null)
        {
            object_ = createTypeRefImpl(PrimitiveKind.pk_objref,
                                        TypeKind.tk_objref);
        }
        return object_;
    }

    /**
     * Obtain the long long type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getLongLongType()
    {
        if (long_long_ == null)
        {
            long_long_ = createTypeRefImpl(PrimitiveKind.pk_longlong,
                                           TypeKind.tk_longlong);
        }
        return long_long_;
    }

    /**
     * Obtain the unsigned long long type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getULongLongType()
    {
        if (unsigned_long_long_ == null)
        {
            unsigned_long_long_ = createTypeRefImpl(PrimitiveKind.pk_ulonglong,
                                                    TypeKind.tk_ulonglong);
        }
        return unsigned_long_long_;
    }

    /**
     * Obtain the long double type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getLongDoubleType()
    {
        if (long_double_ == null)
        {
            long_double_ = createTypeRefImpl(PrimitiveKind.pk_longdouble,
                                             TypeKind.tk_longdouble);
        }
        return long_double_;
    }

    /**
     * Obtain the wchar type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getWCharType()
    {
        if (wchar_ == null)
        {
            wchar_ = createTypeRefImpl(PrimitiveKind.pk_wchar,
                                       TypeKind.tk_wchar);
        }
        return wchar_;
    }

    /**
     * Obtain the wstring type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getWStringType()
    {
        if (wstring_ == null)
        {
	     wstring_ = new TypeRefWithLengthImpl(
                                repository_.get_primitive(PrimitiveKind.pk_wstring),
                                TypeKind.tk_wstring, 0);
        }
        return wstring_;
    }

    /**
     * Obtain the value base type reference.
     *
     * @return The associated TypeRef.
     */
    public TypeRef
    getValueBaseType()
    {
        if (value_base_ == null)
        {
            value_base_ = createTypeRefImpl(PrimitiveKind.pk_value_base,
                                           TypeKind.tk_value);
        }
        return value_base_;
    }

    /**
     * Create an anonymous string<bound> type reference.
     *
     * @param bound The bound of the string.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createStringType(int bound)
    {
        // Create a StringDef into the Interface Repository.
        org.omg.CORBA.StringDef stringDef = repository_.create_string(bound);
        return new TypeRefWithLengthImpl(stringDef, TypeKind.tk_string, bound);
    }

    /**
     * Create an anonymous wstring<bound> type reference.
     *
     * @param bound The bound of the wstring.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createWStringType(int bound)
    {
        // Create a WstringDef into the Interface Repository.
        org.omg.CORBA.WstringDef wstringDef = repository_.create_wstring(bound);
        return new TypeRefWithLengthImpl(wstringDef, TypeKind.tk_wstring, bound);
    }

    /**
     * Create an anonymous sequence<bound,element_type> type reference.
     *
     * @param bound The bound of the sequence.
     * @param element_type The TypeRef of the elements.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createSequenceType(int bound,
                       TypeRef element_type)
    {
        if(element_type == null)
            return null;

        // Create a SequenceDef into the Interface Repository.
        org.omg.CORBA.SequenceDef sequenceDef =
            repository_.create_sequence(bound,
                                        ((IDLTypeWrapper)element_type).getIDLType());
        return new TypeRefWithContentImpl(sequenceDef, TypeKind.tk_sequence,
                                          bound, element_type);
    }

    /**
     * Create an anonymous array, i.e. element_type[length], type reference.
     *
     * @param length The length of the array.
     * @param element_type The TypeRef of the elements.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createArrayType(int length,
                    TypeRef element_type)
    {
        if(element_type == null)
            return null;

        // Create an ArrayDef into the Interface Repository.
        org.omg.CORBA.ArrayDef arrayDef =
             repository_.create_array(length,
                                        ((IDLTypeWrapper)element_type).getIDLType());
        return new TypeRefWithContentImpl(arrayDef, TypeKind.tk_array,
                                          length, element_type);
    }

    /**
     * Create a fixed, i.e. fixed<digits,scale>, type reference.
     *
     * @param digits The number of decimal digits.
     * @param scale The position of the decimal point.
     *
     * @return The new created TypeRef.
     */
    public TypeRef
    createFixedType(short digits,
                    short scale)
    {
        // Create a FixedDef into the Interface Repository.
        org.omg.CORBA.FixedDef fixedDef =
            repository_.create_fixed (digits, scale);
        return new FixedTypeRefImpl(fixedDef, digits, scale);
    }

    /**
     * Create a PSDL type reference for an idl type.
     *
     * @param idl_type - The IDL type that should be referenced.
     *
     * @return The new created PsdlTypeRef.
     */
    public PsdlTypeRef
    createPsdlType(TypeRef idl_type)
    {
        return new PsdlTypeRefImpl(idl_type);
    }

    /**
     * Create a PSDL abstract storagetype type reference.
     *
     * @param isRef - True if it is a reference, else false.
     * i.e. ["strong"] "ref" "<" <abstract_storagetype_name> ">".
     * @param isStrong - True if it is a strong reference, else false.
     * @param ast - The associated abstract storagetype.
     *
     * @return The new created PsdlTypeRef.
     */
    public PsdlTypeRef
    createAbstractStorageTypePsdlType( boolean isStrong,
                                       boolean isRef,
        org.objectweb.corba.ast.api.AbstractStorageTypeDecl ast)
    {
        return new PsdlTypeRefImpl(ast, isRef, isStrong);
    }

    /**
     * Create a PSDL storagetype type reference,
     *
     * @param isRef - True if it is a reference, else false.
     * i.e. "ref" "<" <storagetype_name> ">".
     * @param st - The associated storagetype.
     *
     * @return The new created PsdlTypeRef.
     */
    public PsdlTypeRef
    createStorageTypePsdlType( boolean isRef,
                               org.objectweb.corba.ast.api.StorageTypeDecl st)
    {
        return new PsdlTypeRefImpl(st, isRef, false);
    }
}
