/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/** To access all the AST API. */
import org.objectweb.corba.ast.api.*;

/**
 * ScopeImpl is a wrapper class for all IDL definition scopes.
 *
 * Inherits from:
 * - Declaration as IDL scopes are also IDL declarations.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

abstract public class ScopeImpl
                extends DeclarationImpl
                implements org.objectweb.corba.ast.api.Scope
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * To store Declaration names.
     * It's used to keep the order of the declarations.
     */
    protected java.util.Vector contained_decls_;

    /** To know if the contained objects have been loaded. */
    protected boolean content_loaded_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    ScopeImpl(Repository rep,
              ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        contained_decls_ = new java.util.Vector();
        content_loaded_ = false;
    }

    // ==================================================================
    //
    //  Internal methods.
    //
    // ==================================================================

    /**
     * Add a contained declaration.
     *
     * @param decl The declaration.
     */
    protected void
    addDecl(Declaration decl)
    {
        // Test if the declaration is present
        Declaration current = null;
        String name = decl.getName();
        for (int i=0;i<contained_decls_.size();i++)
        {
            current = (Declaration)contained_decls_.get(i);
            if (current.getName().equalsIgnoreCase(name))
                return; // We don't have to add this declaration
        }
        // Adding the declaration
        contained_decls_.add(decl);
    }

    /**
     * To know if a declaration is a contained declaration.
     *
     * @param decl The declaration.
     *
     * @return True if the declaration is contained, false otherwise.
     */
    protected boolean
    containsDecl(Declaration decl)
    {
        return ( decl.getAbsoluteName().startsWith(getAbsoluteName()+"::") );
    }

    /**
     * Remove a contained declaration.
     */
    protected void
    removeDecl(String name)
    {
        Declaration decl = null;
        for (int i=0;i<contained_decls_.size();i++)
        {
            decl = (Declaration)contained_decls_.get(i);
            if (decl.getName().equalsIgnoreCase(name))
                contained_decls_.remove(i);
        }
    }

    /**
     * Create a declaration.
     */
    protected Declaration
    createDeclaration(int kind,
                      java.lang.String name)
    {
        Declaration decl = null;
        switch (kind)
        {
            // IDL2 objects
            case org.omg.CORBA.DefinitionKind._dk_Interface :
                decl = startInterface(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_LocalInterface :
                decl = startLocalInterface(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_AbstractInterface :
                decl = startAbstractInterface(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Module :
                decl = startModule(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Struct :
                decl = startStruct(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Value :
                decl = startValue(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Union :
                decl = startUnion(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Exception :
                decl = startException(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Constant :
                decl = startConstant(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Enum :
                decl = startEnum(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Alias :
                decl = startAlias(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Attribute :
                decl = startAttribute(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Operation :
                decl = startOperation(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_ValueBox :
                decl = startValueBox(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Native :
                decl = startNative(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_ValueMember :
                decl = startValueMember(name);
                break;

            // IDL3 objects
            case org.omg.CORBA.DefinitionKind._dk_Component :
                decl = startComponent(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Home :
                decl = startHome(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Factory :
                decl = startFactory(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Finder :
                decl = startFinder(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Emits :
                decl = startEmits(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Publishes :
                decl = startPublishes(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Consumes :
                decl = startConsumes(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Provides :
                decl = startProvides(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Uses :
                decl = startUses(name);
                break;
            case org.omg.CORBA.DefinitionKind._dk_Event :
                decl = startEvent(name);
                break;
            default :
                // should not happen
                throw new ASTError("");
        }

        return decl;
    }

    /**
     * To load an Contained object.
     *
     * @param contained The Contained object.
     *
     * @return The newly created Declaration object.
     */
    protected Declaration
    loadContained(org.omg.CORBA.Contained contained)
    {
        DeclarationImpl decl = (DeclarationImpl)
            createDeclaration(contained.def_kind().value(), contained.name());
        decl.load(contained);
        decl.setImported();
        return decl;
    }

    /**
     * To load an Contained object.
     *
     * @param contained The Contained object.
     *
     * @return The newly created Declaration object.
     */
    protected Declaration
    loadContainedAsMapping(org.omg.CORBA.Contained contained)
    {
        DeclarationImpl decl = (DeclarationImpl)
            createDeclaration(contained.def_kind().value(), contained.name());
        decl.loadAsMapping(contained);
        return decl;
    }

    /**
     * Start a struct declaration.
     *
     * @param name The associated name.
     *
     * @return The new created StructDecl.
     */
    protected StructDecl
    startStruct(String name)
    {
        StructDecl decl = new StructDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start an union declaration.
     *
     * @param name The associated name.
     *
     * @return The new created UnionDecl.
     */
    protected UnionDecl
    startUnion(String name)
    {
        UnionDecl decl = new UnionDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start an interface declaration.
     *
     * @param name The associated name.
     *
     * @return The new created InterfaceDecl.
     */
    protected InterfaceDecl
    startInterface(String name)
    {
        InterfaceDecl decl = new InterfaceDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start an local interface declaration.
     *
     * @param name The associated name.
     *
     * @return The new created LocalInterfaceDecl.
     */
    protected LocalInterfaceDecl
    startLocalInterface(String name)
    {
        LocalInterfaceDecl decl = new LocalInterfaceDeclImpl(
                                          getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start an abstract interface declaration.
     *
     * @param name The associated name.
     *
     * @return The new created AbstractInterfaceDecl.
     */
    protected AbstractInterfaceDecl
    startAbstractInterface(String name)
    {
        AbstractInterfaceDecl decl = new AbstractInterfaceDeclImpl(
                                             getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start an value declaration.
     *
     * @param name The associated name.
     *
     * @return The new created ValueDecl.
     */
    protected ValueDecl
    startValue(String name)
    {
        ValueDecl decl = new ValueDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a module declaration.
     *
     * @param name The associated name.
     *
     * @return The new created ModuleDecl.
     */
    protected ModuleDecl
    startModule(String name)
    {
        ModuleDecl module = new ModuleDeclImpl(getRepository(), this);
        module.setName(name);
        return module;
    }

    /**
     * Start an component declaration.
     *
     * @param name The associated name.
     *
     * @return The new created ComponentDecl.
     */
    protected ComponentDecl
    startComponent(String name)
    {
        ComponentDecl decl = new ComponentDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start an event declaration.
     *
     * @param name The associated name.
     *
     * @return The new created EventDecl.
     */
    protected EventDecl
    startEvent(String name)
    {
        EventDecl decl = new EventDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Find a declaration.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    protected Declaration
    findInScope(String name)
    {
        Declaration decl = null;
        for (int i=0;i<contained_decls_.size();i++)
        {
            decl = (Declaration)contained_decls_.get(i);
            if (decl.getName().equalsIgnoreCase(name))
                return decl;
        }
        return null;
    }

    /**
     * Find a declaration in the IR3.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    protected Declaration
    findInIR3(String name)
    {
        org.omg.CORBA.Container container = getContainer();
        if (container == null)
            return null;

        org.omg.CORBA.Contained contained = container.lookup(name);
        if (contained == null)
            return null;

        return loadContained(contained);
    }

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the scope declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        // TODO: Why this method?
        return null;
    }

    /**
     * To avoid casts.
     */
    protected org.omg.CORBA.ComponentIR.ComponentDef
    getComponentDef()
    {
        // TODO: Why this method?
        return null;
    }

    /**
     * To avoid casts
     */
    protected org.omg.CORBA.ExtValueDef
    getExtValueDef()
    {
        // TODO: Why this method?
        return null;
    }

    /**
     * To avoid casts
     */
    protected org.omg.CORBA.ComponentIR.HomeDef
    getHomeDef()
    {
        // TODO: Why this method?
        return null;
    }

    /**
     * To avoid casts
     *
     * @return The Container object associated with the scope declaration.
     */
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
        // TODO: Why this method?
        return null;
    }

    /**
     * Create an attribute definition.
     *
     * @param attribute The attribute declaration.
     * @param type The IDLType of the attribute.
     * @param mode The mode of the attribute ie normal or readonly
     * @param get_exceptions An array containing the exceptions
     *                       that accessor operations can raise.
     * @param set_exceptions An array containing the exceptions
     *                       that mutator operations can raise.
     *
     * @return The new created AttributeDef.
     */
    protected org.omg.CORBA.ExtAttributeDef
    createExtAttribute(AttributeDecl attribute,
                       org.omg.CORBA.IDLType type,
                       org.omg.CORBA.AttributeMode mode,
                       org.omg.CORBA.ExceptionDef[] get_exceptions,
                       org.omg.CORBA.ExceptionDef[] set_exceptions)
    {
        // TODO: Why this method?
        return null;
    }

    /**
     * Create an operation.
     *
     * @param operation The operation declaration.
     * @param type The IDLType of the object that the operations returns.
     * @param mode The mode of the operation ie normal or oneway.
     * @param params An array containing the parameters description.
     * @param exceptions An array containing the exceptions
     *                   that the operation can raise.
     * @param contexts An array containing the possible context values.
     *
     * @return The new created OperationDef.
     */
    protected org.omg.CORBA.OperationDef
    createOperation(OperationDecl operation,
                    org.omg.CORBA.IDLType type,
                    org.omg.CORBA.OperationMode mode,
                    org.omg.CORBA.ParameterDescription[] params,
                    org.omg.CORBA.ExceptionDef[] exceptions,
                    String[] contexts)
    {
        // TODO: Why this method?
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Destroy all the contained declarations of the scope in the IR3.
     */
    public void
    destroy()
    {
        Declaration decl = null;
        for (int i=contained_decls_.size()-1;i>=0;i--)
        {
            decl = (Declaration)contained_decls_.get(i);
            decl.destroy();
        }

        // destroy it as a declaration if there is no content.
        if (contained_decls_.size()==0)
            super.destroy();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    /**
     * Start a new IDL FileScope scope.
     *
     * @param filename The name of the new FileScope.
     *
     * @return The new created FileScope.
     */
    public FileScope
    startFileScope(String filename)
    {
        FileScope decl = new FileScopeImpl(getRepository(),
                                           this,
                                           contained_decls_);
        decl.setName(filename);
        return decl;
    }

    /**
     * Start a new IDL constant declaration.
     *
     * @param name The name of the constant declaration.
     *
     * @return The new created ConstantDecl.
     */
    public ConstantDecl
    startConstant(String name)
    {
        ConstantDecl decl = new ConstantDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Declare an IDL struct.
     * If it doesn't exist then create it.
     *
     * @param name The name of the struct declaration.
     *
     * @return The previous or new created StructDecl.
     */
    public StructDecl
    declareStruct(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof StructDecl))
            return (StructDecl)decl;

        // else create it.
        return startStruct(name);
    }

    /**
     * Declare an IDL union.
     * If it doesn't exist then create it.
     *
     * @param name The name of the union declaration.
     *
     * @return The previous or new created UnionDecl.
     */
    public UnionDecl
    declareUnion(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof UnionDecl))
            return (UnionDecl)decl;

        // else create it.
        return startUnion(name);
    }

    /**
     * Start a new IDL enum declaration.
     *
     * @param name The name of the enum declaration.
     *
     * @return The new created EnumDecl.
     */
    public EnumDecl
    startEnum(String name)
    {
        EnumDecl decl = new EnumDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL alias declaration.
     *
     * @param name The name of the alias declaration.
     *
     * @return The new created AliasDecl.
     */
    public AliasDecl
    startAlias(String name)
    {
        AliasDecl decl = new AliasDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Declare an IDL module.
     * If it doesn't exist then create it.
     *
     * @param name The name of the module declaration.
     *
     * @return The previous or new created ModuleDecl.
     */
    public ModuleDecl
    declareModule(String name)
    {
      // System.out.println("declareModule " + name + " in " + this);

        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        // if it is a module then return it
        // because this is a module reopening.
        if(decl != null && decl instanceof ModuleDecl)
        {
            return (ModuleDecl)decl;
        }

        // else create it.
        ModuleDeclImpl module = (ModuleDeclImpl)startModule(name);

        // Search its ModuleDef into the IR3.
        try
        {
            org.omg.CORBA.Contained contained = getContainer().lookup(name);
            if (contained!=null)
                module.load(contained);
        }
        catch(org.omg.CORBA.SystemException exc)
        {
            // TODO: Do nothing if narrow fails?
        }

        return module;
    }

    /**
     * Declare an IDL interface.
     * If it doesn't exist then create it.
     *
     * @param name The name of the interface declaration.
     *
     * @return The previous or new created InterfaceDecl.
     */
    public InterfaceDecl
    declareInterface(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if (decl != null)
	{
            // if it is an interface then return it.
            if (decl.getClass() == InterfaceDeclImpl.class)
                return (InterfaceDecl)decl;

            // if it is an abstract interface then raise an error.
            if (decl instanceof AbstractInterfaceDeclImpl)
                throw new ASTError("interface `" + name +
                          "' does not match forward abstract declaration");

            // if it is a local interface then raise an error.
            if (decl instanceof LocalInterfaceDeclImpl)
                throw new ASTError("interface `" + name +
                          "' does not match forward local declaration");
	}

        // else create it.
        return startInterface(name);
    }

    /**
     * Declare an IDL local interface.
     * If it doesn't exist then create it.
     *
     * @param name The name of the local interface declaration.
     *
     * @return The previous or new created LocalInterfaceDecl.
     */
    public LocalInterfaceDecl
    declareLocalInterface(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if (decl != null)
        {
            // if it is a local interface then return it.
            if (decl instanceof LocalInterfaceDeclImpl)
                return (LocalInterfaceDecl)decl;

            // if it is an abstract interface then raise an error.
            if (decl instanceof AbstractInterfaceDeclImpl)
                throw new ASTError("interface `" + name +
                          "' does not match forward abstract declaration");

            // if it is an interface then raise an error.
            if (decl instanceof InterfaceDeclImpl)
                throw new ASTError("interface `" + name +
                          "' does not match forward declaration");
        }

        // else create it.
        return startLocalInterface(name);
    }

    /**
     * Declare an IDL abstract interface.
     * If it doesn't exist then create it.
     *
     * @param name The name of the abstract interface declaration.
     *
     * @return The previous or new created AbstractInterfaceDecl.
     */
    public AbstractInterfaceDecl
    declareAbstractInterface(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if (decl != null)
        {
            // if it is an abstract interface then return it.
            if (decl instanceof AbstractInterfaceDeclImpl)
                return (AbstractInterfaceDecl)decl;

            // if it is a local interface then raise an error.
            if (decl instanceof LocalInterfaceDeclImpl)
                throw new ASTError("interface `" + name +
                          "' does not match forward local declaration");

            // if it is an interface then raise an error.
            if (decl instanceof InterfaceDeclImpl)
                throw new ASTError("interface `" + name +
                           "' does not match forward declaration");
        }

        // else create it.
        return startAbstractInterface(name);
    }

   /**
    * Declare an IDL value type.
    * If it doesn't exist then create it.
    *
    * @param name The name of the value type declaration.
    *
    * @return The previous or new created ValueDecl.
    */
    public ValueDecl
    declareValue(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof ValueDecl))
            return (ValueDecl)decl;

        // else create it.
        return startValue(name);
    }

    /**
     * Start a new IDL initializer declaration.
     *
     * @param name The name of the initializer declaration.
     *
     * @return The new created Initializer.
     */
    public Initializer
    startInitializer(String name)
    {
        Initializer init = new InitializerImpl();
        init.setName(name);
        return init;
    }

    /**
     * Start a new IDL value member declaration.
     *
     * @param name The name of the value member declaration.
     *
     * @return The new created ValueMemberDecl.
     */
    public ValueMemberDecl
    startValueMember(String name)
    {
        ValueMemberDecl decl = new ValueMemberDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL value box declaration.
     *
     * @param name The name of the value box declaration.
     *
     * @return The new created ValueBoxDecl.
     */
    public ValueBoxDecl
    startValueBox(String name)
    {
        ValueBoxDecl decl = new ValueBoxDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL exception declaration.
     *
     * @param name The name of the exception declaration.
     *
     * @return The new created ExceptionDecl.
     */
    public ExceptionDecl
    startException(String name)
    {
        ExceptionDecl decl = new ExceptionDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL native declaration.
     *
     * @param name The name of the native declaration.
     *
     * @return The new created NativeDecl.
     */
    public NativeDecl
    startNative(String name)
    {
        NativeDecl decl = new NativeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL attribute declaration.
     *
     * @param name The name of the attribute declaration.
     *
     * @return The new created AttributeDecl.
     */
    public AttributeDecl
    startAttribute(String name)
    {
        AttributeDecl decl = new AttributeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL operation declaration.
     *
     * @param name The name of the operation declaration.
     *
     * @return The new created OperationDecl.
     */
    public OperationDecl
    startOperation(String name)
    {
        OperationDecl decl = new OperationDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Declare an IDL event type.
     * If it not exists then create it.
     *
     * @param name The name of the event type declaration.
     *
     * @return The previous or new created EventDecl.
     */
    public EventDecl
    declareEvent(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof EventDecl))
            return (EventDecl)decl;

        // else create it.
        return startEvent(name);
    }

    /**
     * Declare an IDL component.
     * If it doesn't exist then create it.
     *
     * @param name The name of the component declaration.
     *
     * @return The previous or new created ComponentDecl.
     */
    public ComponentDecl
    declareComponent(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof ComponentDecl))
            return (ComponentDecl)decl;

        // else create it.
        return startComponent(name);
    }

    /**
     * Start a new IDL provides declaration.
     *
     * @param name The name of the provides declaration.
     *
     * @return The new created ProvidesDecl.
     */
    public ProvidesDecl
    startProvides(String name)
    {
        ProvidesDecl decl = new ProvidesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL uses declaration.
     *
     * @param name The name of the uses declaration.
     *
     * @return The new created UsesDecl.
     */
    public UsesDecl
    startUses(String name)
    {
        UsesDecl decl = new UsesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL consumes declaration.
     *
     * @param name The name of the consumes declaration.
     *
     * @return The new created ConsumesDecl.
     */
    public ConsumesDecl
    startConsumes(String name)
    {
        ConsumesDecl decl = new ConsumesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL emits declaration.
     *
     * @param name The name of the emits declaration.
     *
     * @return The new created EmitsDecl.
     */
    public EmitsDecl
    startEmits(String name)
    {
        EmitsDecl decl = new EmitsDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL publishes declaration.
     *
     * @param name The name of the publishes declaration.
     *
     * @return The new created PublishesDecl.
     */
    public PublishesDecl
    startPublishes(String name)
    {
        PublishesDecl decl = new PublishesDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL home declaration.
     *
     * @param name The name of the home declaration.
     *
     * @return The new created HomeDecl.
     */
    public HomeDecl
    startHome(String name)
    {
        HomeDecl decl = new HomeDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL factory declaration.
     *
     * @param name The name of the factory declaration.
     *
     * @return The new created FactoryDecl.
     */
    public FactoryDecl
    startFactory(String name)
    {
        FactoryDecl decl = new FactoryDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new IDL finder declaration.
     *
     * @param name The name of the finder declaration.
     *
     * @return The new created FinderDecl.
     */
    public FinderDecl
    startFinder(String name)
    {
        FinderDecl decl = new FinderDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Declare a PSDL module.
     * If it doesn't exist then create it.
     *
     * @param name The name of the module declaration.
     *
     * @return The previous or new created PsdlModuleDecl.
     */
    public PsdlModuleDecl
    declarePsdlModule(String name)
    {
        // TODO: Should ModuleDecl and PsdlModuleDecl implemented by the same class or not?
        return (PsdlModuleDecl)declareModule(name);
    }

    /**
     * Declare a PSDL abstract storagehome declaration.
     * If it doesn't exist then create it.
     *
     * @param name The name of the abstract storagehome declaration.
     *
     * @return The previous or new created AbstractStorageHomeDecl.
     */
    public AbstractStorageHomeDecl
    declareAbstractStorageHome(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof AbstractStorageHomeDecl))
            return (AbstractStorageHomeDecl)decl;

        // else create it.
        AbstractStorageHomeDecl ash = new AbstractStorageHomeDeclImpl(getRepository(), this);
        ash.setName(name);
        return ash;
    }

    /**
     * Start a new PSDL storagehome declaration.
     *
     * @param name The name of the  storagehome declaration.
     *
     * @return The new created StorageHomeDecl.
     */
    public StorageHomeDecl
    startStorageHome(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof StorageHomeDecl))
            return (StorageHomeDecl)decl;

        StorageHomeDecl sh = new StorageHomeDeclImpl(getRepository(), this);
        sh.setName(name);
        return sh;
    }

    /**
     * Declare a PSDL abstract storagetype declaration.
     * If it doesn't exist then create it.
     *
     * @param name The name of the abstract storagetype declaration.
     *
     * @return The previous or new created AbstractStorageTypeDecl.
     */
    public AbstractStorageTypeDecl
    declareAbstractStorageType(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof AbstractStorageTypeDecl))
            return (AbstractStorageTypeDecl)decl;

        // else create it.
        AbstractStorageTypeDecl ast = new AbstractStorageTypeDeclImpl(getRepository(), this);
        ast.setName(name);
        return ast;
    }

    /**
     * Declare a PSDL storagetype declaration.
     * If it doesn't exist then create it.
     *
     * @param name The name of the storagetype declaration.
     *
     * @return The previous or new created StorageTypeDecl.
     */
    public StorageTypeDecl
    declareStorageType(String name)
    {
        // Find a declaration with the same name.
        Declaration decl = findInScope(name);

        if ((decl != null) && (decl instanceof StorageTypeDecl))
            return (StorageTypeDecl)decl;

        // else create it.
        StorageTypeDecl st = new StorageTypeDeclImpl(getRepository(), this);
        st.setName(name);
        return st;
    }

    /**
     * Start a new PSDL storagetype state member declaration.
     *
     * @param name The name of the storagetype state member declaration.
     *
     * @return The new created StorageTypeStateMemberDecl.
     */
    public StorageTypeStateMemberDecl
    startStorageTypeStateMember(String name)
    {
        StorageTypeStateMemberDecl decl = new StorageTypeStateMemberDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a new PSDL storagetype store directive declaration.
     *
     * @param name The name of the storagetype state member concerned by the store directive.
     *
     * @return The new created StorageTypeStoreDirectiveDecl.
     */
    public StorageTypeStoreDirectiveDecl
    startStorageTypeStoreDirective(String name)
    {
        StorageTypeStoreDirectiveDecl decl = new StorageTypeStoreDirectiveDeclImpl(getRepository(), this);
        decl.setName(name+"_store_directive");
        return decl;
    }

    /**
     * Start a new PSDL operation declaration.
     *
     * @param name The name of the operation declaration.
     *
     * @return The new created PsdlOperationDecl.
     */
    public PsdlOperationDecl
    startPsdlOperation(String name)
    {
        PsdlOperationDecl decl = new PsdlOperationDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Declare a CIDL module.
     * If it doesn't exist then create it.
     *
     * @param name The name of the module declaration.
     *
     * @return The previous or new created CidlModuleDecl.
     */
    public CidlModuleDecl
    declareCidlModule(String name)
    {
        // TODO: Should ModuleDecl and CidlModuleDecl implemented by the same class or not?
        return (CidlModuleDecl)declareModule(name);
    }

    /**
     * Start a new CIDL composition declaration.
     *
     * @param name The name of the CIDL composition declaration.
     *
     * @return The new created CompositionDecl.
     */
    public CompositionDecl
    startComposition(String name)
    {
        CompositionDecl decl = new CompositionDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Find an IDL declaration.
     *
     * Note that the declaration is also recursively
     * searched in the parent scope and finally searched
     * in the Interface Repository.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    public Declaration
    find(String name)
    {
      // System.out.println("ScopeImpl::find " + name + " in " + this + ' ' +this.getAbsoluteName());

        Declaration decl = findInScope(name);
        // If found then returns it.
        if (decl != null) return decl;

        // Finds in the parent scope.
        if (the_parent_ != null)
           decl = the_parent_.find(name);

        // If found then returns it.
        if (decl != null) return decl;

        // System.out.println("ScopeImpl::findInIR3 " + name + " in " + this + ' ' +this.getAbsoluteName());
        return findInIR3(name);
    }

    /**
     * To obtain all the contained Declaration objects.
     *
     * @param exclude_inherited If false return also objects
     *                          contained in inherited scopes.
     * @param limited_types A logical combination of DeclarationKind.
     *
     * @return A sequence of Declaration objects.
     */
    public Declaration[]
    getContents(boolean exclude_inherited,
                long limited_types)
    {
        // System.err.println("Content loaded for "+getAbsoluteName()+" ??? : "+content_loaded_);
        if (!content_loaded_)
        {
            // if it's a mapping scope, then use the OpenCCM repository
            // as an IDL2 repository to retrieve contained objects.
            if (is_mapping_)
                getRepository().useIDL2Repository();

            org.omg.CORBA.Container container = getContainer();
            if(container == null)
                return null;
            org.omg.CORBA.Contained[] content = container.contents(org.omg.CORBA.DefinitionKind.dk_all, true);

            // we need to reorder the contained declarations !!!
            // because the could have been loaded one by one without any
            // garanties on the order.
            java.util.Vector ordered_decls = new java.util.Vector();
            Declaration decl = null;

            for (int i=0; i<content.length; i++)
            {
                // System.err.println("Content["+i+"] = "+content[i].name());
                decl = findInScope(content[i].name());
                if (decl==null)
                {
                    if (is_mapping_)
                        decl = loadContainedAsMapping(content[i]);
                    else
                        decl = loadContained(content[i]);
                    // System.err.println("### Loading decl : "+decl.getAbsoluteName());
                }
                // Fixed a bug on constant
                try{
                    // Test if decl is a constant
                    org.objectweb.corba.ast.api.ConstantDecl c = (org.objectweb.corba.ast.api.ConstantDecl)decl;
                    // Load the constant
                    // System.err.println("### Loading decl : "+decl.getAbsoluteName());
                    decl = loadContained(content[i]);
                }catch(ClassCastException ex){
                    // nothing to do
                }
                 // System.err.println(" Decl found : "+decl.getAbsoluteName());
                 ordered_decls.add(decl);
            }

            // Add declarations that are not in the repository (psdl, cidl)
            for(int i=0; i<contained_decls_.size(); i++)
            {
                decl = (Declaration)contained_decls_.get(i);
                if ( (decl.getCategory() == org.objectweb.corba.ast.api.DeclarationCategory.dc_psdl)
                     || (decl.getCategory() == org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl) )
                    ordered_decls.add(decl);
            }

            contained_decls_ = ordered_decls;

            // get back to an IDL3 repository.
            if (is_mapping_)
                getRepository().useIDL3Repository();

            content_loaded_ = true;
        }

        Declaration decl = null;
        java.util.Vector selection = new java.util.Vector();
        for (int i=0; i<contained_decls_.size(); i++)
        {
            decl = (Declaration)contained_decls_.get(i);
            // System.err.println("@@@@@@@ Contained_decls : "+ decl.getAbsoluteName());
            if ((decl.getDeclKind()&limited_types)!=0)
                selection.add(decl);
        }
        //  System.err.println("fin content loaded");
        return (Declaration[])selection.toArray(new Declaration[0]);
    }

    /**
     * To find a contained Declaration according to it's scoped name.
     *
     * @param scoped_name The scoped name.
     *
     * @return The found Declaration (or loaded from the Interface
     *         Repository) or null.
     */
    public Declaration
    lookup(String scoped_name)
    {
        if (scoped_name.startsWith("::")) return the_repository_.lookup(scoped_name);

        int idx = scoped_name.indexOf("::");
        String s1 = null;
        String s2 = null;
        if (idx==-1)
        {
            s1 = scoped_name;
            s2 = "";
        }
        else
        {
            s1 = scoped_name.substring(0,idx);
            s2 = scoped_name.substring(idx+2);
        }

        Declaration decl = findInScope(s1);
        if (decl==null)
        {
	    // try to load it from the IR3
            decl = findInIR3(s1);
        }

        if (s2.equals(""))
            return decl;

        if (decl instanceof Scope)
            return ((Scope)decl).lookup(s2);

        return null;
    }

    /**
     * Print the Declaration.
     */
    public void
    print()
    {
        System.out.println(getAbsoluteName());
        Declaration[] decls = getContents(true, DeclarationKind.dk_all);
        for(int i=0; i<decls.length; i++)
        {
            System.out.print("+--");
            decls[i].print();
        }
    }

}
