/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST AbstractStorageHomeList. */
import org.objectweb.corba.ast.api.AbstractStorageHomeList;

import org.objectweb.corba.ast.api.StorageHomeFactoryDecl;

import org.objectweb.corba.ast.api.StorageHomeKeyDecl;

/** To use AST AbstractStorageHomeDecl. */
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/**
 * StorageHomeBaseImpl is a wrapper class for
 * PSDL storagehome declarations.
 *
 *
 * Inherits from:
 *
 * - ForwardScopePsdlImpl as storagehomes are also PSDL forward scopes.
 *
 * - StorageHomeBase as OMG IDL for PSDL storagehome declarations.
 *
 *
 * Provides:
 *
 * - The list of inherited abstract storagehomes,
 *   i.e. the getAbstractStorageHomeList operation.
 *
 * - The factory for StorageHomeKeyDecl,
 *   i.e. the startStorageHomeKey operation.
 *
 * - The factory for StorageHomeFactoryDecl,
 *   i.e. the startStorageHomeFactory operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

abstract public class StorageHomeBaseImpl
                extends ForwardScopePsdlImpl
                implements org.objectweb.corba.ast.api.StorageHomeBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The list of inherited abstract storagehomes. */
    protected AbstractStorageHomeListImpl abstract_storagehomes_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    StorageHomeBaseImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the ForwardScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        abstract_storagehomes_ = new AbstractStorageHomeListImpl();
        content_loaded_ = true;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for DeclarationImpl
    //
    // ==================================================================

    /**
     * Check if the declaration is valid.
     */
    public void
    check(StorageTypeBaseImpl st)
    {
        Declaration[] decls = null;

        // Retrieve keys defined in this storage home
        decls = getKeys();
        if(decls != null){
            for (int i=0; i<decls.length; i++)
            {
//                System.out.println("found " + decls[i].getName());
                // For each key, check the validity of key state members
                ((StorageHomeKeyDeclImpl)decls[i]).check(st);
            }
//            System.out.println("finish");
        }

        // Retrieve factories defined in this storage home
        decls = getFactories();
        if(decls != null){
            for (int i=0; i<decls.length; i++)
            {
//                System.out.println("found " + decls[i].getName());
                // For each factory, check the validity of factory state members
                ((StorageHomeFactoryDeclImpl)decls[i]).check(st);
            }
//            System.out.println("finished");
        }

    }

    // ==================================================================
    //
    // Methods for ScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopePsdlImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Get keys defined in this storage home.
     *
     * @return a list of Keys.
     */
    public Declaration[]
    getKeys()
    {
       return getContents(true, DeclarationKind.dk_storage_home_key);
    }

    /**
     * Get factories defined in this storage home.
     *
     * @return a list of factories.
     */
    public Declaration[]
    getFactories()
    {
       return getContents(true, DeclarationKind.dk_storage_home_factory);
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        // Call the ForwardScopeImpl "create" method.
        super.create();

        // Check if an abstract storagehome is not specify more that once
        int index = abstract_storagehomes_.checkSameItem();
        if( index != -1 )
        {
            throw new ASTError(getName() + " implements twice " + ((AbstractStorageHomeDecl)abstract_storagehomes_.get(index)).getName() );
        }
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageHomeBase
    //
    // ==================================================================

    /**
     * Obtain the list of inherited abstract storagehomes.
     *
     * @return The list of inherited abstract storagehomes.
     */
    public AbstractStorageHomeList
    getAbstractStorageHomeList()
    {
        return abstract_storagehomes_;
    }

    /**
     * Start a storagehome key declaration.
     *
     * @param name The name of the storagehome key declaration.
     *
     * @return The created StorageHomeKeyDecl.
     */
    public StorageHomeKeyDecl
    startStorageHomeKey(String name)
    {
        StorageHomeKeyDecl decl = new StorageHomeKeyDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Start a storagehome factory declaration.
     *
     * @param name The name of the storagehome factory declaration.
     *
     * @return The created StorageHomeFactoryDecl.
     */
    public StorageHomeFactoryDecl
    startStorageHomeFactory(String name)
    {
        StorageHomeFactoryDecl decl = new StorageHomeFactoryDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }
}
