/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

import org.objectweb.corba.ast.api.StorageTypeDecl;

import org.objectweb.corba.ast.api.StorageHomeDecl;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/**
 * StorageHomeDeclImpl is a wrapper class for
 * PSDL storagehome declarations.
 *
 *
 * Inherits from:
 *
 * - StorageHomeBaseImpl as storagehomes are also IDL forward scopes.
 *
 *
 * Provides:
 *
 * - The managed storagetype,
 *   i.e. the setStorageType and getStorageType operations.
 *
 * - The inherited base storagehome,
 *   i.e. the setBaseStorageHome and getBaseStorageHome operations.
 *
 * - The primary key ref property,
 *   i.e. the setPrimaryKeyName and getPrimaryKeyName operations.
 *
 * - The primary key name,
 *   i.e. the setPrimaryKeyRef and isPrimaryKeyRef operations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class StorageHomeDeclImpl
     extends StorageHomeBaseImpl
  implements org.objectweb.corba.ast.api.StorageHomeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The managed storagetype. */
    protected StorageTypeDeclImpl managedStorageType_;

    /** The inherited base storagehome. */
    protected StorageHomeDeclImpl baseStorageHome_;

    /** The primarykey name. */
    protected String primaryKeyName_;

    /** Is a primarykey ref. */
    protected boolean isPrimaryKeyRef_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    StorageHomeDeclImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the StorageHomeBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
        managedStorageType_ = null;
        baseStorageHome_ = null;
        primaryKeyName_ = null;
        isPrimaryKeyRef_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for DeclarationImpl
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_storage_home;
    }


    /**
     * Create the declaration.
     */
    public void
    create()
    {
        // Call the StorageHomeBaseImpl "create" method.
        super.create();

        // Check the primary key
        if(primaryKeyName_ != null)
        {
            java.util.Vector keys = new java.util.Vector();

            int size = 0;
            size = abstract_storagehomes_.getSize();
            for (int i=0; i<size; i++)
            {
                AbstractStorageHomeDeclImpl ash = (AbstractStorageHomeDeclImpl)abstract_storagehomes_.get(i);
                Declaration decls[] = ash.getKeys();
                if(decls != null)
                {
                    for(int j=0; j<decls.length; j++)
                        keys.add( decls[j] );
                }
            }

            int i = 0;
            boolean found = false;

            size = keys.size();
            while( (i<size) && !found)
            {
                if( primaryKeyName_.compareTo(((Declaration)keys.get(i)).getName()) == 0 )
                    found = true;
                i++;
            }
            if(!found)
            {
                throw new ASTError(getName() +" : " + primaryKeyName_ + " is not a valid primary key.");
            }
        }

        // Check the storage type managed
        java.util.Vector ast_required = new java.util.Vector();
        java.util.Vector ast_found = null;
        AbstractStorageHomeListImpl ash_list = null;
        AbstractStorageTypeListImpl ast_list = null;
        int size = 0;

        // Retrieve abstract storage homes implemented
        ash_list = (AbstractStorageHomeListImpl)getAbstractStorageHomeList();
        size = ash_list.getSize();
        for(int i=0; i<size; i++)
        {
            AbstractStorageHomeDeclImpl ash = (AbstractStorageHomeDeclImpl)ash_list.get(i);
            AbstractStorageTypeDeclImpl ast = (AbstractStorageTypeDeclImpl)ash.getAbstractStorageType();
            ast_required.add(ast);
        }

        // Retrieve abstract storage types implemented by the managed storage type
        ast_found = managedStorageType_.getImplementedAbstractStorageTypes();

        if(!ast_found.containsAll(ast_required))
             throw new ASTError(getName() + " has not a valid storage type.");

    }

    // ==================================================================
    //
    // Methods for ScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopePsdlImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for StorageHomeBaseImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Check if the declaration is valid.
     */
    public void
    check()
    {
        // Call the StorageHomeBase "check" method
        super.check(managedStorageType_);
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageHomeBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageHomeDecl
    //
    // ==================================================================

    /**
     * Set the managed storagetype.
     *
     * @param st The managed storagetype.
     */
    public void
    setStorageType(StorageTypeDecl st)
    {
        if(st != null) {
            managedStorageType_ = (StorageTypeDeclImpl)st;
	}
    }

    /**
     * Obtain the managed storagetype.
     *
     * @return The managed storagetype.
     */
    public StorageTypeDecl
    getStorageType()
    {
        return managedStorageType_;
    }

    /**
     * Set the inherited base storagehome.
     *
     * @param sh The inherited base storagehome.
     */
    public void
    setBaseStorageHome(StorageHomeDecl sh)
    {
        if(sh != null) {
            baseStorageHome_ = (StorageHomeDeclImpl)sh;
        }
    }

    /**
     * Obtain the inherited base storagehome.
     *
     * @return The inherited base storagehome.
     */
    public StorageHomeDecl
    getBaseStorageHome()
    {
        return baseStorageHome_;
    }

    /**
     * Set the primary key name.
     *
     * @param name The primary key name.
     */
    public void
    setPrimaryKeyName(String name)
    {
        primaryKeyName_ = name;
    }

    /**
     * Obtain the primary key name.
     *
     * @return The primary key name.
     */
    public String
    getPrimaryKeyName()
    {
        return primaryKeyName_;
    }

    /**
     * Set as a primary key ref.
     *
     * @param b True if a primary key ref, or false otherwise.
     */
    public void
    setPrimaryKeyRef(boolean b)
    {
        isPrimaryKeyRef_ = b;
    }

    /**
     * Is it a primary key ref?
     *
     * @return True if a primary key ref, or false otherwise.
     */
    public boolean
    isPrimaryKeyRef()
    {
        return isPrimaryKeyRef_;
    }
}
