/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To use AST AbstractStorageTypeDecl. */
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;

/** To use AST AbstractStorageTypeList. */
import org.objectweb.corba.ast.api.AbstractStorageTypeList;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/**
 * StorageTypeBaseImpl is a wrapper class for
 * PSDL storagetype declarations.
 *
 *
 * Inherits from:
 *
 * - ForwardScopeImpl as storagetypes are also IDL forward scopes.
 *
 * - StorageTypeBase as OMG IDL for PSDL storagetype declarations.
 *
 *
 * Provides:
 *
 * - The list of inherited abstract storagetypes,
 *   i.e. the getAbstractStorageTypeList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

abstract public class StorageTypeBaseImpl
              extends ForwardScopePsdlImpl
           implements org.objectweb.corba.ast.api.StorageTypeBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The list of inherited abstract storagetypes. */
    protected AbstractStorageTypeListImpl abstract_storagetypes_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    StorageTypeBaseImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the ForwardScopePsdlImpl constructor.
        super(rep, parent);

        // Init internal state.
        abstract_storagetypes_ = new AbstractStorageTypeListImpl();
        content_loaded_ = true;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * This method check if the string passed match with a declared state member.
     *
     * @param state The name of the state member to check.
     *
     * @return True if it's a valid state member.
     */
    protected boolean
    isValidState(String state)
    {
        DeclarationImpl decl = null;

        // Find in the current storageType
        decl = (DeclarationImpl)findInScope(state);
        if(decl == null)
        {
            // Find in the implemented classes
            boolean found = false;
            int size = abstract_storagetypes_.getSize();
            if(size>0)
            {
                int i = 0;
                while( (i<size) && !found )
                {
                    found = ((AbstractStorageTypeDeclImpl)abstract_storagetypes_.get(i)).isValidState(state);
                    i++;
                }
            }
            if(!found) return false;
        }
        return true;
    }

    // ==================================================================
    //
    // Methods for DeclarationImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopePsdlImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the list of all inherited abstract storagetypes.
     *
     * @return The list of all inherited abstract storagetypes.
     */
    public java.util.Vector
    getImplementedAbstractStorageTypes()
    {
        int size = 0;
        java.util.Vector local_list = null;
        java.util.Vector list = new java.util.Vector();

        local_list = abstract_storagetypes_.toVector();
        size = local_list.size();
        for(int i=0; i<size; i++)
        {
            AbstractStorageTypeDeclImpl ast = (AbstractStorageTypeDeclImpl)local_list.get(i);
            list.add(ast);
            list.addAll( ast.getImplementedAbstractStorageTypes() );
        }

        return list;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        // Call the ForwardScopeImpl "create" method.
        super.create();

        // Check if an abstract storagetype is not specify more that once
        int index = abstract_storagetypes_.checkSameItem();
        if( index != -1 )
        {
            throw new ASTError(getName() + " implements twice " + ((AbstractStorageTypeDecl)abstract_storagetypes_.get(index)).getName() );
        }
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageTypeBase
    //
    // ==================================================================

    /**
     * Obtain the list of inherited abstract storagetypes.
     *
     * @return The list of inherited abstract storagetypes.
     */
    public AbstractStorageTypeList
    getAbstractStorageTypeList()
    {
        return abstract_storagetypes_;
    }
}
