/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

import org.objectweb.corba.ast.api.StringList;

import org.objectweb.corba.ast.api.StorageTypeDecl;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/**
 * StorageTypeDeclImpl is a wrapper class for
 * PSDL storagetype declarations.
 *
 *
 * Inherits from:
 *
 * - StorageTypeBaseImpl as storagetypes are also IDL forward scopes.
 *
 *
 * Provides:
 *
 * - The base inherited storagetype,
 *   i.e. the setBaseStorageType and getBaseStorageType operations.
 *
 * - The list of ref rep directives,
 *   i.e. the getRefRepDirectiveList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class StorageTypeDeclImpl
     extends StorageTypeBaseImpl
  implements org.objectweb.corba.ast.api.StorageTypeDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Base storage type. */
    protected StorageTypeDeclImpl baseStorageType_;

    /** Ref rep directive list. */
    protected StringListImpl refRepDirectiveList_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param parent The parent scope of the declaration.
     */
    protected
    StorageTypeDeclImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the StorageTypeBaseImpl constructor.
        super(rep, parent);

        // Init internal state.
        baseStorageType_ = null;
        refRepDirectiveList_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * This method check if the string passed match with a declared state member.
     *
     * @param state The name of the state member to check.
     *
     * @return True if it's a valid state member.
     */
    protected boolean
    isValidState(String state)
    {
        // Call the StorageTypeBase "isValidState" method
        boolean found = super.isValidState(state);

        if(baseStorageType_ == null)
            return found;
        if(!found)
            // if not found, check in the base storage type
            return baseStorageType_.isValidState(state);
        else
            return true;
    }

    // ==================================================================
    //
    // Methods for DeclarationImpl
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_storage_type;
    }

    // ==================================================================
    //
    // Methods for ScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for ForwardScopeImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for StorageTypeBaseImpl
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        // Call the ForwardScopeImpl "create" method.
        super.create();

        // Check if a member is not specify more that once
        int index = refRepDirectiveList_.checkSameItem();
        if( index != -1 )
        {
            throw new ASTError( (String)refRepDirectiveList_.get(index) + " must be specified once and once only.");
        }
    }

    /**
     * Check if the declaration is valid.
     */
    public void
    check()
    {
        /** Check if the reference representation is valid */
        int size = refRepDirectiveList_.getSize();

        // 1. baseStorageType_ must be null
        if( (size>0) && (baseStorageType_ != null) )
        {
            throw new ASTError( getName() +
               " has a based storage type. This is not consistent with a reference representation directive!" );
        }

        // 2. state members must be valid
        for(int i=0; i<size; i++)
        {
            String state = (String)refRepDirectiveList_.get(i);
            if( isValidState(state) == false)
            {
                throw new ASTError(getName() + " => " + state + " is not a valid state member." );
            }
        }

        /* A storagetype that directly implements an abstract storagetype
           that declares a state member whose type is an abstract storagetype
           or an array or a sequence of abstract storagetypes must provide
           a store directive for this state member.
        */
        org.objectweb.corba.ast.api.AbstractStorageTypeDecl[] ast_decls = null;

        // get abstract storage types implemented
        ast_decls = getAbstractStorageTypeList().getAbstractStorageTypes();
        for(int i=0; i<ast_decls.length; i++)
        {
            // System.err.println("+--> "+ast_decls[i].getName());
            // get storage type state members
            org.objectweb.corba.ast.api.Declaration[] decls = null;
            decls = ast_decls[i].getContents(true, org.objectweb.corba.ast.api.DeclarationKind.dk_storage_type_state_member);
            for(int j=0; j<decls.length; j++)
            {
                org.objectweb.corba.ast.api.StorageTypeStateMemberDecl
                            member = (org.objectweb.corba.ast.api.StorageTypeStateMemberDecl)decls[j];
                org.objectweb.corba.ast.api.StorageTypeBase type = member.getType().getStorageTypeBase();
                if( type != null)
                {
                    // State member is a storage type or an abstract storage type
                    String name = getAbsoluteName()+"::"+decls[j].getName()+"_store_directive";
                    org.objectweb.corba.ast.api.Declaration store_directive = lookup(name);
                    if (store_directive == null)
                        throw new ASTError("No store directive found for "+decls[j].getName());
                }
            }
        }

    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageTypeBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageTypeDecl
    //
    // ==================================================================

    /**
     * Set the base inherited storagetype.
     *
     * @param st The base inherited storagetype.
     **/
    public void
    setBaseStorageType(StorageTypeDecl st)
    {
        if(st != null) {
            baseStorageType_ = (StorageTypeDeclImpl)st;
	}
    }

    /**
     * Obtain the base inherited storagetype.
     *
     * @return The base inherited storagetype.
     **/
    public StorageTypeDecl
    getBaseStorageType()
    {
        return baseStorageType_;
    }

    /**
     * Obtain the list of ref rep directives.
     *
     * @return all ref rep directives.
     **/
    public StringList
    getRefRepDirectiveList()
    {
       return refRepDirectiveList_;
    }
}
