/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To use AST PsdlTypeRef */
import org.objectweb.corba.ast.api.PsdlTypeRef;

/** To use AST StorageHomeDecl */
import org.objectweb.corba.ast.api.StorageHomeDecl;

/** To use AST StorageTypeStateMemberDecl */
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;

/** To report ASTError exceptions. */
import org.objectweb.corba.ast.api.ASTError;

/**
 * StorageTypeStoreDirectiveDeclImpl is a wrapper class for
 * PSDL storagetype state member declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - StorageTypeStateMemberDecl: OMG IDL for PSDL storagetype store
 *   directive declarations.
 *
 *
 * Provides:
 *
 * - The 'Readonly' property,
 *   i.e. the setReadonly and isReadonly operations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class StorageTypeStoreDirectiveDeclImpl
     extends DeclarationImpl
  implements org.objectweb.corba.ast.api.StorageTypeStoreDirectiveDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The name of the state member concerned. */
    private String state_member_;

    /** The state member's type. */
    private PsdlTypeRef type_;

    /** The storage home where the state member must be stored. */
    private StorageHomeDeclImpl storage_home_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the value member declaration.
     **/
    protected
    StorageTypeStoreDirectiveDeclImpl(Repository rep,
                                      ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        state_member_ = null;
        type_ = null;
        storage_home_ = null;;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the value
     *         member declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        // TODO
        return null;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_psdl;
    }

    // ==================================================================
    //
    // Internal methods for DeclarationWithTypeRefImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        // TODO
        return null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_storage_type_store_directive;
    }

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        boolean found = false;

        // Call the DeclarationImpl "create" method.
        super.create();

        org.objectweb.corba.ast.api.StorageTypeDecl st = null;

        /* First check :
          <store_directive> ::= "stores" <simple_declarator> "as"
                               <psdl_concrete_state_type> [<storagehome_scope>]
           <simple_declarator> must be the name of a state member declared
           in this storagetype or in one of the abstract storagetype it
           implements.
        */
        org.objectweb.corba.ast.api.AbstractStorageTypeDecl[] ast_decls = null;

        st = (org.objectweb.corba.ast.api.StorageTypeDecl)getParent();
        // System.err.println("<= "+st.getName()+" =>");
        // Get abstract storage types implemented
        ast_decls = st.getAbstractStorageTypeList().getAbstractStorageTypes();
        int i = 0;
        while ((!found) && (i<ast_decls.length))
        {
            // System.err.println("+--> "+ast_decls[i].getName());
            Declaration[] decls = null;
            // Get state members
            decls = ast_decls[i].getContents(true, org.objectweb.corba.ast.api.DeclarationKind.dk_storage_type_state_member);
            int j=0;
            while ((!found) && (j<decls.length))
            {
                StorageTypeStateMemberDecl member = (StorageTypeStateMemberDecl)decls[j];
                org.objectweb.corba.ast.api.StorageTypeBase type = member.getType().getStorageTypeBase();
                // System.err.println("+------>"+member.getName());
                if( member.getName().equals(getStateMember()) )
                    found = true;
                j++;
            }
            i++;
        }
        if (!found)
            throw new ASTError( getStateMember() + " must be the name of a state member declared"
                                + " in this storagetype or in one of the abstract storagetype it implements.");

        /* Second check :
           if <psdl_concrete_state_type> denotes a storagetype S then the type
           of the state member must be an abstract storagetype S' or a sequence/array
           of an abstract storagetype S', S must implement directly or indirectly S'.
           Idem for references.
        */

    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StorageTypeStoreDirectiveDecl
    //
    // ==================================================================

    /**
     * Set the state member.
     *
     * @param name The state member's name.
     */
    public void
    setStateMember(String name)
    {
        state_member_ = name;
    }

    /**
     * Get the state member
     *
     * @return The state member.
     */
    public String
    getStateMember()
    {
        return state_member_;
    }

    /**
     * Set the state member's type.
     *
     * @param st The state member's type.
     */
    public void
    setType(PsdlTypeRef type)
    {
        type_ = type;
    }

    /**
     * Get the state member's type
     *
     * @return The state member's type.
     */
    public PsdlTypeRef
    getType()
    {
        return type_;
    }

    /**
     * Set the storage home.
     *
     * @param sh The storage home where the state member must be stored.
     */
    public void
    setStorageHome(StorageHomeDecl sh)
    {
        storage_home_ = (StorageHomeDeclImpl)sh;
    }

    /**
     * Get the storage home.
     *
     * @return The storage home where the state member must be stored.
     */
    public StorageHomeDecl
    getStorageHome()
    {
        return storage_home_;
    }
}
