/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST StructMember. */
import org.objectweb.corba.ast.api.StructMember;

/** To access AST StructMemberList. */
import org.objectweb.corba.ast.api.StructMemberList;

/** To use CORBA::StructDef. */
import org.omg.CORBA.StructDef;
import org.omg.CORBA.StructDefHelper;

/**
 * StructDeclImpl is a wrapper class for IDL struct declarations.
 *
 * Inherits from:
 * - ForwardScopeIDLImpl as structs are also IDL forward scopes.
 * - TypeRef as structs are IDL types,
 * - WithDeclarators to access struct members
 * - IDLTypeWrapper as structs are IDL types.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class StructDeclImpl
     extends ForwardScopeIDLImpl
  implements org.objectweb.corba.ast.api.StructDecl,
             IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 StructDef. */
    protected StructDef struct_def_;

    /** List of members. */
    protected StructMemberListImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    StructDeclImpl(Repository rep,
                   ScopeImpl parent)
    {
        // Call the ForwardScopeIDLImpl constructor.
        super(rep, parent);

        // Init internal state.
        struct_def_ = null;
        members_ = new StructMemberListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 StructDef.
     *
     * @param contained The StructDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        struct_def_ = StructDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = struct_def_.members();

        // load as scope now to prevent infinite loop in the case
        // of recursive struct.
        super.load(contained);

        for (int i=0;i<members.length;i++)
            members_.addMember(members[i].name,
                               getRepository().
                                   getAsTypeRef(members[i].type_def));
    }

    /**
     * Loads infos of the CORBA 3.0 StructDef.
     *
     * @param contained The StructDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        struct_def_ = StructDefHelper.narrow(contained);
        org.omg.CORBA.StructMember[] members = struct_def_.members();

        // load as scope now to prevent infinite loop in the case
        // of recursive struct.
        super.loadAsMapping(contained);

        for (int i=0;i<members.length;i++)
            members_.addMember(members[i].name,
                               getRepository().
                                 getAsMappedTypeRef(members[i].type_def));
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the struct declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        return struct_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the struct declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return struct_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        struct_def_ = the_parent_.getContainer().
                     create_struct(getId(), getName(), getVersion(),
                                   members_.getStructMemberSeq());
    }

    /**
     * Complete the container object.
     */
    protected void
    completeContainer()
    {
        struct_def_.members(members_.getStructMemberSeq());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];

        // members
        java.util.Vector struct_depend = new java.util.Vector();
        Declaration[] depend = null;

        StructMember[] members = members_.getStructMembers();
        for (int i=0; i<members.length; i++)
        {
            TypeRef member_type = members[i].getType();
            if (member_type.isDeclaration())
            {
                if ((!containsDecl((Declaration)member_type)) &&
                    (struct_depend.indexOf(member_type)==-1))
                    struct_depend.add(member_type);
            }

            depend = member_type.getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((struct_depend.indexOf(depend[j])==-1) &&
                    (!containsDecl(depend[j])) &&
                    (depend[j]!=this))
                    struct_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])struct_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_struct;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDeclarators
    //
    // ==================================================================

    /**
     * Find the declarator associated to an index.
     *
     * @param index The index of the declarator that should be returned.
     *
     * @return The name of the declarator at the specified index.
     */
    public String
    declaratorAt(int index)
    {
        return members_.nameAt(index);
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_struct;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.StructDecl
    //
    // ==================================================================

    /**
     * Obtain the struct member list.
     *
     * @return The members of the struct declaration.
     */
    public StructMemberList
    getMemberList()
    {
        return members_;
    }

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the struct declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return struct_def_;
    }
}
