/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST AnyValue. */
import org.objectweb.corba.ast.api.AnyValue;

/** To access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** To access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST UnionMemberList. */
import org.objectweb.corba.ast.api.UnionMemberList;

/** To use CORBA::TypeCode. */
import org.omg.CORBA.TypeCode;

/** To use CORBA::UnionDef. */
import org.omg.CORBA.UnionDef;
import org.omg.CORBA.UnionDefHelper;

/** To use CORBA::UnionMember. */
import org.omg.CORBA.UnionMember;

/**
 * UnionDeclImpl is a wrapper class for IDL union declarations.
 *
 * Inherits from:
 * - ForwardScopeIDLImpl as unions are also IDL forward scopes,
 * - TypeRef as unions are IDL types.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UnionDeclImpl
     extends ForwardScopeIDLImpl
  implements org.objectweb.corba.ast.api.UnionDecl,
             IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 UnionDef. */
    private UnionDef union_def_;

    /** The default index. */
    private int default_index_;

    /** The discriminator. */
    private TypeRef discriminator_;

    /** The discriminator. */
    private TypeCode tc_discriminator_;

    /** List of members. */
    private UnionMemberListImpl members_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    UnionDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ForwardScopeIDLImpl constructor.
        super(rep, parent);

        // Init internal state.
        union_def_ = null;
        default_index_ = -1;
        discriminator_ = null;
        tc_discriminator_ = null;
        members_ = new UnionMemberListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtain the associated CORBA::IDLType of the discriminator.
     *
     * @return The associated CORBA::IDLType of the discriminator.
     */
    protected org.omg.CORBA.IDLType
    getDiscriminatorIDLType()
    {
      return ( discriminator_ != null ) ?
             (((IDLTypeWrapper)discriminator_).getIDLType()) : null;
    }

    /**
     * Obtain the associated CORBA::TypeCode of the discriminator.
     *
     * @return The associated CORBA::TypeCode of the discriminator.
     */
    protected TypeCode
    getDiscriminatorTypeCode()
    {
        if(tc_discriminator_ == null)
	{
            org.omg.CORBA.IDLType type = getDiscriminatorIDLType();
            if(type != null)
                tc_discriminator_ = type.type();
	}
        return tc_discriminator_;
    }

    /**
     * Obtain the associated sequence of union members.
     *
     * @return The associated sequence of union members.
     */
    protected org.omg.CORBA.UnionMember[]
    getUnionMemberSeq()
    {
        TypeCode tc = getDiscriminatorTypeCode();
	return (tc != null) ? members_.getUnionMemberSeq(tc)
	                    : new org.omg.CORBA.UnionMember[0];
    }

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 UnionDef.
     *
     * @param contained The UnionDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        union_def_ = UnionDefHelper.narrow(contained);
        setDiscriminator(getRepository().
                         getAsTypeRef(union_def_.discriminator_type_def()));
        try
        {
            default_index_ = union_def_.type().default_index();
        }
        catch (org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            // Wrap the CORBA::TypeCode::BadKind exception.
            throw new org.objectweb.ccm.CORBA.UserExceptionWrapper(ex);
        }

        // load as scope now to prevent infinite loop in the case
        // of recursive union.
        super.load(contained);

        UnionMember[] members = union_def_.members();
        for (int i=0; i<members.length; i++)
        {
            AnyValueImpl any = new AnyValueImpl();
            any.loadAny(members[i].label);
            members_.addMember(members[i].name,
                      getRepository().getAsTypeRef(members[i].type_def),
                      any);
        }
    }

    /**
     * Loads infos of the CORBA 3.0 UnionDef.
     *
     * @param unionDef The UnionDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        union_def_ = UnionDefHelper.narrow(contained);
        setDiscriminator(getRepository().
              getAsMappedTypeRef(union_def_.discriminator_type_def()));
        try
        {
            default_index_ = union_def_.type().default_index();
        }
        catch (org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            // Wrap the CORBA::TypeCode::BadKind exception.
            throw new org.objectweb.ccm.CORBA.UserExceptionWrapper(ex);
        }

        // load as scope now to prevent infinite loop in the case
        // of recursive struct.
        super.loadAsMapping(contained);

        UnionMember[] members = union_def_.members();
        for (int i=0; i<members.length; i++)
        {
            AnyValueImpl any = new AnyValueImpl();
            any.loadAny(members[i].label);
            members_.addMember(members[i].name,
                      getRepository().getAsMappedTypeRef(members[i].type_def),
                      any);
        }
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the union declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
        return union_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the union declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return union_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        // Create an UnionDef into the IFR.
        union_def_ = the_parent_.getContainer().
                     create_union(getId(), getName(), getVersion(),
                                  getDiscriminatorIDLType(),
                                  getUnionMemberSeq());
    }

    /**
     * Complete the container object.
     */
    protected void
    completeContainer()
    {
        union_def_.discriminator_type_def(getDiscriminatorIDLType());
        union_def_.members(getUnionMemberSeq());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        java.util.Vector union_depend = new java.util.Vector();
        Declaration[] depend = null;

        // discriminator
        if (getDiscriminator().isDeclaration())
            union_depend.add(getDiscriminator());

        depend = getDiscriminator().getDependencies();
        for (int i=0;i<depend.length;i++)
            union_depend.add(depend[i]);

        // members
        org.objectweb.corba.ast.api.UnionMember[] members =
            members_.getUnionMembers();
        for (int i=0; i<members.length; i++)
        {
            TypeRef member_type = members[i].getType();
            if (member_type.isDeclaration())
            {
                if ((!containsDecl((Declaration)member_type)) &&
                    (union_depend.indexOf(member_type)==-1))
                    union_depend.add(member_type);
            }

            depend = member_type.getDependencies();
            for (int j=0;j<depend.length;j++)
            {
                if ((union_depend.indexOf(depend[j])==-1) &&
                    (!containsDecl(depend[j])) &&
                    (depend[j]!=this))
                    union_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])union_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_union;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_union;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.UnionDecl
    //
    // ==================================================================

    /**
     * Set the associated discriminator type.
     *
     * @param type The TypeRef of the associated discriminator.
     */
    public void
    setDiscriminator(TypeRef type)
    {
        if(type!=null)
        {
            discriminator_ = type;
        }
    }

    /**
     * Obtain the associated discriminator type.
     *
     * @return The associated discriminator type.
     */
    public TypeRef
    getDiscriminator()
    {
        return discriminator_;
    }

    /**
     * Obtain the union member list.
     *
     * @return The union member list.
     */
    public UnionMemberList
    getMemberList()
    {
        return members_;
    }

    /**
     * Obtain the default index.
     *
     * @return The default index.
     */
    public int
    getDefaultIndex()
    {
        return default_index_;
    }

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the union declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return union_def_;
    }
}
