/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** Used to access AST ComponentDecl. */
import org.objectweb.corba.ast.api.ComponentDecl;

/** To use CORBA::ComponentIR::UsesDef. */
import org.omg.CORBA.ComponentIR.UsesDef;
import org.omg.CORBA.ComponentIR.UsesDefHelper;

/**
 * UsesDeclImpl is a wrapper class for IDL uses port declarations.
 *
 * Inherits from:
 * - InterfacePortDeclImpl as they are ports for an interface,
 * - LocalContextOperationMapping 
 *   to obtain the local context operation mapping.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class UsesDeclImpl
       extends InterfacePortDeclImpl
       implements org.objectweb.corba.ast.api.UsesDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 UsesDef. */
    private UsesDef uses_def_;

    /** Is multiple. */
    private boolean is_multiple_;

    /** Local context operation mapping. */
    private OperationDecl[] local_context_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the uses declaration.
     */
    protected
    UsesDeclImpl(Repository rep,
                 ScopeImpl parent)
    {
        // Call the InterfacePortDeclImpl constructor.
        super(rep, parent);

        // Init internal state.
        uses_def_ = null;
        is_multiple_ = false;
        local_context_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 UsesDef.
     *
     * @param contained The UsesDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        uses_def_ = UsesDefHelper.narrow(contained);
        setInterface((InterfaceDecl)getRepository().
                     lookupId(uses_def_.interface_type().id()));
        is_multiple_ = uses_def_.is_multiple();
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the uses declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return uses_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector uses_depend = new java.util.Vector();

        // content
        // TODO: The next test is not needed
        // as InterfaceDecl is a Declaration, isn't it? 
        if (getInterface().isDeclaration())
            uses_depend.add(getInterface());

        Declaration[] depend = getInterface().getDependencies();
        for (int i=0; i<depend.length; i++)
            uses_depend.add(depend[i]);

        dependencies_ = (Declaration[])uses_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_uses;
    }

    /**
     * Create the uses declaration into the IFR.
     */
    public void
    create()
    {
        uses_def_ = the_parent_.getComponentDef().
                    create_uses(getId(), getName(), getVersion(),
                                interface_.getInterfaceDef(),
                                is_multiple_);
        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ClientOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the client operation mapping.
     *
     * @return The client operation mapping.
     */
    public OperationDecl[]
    getClientMapping()
    {
        if (client_mapping_!=null)
            return client_mapping_;

        // retrieve the client mapping interface.
        InterfaceDecl client = ((ComponentDecl)the_parent_).getClientMapping();
        client.getContents(true, DeclarationKind.dk_null);

        client_mapping_ = new OperationDecl[3];
        client_mapping_[0] = (OperationDecl)client.find("connect_"+getName());
        client_mapping_[1] = (OperationDecl)client.find("disconnect_"+getName());

        if (is_multiple_)
            client_mapping_[2] = (OperationDecl)
                                 client.find("get_connections_"+getName());
        else
            client_mapping_[2] = (OperationDecl)
                                 client.find("get_connection_"+getName());

        return client_mapping_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.InterfacePortDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.LocalContextOperationMapping
    //
    // ==================================================================

    /**
     * Obtain the local context operation mapping.
     *
     * @return The local context operation mapping.
     */
    public OperationDecl[]
    getLocalContextMapping()
    {
        if (local_context_mapping_!=null)
            return local_context_mapping_;

        // retrieve the context mapping local interface.
        InterfaceDecl context = ((ComponentDecl)the_parent_).
                                getLocalContextMapping();
        context.getContents(true, DeclarationKind.dk_null);

        local_context_mapping_ = new OperationDecl[1];
        if (is_multiple_)
            local_context_mapping_[0] = (OperationDecl)
                             context.find("get_connections_"+getName());
        else
            local_context_mapping_[0] = (OperationDecl)
                             context.find("get_connection_"+getName());

        return local_context_mapping_;
    }
    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.UsesDecl
    //
    // ==================================================================

    /**
     * Set as multiple.
     */
    public void
    setMultiple()
    {
        is_multiple_ = true;
    }

    /**
     * Is it multiple?
     *
     * @return True if it is multiple, else false.
     */
    public boolean
    isMultiple()
    {
        return is_multiple_;
    }
}
