/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To use CORBA::ValueBoxDef. */
import org.omg.CORBA.ValueBoxDef;
import org.omg.CORBA.ValueBoxDefHelper;

/**
 * ValueBoxDeclImpl is a wrapper class for IDL value box declarations.
 *
 * Inherits from:
 * - DeclarationWithTypeRefImpl as value boxes are IDL declarations
 *   and have an associated type.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueBoxDeclImpl
       extends DeclarationWithTypeRefImpl
       implements org.objectweb.corba.ast.api.ValueBoxDecl,
                  IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ValueBoxDef. */
    private ValueBoxDef value_box_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the value box declaration.
     */
    protected 
    ValueBoxDeclImpl(Repository rep,
                     ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        value_box_def_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ValueBoxDef.
     *
     * @param contained The ValueBoxDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        value_box_def_ = ValueBoxDefHelper.narrow(contained);
        setType(getRepository().
                getAsTypeRef(value_box_def_.original_type_def()));
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 ValueBoxDef.
     *
     * @param contained The ValueBoxDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        value_box_def_ = ValueBoxDefHelper.narrow(contained);
        setType(getRepository().
                getAsMappedTypeRef(value_box_def_.original_type_def()));
        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the
     *         value box declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return value_box_def_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector vbox_depend = new java.util.Vector();

        // content
        if (getType().isDeclaration())
            vbox_depend.add(getType());

        Declaration[] depend = getType().getDependencies();
        for (int i=0;i<depend.length;i++)
            vbox_depend.add(depend[i]);

        dependencies_ = (Declaration[])vbox_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_value_box;
    }

    /**
     * Create the value box declaration into the IFR.
     */
    public void
    create()
    {
        value_box_def_ = the_parent_.getContainer().
                         create_value_box(getId(),
                                          getName(),
                                          getVersion(),
                                          super.getIDLType());
        super.create();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_value_box;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ValueBoxDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the type reference.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return value_box_def_;
    }
}
