/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** Used to access AST TypeRef. */
import org.objectweb.corba.ast.api.TypeRef;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** Used to access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** Used to access AST AttributeDecl. */
import org.objectweb.corba.ast.api.AttributeDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.corba.ast.api.OperationDecl;

/** Used to access AST ValueDecl. */
import org.objectweb.corba.ast.api.ValueDecl;

/** Used to access AST ValueList. */
import org.objectweb.corba.ast.api.ValueList;

/** Used to access AST InterfaceList. */
import org.objectweb.corba.ast.api.InterfaceList;

/** Used to access AST Initializer. */
import org.objectweb.corba.ast.api.Initializer;

/** Used to access AST InitializerList. */
import org.objectweb.corba.ast.api.InitializerList;

/** Used to access AST Parameter. */
import org.objectweb.corba.ast.api.Parameter;

/** Used to access AST Exception. */
import org.objectweb.corba.ast.api.ExceptionDecl;

/** To use CORBA::ValueDef. */
import org.omg.CORBA.ValueDef;

/** To use CORBA::ExtValueDef. */
import org.omg.CORBA.ExtValueDef;
import org.omg.CORBA.ExtValueDefHelper;

/** To use CORBA::InterfaceDef. */
import org.omg.CORBA.InterfaceDef;

/**
 * ValueDeclImpl is a wrapper class for IDL value type declarations.
 *
 *
 * Inherits from:
 *
 * - ForwardScopeImpl as valuetypes are also IDL forward scopes.
 *
 * - ValueDecl: OMG IDL for value type declarations.
 *
 * - IDLTypeWrapper as values are IDL types.
 *
 *
 * Provides:
 *
 * - the boolean 'Custom' property
 *   i.e. the setCustom and isCustom operations.
 *
 * - the boolean 'Abstract' property
 *   i.e. the setAbstract and isAbstract operations.
 *
 * - the boolean 'Truncatable' property
 *   i.e. the setTruncatable and isTruncatable operations.
 *
 * - the inherited 'BaseValue' property
 *   i.e. the setBaseValue and getBaseValue operations.
 *
 * - the boolean 'PrimaryKey' property
 *   i.e. the isPrimaryKey operation.
 *
 * - the list of inherited abstract valuetypes
 *   i.e. the getAbstractValueList operation.
 *
 * - the list of supported interfaces
 *   i.e. the getSupportedInterfaceList operation.
 *
 * - the list of initializers
 *   i.e. the getInitializerList operation.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueDeclImpl
     extends ForwardScopeIDLImpl
  implements ValueDecl,
             IDLTypeWrapper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ValueDef. */
    protected ExtValueDef ext_value_def_;

    /** To store the 'Custom' property. */
    protected boolean is_custom_;

    /** To store the 'Abstract' property. */
    protected boolean is_abstract_;

    /** To store the 'Truncatable' property. */
    protected boolean is_truncatable_;

    /** The inherited base value. */
    protected ValueDeclImpl base_value_;

    /** The inherited abstract base values. */
    protected ValueListImpl abstract_base_values_;

    /** The supported interfaces.  */
    protected InterfaceListImpl supported_interfaces_;

    /** The initializers.  */
    protected InitializerListImpl initializers_;
 
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the value declaration.
     */
    public
    ValueDeclImpl(Repository rep, ScopeImpl parent)
    {
        // Call the ForwardScopeIDLImpl constructor.
        super(rep, parent);

        // Init internal state.
        ext_value_def_ = null;
        is_custom_ = false;
        is_abstract_ = false;
        is_truncatable_ = false;
        base_value_ = null;
        abstract_base_values_ = new ValueListImpl();
        supported_interfaces_ = new InterfaceListImpl();
        initializers_ = new InitializerListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ValueDef.
     *
     * @param contained The ValueDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        ext_value_def_ = ExtValueDefHelper.narrow(contained);
        is_custom_ = ext_value_def_.is_custom();
        is_abstract_ = ext_value_def_.is_abstract();
        is_truncatable_ = ext_value_def_.is_truncatable();

        ValueDef baseValue = ext_value_def_.base_value();
        if (baseValue!=null)
            setBaseValue((ValueDecl)getRepository().lookupId(baseValue.id()));

        // load as scope now to prevent infinite loop in the case
        // of recursive union.
        super.load(contained);

        ValueDef[] abstract_base_values = ext_value_def_.abstract_base_values();
        for (int i=0; i<abstract_base_values.length; i++)
            abstract_base_values_.add(
                (ValueDecl)getRepository().
                lookupId(abstract_base_values[i].id()));

        InterfaceDef[] supported_interfaces = 
             ext_value_def_.supported_interfaces();
        for (int i=0; i<supported_interfaces.length; i++)
            supported_interfaces_.add(
                (InterfaceDecl)getRepository().
                lookupId(supported_interfaces[i].id()));

        org.omg.CORBA.ExtInitializer[] initializers =
            ext_value_def_.ext_initializers();
        for (int i=0; i<initializers.length; i++)
        {
            Initializer initializer = startInitializer(initializers[i].name);
            initializers_.add(initializer);
            // Load initializer parameters
            org.omg.CORBA.StructMember[] params = initializers[i].members;
            for (int j=0; j<params.length; j++)
            {
                initializer.getParameterList().
                            addInParam(params[j].name,
                                       getRepository().
                                       getAsTypeRef(params[j].type_def));
            }
            // Load initializer exceptions
            org.omg.CORBA.ExceptionDef[] exc = initializers[i].exceptions_def;
            for (int j=0; j<exc.length; j++)
            {
                initializer.getExceptionList().add(
                        (ExceptionDecl)getRepository().lookupId(exc[j].id()) );
            }
        }
    }

    /**
     * Loads infos of the CORBA 3.0 ValueDef.
     *
     * @param contained The ValueDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        ext_value_def_ = ExtValueDefHelper.narrow(contained);
        is_custom_ = ext_value_def_.is_custom();
        is_abstract_ = ext_value_def_.is_abstract();
        is_truncatable_ = ext_value_def_.is_truncatable();

        ValueDef baseValue = ext_value_def_.base_value();
        if (baseValue!=null)
            setBaseValue((ValueDecl)getRepository().
            lookupMappedId(baseValue.id()));

        // load as scope now to prevent infinite loop in the case
        // of recursive union.
        super.loadAsMapping(contained);


        ValueDef[] abstract_base_values =
            ext_value_def_.abstract_base_values();
        for (int i=0; i<abstract_base_values.length; i++)
            abstract_base_values_.add(
                (ValueDecl)getRepository().
                lookupMappedId(abstract_base_values[i].id()));

        InterfaceDef[] supported_interfaces =
            ext_value_def_.supported_interfaces();
        for (int i=0; i<supported_interfaces.length; i++)
            supported_interfaces_.add(
                (InterfaceDecl)getRepository().
                lookupMappedId(supported_interfaces[i].id()));

        org.omg.CORBA.ExtInitializer[] initializers = 
            ext_value_def_.ext_initializers();
        for (int i=0; i<initializers.length; i++)
        {
            Initializer initializer = startInitializer(initializers[i].name);
            initializers_.add(initializer);
            // Load initializer parameters
            org.omg.CORBA.StructMember[] params = initializers[i].members;
            for (int j=0; j<params.length; j++)
            {
                initializer.getParameterList().
                            addInParam(params[j].name,
                                       getRepository().
                                       getAsMappedTypeRef(params[j].type_def));
            }
            // Load initializer exceptions
            org.omg.CORBA.ExceptionDef[] exc = initializers[i].exceptions_def;
            for (int j=0; j<exc.length; j++)
            {
                initializer.getExceptionList().add(
                        (ExceptionDecl)getRepository().lookupId(exc[j].id()) );
            }
        }
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the value declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return ext_value_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Create an attribute definition.
     *
     * @param attribute The attribute declaration.
     * @param type The IDLType of the attribute.
     * @param mode The mode of the attribute.
     * @param get_exceptions An array containing the exceptions that
     *                       the accessor method can raise.
     * @param put_exceptions An array containing the exceptions that
     *                       the mutator method can raise.
     *
     * @return The new created AttributeDef.
     */
    protected org.omg.CORBA.ExtAttributeDef
    createExtAttribute(AttributeDecl attribute,
                       org.omg.CORBA.IDLType type,
                       org.omg.CORBA.AttributeMode mode,
                       org.omg.CORBA.ExceptionDef[] get_exceptions,
                       org.omg.CORBA.ExceptionDef[] set_exceptions)
    {
        // Create an AttributeDef into the IFR!;
        org.omg.CORBA.ExtAttributeDef ext_attribute_def =
            getExtValueDef().create_ext_attribute(attribute.getId(),
                                                  attribute.getName(),
                                                  attribute.getVersion(),
                                                  type, mode,
                                                  get_exceptions,
                                                  set_exceptions);
        return ext_attribute_def;
    }

    /**
     * Create an operation
     *
     * @param operation The operation declaration.
     * @param type The IDLType of the object that the operations returns.
     * @param mode The mode of the operation ie normal or oneway.
     * @param params An array containing the parameters description.
     * @param exceptions An array containing the exceptions that
     *                   the operation can raise.
     * @param contexts An array containing the possible context values.
     *
     * @return The new created OperationDef.
     */
    protected org.omg.CORBA.OperationDef
    createOperation(OperationDecl operation,
                    org.omg.CORBA.IDLType type,
                    org.omg.CORBA.OperationMode mode,
                    org.omg.CORBA.ParameterDescription[] params,
                    org.omg.CORBA.ExceptionDef[] exceptions,
                    String[] contexts)
    {
        // Create an OperationDef into the IFR.
        org.omg.CORBA.OperationDef operation_def =
            getExtValueDef().create_operation(operation.getId(),
                                              operation.getName(),
                                              operation.getVersion(),
                                              type, mode,
                                              params,
                                              exceptions,
                                              contexts);
        return operation_def;
    }

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the value declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return ext_value_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        // Create a ValueDef into the IFR.
        ext_value_def_ = the_parent_.getContainer().
                         create_ext_value(getId(), getName(), getVersion(),
                         is_custom_, is_abstract_,
                         getBaseValueDef(),
                         is_truncatable_,
                         abstract_base_values_.getValueDefSeq(),
                         supported_interfaces_.getInterfaceDefSeq(),
                         new org.omg.CORBA.ExtInitializer[0]);
    }

    /**
     * Complete the container object.
     */
    protected void
    completeContainer()
    {
        ValueDef base_value = getBaseValueDef();
        if(base_value != null)
            getExtValueDef().base_value(base_value);

        if(abstract_base_values_.getSize() != 0)
            getExtValueDef().abstract_base_values(abstract_base_values_.getValueDefSeq());

        if(supported_interfaces_.getSize() != 0)
            getExtValueDef().supported_interfaces(supported_interfaces_.getInterfaceDefSeq());

        if(initializers_.getSize() != 0)
            getExtValueDef().ext_initializers(initializers_.getExtInitializerSeq());
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the base value.
     *
     * @return The ExtValueDef of the base value or null if it not exists.
     **/
    public ExtValueDef
    getBaseValueDef()
    {
        return (base_value_ != null)
	     ? (base_value_.getExtValueDef()) : null;
    }

    /**
     * Obtain its ValueDef reference.
     *
     * @return The ValueDef associated with the value declaration.
     */
    public ExtValueDef
    getExtValueDef()
    {
        return ext_value_def_;
    }

    /**
     * Obtain its ValueDef reference.
     *
     * @return The ValueDef associated with the value declaration.
     */
    public ValueDef
    getValueDef()
    {
        return ext_value_def_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        dependencies_ = new Declaration[0];
        java.util.Vector value_depend = new java.util.Vector();
        Declaration[] depend = null;

        // base value
        if (getBaseValue()!=null)
        {
            value_depend.add(getBaseValue());
            depend = getBaseValue().getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if (depend[j]!=this)
                    value_depend.add(depend[j]);
            }
        }

        // abstract base values
        ValueDecl[] abs = abstract_base_values_.getValues();
        for (int i=0; i<abs.length; i++)
        {
            value_depend.add(abs[i]);
            depend = abs[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((depend[j]!=this) &&
                    (value_depend.indexOf(depend[j])==-1))
                    value_depend.add(depend[j]);
            }
        }

        // supported interfaces
        InterfaceDecl[] supp = supported_interfaces_.getInterfaces();
        for (int i=0; i<supp.length; i++)
        {
            value_depend.add(supp[i]);
            depend = supp[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((depend[j]!=this) &&
                    (value_depend.indexOf(depend[j])==-1))
                    value_depend.add(depend[j]);
            }
        }

        // initializers
        Initializer[] inits = initializers_.getInitializers();
        for (int i=0; i<inits.length; i++)
        {
            Parameter[] parameters = inits[i].getParameterList().
                                     getParameters();
            for (int j=0; j<parameters.length; j++)
            {
                TypeRef param_type = parameters[j].getType();
                depend = param_type.getDependencies();
                for (int k=0; k<depend.length; k++)
                {
                    if ((!containsDecl(depend[k])) &&
                        (depend[k]!=this) &&
                        (value_depend.indexOf(depend[k])==-1))
                        value_depend.add(depend[k]);
                }
            }
        }

        // contents
        Declaration[] decls = getContents(true, DeclarationKind.dk_all);
        for (int i=0; i<decls.length; i++)
        {
            depend = decls[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (depend[j]!=this) &&
                    (value_depend.indexOf(depend[j])==-1))
                    value_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])value_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_value;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    /**
     * Find a declaration.
     *
     * Note that the declaration is also recursively searched in the
     * parent scope.
     *
     * @param name The name of the searched declaration.
     *
     * @return The declaration that was searched
     *         or null if it does not exist.
     */
    public Declaration
    find(String name)
    {
        // Finds in the current scope.
        Declaration decl = super.find(name);

        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in base value.           
        if(base_value_ != null)
        {
            decl = base_value_.find(name);

            // If found then returns it.
            if(decl != null) return decl;
        }

        // If not found then finds in abstract values.
        decl = abstract_base_values_.find(name);

        // If found then returns it.
        if(decl != null) return decl;

        // If not found then finds in supported interfaces.
        decl = supported_interfaces_.find(name);

        // Else not found.
        return null;
    }

    /**
     * To obtain all the contained Declaration objects.
     *
     * @param exclude_inherited If false return also objects contained in inherited scopes.
     * @param limited_types A logical combination of DeclarationKind.
     *
     * @return An array of Declaration objects.
     */
    public Declaration[]
    getContents(boolean exclude_inherited,
                long limited_types)
    {
        if (exclude_inherited)
            return super.getContents(exclude_inherited, limited_types);

        Declaration[] res = super.getContents(exclude_inherited, limited_types);

        InterfaceDecl[] itfs = supported_interfaces_.getInterfaces();
        for (int i=0; i<itfs.length; i++)
        {
            Declaration[] tmp1 = itfs[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }
        ValueDecl[] vals = abstract_base_values_.getValues();
        for (int i=0; i<vals.length; i++)
        {
            Declaration[] tmp1 = vals[i].getContents(exclude_inherited, limited_types);
            Declaration[] tmp2 = res;
            res = new Declaration[tmp1.length+tmp2.length];
            System.arraycopy(tmp2, 0, res, 0, tmp2.length);
            System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        }
        if (base_value_==null)
            return res;

        Declaration[] tmp1 = base_value_.getContents(exclude_inherited, limited_types);
        Declaration[] tmp2 = res;
        res = new Declaration[tmp1.length+tmp2.length];
        System.arraycopy(tmp2, 0, res, 0, tmp2.length);
        System.arraycopy(tmp1, 0, res, tmp2.length, tmp1.length);
        return res;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.TypeRef
    //
    // ==================================================================

    /**
     * Obtain its associated TypeKind.
     *
     * @return The associated TypeKind.
     */
    public TypeKind
    getTypeKind()
    {
        return TypeKind.tk_value;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ValueDecl
    //
    // ==================================================================

    /**
     * Set as a custom valuetype.
     *
     * @param b True if it is a custom valuetype, else false.
     */
    public void
    setCustom(boolean b)
    {
        is_custom_ = b;
    }

    /**
     * Is it a custom valuetype?
     *
     * @return True if it is a custom valuetype, else false.
     */
    public boolean
    isCustom()
    {
        return is_custom_;
    }

    /**
     * Set as an abstract valuetype.
     *
     * @param b True if it is an abstract valuetype, else false.
     */
    public void
    setAbstract(boolean b)
    {
        is_abstract_ = b;
    }

    /**
     * Is it abstract?
     *
     * @return True if it is an abstract valuetype, else false.
     */
    public boolean
    isAbstract()
    {
        return is_abstract_;
    }

    /**
     * Set as a truncatable valuetype.
     *
     * @param b True if it is a truncatable valuetype, else false.
     */
    public void
    setTruncatable(boolean b)
    {
        is_truncatable_ = b;
    }

    /**
     * Is it truncatable?
     *
     * @return True if it is a truncatable valuetype, else false.
     */
    public boolean
    isTruncatable()
    {
        return is_truncatable_;
    }

    /**
     * Set the base value.
     *
     * @param value The base value.
     */
    public void
    setBaseValue(ValueDecl value)
    {
        if(value != null)
        {
            base_value_ = (ValueDeclImpl)value;
        }
    }

    /**
     * Obtain the base value.
     *
     * @return The base value.
     */
    public ValueDecl
    getBaseValue()
    {
        return base_value_;
    }

    /**
     * Is it a primary key valuetype?
     *
     * @return True if it is a primary key, else false.
     */
    public boolean
    isPrimaryKey()
    {
        boolean is_pk = false;
        ValueDecl[] abs = abstract_base_values_.getValues();
        for (int i=0;i<abs.length;i++)
        {
            if (abs[i].getAbsoluteName().equals("::Components::PrimaryKeyBase"))
                return true;

            is_pk = abs[i].isPrimaryKey();
            if (is_pk)
                return true;
        }

        if (getBaseValue()!=null)
            is_pk = getBaseValue().isPrimaryKey();

        return is_pk;
    }

    /**
     * Obtain the list of inherited abstract values.
     *
     * @param itf The list of inherited abstract values.
     */
    public ValueList
    getAbstractValueList()
    {
        return abstract_base_values_;
    }


    /**
     * Obtain the list of supported interfaces.
     *
     * @param itf The list of supported interfaces.
     */
    public InterfaceList
    getSupportedInterfaceList()
    {
        return supported_interfaces_;
    }

    /**
     * Obtain the list of initializers.
     *
     * @param itf The list of initializers.
     */
    public InitializerList
    getInitializerList()
    {
        return initializers_;
    }

    // ==================================================================
    //
    // Methods for interface IDLTypeWrapper
    //
    // ==================================================================

    /**
     * Obtain its IDLType reference.
     *
     * @return The IDLType associated with the value declaration.
     */
    public org.omg.CORBA.IDLType
    getIDLType()
    {
        return ext_value_def_;
    }
}
