/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To use CORBA::ValueMemberDef. */
import org.omg.CORBA.ValueMemberDef;
import org.omg.CORBA.ValueMemberDefHelper;

/**
 * ValueMemberDeclImpl is a wrapper class
 * for IDL value member declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationWithTypeRefImpl as value members are IDL declarations
 *   and have an associated type.
 *
 * - ValueMemberDecl: OMG IDL for value member declarations.
 *
 *
 * Provides:
 *
 * - The boolean 'Public' property,
 *   i.e. the setPublic and isPublic operations.
 *
 * - The boolean 'Private' property,
 *   i.e. the setPrivate and isPrivate operations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ValueMemberDeclImpl
       extends DeclarationWithTypeRefImpl
       implements org.objectweb.corba.ast.api.ValueMemberDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ValueMemberDef. */
    private ValueMemberDef value_member_def_;

    /** The visibility of the value member. */
    private short access_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the value member declaration.
     **/
    protected
    ValueMemberDeclImpl(Repository rep,
                        ScopeImpl parent)
    {
        // Call the DeclarationWithTypeRefImpl constructor.
        super(rep, parent);

        // Init internal state.
        value_member_def_ = null;
        type_ = null;
        access_ = (short)0;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ValueMemberDef.
     *
     * @param contained The ValueMemberDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        value_member_def_ = ValueMemberDefHelper.narrow(contained);
        setType(getRepository().getAsTypeRef(value_member_def_.type_def()));
        access_ = value_member_def_.access();
        super.load(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the value
     *         member declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return value_member_def_;
    }

    // ==================================================================
    //
    // Internal methods for DeclarationWithTypeRefImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector vmember_depend = new java.util.Vector();

        // content
        if (getType().isDeclaration())
            vmember_depend.add(getType());

        Declaration[] depend = getType().getDependencies();
        for (int i=0; i<depend.length; i++)
            vmember_depend.add(depend[i]);


        dependencies_ = (Declaration[])vmember_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_value_member;
    }

    /**
     * Create the value member declaration into the IFR.
     */
    public void
    create()
    {
        // Create a ValueMemberDef into the IFR.
        value_member_def_ = the_parent_.getExtValueDef().
                            create_value_member(getId(),
                                                getName(),
                                                getVersion(),
                                                getIDLType(),
                                                access_);
        super.create();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.corba.ast.api.DeclarationWithTypeRef
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ValueMemberDecl
    //
    // ==================================================================

    /**
     * Set as a public value member.
     */
    public void
    setPublic()
    {
        access_ = (short)1;
    }

    /**
     * Is it a public value member?
     *
     * @return True if it is public, else false.
     */
    public boolean
    isPublic()
    {
        return access_ == (short)1;
    }

    /**
     * Set as a private value member.
     */
    public void
    setPrivate()
    {
        access_ = (short)0;
    }

    /**
     * Is it a private value member?
     *
     * @return True if it is private, else false.
     */
    public boolean
    isPrivate()
    {
        return access_ == (short)0;
    }
}
