/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.common;

/**
 ** This class uses the Velocity (http://jakarta.apache.org/Velocity)
 ** engine to creates mapped files.
 **/
public class Generator
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     ** The file ressource loader path
     **/
    protected String ressource_path_;

    /**
     ** The template path
     **/
    protected String path_;

    /**
     ** The velocity engine.
     **/
    protected org.apache.velocity.app.VelocityEngine engine_;

    /**
     ** The current context.
     **/
    protected org.apache.velocity.context.Context context_;

    /**
     ** The writer
     **/
    protected java.io.Writer stream_;

    /**
     ** The file output name
     **/
    protected String output_name_;

    /**
     ** The log file name
     **/
    protected String log_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    public Generator()
    {
        engine_ = new org.apache.velocity.app.VelocityEngine();
        ressource_path_ = ".";
        path_ = "";
        stream_ = null;
        output_name_ = "generated.txt";
        log_ = "velocityError.log";
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     **/
    public void
    init()
    {
        // Initialize the Velocity Engine
        try{
            engine_.setProperty(org.apache.velocity.app.VelocityEngine.RESOURCE_LOADER, "file");
            engine_.setProperty(org.apache.velocity.app.VelocityEngine.FILE_RESOURCE_LOADER_PATH, ressource_path_);
            engine_.init();
        }catch(Exception e){
            e.printStackTrace();
        };

        // Initialize the output file
        try {
            java.io.FileOutputStream file = new java.io.FileOutputStream(new java.io.File(output_name_));
            stream_ = new java.io.PrintWriter(file);
        } catch (Exception e ) {
            throw new Error("Problem with io : " + e );
        }
        stream_ = new org.apache.velocity.io.VelocityWriter(stream_);//new Printer(stream_);

        // Create a new context
        setContext(new org.apache.velocity.VelocityContext());
    }

    /**
     * Sets the file ressource loader path.
     *
     * @param path - The path from where files will be loaded.
     **/
    public void
    setRessourcePath(java.util.Vector v)
    {
        if ( (v!=null) && (v.size()>0) )
        {
            String path = null;
            ressource_path_ = (String)v.get(0);

            for (int i=1; i<v.size(); i++)
            {
                path = (String)v.get(i);
                ressource_path_ += "," + path;
            }
        }
    }

    /**
     * Sets the template path.
     *
     * @param path - A path from a ressource declared in your classpath.
     **/
    public void
    setTemplatePath(String path)
    {
        path_ = path;
    }

    /**
     * Sets the name of the local Velocimacro library template.
     *
     * @param name - The local Velocimacro library template path.
     **/
    public void
    setLibrary(java.util.Vector v)
    {
        if ( (v!=null) && (v.size()>0) )
        {
            String name = (String)v.get(0);

            for (int i=1; i<v.size(); i++)
            {
                name += "," + (String)v.get(i);
            }
            try{
                engine_.setProperty(org.apache.velocity.app.VelocityEngine.VM_LIBRARY, name);
            }catch(Exception e){
                e.printStackTrace();
            };
        }
    }

    /**
     * Sets the current context.
     *
     * @param context - The context to be set.
     **/
    public void
    setContext(org.apache.velocity.context.Context context)
    {
        context_ = context;
    }

    /**
     * Sets the output file name.
     *
     * @param name - The output file name to be set.
     **/
    public void
    setOutputName(String file_name)
    {
        output_name_ = file_name;
    }

    /**
     * Gets a value from the context.
     *
     * @param key - The name to key the provided value with.
     *
     * @return The associated value.
     **/
    public Object
    get(String key)
    {
        return context_.get(key);
    }

    /**
     * Adds a name/value pair to the context.
     *
     * @param key - The name to key the provided value with.
     *        name - The corresponding value.
     **/
    public void
    put(String key, Object value)
    {
        context_.put(key, value);
    }

    /**
     * Removes the value associated with the specified key from the context.
     *
     * @param key - The name of the value to remove.
     **/
    public void
    remove(Object key)
    {
        context_.remove(key);
    }

    /**
     * Uses velocity to map a template file.
     *
     * @param template_name - The name of the template file.
     **/
    public void
    map_file(String template_name)
    {
        try {
            engine_.mergeTemplate(path_ + template_name, context_, stream_);
            stream_.flush();
        } catch (java.io.FileNotFoundException ex) {
            throw new Error("Problem with io : " + ex );
        } catch (SecurityException ex) {
            throw new Error("Problem with io : " + ex );
        } catch (Exception e ) {
            throw new Error("Problem merging template : " + e );
        }
    }

    /**
     * Uses velocity to map a currently registered Velocimacro.
     *
     * @param macro_name - The name of the macro to invoke (map)
     *        params - args used to invoke Velocimacro. In context key format : eg "foo","bar" (rather than "$foo","$bar")
     **/
    public void
    map(String macro_name, java.lang.String[] params)
    {
        try {
            engine_.invokeVelocimacro(macro_name, log_, params, context_, stream_);
            stream_.flush();
        } catch (Exception e ) {
            throw new Error("Problem invoking macro : " + e );
        }
    }

    /**
     * Uses velocity to map a currently registered Velocimacro.
     *
     * @param macro_name - The name of the macro to invoke (map)
     **/
    public void
    map(String macro_name)
    {
        map(macro_name, new String[0]);
    }

    /**
     * Print immediatly the text in the output stream.
     *
     * @param text - The text to be written.
     **/
    public void
    print(String text)
    {
        try {
            stream_.write(text);
        } catch (java.io.IOException e ) {
            throw new Error("Problem with io : " + e );
        }
    }

}
