/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.common;

/**
 *
 */
public class Indentor
{
    // ===========================================================
    //
    // Constants.
    //
    // ===========================================================

    /**
     ** The number of spaces to print for an indentation
     **/
    public static final int NB_CAR_INDENT = 4;

    /**
     ** The tag used to increment indentation
     **/
    public static final String INCREMENT_TAG = new String("/inc");

    /**
     ** The tag used to decrement indentation
     **/
    public static final String DECREMENT_TAG = new String("/dec");

    /**
     ** The end-line tag
     **/
    public static final String END_LINE = new String("\n");

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     ** The number of indentation to do
     **/
    private int indent_;

    /**
     ** The input stream to indent
     **/
    private java.io.BufferedReader in_;

    /**
     ** The resulting output
     **/
    private java.util.Vector out_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor
     */
    public Indentor() {}

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Initialize the indentor
     *
     * @param file_name - The file to indent.
     */
    private void
    init(String file_name)
    {
        // init internal states
        indent_ = 0;
        in_ = null;
        out_ = new java.util.Vector();

        // init input file
        try{
            in_ = new java.io.BufferedReader( new java.io.FileReader(file_name) );
        }catch(java.io.IOException ex){
            System.err.println("I/O error : ");
            ex.printStackTrace();
        }
    }

    /**
     * Our own little min method.
     *
     * @param a - An integer value.
     * @param b - An integer value.
     *
     * @return The min value.
     **/
    private int
    min(int a, int b)
    {
        if (a < b) return a;
            return b;
    }

    /**
     * Search the first occurence between :
     *  - INCREMENT_TAG
     *  - DECREMENT_TAG
     *  - END_LINE
     *
     * @param src - The string to search in.
     *
     * @return The index of the first occurence found, else -1.
     **/
    private int searchFirst(String src)
    {
        int index_inc = -1;
        int index_dec = -1;
        int index_end = -1;

        index_inc = src.indexOf(INCREMENT_TAG);
        index_dec = src.indexOf(DECREMENT_TAG);
        index_end = src.indexOf(END_LINE);
        if (index_inc != -1)
        {
            if (index_dec != -1)
            {
                if (index_end != -1)
                {
                    if (index_inc < index_dec)
                        return min(index_inc, index_end);
                    else
                        return min(index_dec, index_end);
                }
                else
                {
                    return min(index_inc, index_dec);
                }
            }
            else
            {
                if (index_end != -1)
                    return min(index_inc, index_end);
                else
                    return index_inc;
            }
        }
        else if (index_dec != -1)
        {
            if (index_end != -1)
            {
                return min(index_dec, index_end);
            }
            else
            {
                return index_dec;
            }
        }
        else
        {
            return index_end;
        }

    }

    /**
     * Increment the output
     **/
    private void
    increment()
    {
        indent_ += NB_CAR_INDENT;
    }

    /**
     * Decrement the output
     **/
    private void
    decrement()
    {
        indent_ -= NB_CAR_INDENT;
        if(indent_ < 0)
            indent_ = 0;
    }

    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Replace the first occurence found in str with the new string
     *
     * @param str - The string to search in.
     * @param old_pattern - The pattern to search.
     * @param new_pattern - The replacing pattern.
     *
     * @return The resulting string.
     **/
    public String
    replaceFirst(String str, String old_pattern, String new_pattern)
    {
        int index = str.indexOf(old_pattern);
        int l = old_pattern.length();
        try{
            return str.substring(0, index) + new_pattern
                   + str.substring(index+l, str.length());
        }catch(Exception e){
            return str;
        }
    }

    /**
     * Management of indentation
     *
     * @param file_name - The file to indent.
     **/
    public void
    indent(String file_name) throws java.io.IOException
    {
        String current = null;

        // Init the indentor
        init(file_name);

        // Processing the file
        while ( (current = in_.readLine() ) != null )
        {
            // Indent the new line
            String res = new String("");

            for(int i=0; i<indent_; i++)
                res += " ";

            // Search incrementation / decrementation tag
            int index = searchFirst(current);

            // Generate indentation
            if ( current.startsWith(INCREMENT_TAG, index) )
            {
                increment();
                current = replaceFirst(current, INCREMENT_TAG, "");
            }
            else if( current.startsWith(DECREMENT_TAG, index) )
            {
                decrement();
                current = replaceFirst(current, DECREMENT_TAG, "");
                res = new String("");
                for(int i=0; i<indent_; i++)
                    res += " ";
            }
            res += current + END_LINE;
            out_.add( res );
        }

        in_.close();
        apply(file_name);
    }

    /**
     * Apply changes on the file
     *
     * @param file_name - The file to indent.
     **/
    public void
    apply(String file_name)
    {
        java.io.File fic = null;
        java.io.PrintWriter writer = null;

        // Overwrite the input file
        try{
            fic = new java.io.File(file_name);
            fic.delete();

            writer = new java.io.PrintWriter( new java.io.FileWriter(fic) );
        }catch(java.io.IOException ex){
            System.err.println("I/O error : ");
            ex.printStackTrace();
        }

        // Print contents
        for(int i=0; i<out_.size(); i++)
        {
            writer.write( (String)out_.get(i) );
        }
        writer.flush();
        writer.close();
    }

}

