/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.parser.lib;

import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.WithDeclarators;

/**
 * This class manages errors.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ErrorManager
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The file name.
     **/
    private String fileName_;

    /**
     **  The number of warnings.
     **/
    private int nbWarnings_;

    /**
     **  The number of errors.
     **/
    private int nbErrors_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param fileName The name of the file that is being parsed.
     **/
    public
    ErrorManager(String fileName)
    {
        // Init internal state.
        fileName_ = fileName;
        nbWarnings_ = 0;
        nbErrors_ = 0;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     **  Display a message on the standart error stream.
     **
     **  @param line The line number where the error (or warning) occured.
     **  @param kind The kind of the message ie "error" or "warning"
     **              or anything else.
     **  @param text The message to display.
     **/
    protected void
    log(int line, String kind, String text)
    {
        System.out.println(fileName_ + ':' + line + ": " + kind + ": " + text);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
 
    /**
     **  Obtain the number of warnings.
     **
     **  @return The number of warnings.
     **/
    public int
    getNbWarnings()
    {
        return nbWarnings_;
    }

    /**
     **  Obtain the number of errors.
     **
     **  @return The number of errors.
     **/
    public int
    getNbErrors()
    {
        return nbErrors_;
    }

    /**
     **  Set the name of the file that is read.
     **
     **  @param The name of the file.
     **/
    public void
    setFileName(String fileName)
    {
        fileName_ = fileName;
    }

    /**
     **  Display a warning.
     **
     **  @param line The line number where the warning occured.
     **  @param text The message to display.
     **/
    public void
    warning(int line, String text)
    {
        log(line, "warning", text);
        nbWarnings_ ++;
    }

    /**
     **  Display a error.
     **
     **  @param line The line number where the error occured.
     **  @param text The message to display.
     **/
    public void
    error(int line, String text)
    {
        log(line, "error", text);
        nbErrors_ ++;
    }

    /**
     **  Display a error.
     **
     **  @param line The line number where the error occured.
     **  @param sysExc The system exception to display.
     **/
    public void
    error(int line, org.omg.CORBA.SystemException sysExc)
    {
        error(line, sysExc, null);
    }

    /**
     **  Display a error.
     **
     **  @param line The line number where the error occured.
     **  @param sysExc The system exception to display.
     **  @param decl The declaration that threw the system exception.
     **/
    public void
    error(int line,
	  org.omg.CORBA.SystemException sysExc,
	  Declaration decl)
    {
        int minor = sysExc.minor;

        if(sysExc instanceof org.omg.CORBA.BAD_PARAM)
	{
            switch(minor)
            {
            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.RidAlreadyDefined:
                error(line, '`' + decl.getId() +
                        "' already defined in the IFR");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.NameAlreadyUsed:
                error(line, '`' + decl.getName() +
                        "' already defined in the context");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.InvalidContainer:
                error(line, "Target is an invalid container");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.NameClashInInheritedContext:
                error(line, '`' + decl.getName() +
                        "' already defined in inherited context");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.InvalidAbstractInterfaceInheritance:
                error(line, "invalid abstract interface inheritance");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.InvalidLocalInterfaceInheritance:
                error(line, "invalid local interface inheritance");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.NotLocalObject:
	        // Do nothing because the error was already reported! 
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.NameAlreadyUsedByImmediateScope:
                error(line, '`' + decl.getName() +
                            "' already used by immediate scope");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.NeedOnlyInParams:
                error(line, '`' + decl.getName() +
                            "' only needs in parameters");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.NoExceptionsForOnewayOperation:
                error(line, '`' + decl.getName() +
                            "' no exceptions expected for oneway operation");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.InvalidValueBoxType:
                error(line, '`' + decl.getName() +
                            "' invalid valuebox type");
                return;
            }

            if (minor >= org.objectweb.ccm.IR3.SystemExceptionMinorValues.DuplicateDeclarator)
	    {
                int index = minor - org.objectweb.ccm.IR3.SystemExceptionMinorValues.DuplicateDeclarator;
                error(line, '`' + ((WithDeclarators)decl).declaratorAt(index) +
                            "' duplicate declarator in " + decl.getAbsoluteName());
                return;
            }

        }

        if(sysExc instanceof org.omg.CORBA.BAD_INV_ORDER)
	{
            switch(minor)
            {
            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.Dependency:
                error(line, "dependency exits in IFR preventing destruction");
                return;

            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.CanNotBeDestroyed:
                error(line, "indestructible object");
                return;
	    }
	}

        if(sysExc instanceof org.omg.CORBA.INTF_REPOS)
	{
            switch(minor)
            {
            case org.objectweb.ccm.IR3.SystemExceptionMinorValues.AmbiguousLookup:
                error(line, "ambiguous lookup");
                return;
	    }
	}

        if (decl != null)
            error(line, sysExc.toString() + " on " + decl.getAbsoluteName());
        else
            error(line, sysExc.toString());

	sysExc.printStackTrace();
    }
}
