/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

import org.objectweb.util.cmdline.api.Option;
import org.objectweb.util.cmdline.api.Usage;

/**
 * This provides a default implementation of the interface
 * org.objectweb.util.CommandLine.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultCommandLine
     extends DefaultUsage
  implements org.objectweb.util.cmdline.api.CommandLine
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** To store command line options. */
    private java.util.Vector options_;

    /** To store the maximun option length. */
    private int maxOptionLength_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * It inits the Labels, Arguments, and Description usage properties
     * with an empty string.
     */
    public
    DefaultCommandLine()
    {
        this("", "", (String[])null, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param with_help true if default help option is required.
     */
    public
    DefaultCommandLine(String label,
                       String arguments,
                       String description,
                       boolean with_help)
    {
        this(label, arguments, new String[] { description }, with_help);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param with_help true if default help option is required.
     */
    public
    DefaultCommandLine(String[] labels,
                       String arguments,
                       String description,
                       boolean with_help)
    {
        this(labels, arguments, new String[] { description }, with_help);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param with_help true if default help option is required.
     */
    public
    DefaultCommandLine(String label,
                       String arguments,
                       String[] description,
                       boolean with_help)
    {
        this(new String[] { label }, arguments, description, with_help);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param with_help true if default help option is required.
     */
    public
    DefaultCommandLine(String[] labels,
                       String arguments,
                       String[] description,
                       boolean with_help)
    {
        super(labels, arguments, description);

        options_ =  new java.util.Vector();
        maxOptionLength_ = 0;

        if(with_help) {
            addOption(new DefaultOptionHelp());
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Exists when error.
     *
     * @param message The error message.
     */
    protected void
    exit(String message)
    {
        System.err.println(getLabels()[0] + ": " + message + '!');
        print(System.err);
        System.exit(-1);
    }

    /**
     * Sums the length of a set of strings.
     *
     * @param strings The string array.
     *
     * @return The sum of the length of a set of strings.
     */
    static protected int
    sumStringLength(String[] strings)
    {
      int result = 0;
      for(int i=0; i<strings.length; i++)
        result += strings[i].length();
      return result;
    }

    /**
     * Prints a set of strings.
     *
     * @param stream The output stream.
     * @param strings The string array.
     * @param sep_first The separator.
     * @param sep_next The separator.
     */
    static protected void
    print(java.io.PrintStream stream,
          String[] strings,
          String sep_first,
          String sep_next)
    {
        if(strings.length > 0)
        {
            stream.print(sep_first);
            stream.print(strings[0]);
            for(int i=1; i<strings.length; i++)
            {
                stream.print(sep_next);
                stream.print(strings[i]);
            }
        }
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.api.Printable
    //
    // ==================================================================

    /**
     * Prints the target object to a java.io.PrintStream output stream.
     *
     * @param stream The java.io.PrintStream output stream.
     */
    public void
    print(java.io.PrintStream stream)
    {
      //
      // Prints the Usage part.
      //
	stream.print("Usage: ");
        stream.print(getLabels()[0]);

	// Iterate over all the command line options.
	for(java.util.Iterator iterator=options_.iterator();
	    iterator.hasNext();
	    )
	{
            stream.print(' ');

	    Option option = (Option)iterator.next();

            if(!option.isMandatory())
                stream.print('[');

	    print(stream, option.getLabels(), "", "|");

	    print(stream, option.getArguments(), " ", " ");

            if(!option.isMandatory())
                stream.print(']');
        }

        print(stream, getArguments(), " ", " ");

      //
      // Prints the Options part.
      //
        stream.println("\n\nOptions:");

	// Iterate over all the command line options.
	for(java.util.Iterator iterator=options_.iterator();
	    iterator.hasNext();
	    )
	{
            int nb_characters = 0;

	    Option option = (Option)iterator.next();

            String[] labels = option.getLabels();
            print(stream, labels, "  ", ", ");
            nb_characters = nb_characters +
	        sumStringLength(labels) +
	        labels.length * 2;

            String[] arguments = option.getArguments();
            print(stream, arguments, " ", " ");
            nb_characters = nb_characters +
	        sumStringLength(arguments) +
	        arguments.length;

            String[] description = option.getDescription();
            for(int i=0; i<description.length; i++)
            {
	        for(int j = nb_characters;
                    j <= maxOptionLength_; j++)
	        {
                   stream.print(' ');
                }
                stream.println(description[i]);
	        nb_characters = 0;
            }
        }
	stream.println("");

      //
      // Prints the Description part.
      //
	stream.println("Description: ");
        print(stream, getDescription(), "  ", "\n  ");

	stream.println("\n");
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.CommandLine
    //
    // ==================================================================

    /**
     * Gets the command line options.
     *
     * @return The command line options.
     */
    public Option[]
    getOptions()
    {
        return ((Option[])(options_.toArray(new Option[0])));
    }

    /**
     * Adds a command line option.
     *
     * @param option The command line option object.
     */
    public void
    addOption(Option option)
    {
        options_.addElement(option);

        int size = sumStringLength(option.getLabels()) +
                   option.getLabels().length * 2 +
                   sumStringLength(option.getArguments()) +
	           option.getArguments().length;

        if(size > maxOptionLength_)
        {
            maxOptionLength_ = size;
        }
    }

    /**
     * Parses given command line arguments and 
     * feeds the command line option objects.
     *
     * @param args The command line arguments.
     *
     * @return The command line arguments not parsed.
     */
    public String[]
    parse(String[] args)
    {
        // Creates a command line arguments iterator.
        DefaultIterator argumentsIterator = new DefaultIterator(args, this);

        // Iterates over all command line arguments.
        for(; argumentsIterator.hasNext(); )
        {
            String currentArgument = argumentsIterator.next();

            // Iterates over all command line options.
            for(java.util.Iterator iterator=options_.iterator();
                iterator.hasNext(); )
            {
                Option option = (Option)iterator.next();
                if(option.check(currentArgument))
                {
                    argumentsIterator.remove();
                    try
                    {
                        option.consume(argumentsIterator);
                    }
                    catch(Error exc)
                    {
		      exit(exc.getMessage() + " for option " +
                           OptionHelper.toString(option));
                    }
                    break;
		}
            }
        }

        // Checks if all mandatory options are set during parsing.
        //
        // Iterates over all command line options.
        //
        for(java.util.Iterator iterator=options_.iterator();
            iterator.hasNext(); )
        {
            Option option = (Option)iterator.next();

            // Checks if the option is mandatory and not set.
            if(option.isMandatory() && !option.isSet())
            {
                exit("Missed mandatory option " +
                     OptionHelper.toString(option));
	    }
	}

        // Obtains unparsed command line arguments.
        String[] arguments = argumentsIterator.getUnparsedArguments();

        // Checks if enough.
        if(arguments.length < getArguments().length)
            exit("Not enough arguments");

        if(arguments.length > getArguments().length)
	{
            exit("Too many arguments");
        }

        // Returns unparsed command line arguments.
        return arguments;
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
