/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

import org.objectweb.util.cmdline.api.Iterator;

/**
 * This is a default abstract implementation of the
 * org.objectweb.util.cmdline.api.Option interface.
 *
 * Subclasses must implement the consume method.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class DefaultOptionBase
        extends DefaultUsage
     implements org.objectweb.util.cmdline.api.Option
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The mandatory flag. */
    protected boolean mandatory_;

    /** The set flag. */
    protected boolean set_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * It inits the Labels, Arguments, and Description usage properties
     * with an empty string.
     */
    public
    DefaultOptionBase()
    {
        this("", new String[0], null, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param argument The usage argument.
     * @param description The usage description.
     * @param is_mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String label,
                      String argument,
                      String[] description)
    {
        this(label, new String[] { argument }, description, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param is_mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String label,
                      String[] arguments,
                      String[] description)
    {
        this(label, arguments, description, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param argument The usage argument.
     * @param description The usage description.
     * @param is_mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String[] labels,
                      String argument,
                      String[] description)
    {
        this(labels, new String[] { argument }, description, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param is_mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String[] labels,
                      String[] arguments,
                      String[] description)
    {
        this(labels, arguments, description, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param argument The usage argument.
     * @param description The usage description.
     * @param is_mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String[] labels,
                      String argument,
                      String description)
    {
        this(labels, new String[] { argument },
             new String[] { description }, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param is_mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String[] labels,
                      String[] arguments,
                      String description)
    {
        this(labels, arguments, new String[] { description }, false);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param argument The usage argument.
     * @param description The usage description.
     * @param mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String label,
                      String argument,
                      String[] description,
                      boolean mandatory)
    {
        this(new String[] { label }, new String[] { argument },
             description, mandatory);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param label The usage label.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String label,
                      String[] arguments,
                      String[] description,
                      boolean mandatory)
    {
        this(new String[] { label }, arguments, description, mandatory);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param argument The usage argument.
     * @param description The usage description.
     * @param mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String[] labels,
                      String argument,
                      String[] description,
                      boolean mandatory)
    {
        this(labels, new String[] { argument }, description, mandatory);
    }

    /**
     * The constructor with the initial values for the Labels,
     * Arguments, and Description usage properties.
     *
     * @param labels The usage labels.
     * @param arguments The usage arguments.
     * @param description The usage description.
     * @param mandatory True if a mandatory option.
     */
    public
    DefaultOptionBase(String[] labels,
                      String[] arguments,
                      String[] description,
                      boolean mandatory)
    {
        super(labels, arguments, description);
        setMandatory(mandatory);
        set_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Consumes a command line argument from an iterator.
     *
     * @param iterator The command line argument iterator.
     *
     * @return The consumed argument.
     */
    protected String
    consumeArgument(Iterator iterator)
    {
      try {
          String argument = iterator.next();
          iterator.remove();
          return argument;
      } catch(java.util.NoSuchElementException exc) {
          throw new Error("Missed argument");
      }
    }

    /**
     * Checks if the option was already set.
     *
     * @param iterator The command line argument iterator.
     */
    protected void
    checkAlreadySet(org.objectweb.util.cmdline.api.Iterator iterator)
    {
      if(set_)
          throw new Error("Twice uses");
      set_ = true;
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Is it a mandatory option?
     *
     * @return true is it is a mandatory option.
     */
    public boolean
    isMandatory()
    {
       return mandatory_;
    }

    /**
     * Sets the mandatory option status.
     *
     * @param mandatory true if it is a mandatory option.
     */
    public void
    setMandatory(boolean mandatory)
    {
       mandatory_ = mandatory;
    }

    /**
     * Obtains the Set status of the option.
     *
     * @return true the option was set during
     *         the command line argument parsing.
     */
    public boolean
    isSet()
    {
       return set_;
    }

    /**
     * Checks the current command line argument.
     *
     * @param current The current command line argument.
     *
     * @return true if the option accepts this argument.
     */
    public boolean
    check(String current)
    {
        String[] labels = getLabels();
        for(int i=0; i<labels.length; i++)
	{
            if(current.equals(labels[i]))
                return true;
	}

        return false;
    }

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     *
     * @return true if the option has consumed arguments.
     */
    abstract public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator);

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
