/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.zeusx;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.BitSet;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.ZeusDefaults;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.BaseBinding;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.util.CapitalizationUtils;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;
import org.enhydra.zeus.util.XMLName;

import org.enhydra.zeus.generator.*;

//import org.objectweb.ccm.descriptor.guirepres.*;

/**
 * <p>
 *  <code>{@link Generator}</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a 
 *    <code>File</code> [directory]).
 * </p><p>
 *  <code>ObjectWebBeansGenerator</code> is the ObjectWeb XmlTool additional
 *    <code>Generator</code> and takes in either a single <code>Binding</code>
 *    or a <code>List</code> of them, and directly generates Java classes.
 * </p>
 *
 * @author Christophe Contreras
 */
public class ObjectWebBeansGenerator implements Generator {

    /** A list of bindings to ignore */
    protected List ignoredBindings;
    
    /** A list of classes to import for all interfaces */
    protected List globalInterfaceImports;
    
    /** A list of classes to import for all implementations */
    protected List globalImplementationImports;
    
    /** This instances's collapsed elements */
    protected List collapsedElements;
    
    /** Base directory for generated files. */
    protected File outputDir;
    
    /** the package where we'll find the interfaces */
    protected String interfacesPackage;
    
    /** Keeps track of whether the Generator needs to be reinitialized. */
    protected boolean initialized;

    /**
     * <p>
     *  This will construct a new instance of <code>ObjectWebBeansGenerator</code>.
     * </p>
     */
    public ObjectWebBeansGenerator(String wantedInterfacesPackage) {
        initialized = false;
        ignoredBindings = new LinkedList();
        globalInterfaceImports = new LinkedList();
        globalImplementationImports = new LinkedList();
        collapsedElements = new LinkedList();
        
	this.interfacesPackage = wantedInterfacesPackage;
	if ( this.interfacesPackage != "" ) {
		this.interfacesPackage = this.interfacesPackage + ".";
	}

        // Add some required imports for all interfaces
/*        globalInterfaceImports.add("java.io.File");
        globalInterfaceImports.add("java.io.IOException");
        globalInterfaceImports.add("java.io.OutputStream");
        globalInterfaceImports.add("java.io.Writer");
*/        
        // Add some required imports for all implementations
        globalImplementationImports.add("java.io.File");
        globalImplementationImports.add("java.io.FileReader");
        globalImplementationImports.add("java.io.FileWriter");
        globalImplementationImports.add("java.io.IOException");
        globalImplementationImports.add("java.io.Reader");
        globalImplementationImports.add("java.io.Writer");
        globalImplementationImports.add("java.util.Map");
        globalImplementationImports.add("org.xml.sax.EntityResolver");
        globalImplementationImports.add("org.xml.sax.ErrorHandler");
        globalImplementationImports.add("org.xml.sax.InputSource");
        globalImplementationImports.add("org.xml.sax.SAXException");
        globalImplementationImports.add("org.xml.sax.XMLReader");
        globalImplementationImports.add("org.xml.sax.helpers.XMLReaderFactory");
	
    }

    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param baseDir the name of the base directory where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(String outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = new File(outputDir);
        uninitialize();
    }
    
    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the <code>File</code> (directory) where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(File outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = outputDir;
        uninitialize();
    }

    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     * @param xmlNamespaceURI the XML namespace URI of the binding to ignore. 
     *        The empty string indicates there is no namespace associated 
     *        with the binding.
     */
    public void setIgnoreBinding(String xmlName, String xmlNamespaceURI) {
        ignoredBindings.add(new XMLName(xmlName, xmlNamespaceURI));
    }
    
    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur. This
     *    convenience version supplies an empty string for the namespace URI.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     */
    public void setIgnoreBinding(String xmlName) {
        setIgnoreBinding(xmlName, "");
    }
    
    /**
     * This will determine if a particular binding should be ignored. The
     * binding is identified by its XML local name and its XML namespace URI.
     *
     * @param xmlName the XML local name of the binding.
     * @param xmlNamespaceURI the XML namespace URI associated with the binding. 
     *        An empty string indicates that there is no namespace associated 
     *        with the binding.
     *
     * @return true if and only if the binding should be ignored
     */
    protected boolean isIgnoredBinding(String xmlName, String xmlNamespaceURI) {
        if (ignoredBindings != null) {
            for (Iterator i = ignoredBindings.iterator(); i.hasNext(); ) {
                XMLName bindingName = (XMLName)i.next();
                if ((bindingName.getXMLName().equals(xmlName)) &&
                    (bindingName.getNamespaceURI().equals(xmlNamespaceURI))) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * <p>
     *  This will convert from a set of constraints to Java classes.
     *    It accesses those constraints through the
     *    <code>{@link Binding}</code> interface, which provides
     *    them in a representation-independent format, and then
     *    converts them into Java code.
     * </p>
     *
     * @param binder <code>Binder</code> with ability to convert from
     *        constraints to Zeus <code>Binding</code>s.
     * @param root whether this is the root element
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     * @throws <code>ZeusException</code> - when errors in class generation
     *         occur.
     */
    public void generate(Binding binding) 
        throws IOException, ZeusException {
            
        collapsedElements.clear();
            
        if (!isInitialized()) {
            initialize();
        }
    
        /**
         * Currently, only Containers can be generated (which
         *   includes ContainerProperties), as AtomicProperties
         *   won't result in classes.
         */
        if (binding instanceof Container) {
            Container container = (Container)binding;
            
            // Ignore Java primitive types (from collapsed elements)
            if (ClassUtils.isJavaPrimitive(binding.getJavaType())) {
                return;
            }
        
	    /** the beans classes set */
	    /** Generate bean interface */
	    generateInterfaceBean(container);

	    /** Generate the interaction bean */
	    generateImplBean(container);
	    
	    /** Generate the Listener class */
	    generateListener(container);

	    /** Generate the visual bean class */
	    generateListenerImpl(container);
	    
	    /** Generate the Event class */
	    generateEventClass(container);

	    /** Generate the Extension Manager class */
	    generateExtensionManager(container);

        } else  {
            throw new UnsupportedBindingTypeException(binding);
        }
    }

    /**
     * <p>
     *  This will generate the interface bean
     *    for the supplied <code>Container</code>.
     * </p>
     *
     * @param container <code>Container</code> to generate interfacebean from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateInterfaceBean(Container container)
        throws IOException {

        List interfaceBeanImports = new LinkedList();

        // Get a new writer
        String filename =
            new StringBuffer(container.getJavaType())
                .append("Bean.java")
                .toString();
        File outputFile = new File(getDir(container), filename);
        FileWriter writer = new FileWriter(outputFile);

        // Write out the source file header comment
        writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);

        // Write the package declaration, if needed
        writePackageDeclaration(writer, container.getJavaInterfacePackage());

        // Adds the global imports
        for (Iterator i = globalInterfaceImports.iterator(); i.hasNext(); ) {
            interfaceBeanImports.add((String)i.next());
        }

        /**
         * If any of the properties do not have a Java package then we
         *   must import them, unless they are primitive types.
         */
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getJavaInterfacePackage().equals("")) {
                if (!ClassUtils.isJavaPrimitive(property.getJavaType())) {
                    interfaceBeanImports.add(property.getJavaType()+"Bean");
                }
            }

            if (property.isCollection()) {
                String collectionClass = property.getJavaCollectionClass();
                if (!interfaceBeanImports.contains(collectionClass)) {
                    interfaceBeanImports.add(collectionClass);
                }
            }
        }

        // Write out import statements
        writeImportStatements(writer, interfaceBeanImports);

        // Write out class header
        writeDeclaration(writer, container,
		"interface",
		"Bean",
		container.getJavaType()+","+this.interfacesPackage+"Bean",
		"");

	writer.write("    public void add");
	writer.write(container.getJavaType());
	writer.write("Listener(");
	writer.write(container.getJavaType());
	writer.write("Listener l);\n");
	writer.write("    public void remove");
	writer.write(container.getJavaType());
	writer.write("Listener(");
	writer.write(container.getJavaType());
	writer.write("Listener l);\n");

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This will generate the implementation bean 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateImplBean(Container container) 
        throws IOException {
	
	List implBeanImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "ImplBean.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());
				
        // Adds the global imports
        for (Iterator i = globalImplementationImports.iterator(); i.hasNext(); ) {
            implBeanImports.add((String)i.next());
        }

        /**
         * If any of the properties do not have a Java package then we
         *   must import them, unless they are primitive types.
         */
        boolean firstCollection = true;
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getJavaInterfacePackage().equals("")) {
                if (!ClassUtils.isJavaPrimitive(property.getJavaType())) {
                    implBeanImports.add(property.getJavaType()+"Bean");
                }
            }
            
            if (property.isCollection()) {
                if (firstCollection) {
                    implBeanImports.add("java.util.Iterator");
                    firstCollection = false;
                }
                String collectionClass = property.getJavaCollectionClass();
                if (!implBeanImports.contains(collectionClass)) {
                    implBeanImports.add(collectionClass);
                    implBeanImports.add(ClassUtils.getCollectionImplClass(
                        collectionClass));
                }
            }
        }

	implBeanImports.add("java.util.ArrayList");
//	implBeanImports.add(this.interfacesPackage+"GUIManager");
	implBeanImports.add(this.interfacesPackage+"ExtensionManager");

        // Write out import statements (nearly the same as in Implementation)
        writeImportStatements(writer, implBeanImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"ImplBean",
		container.getJavaType()+"Impl",
		container.getJavaType()+"Bean");

        // Write out each property
        writeImplBeanProperties(writer, container);

        // Write out unmarshaller properties
        writeImplBeanUnmarshallerProperties(writer, container);

        // Write class constructor
        writeImplBeanConstructor(writer, 
                                       container.getJavaType(), 
                                       properties);       
        
	// Write out the modifier main method for Zeus interaction
	writeImplBeanMethods(writer, container);

        // Write out the implementation of the unmarshal() methods
        writeImplBeanMarshalUnmarshalMethods(writer, container);

        // Write out SAX Handler methods
        writeImplBeanSAXHandlerMethods(writer, container);

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the listener interface for interaction in beans
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateListener(Container container) 
        throws IOException {
	
	// Get a new writer
	String filename = container.getJavaType() + "Listener.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());
	
	writeDeclaration(writer,container,
		"interface",
		"Listener",
		"java.util.EventListener",
		"");

	writer.write("    public abstract void refreshData(");
	writer.write(container.getJavaType());
	writer.write("ModificationEvent e);\n");

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the listener implementation for interaction in beans
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateListenerImpl(Container container) 
        throws IOException {
	
	List listenerImplImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "ListenerImpl.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

        // Write out import statements
        writeImportStatements(writer, listenerImplImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"ListenerImpl",
		"",
		container.getJavaType()+"Listener");

	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("GUI _client;\n\n");

        writer.write("    public ");
        writer.write(container.getJavaType());
        writer.write("ListenerImpl(");
	writer.write(container.getJavaType());
	writer.write("GUI client) {\n");
	writer.write("        _client = client ;\n");
	writer.write("        ((");
	writer.write(container.getJavaType());
	writer.write("Bean)_client.getManager().getSource()).add");
	writer.write(container.getJavaType());
	writer.write("Listener(this);\n");
	writer.write("    }\n\n");

	writer.write("    public void refreshData(");
	writer.write(container.getJavaType());
	writer.write("ModificationEvent event) {\n");
	writer.write("        _client.refresh();\n");
	writer.write("    }\n\n");

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the event class for interaction in beans
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateEventClass(Container container) 
        throws IOException {
	
	// Get a new writer
	String filename = container.getJavaType() + "ModificationEvent.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());
	
	// write oui the class code
	writer.write("public class ");
	writer.write(container.getJavaType());
	writer.write("ModificationEvent extends java.util.EventObject {\n");
	writer.write("    public ");
	writer.write(container.getJavaType());
	writer.write("ModificationEvent (");
	writer.write(container.getJavaType());
	writer.write("ImplBean source) {\n");
	writer.write("        super(source);\n");
	writer.write("    }\n\n");

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }
	
    /**
     * <p>
     *  This will generate the ExtensionManager class 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateExtensionManager(Container container)
	throws IOException {

	List extManagerImports = new LinkedList();
	
	// Get a new writer
	String filename = container.getJavaType() + "ExtensionManagerImpl.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

	extManagerImports.add(this.interfacesPackage+"*");
	extManagerImports.add("java.util.Iterator");
	extManagerImports.add("java.util.List");
	extManagerImports.add("java.util.LinkedList");

        // Write out import statements
        writeImportStatements(writer, extManagerImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"ExtensionManagerImpl",
		"",
		"ExtensionManager");

	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("Bean _source;\n\n");
	writer.write("    private List extensionList;\n");

	/** Constructor */
        writer.write("    public ");
        writer.write(container.getJavaType());
        writer.write("ExtensionManagerImpl(");
	writer.write(container.getJavaType());
	writer.write("ImplBean source) {\n");
	writer.write("        _source = source ;\n");
	writer.write("        extensionList = new LinkedList();\n");
	writer.write("    }\n\n");

	/** set get methods */
	writer.write("    public ");
	writer.write("Bean getSource() {\n");
	writer.write("        return (_source);\n");
	writer.write("    }\n\n");

	writer.write("    public ");
	writer.write(this.interfacesPackage);
	writer.write("Extension getExtensionByName(String wantedExtension) {\n");
	writer.write("        ");
	writer.write(this.interfacesPackage);
	writer.write("ExtensionImpl extension = null;\n");
	writer.write("        for (Iterator i = extensionList.iterator() ; i.hasNext() ; ) {\n");
	writer.write("            extension = (");
	writer.write(this.interfacesPackage);
	writer.write("ExtensionImpl) ( i.next() );\n");
	writer.write("            if (extension.toString().startsWith(wantedExtension))\n");
	writer.write("                return (extension);\n");
	writer.write("        }\n");
	writer.write("        // Extension not found, need a new instance\n");
	writer.write("        this.addExtension (wantedExtension);\n");
	writer.write("        return (this.getExtensionByName(wantedExtension));\n");
	writer.write("    }\n\n");

	writer.write("    public void addExtension (String wantedExtensionClass) {\n");
	writer.write("        try {\n");
	writer.write("            ");
	writer.write(this.interfacesPackage);
	writer.write("ExtensionImpl extension = (");
	writer.write(this.interfacesPackage);
	writer.write("ExtensionImpl) ( Class.forName(\"");
	writer.write(container.getJavaInterfacePackage());
	writer.write(".");
	writer.write(container.getJavaType());
	writer.write("\" + wantedExtensionClass + \"Accessor\").newInstance() );\n");
	writer.write("            extension.setManager(this);\n");
/*	writer.write("            ((");
	writer.write(this.interfacesPackage);
	writer.write(".GUI) extension).startListening();\n");
*/
	writer.write("            extensionList.add( (");
	writer.write(this.interfacesPackage);
	writer.write("ExtensionImpl) extension );\n");
	writer.write("        } catch (Exception e) {\n");
	writer.write("            e.printStackTrace();\n");
	writer.write("        }\n");
	writer.write("    }\n");

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This writes out the package declaration for this class, if needed.
     * </p>
     *
     * @param writer <code>Writer</code> to perform output to
     * @param packageName the package of this class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writePackageDeclaration(Writer writer, String packageName) 
        throws IOException {
            
        // Generate a package statement if necessary.
        if (!packageName.equals("")) {
            writer.write("package");
            writer.write(" ");
            writer.write(packageName);
            writer.write(";\n\n");
        }
    }
    
    /**
     * <p>
     *  This will take the supplied list of class names, and add import
     *    statements for each of the classes in the list.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param importClasses <code>List</code> of classes to add imports for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImportStatements(Writer writer, 
                                                  List importClasses) 
        throws IOException {
        
        // Write out imports
        if (importClasses.size() > 0) {
            writer.write("// Local Interface Import Statements\n");
        }
        for (Iterator i = importClasses.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (importClasses.size() > 0) {
            writer.write("\n");
        }
    }
    
    /**
     * <p>
     *  This writes out the interface bean declaration (the class header).
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write the interface for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeDeclaration(Writer writer, 
                                             Container container,
					     String modifier,
					     String suffix,
					     String extended,
					     String implemented)
        throws IOException {
            
        writer.write("public ");
	writer.write(modifier);
	writer.write(" ");
        writer.write(container.getJavaType());
	writer.write(suffix);
	if (!extended.equals("")) {
	    writer.write(" extends ");
	    writer.write(extended);
	}
	
	if (!implemented.equals("")) {
	    writer.write(" implements ");
	    writer.write(implemented);
	}
	
	writer.write(" {\n\n");
    }
    
    /**
     * <p>
     *  This writes out the properties for an interaction bean as variables.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param properties <code>List</code> of properties to write out as
     *        member variables
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplBeanProperties(Writer writer, Container container)
        throws IOException {

//	writer.write("    private ");
//	writer.write(container.getJavaType());
//	writer.write("GUIManager _GUIManager;\n");
	writer.write("    private ArrayList ");
	writer.write(container.getJavaVariableName());
	writer.write("Listeners = new ArrayList();\n");
	writer.write("    private ");
//	writer.write(container.getJavaType());
	writer.write("ExtensionManager _extManager;\n");

    }

    /**
     * <p>
     *  This will write out properties required for unmarshalling from XML
     *    into this Java class.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container writing out for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplBeanUnmarshallerProperties(
        Writer writer, Container container) throws IOException {
	
        writer.write("    /** The current node in unmarshalling */\n");
        writer.write("    private Unmarshallable zeus_currentUNode;\n\n");
        
        writer.write("    /** The parent node in unmarshalling */\n");
        writer.write("    private Unmarshallable zeus_parentUNode;\n\n");
        
        writer.write("    /** Whether this node has been handled */\n");
        writer.write("    private boolean zeus_thisNodeHandled = false;\n\n");
        
        writer.write("    /** Whether a DTD exists for an unmarshal call */\n");
        writer.write("    private boolean hasDTD;\n\n");
        
        writer.write("    /** Whether validation is occurring */\n");
        writer.write("    private boolean validate;\n\n");
        
        writer.write("    /** The namespace mappings on this element */\n");
        writer.write("    private Map namespaceMappings;\n\n");
        
        writer.write("    /** The EntityResolver for SAX parsing to use */\n");
        writer.write("    private static EntityResolver entityResolver;\n\n");
        
        writer.write("    /** The ErrorHandler for SAX parsing to use */\n");
        writer.write("    private static ErrorHandler errorHandler;\n\n");
	
    }

    /**
     * <p>
     *  This writes out the constructor for a Bean class
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param className the name of the Bean class
     * @param properties <code>List</code> of properties to write out as
     *        member variables
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplBeanConstructor(Writer writer, 
                                                  String className,
                                                  List properties)
        throws IOException {

        BitSet modifiers = null;

        writer.write("    public ");
        writer.write(className);
        writer.write("ImplBean() {\n");

	writer.write("        super();\n");
//	writer.write("        this._GUIManager = new ");
//	writer.write(className);
//	writer.write("GUIManagerImpl(this);\n");
	writer.write("        this._extManager = new ");
	writer.write(className);
	writer.write("ExtensionManagerImpl(this);\n");

        writer.write("    }\n\n"); 
    }

    /**
     * <p>
     *  This writes out the specific beans methods for the Impl Bean
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param className the name of the Bean class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplBeanMethods(Writer writer, 
                                                  Container container)
        throws IOException {
	
	writer.write("    synchronized public void add");
	writer.write(container.getJavaType());
	writer.write("Listener(");
	writer.write(container.getJavaType());
	writer.write("Listener l) {\n");
	writer.write("        ");
	writer.write(container.getJavaVariableName());
	writer.write("Listeners.add(l);\n");
	writer.write("    }\n\n");
	
	writer.write("    synchronized public void remove");
	writer.write(container.getJavaType());
	writer.write("Listener(");
	writer.write(container.getJavaType());
	writer.write("Listener l) {\n");
	writer.write("        ");
	writer.write(container.getJavaVariableName());
	writer.write("Listeners.remove(l);\n");
	writer.write("    }\n\n");
	
	writer.write("    public void notifyListeners() {\n");
	writer.write("        ");
	writer.write(container.getJavaType());
	writer.write("ModificationEvent modifEvent = new ");
	writer.write(container.getJavaType());
	writer.write("ModificationEvent (this);\n");
	writer.write("        ArrayList lv = null;\n");
	writer.write("        synchronized(this) { lv = (ArrayList)");
	writer.write(container.getJavaVariableName());
	writer.write("Listeners.clone(); }\n");
	writer.write("        for (int i = 0 ; i < lv.size() ; i++)\n");
	writer.write("            ((");
	writer.write(container.getJavaType());
	writer.write("Listener) lv.get(i)).refreshData(modifEvent);\n");
	writer.write("    }\n\n");
	
	writer.write("    public ");
	writer.write("ExtensionManager getExtensionManager() {\n");
	writer.write("        return (_extManager);\n");
	writer.write("    }\n\n");

        List properties = container.getProperties();

	writer.write("    public java.util.Vector getAddableChildren () {\n");
	writer.write("        java.util.Vector addableChildren = new java.util.Vector();\n\n");
	for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();

	    if (property.getModifier().get(Property.SOURCE_ELEMENT))
		if (property.isCollection()) {
		    writer.write("        addableChildren.addElement(\"");
		    writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		    writer.write("\");\n");
		}
		else {
		    writer.write("        if ( get");
		    writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		    writer.write("() == null)\n");
		    writer.write("            addableChildren.addElement(\"");
		    writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		    writer.write("\");\n");
		}
	}

	writer.write("        return (addableChildren);\n");
	writer.write("    }\n\n");
    }

    /**
     * <p>
     *  This will write out the implementation of the various 
     *    <code>unmarshal</code> methods supported by generated bean classes.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write methods for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplBeanMarshalUnmarshalMethods(Writer writer,
                                                       Container container)
        throws IOException {
            
        String className = container.getJavaType();
        String variableName = container.getJavaVariableName();
        
        // marshal(File file)
        writer.write("    public void marshalBean(File file)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the marshal(File) method\n");
        writer.write("        marshal(file);\n");
        writer.write("    }\n\n");

        // unmarshal(File file)
        writer.write("    public static ");
        writer.write(className);
        writer.write("Bean unmarshalBean(File file)");
        writer.write(" throws IOException {\n");
        writer.write("        // Delegate to the unmarshalBean(Reader) method\n");
        writer.write("        return unmarshalBean(new FileReader(file), true);\n");
        writer.write("    }\n\n");

        // unmarshal(Reader reader, boolean validate)
        writer.write("    public static ");
        writer.write(className);
        writer.write("Bean unmarshalBean(Reader reader, boolean validate)");
        writer.write(" throws IOException {\n");

        // Create new instance
        writer.write("        ");
        writer.write(className);
        writer.write("ImplBean ");
        writer.write(variableName);
        writer.write(" = new ");
        writer.write(className);
	writer.write("ImplBean();\n");
        
        // Set validation state
        writer.write("        ");
        writer.write(variableName);
        writer.write(".setValidating(validate);\n");
        
        // Set up class heirarchy
        writer.write("        ");
        writer.write(variableName);
        writer.write(".setCurrentUNode(");
        writer.write(variableName);
        writer.write(");\n");
        writer.write("        ");
        writer.write(variableName);
        writer.write(".setParentUNode(null);\n");
            
        // Load the XML parser
        writer.write("        // Load the XML parser\n");
        writer.write("        XMLReader parser = null;\n");
        writer.write("        String parserClass = System.getProperty(");
        writer.write("\"org.xml.sax.driver\",\n");
        writer.write("            \"");
        writer.write(ZeusDefaults.DEFAULT_PARSER);
        writer.write("\");\n");
        writer.write("        try {\n");
        writer.write("            parser = ");
        writer.write("XMLReaderFactory.createXMLReader(parserClass);\n\n");
        writer.write("            // Set entity resolver, if needed\n");
        writer.write("            if (entityResolver != null) {\n");
        writer.write("                parser.setEntityResolver(");
        writer.write("entityResolver);\n");
        writer.write("            }\n\n");
        writer.write("            // Set error handler\n");
        writer.write("            parser.setErrorHandler(");
        writer.write(variableName);
        writer.write(");\n\n");
        writer.write("            // Register lexical handler\n");
        writer.write("            parser.setProperty(\"");
        writer.write("http://xml.org/sax/properties/lexical-handler");
        writer.write("\", ");
        writer.write(variableName);
        writer.write(");\n\n");
        writer.write("            // Register content handler\n");
        writer.write("            parser.setContentHandler(");
        writer.write(variableName);
        writer.write(");\n");
        writer.write("        } catch (SAXException e) {\n");
        writer.write("            throw new IOException(");
        writer.write("\"Could not load XML parser: \" + \n");
        writer.write("                e.getMessage());\n");
        writer.write("        }\n\n");
        
        // Get the input source
        writer.write("        InputSource inputSource = new InputSource(");
        writer.write("reader);\n");
        writer.write("        try {\n");
            
        // Set validation
        writer.write("            parser.setFeature(");
        writer.write("\"http://xml.org/sax/features/validation\", ");
        writer.write("new Boolean(validate).booleanValue());\n");
        
        // Set namespace processing
        writer.write("            parser.setFeature(");
        writer.write("\"http://xml.org/sax/features/namespaces\", ");
        writer.write("true);\n");
        writer.write("            parser.setFeature(");
        writer.write("\"http://xml.org/sax/features/namespace-prefixes\", ");
        writer.write("false);\n");
        
        // Parse
        writer.write("            parser.parse(inputSource);\n");
        writer.write("        } catch (SAXException e) {\n");
        writer.write("            throw new IOException(");
        writer.write("\"Error parsing XML document: \" + \n");
        writer.write("                e.getMessage());\n");
        writer.write("        }\n\n");
        
        // Return the resulting object
        writer.write("        // Return the resultant object\n");
//        writer.write("        System.out.println(");
//	writer.write(variableName);
//	writer.write(".getClass().toString());\n");
	writer.write("        return ");
        writer.write(variableName);
        writer.write(";\n");
        writer.write("    }\n\n");
        
    }

    /**
     * <p>
     *  This will write out methods that implements various SAX handlers
     *    for use in unmarshalling.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write class for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImplBeanSAXHandlerMethods(Writer writer, 
                                                        Container container)
        throws IOException {
            
        String className = 
            CapitalizationUtils.initialUpper(container.getJavaName());
        String variableName = container.getJavaVariableName();

        // SAX startElement()
        writer.write("    public void startElement(String namespaceURI, ");
        writer.write("String localName,\n");
        writer.write("                             String qName, ");
        writer.write("org.xml.sax.Attributes atts)\n");
        writer.write("        throws SAXException {\n\n");
            
        writer.write("        // Feed this to the correct ContentHandler\n");
        writer.write("        Unmarshallable current = getCurrentUNode();\n");
        writer.write("        if (current != this) {\n");
        writer.write("            current.startElement(");
        writer.write("namespaceURI, localName, qName, atts);\n");
        writer.write("            return;\n");
        writer.write("        }\n\n");

        writer.write("        // See if we handle, or we delegate\n");
        writer.write("        if ((localName.equals(\"");
        writer.write(container.getXMLName());
        writer.write("\")) && (!zeus_thisNodeHandled)) {\n");
            
        writer.write("            // Handle ourselves\n");
        writer.write("            for (int i=0, len=atts.getLength(); i<len;");
        writer.write(" i++) {\n");
        writer.write("                String attName");
        writer.write("= atts.getLocalName(i);\n");
        writer.write("                String attValue = atts.getValue(i);\n");
        
        // Write out conditionals for this binding's attributes
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            BitSet modifiers = property.getModifier();
            String propertyXMLName = property.getXMLName();
            
            // Only delegate to child elements
            if ((modifiers.get(Property.SOURCE_ATTLIST)) &&
                (!modifiers.get(Property.MUTABILITY_FINAL)) &&
                (!propertyXMLName.startsWith("xmlns"))) {
                writer.write("                ");
                writer.write("if (attName.equals(\"");
                writer.write(propertyXMLName);
                writer.write("\")) {\n");
                writer.write("                    set");
                writer.write(CapitalizationUtils.initialUpper(
                    property.getJavaName()));
                writer.write("(attValue);\n");
                writer.write("                }\n");
            }
        }
        
        writer.write("            }\n");
        writer.write("            zeus_thisNodeHandled = true;\n");
        writer.write("            return;\n");
        writer.write("        } else {\n");
        writer.write("            // Delegate handling\n");
        
        // Delegation to properties
        String pcDataVariableName = null;      
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            BitSet modifiers = property.getModifier();
            String propertyXMLName = property.getXMLName();
            
            // Only delegate to child elements
            if (modifiers.get(Property.SOURCE_ELEMENT)) {
                String propertyClassName = 
                    CapitalizationUtils.initialUpper(property.getJavaName());
                String propertyVariableName = property.getJavaVariableName();
                
                writer.write("            if (localName.equals(\"");
                writer.write(propertyXMLName);
                writer.write("\")) {\n");
                    
                if (collapsedElements.contains(property)) {
                    writer.write("                this.zeus_in");
                    writer.write(CapitalizationUtils.initialUpper(
                        propertyVariableName));
                    writer.write(" = true;\n");
                } else {
                    writer.write("                ");
                    writer.write(property.getJavaType());
                    writer.write("ImplBean ");
                    writer.write(propertyVariableName);
                    writer.write(" = new ");
                    writer.write(property.getJavaType());
                    writer.write("ImplBean();\n");
                
                    // Set up this new object's delegation heirarchy
                    writer.write("                current = ");
                    writer.write("getCurrentUNode();\n");
                    writer.write("                ");
                    writer.write(propertyVariableName);
                    writer.write(".setParentUNode(current);\n");
                    writer.write("                ");
                    writer.write(propertyVariableName);
                    writer.write(".setCurrentUNode(");
                    writer.write(propertyVariableName);
                    writer.write(");\n");
                    writer.write("                this.setCurrentUNode(");
                    writer.write(propertyVariableName);
                    writer.write(");\n");
                    writer.write("                ");
                    writer.write(propertyVariableName);
                    writer.write(".startElement(namespaceURI, localName, ");
                    writer.write("qName, atts);\n");
                
                    // Add the (populated) object value
                    writer.write("                // Add this value in\n");
                    if (property.isCollection()) {
                        writer.write("                this.get");
                	writer.write(CapitalizationUtils.initialUpper(
                    	    propertyVariableName));
                        writer.write("List().add(");
                        writer.write(propertyVariableName);
                        writer.write(");\n");
                    } else {
                        writer.write("                this.set");
                	writer.write(CapitalizationUtils.initialUpper(
                    	    propertyVariableName));
                        writer.write("(");
                        writer.write(propertyVariableName);
                        writer.write(");\n");
                    }
                }
                
		writer.write("                ((javax.swing.tree.DefaultMutableTreeNode) ((");
		writer.write(container.getJavaType());
		writer.write("NodeAccessor) this.getExtensionManager().getExtensionByName(\"Node\") ).getInstance()).add( (javax.swing.tree.DefaultMutableTreeNode) ((");
        	writer.write(property.getJavaType());
		writer.write("NodeAccessor) ");
		writer.write(propertyVariableName);
		writer.write(".getExtensionManager().getExtensionByName(\"Node\") ).getInstance());\n");
                writer.write("                return;\n");
                writer.write("            }\n");
            } else if (propertyXMLName.equals(ZeusDefaults.PCDATA_XML_NAME)) {
                pcDataVariableName = property.getJavaVariableName();
            }
        }
        
        writer.write("        }\n");
        writer.write("    }\n\n");
	
    }

    
    /**
     * <p>
     *  This will initialize this instance of <code>Generator</code>,
     *    handling tasks like directory creation.
     * </p>
     *
     * @throws <code>IllegalOutputDirectoryException</code> - when an illegal
     *         output directory has been supplied to this <code>Generator</code>
     */    
    private void initialize() throws IllegalOutputDirectoryException {        
        // Determine the output directory
        if (outputDir == null) {
            outputDir = new File(".");
        }
        
        // Ensure the output directory exists and is a directory
        if ((outputDir.exists()) && (!outputDir.isDirectory())) {
            throw new IllegalOutputDirectoryException(
                outputDir.getAbsolutePath());
        }

        // Create the directory if it does not exist
        if (!outputDir.exists()) {
            outputDir.mkdir();
        }

        initialized = true;
    }        

    /**
     * <p>
     *  This will reset the initialization state of this <code>Generator</code>
     *    instance to <code>false</code>.
     * </p>
     */    
    private void uninitialize() {
        initialized = false;
    }
    
    /**
     * <p>
     *  This will indicate if this <code>Generator</code> instance has been
     *    initialized.
     * </p>
     *
     * @return <code>boolean</code> - whether this instance is initialized.
     */
    private boolean isInitialized() {
        return initialized;
    }
    
    /**
     * <p>
     *  This will determine the directory to use for generation
     *    based on the packages of a <code>{@link Container}</code>.
     * </p>
     *
     * @param container the container to obtain the directory for.
     * @return <code>File</code> - the directory for generation.
     */
    private File getDir(Container container) {
        StringBuffer dirName = new StringBuffer();
        String thisPackage = container.getJavaInterfacePackage();

        for (int i=0, len=thisPackage.length(); i<len; i++) {
            char ch = thisPackage.charAt(i);
            if (ch != '.') {
                dirName.append(ch);
            } else {
                dirName.append(System.getProperty("file.separator"));
            }
        }
        
        File dir = 
            new File(outputDir, dirName.toString());
        
        // Create the directory if it does not exist
        if (!dir.exists()) {
            dir.mkdirs();
        }
        
        return dir;
    }
}
