/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.zeusx;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binder;
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.Transformer;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.binder.DTDBinder;
import org.enhydra.zeus.generator.SimpleGenerator;
import org.enhydra.zeus.source.DTDSource;
import org.enhydra.zeus.source.StreamDTDSource;
import org.enhydra.zeus.transform.DefaultsTransformer;

import org.enhydra.zeus.util.*;

/**
 * <p>
 *  This is the class used by the ObjectWeb XmlTool.
 *    It allows the call of simple generation from a DTD,
 *    and calls the ObjectWeb Generator to gnerate additional classes.
 * </p>
 *
 * @author Christophe Contreras
 */
public class ObjectWebDTDSourceGenerator extends BaseSourceGenerator
    implements SourceGenerator {

    /** The interface package name to use. */
    private static String interfacePackageName;

    /** The root element to use */
    private String rootElementName;

    /** the package where the framework interfaces will be put in. */
    private String interfacesPackage;

	/** the directory where generation models will be found */
	private String modelsDirectory;

    /**
     * <p>
     *  This sets up defaults for the generator.
     * </p>
     */
    public ObjectWebDTDSourceGenerator() {
        super();
	interfacesPackage = "";
    }

    /**
     * <p>
     *  This defines a contract for subclasses to allow them to get the
     *    Zeus <code>{@link org.enhydra.zeus.Binding}</code> objects to
     *    generate code from.
     * </p>
     *
     * @return <code>List</code> - a list of the Binding
     *         objects to generate code from.
     * @exception <code>IOException</code> - when binding creation fails
     */
    protected List getConstraintBindings() throws IOException {

        // Create a source to read the DTD from.
        DTDSource dtdSource = new StreamDTDSource(constraintsReader);

        // Create the binder we will use.
        DTDBinder dtdBinder  = new DTDBinder(dtdSource, rootElementName);
        dtdBinder.setIsCollapsingSimpleElements(collapseSimpleElements);
        dtdBinder.setIsIgnoringIDAttributes(ignoreIDAttributes);

        // return the bindings.
        return dtdBinder.getBindings();
    }

    /**
     * <p>
     *  This handles the process of creating a <code>{@link Generator}</code>
     *    for use in data binding.
     * </p><p>
     *  This version returns determines the <code>Generator</code>
     *    implementation to use based on user input.
     * </p>
     *
     * @return <code>Generator</code> - the <code>Generator</code> to use.
     */
    protected Generator[] getGenerators(String wantedInterfacesPackage, String wantedModelsDirectory) {
	Generator[] returnedGenerators =
	    {new SimpleGenerator(),
	     new ObjectWebBeansGenerator(wantedInterfacesPackage),
	     new ObjectWebGUIsGenerator(wantedInterfacesPackage, wantedModelsDirectory),
	     new ObjectWebDeployInfoGenerator(wantedInterfacesPackage, wantedModelsDirectory)};

//        Generator generator = new SimpleGenerator();
        return returnedGenerators;
    }       
    
    /**
     * <p>
     *  This sets the name of the root element to use for generation of classes
     *    in this DTD.
     * </p>
     * 
     * @param rootElementName the root element's name to use.
     */
    public void setRootElementName(String rootElementName) {
        this.rootElementName = rootElementName;
    }

    /**
     * <p>
     *  This sets the name for the package of the interfaces for the framework
     * </p>
     *
     * @param wantedInterfacesPackage the package's name to use.
     */
    public void setInterfacesPackage(String wantedInterfacesPackage) {
        this.interfacesPackage = wantedInterfacesPackage;
    }

    /**
     * <p>
     *  This sets the directory where generation models will be looked for
     * </p>
     *
     * @param wantedModelsDirectory the directory to use.
     */
    public void setModelsDirectory(String wantedModelsDirectory) {
        this.modelsDirectory = wantedModelsDirectory;
    }

    /**
     * <p>
     *  This method performs class generation.
     * </p>
     *
     * @throws <code>IOException</code> - when class generation fails
     * @throws <code>ZeusException</code> - when class generation fails.
     */
    public void generate() throws IOException, ZeusException {
	// Generate the interfaces for the target framework
	ObjectWebInterfacesGenerator interfacesGenerator = new ObjectWebInterfacesGenerator(interfacesPackage);
	interfacesGenerator.setOutputDirectory(outputDir);
	interfacesGenerator.generate();

        // Transformer for bindings
        Transformer transformer = new DefaultsTransformer();
        transformer.getTransformerOptions().setDefaultPackage(javaPackage);

        // Generator for source code
        Generator[] generators = getGenerators(interfacesPackage, modelsDirectory);
		for (int i=0;i<generators.length;i++) {
			generators[i].setOutputDirectory(outputDir);
	}

        // Get the DTD bindings
        List bindings = getConstraintBindings();

        // Transform the bindings
        List transformedBindings = transformer.transform(bindings);


        // Generate source code
        for (Iterator i = transformedBindings.iterator(); i.hasNext(); ) {
            Binding binding = (Binding)i.next();
	    for (int j=0 ; j<generators.length ; j++) {
        	generators[j].generate(binding);
	    }
        }
    }

    /**
     * <p>
     *  This provides command-line class generation.
     * </p>
     *
     * @param args <code>String</code> array of command-line arguments.
     */
    public static void main(String[] args) {
        try {
            Arguments arguments = new Arguments(args);

            // Generate source
            ObjectWebDTDSourceGenerator sourceGenerator =
                new ObjectWebDTDSourceGenerator();

            // Ensure correct usage
            String constraintsFilename =
                arguments.getValue("constraints");
            String rootElementName =
		arguments.getValue("root");
            if (constraintsFilename == null || rootElementName == null) {
//                System.out.println("Usage: java " +
//                    sourceGenerator.getClass().getName() +
		System.out.println("Usage: ccm_dtd2java" +
                    "\n      -constraints=<constraints filename>" +
		    "\n      -root=<root element name>" +
                    "\n     [-outputDir=<output directory>]" +
//                    "\n     [-collapseSimpleElements=<true | false>]" +
//                    "\n     [-ignoreIDAttributes=<true | false>]" +
                    "\n     [-javaPackage=<Java package name>]" +
		    "\n     [-interfacesPackage=<Java package name for the framework interfaces>]" +
                    "\n     [-modelsDirectory=<the directory where to find generation models>]" +
		    "\n\ndefaults are:  -outputDir         = " + System.getProperty("openccm.homedir") + "/generated" +
		    "\n               -javaPackage       = org.objectweb.ccm.descriptor.<the root element you'll specify>" +
		    "\n               -interfacesPackage = org.objectweb.ccm.descriptor" +
                    "\n               -modelsDirectory   = " + System.getProperty("openccm.homedir") + "/xml/" +
		    "\n\n    you'll need the GUIREPRES and DEPLOYINFO classes, usually generated during the OpenCCM compilation process");
                return;
            }

            // Get command-line options
            String outputDirName = arguments.getValue("outputDir");
            String collapseSimpleElements =
                arguments.getValue("collapseSimpleElements");
            String ignoreIDAttributes =
                arguments.getValue("ignoreIDAttributes");
            String javaPackage = arguments.getValue("javaPackage");

            String wantedInterfacesPackage = arguments.getValue("interfacesPackage");

            String wantedModelsDirectory = arguments.getValue("modelsDirectory");

            sourceGenerator.setConstraintsInput(constraintsFilename);
			sourceGenerator.setRootElementName(rootElementName);

	    if (outputDirName != null) {
                sourceGenerator.setOutputDir(outputDirName);
            } else {
		sourceGenerator.setOutputDir(System.getProperty("openccm.homedir") + "/generated");
	    }

            if (javaPackage != null) {
                sourceGenerator.setJavaPackage(javaPackage);
            } else {
                sourceGenerator.setJavaPackage("org.objectweb.ccm.descriptor." + rootElementName);
            }
            if (wantedInterfacesPackage != null) {
                sourceGenerator.setInterfacesPackage(wantedInterfacesPackage);
	    } else {
		sourceGenerator.setInterfacesPackage("org.objectweb.ccm.descriptor");
	    }
	    if (wantedModelsDirectory != null) {
		sourceGenerator.setModelsDirectory(wantedModelsDirectory);
            } else {
		sourceGenerator.setModelsDirectory(System.getProperty("openccm.homedir") + "/xml/");
	    }
            if ((collapseSimpleElements != null) &&
                (collapseSimpleElements.equalsIgnoreCase("true"))) {
                if ((ignoreIDAttributes != null) &&
                    (ignoreIDAttributes.equalsIgnoreCase("true"))) {
                    sourceGenerator.setCollapseSimpleElements(true, true);
                } else {
                    sourceGenerator.setCollapseSimpleElements(true, false);
                }
            }

            // Perform actual class generation
            sourceGenerator.generate();

        } catch (IOException e) {
            System.out.println("Error generating source code: " +
                               e.getMessage());
            e.printStackTrace();
        } catch (ZeusException e) {
            System.out.println("Error generating source code: " +
                               e.getMessage());
            e.printStackTrace();
        }
    }
}
