/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.zeusx;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.BitSet;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.ZeusDefaults;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.BaseBinding;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.util.CapitalizationUtils;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;
import org.enhydra.zeus.util.XMLName;

import org.enhydra.zeus.generator.*;

//import org.objectweb.ccm.descriptor.guirepres.*;

/**
 * <p>
 *  <code>{@link Generator}</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a
 *    <code>File</code> [directory]).
 * </p><p>
 *  <code>ObjectWebGUIsGenerator</code> is the ObjectWeb XmlTool additional
 *    <code>Generator</code> and takes in either a single <code>Binding</code>
 *    or a <code>List</code> of them, and directly generates Java classes.
 * </p>
 *
 * @author Christophe Contreras
 */
public class ObjectWebGUIsGenerator implements Generator {

    /** A list of bindings to ignore */
    protected List ignoredBindings;

    /** This instances's collapsed elements */
    protected List collapsedElements;

    /** Base directory for generated files. */
    protected File outputDir;

    /** package where we'll find the interfaces. */
    protected String interfacesPackage;

    /** directory where we'll find the models. */
    protected String modelsDirectory;

    /** Keeps track of whether the Generator needs to be reinitialized. */
    protected boolean initialized;

    /**
     * <p>
     *  This will construct a new instance of <code>ObjectWebGUIsGenerator</code>.
     * </p>
     */
    public ObjectWebGUIsGenerator(String wantedInterfacesPackage, String wantedModelsDirectory) {
        initialized = false;
        ignoredBindings = new LinkedList();
        collapsedElements = new LinkedList();

		this.interfacesPackage = wantedInterfacesPackage;
		this.modelsDirectory = wantedModelsDirectory;
    }

    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param baseDir the name of the base directory where
     *        all generated classes should be placed
     */
    public void setOutputDirectory(String outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = new File(outputDir);
        uninitialize();
    }
    
    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the <code>File</code> (directory) where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(File outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = outputDir;
        uninitialize();
    }

    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     * @param xmlNamespaceURI the XML namespace URI of the binding to ignore. 
     *        The empty string indicates there is no namespace associated 
     *        with the binding.
     */
    public void setIgnoreBinding(String xmlName, String xmlNamespaceURI) {
        ignoredBindings.add(new XMLName(xmlName, xmlNamespaceURI));
    }
    
    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur. This
     *    convenience version supplies an empty string for the namespace URI.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     */
    public void setIgnoreBinding(String xmlName) {
        setIgnoreBinding(xmlName, "");
    }
    
    /**
     * This will determine if a particular binding should be ignored. The
     * binding is identified by its XML local name and its XML namespace URI.
     *
     * @param xmlName the XML local name of the binding.
     * @param xmlNamespaceURI the XML namespace URI associated with the binding. 
     *        An empty string indicates that there is no namespace associated 
     *        with the binding.
     *
     * @return true if and only if the binding should be ignored
     */
    protected boolean isIgnoredBinding(String xmlName, String xmlNamespaceURI) {
        if (ignoredBindings != null) {
            for (Iterator i = ignoredBindings.iterator(); i.hasNext(); ) {
                XMLName bindingName = (XMLName)i.next();
                if ((bindingName.getXMLName().equals(xmlName)) &&
                    (bindingName.getNamespaceURI().equals(xmlNamespaceURI))) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * <p>
     *  This will convert from a set of constraints to Java classes.
     *    It accesses those constraints through the
     *    <code>{@link Binding}</code> interface, which provides
     *    them in a representation-independent format, and then
     *    converts them into Java code.
     * </p>
     *
     * @param binder <code>Binder</code> with ability to convert from
     *        constraints to Zeus <code>Binding</code>s.
     * @param root whether this is the root element
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     * @throws <code>ZeusException</code> - when errors in class generation
     *         occur.
     */
    public void generate(Binding binding) 
        throws IOException, ZeusException {
            
        collapsedElements.clear();
            
        if (!isInitialized()) {
            initialize();
        }
    
        /**
         * Currently, only Containers can be generated (which
         *   includes ContainerProperties), as AtomicProperties
         *   won't result in classes.
         */
        if (binding instanceof Container) {
            Container container = (Container)binding;
            
            // Ignore Java primitive types (from collapsed elements)
            if (ClassUtils.isJavaPrimitive(binding.getJavaType())) {
                return;
            }
        
	    /** the GUI classes set */
	    /** Generate the GUI Manager interface */
//	    generateGUIManagerInterface(container);

	    /** Generate the GUI Manager */
//	    generateGUIManager(container);
	    
	    /** Generate the GUI interface */
	    generateGUIInterface(container);

	    /** Generate the GUI class */
	    generatePanelAccessor(container);
	    generatePanel(container);

	    /** Generate the node bean class */
	    generateNodeAccessor(container);
	    generateNode(container);

	    /** the deployment class set */
	    
        } else  {
            throw new UnsupportedBindingTypeException(binding);
        }
    }



    /**
     * <p>
     *  This will generate the GUIManager interface 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
/*    protected void generateGUIManagerInterface(Container container)
	throws IOException {

	// Get a new writer
	String filename = container.getJavaType() + "GUIManager.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"interface",
		"GUIManager",
		"org.objectweb.ccm.descriptor.GUIManager",
		"");
	
        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }
*/
    /**
     * <p>
     *  This will generate the GUIManager class 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
/*    protected void generateGUIManager(Container container)
	throws IOException {

	List guiManagerImports = new LinkedList();
	
	// Get a new writer
	String filename = container.getJavaType() + "GUIManagerImpl.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

	guiManagerImports.add("org.objectweb.ccm.descriptor.*");

        // Write out import statements
        writeImportStatements(writer, guiManagerImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"GUIManagerImpl",
		"",
		container.getJavaType()+"GUIManager");

	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("Bean _source;\n\n");

	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("GUI _node;\n\n");
	
*/	/** Constructor */
/*        writer.write("    public ");
        writer.write(container.getJavaType());
        writer.write("GUIManagerImpl(");
	writer.write(container.getJavaType());
	writer.write("ImplBean source) {\n");
	writer.write("        _source = source ;\n");
	writer.write("        _node = new ");
	writer.write(container.getJavaType());
        writer.write("GUINode(this);\n");
	writer.write("    }\n\n");

*/	/** set get methods */
/*	writer.write("    public ");
	writer.write("Bean getSource() {\n");
	writer.write("        return (_source);\n");
	writer.write("    }\n\n");

	writer.write("    public ");
	writer.write("GUI getGUINode() {\n");
	writer.write("        return ( _node);\n");
	writer.write("    }\n\n");

	writer.write("    public ");
	writer.write("GUI getGUIPanel() {\n");
	writer.write("        return ( (GUI) new ");
	writer.write(container.getJavaType());
	writer.write("GUIPanel(this) );\n");
	writer.write("    }\n\n");
	
        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }
*/
    /**
     * <p>
     *  This will generate the interface for an GUI connected to a listener
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateGUIInterface(Container container)
	throws IOException {
	
	// Get a new writer
	String filename = container.getJavaType() + "GUI.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"interface",
		"GUI",
		this.interfacesPackage+".GUI",
		"");

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the visual representation 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generatePanelAccessor(Container container) 
        throws IOException {
	
	List panelAccessorImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "PanelAccessor.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

	panelAccessorImports.add(this.interfacesPackage+".ExtensionImpl");
	panelAccessorImports.add(this.interfacesPackage+".ExtensionManager");

        // Write out import statements
        writeImportStatements(writer, panelAccessorImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"PanelAccessor",
		"ExtensionImpl",
		"");

        List properties = container.getProperties();

        // Write out each property
	writer.write("    private ExtensionManager _extManager;\n\n");
	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("Panel _panel;\n\n");
	
        // Write class constructor
	writer.write("    public ");
	writer.write(container.getJavaType());
	writer.write("PanelAccessor(");
//	writer.write(container.getJavaType());
//	writer.write("ExtensionManagerImpl manager");
	writer.write(") {\n");
//	writer.write("        this._extManager = manager ;\n");
	writer.write("        _panel = new ");
	writer.write(container.getJavaType());
	writer.write("Panel(this);\n");
	writer.write("    }\n\n");

//	writer.write("    public ExtensionManager getManager() {\n");
//	writer.write("        return (_extManager);\n");
//	writer.write("    }\n\n");
	
	writer.write("    public ");
//	writer.write(container.getJavaType());
	writer.write("Object getInstance() {\n");
	writer.write("        _panel.startListening();\n");
	writer.write("        return (_panel);\n");
	writer.write("    }\n\n");
	
	// Write out the methods involving properties
	writer.write("    public String toString() {\n");
	writer.write("        return(\"Panel\");\n");
	writer.write("    }\n\n");
	
        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the visual representation 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generatePanel(Container container) 
        throws IOException {
	
	List panelImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "Panel.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

	panelImports.add("java.awt.Component");
	panelImports.add("java.awt.BorderLayout");
	panelImports.add("java.awt.event.ActionEvent");
	panelImports.add("java.awt.event.ActionListener");
//	panelImports.add("java.io.File");
	panelImports.add("java.util.List");
	panelImports.add("javax.swing.JButton");
	panelImports.add("javax.swing.JLabel");
	panelImports.add("javax.swing.JPanel");
//	panelImports.add("java.awt.Dimension");
//	panelImports.add("org.objectweb.ccm.descriptor.guirepres.*");

	panelImports.add(this.interfacesPackage+".GUI");

        // Write out import statements
        writeImportStatements(writer, panelImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"Panel",
		"JPanel",
		container.getJavaType()+"GUI,ActionListener");

        List properties = container.getProperties();

        // Write out each property
        writeGUIPanelProperties(writer,
					container.getJavaType(),
					properties);

        // Write class constructor
        writeGUIPanelConstructor(writer, 
                                       container.getJavaType(),
				       properties);
        
	// Write out the methods involving properties
	writeGUIPanelMethods(writer, container.getJavaType(), properties, container.getJavaInterfacePackage());

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the visual representation 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateNodeAccessor(Container container) 
        throws IOException {
	
	List panelAccessorImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "NodeAccessor.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

	panelAccessorImports.add(this.interfacesPackage+".ExtensionImpl");
	panelAccessorImports.add(this.interfacesPackage+".ExtensionManager");

        // Write out import statements
        writeImportStatements(writer, panelAccessorImports);

        // Write out the implementation declaration
        writeDeclaration(writer, container,
		"class",
		"NodeAccessor",
		"ExtensionImpl",
		"");

        List properties = container.getProperties();

        // Write out each property
	writer.write("    private ExtensionManager _extManager;\n\n");
	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("Node _node;\n\n");
	
        // Write class constructor
	writer.write("    public ");
	writer.write(container.getJavaType());
	writer.write("NodeAccessor(");
//	writer.write(container.getJavaType());
//	writer.write("ExtensionManagerImpl manager");
	writer.write(") {\n");
//	writer.write("        this._extManager = manager ;\n");
	writer.write("        _node = new ");
	writer.write(container.getJavaType());
	writer.write("Node(this);\n");
	writer.write("    }\n\n");

//	writer.write("    public ExtensionManager getManager() {\n");
//	writer.write("        return (_extManager);\n");
//	writer.write("    }\n\n");
	
	writer.write("    public ");
//	writer.write(container.getJavaType());
	writer.write("Object getInstance() {\n");
	writer.write("        _node.startListening();\n");
	writer.write("        return (_node);\n");
	writer.write("    }\n\n");

	// Write out the methods involving properties
	writer.write("    public String toString() {\n");
	writer.write("        return(\"Node\");\n");
	writer.write("    }\n\n");
	
        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate the visual node representation 
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateNode(Container container) 
        throws IOException {
	
	// Get a new writer
	String filename = container.getJavaType() + "Node.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());
				
        writeDeclaration(writer, container,
		"class",
		"Node",
		"javax.swing.tree.DefaultMutableTreeNode",
		container.getJavaType()+"GUI");

	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("NodeAccessor _accessor;\n\n");
	writer.write("    private ");
	writer.write(container.getJavaType());
	writer.write("Listener _listener;\n\n");

        writer.write("    public ");
        writer.write(container.getJavaType());
        writer.write("Node(");
	writer.write(container.getJavaType());
	writer.write("NodeAccessor accessor) {\n");
	writer.write("        _accessor = accessor ;\n");
//	writer.write("        while (this._accessor.getManager() == null );\n");

	writer.write("    }\n\n");

	writer.write("    public void startListening() {\n");
	writer.write("        _listener = new ");
	writer.write(container.getJavaType());
	writer.write("ListenerImpl( (");
	writer.write(container.getJavaType());
	writer.write("GUI) this);\n");
	writer.write("    }\n\n");

	writer.write("    public ");
	writer.write(this.interfacesPackage);
	writer.write(".ExtensionManager getManager() {\n");
	writer.write("        return (_accessor.getManager());\n");
	writer.write("    }\n\n");

	writer.write("    public void refresh() {\n");

	writer.write("        removeAllChildren();\n");

        List properties = container.getProperties();

	for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
	    if (property.getModifier().get(Property.SOURCE_ELEMENT))
	    {
		if (property.isCollection())
		{
		    writer.write("        java.util.List ");
		    writer.write(property.getJavaName());
		    writer.write("List = ((");
		    writer.write(container.getJavaType());
		    writer.write("Bean)_accessor.getManager().getSource()).get");
		    writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		    writer.write("List();\n");
		    writer.write("        for (java.util.Iterator i = ");
		    writer.write(property.getJavaName());
		    writer.write("List.iterator(); i.hasNext(); ) {\n");
		    writer.write("            ");
		    writer.write(this.interfacesPackage);
		    writer.write(".Bean child = (");
		    writer.write(this.interfacesPackage);
		    writer.write(".Bean)i.next();\n");
		    writer.write("            this.add ( (javax.swing.tree.DefaultMutableTreeNode) (child.getExtensionManager().getExtensionByName(\"Node\").getInstance()) );\n");
		    writer.write("        }\n");
		}
		else
		{
		    writer.write("        if ( ((");
		    writer.write(container.getJavaType());
		    writer.write("Bean)_accessor.getManager().getSource()).get");
		    writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		    writer.write("() != null ) \n");
		    writer.write("            this.add(   (javax.swing.tree.DefaultMutableTreeNode) ( (");
		    writer.write(CapitalizationUtils.initialUpper(property.getXMLName()));
		    writer.write("Bean) (((");
		    writer.write(container.getJavaType());
		    writer.write("Bean)_accessor.getManager().getSource()).get");
		    writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		    writer.write("()) ).getExtensionManager().getExtensionByName(\"Node\").getInstance()    ) ;\n");
		}
	    }
	}

	writer.write("    }\n\n");

	writer.write("    public void validateChanges() {\n");
	writer.write("    }\n");	

	writer.write("    public String toString() {\n");
	writer.write("        return(\"");
	writer.write(container.getJavaType());
	writer.write("\");\n");
	writer.write("    }\n");

        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }

    /**
     * <p>
     *  This will generate an operator manager interface
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
/*    protected void generateOperatorManagerInterface(Container container)
        throws IOException {
	
	List gUIBeanImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "OperatorManager.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

        writer.write("public interface ");
        writer.write(container.getJavaType());
        writer.write("OperatorManager {\n\n");
	
        writer.write("}\n");
        
        writer.flush();
        writer.close();

    }
*/
    /**
     * <p>
     *  This will generate an operator manager
     *  for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
/*    protected void generateOperatorManager(Container container)
        throws IOException {
	
	List gUIBeanImports = new LinkedList();

	// Get a new writer
	String filename = container.getJavaType() + "OperatorManagerImpl.java";
	File outputFile = new File(getDir(container), filename);
	FileWriter writer = new FileWriter(outputFile);
	
	// Write out the source file header comment
	writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
	
	// Add a package statement if necessary.
	writePackageDeclaration(writer,
				container.getJavaInterfacePackage());

        writer.write("public class ");
        writer.write(container.getJavaType());
        writer.write("OperatorManagerImpl {\n\n");
	
	writer.write("    private ");
	writer.write(container.getJavaType());
        writer.write("Bean _source;\n\n");

	writer.write("    public ");
	writer.write(container.getJavaType());
        writer.write("OperatorManagerImpl(");
	writer.write(container.getJavaType());
        writer.write("ImplBean source) {\n");
	writer.write("        _source=source;\n");
	writer.write("    }\n\n");

        writer.write("}\n");

        writer.flush();
        writer.close();

    }
*/
    /**
     * <p>
     *  This writes out the package declaration for this class, if needed.
     * </p>
     *
     * @param writer <code>Writer</code> to perform output to
     * @param packageName the package of this class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writePackageDeclaration(Writer writer, String packageName) 
        throws IOException {
            
        // Generate a package statement if necessary.
        if (!packageName.equals("")) {
            writer.write("package");
            writer.write(" ");
            writer.write(packageName);
            writer.write(";\n\n");
        }
    }
    
    /**
     * <p>
     *  This will take the supplied list of class names, and add import
     *    statements for each of the classes in the list.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param importClasses <code>List</code> of classes to add imports for
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeImportStatements(Writer writer, 
                                                  List importClasses) 
        throws IOException {
        
        // Write out imports
        if (importClasses.size() > 0) {
            writer.write("// Local Interface Import Statements\n");
        }
        for (Iterator i = importClasses.iterator(); i.hasNext(); ) {
            writer.write("import ");
            writer.write((String)i.next());
            writer.write(";\n");
        }
        if (importClasses.size() > 0) {
            writer.write("\n");
        }
    }
    
    /**
     * <p>
     *  This writes out the interface bean declaration (the class header).
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write the interface for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeDeclaration(Writer writer, 
                                             Container container,
					     String modifier,
					     String suffix,
					     String extended,
					     String implemented)
        throws IOException {
            
        writer.write("public ");
	writer.write(modifier);
	writer.write(" ");
        writer.write(container.getJavaType());
	writer.write(suffix);
	if (!extended.equals("")) {
	    writer.write(" extends ");
	    writer.write(extended);
	}
	
	if (!implemented.equals("")) {
	    writer.write(" implements ");
	    writer.write(implemented);
	}
	
	writer.write(" {\n\n");
    }
    
    /**
     * <p>
     *  This writes out the properties for an GUI Bean as variables.
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param properties <code>List</code> of properties to write out as
     *        member variables
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeGUIPanelProperties(Writer writer, String className, List properties)
        throws IOException {

	writer.write("    private ");
	writer.write(className);
	writer.write("PanelAccessor _accessor;\n");
	writer.write("    private ");
	writer.write(className);
	writer.write("Listener _listener;\n\n");
	writer.write("    private JPanel contenu = new JPanel(new java.awt.GridLayout(0,1));\n");
	writer.write("    private JButton close = new JButton(\"X\");\n");
	writer.write("    private JButton validate = new JButton(\"OK\");\n\n");

        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (isIgnoredBinding(property.getXMLName(),
                                 property.getXMLNamespaceURI())) {
                continue;
            }
            writeGUIPanelProperty(writer, property);
        }
        
    }
        
    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as a member variable SPECIFIC to GUIs.
     * </p>
     *
     * @param writer <code>Writer</code> to write variable declaration to.
     * @param property <code>Property</code> to generate code for.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */
    protected void writeGUIPanelProperty(Writer writer, Property property) 
        throws IOException {
            
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if ((property instanceof AtomicProperty) &&
            (!property.getXMLName().startsWith("xmlns")) &&
	    (	(property.getModifier().get(Property.SOURCE_ATTLIST))) ||
		(ClassUtils.isJavaPrimitive(property.getJavaType()))
	    )
	{
            AtomicProperty atomicProperty = (AtomicProperty)property;
            // Write out the property
            writer.write("    private JLabel label");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getJavaName()));
	    writer.write(" = new JLabel(\"");
	    writer.write(atomicProperty.getXMLName());
	    writer.write("\");\n");
	    writer.write("    private Component visualField");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getJavaName()));
	    writer.write(";\n");
	    writer.write("    private String stringClass");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getJavaName()));
	    writer.write(";\n");
	    writer.write("    private String stringGetmethod");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getJavaName()));
	    writer.write(";\n");
	    writer.write("    private String stringSetmethod");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getJavaName()));
	    writer.write(";\n");
	    writer.write("    private String stringSetsizemethod");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getJavaName()));
	    writer.write(";\n\n");
        }
    }
                                                             
    /**
     * <p>
     *  This writes out the constructor for an GUI Bean class
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param className the name of the Bean class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeGUIPanelConstructor(Writer writer, 
                                                  String className,
						  List properties)
        throws IOException {

        BitSet modifiers = null;

        writer.write("    public ");
        writer.write(className);
        writer.write("Panel(");
	writer.write(className);
	writer.write("PanelAccessor accessor) {\n");

        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
	    if (property.getModifier().get(Property.SOURCE_ATTLIST) ||
		(ClassUtils.isJavaPrimitive(property.getJavaType())) )
	    {
		writer.write("    label");
        	writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(".setPreferredSize(new java.awt.Dimension(100,20));\n");

		writer.write("        visualField");
        	writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(" = new javax.swing.JTextField();\n");
		writer.write("        stringClass");
        	writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(" = \"javax.swing.JTextField\";\n");
		writer.write("        stringGetmethod");
        	writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(" = \"getText\";\n");
		writer.write("        stringSetmethod");
        	writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(" = \"setText\";\n");
		writer.write("        stringSetsizemethod");
        	writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(" = \"setPreferredSize\";\n");
	    }
	}
	writer.write("        _accessor = accessor ;\n");
	writer.write("        this.setLayout(new BorderLayout());\n");
	writer.write("        this.setBorder(new javax.swing.border.EmptyBorder(1,1,1,1));\n");
	writer.write("        this.add(new JLabel(\"");
	writer.write(className);
	writer.write("\"), BorderLayout.NORTH);\n");
	writer.write("        this.add(contenu, BorderLayout.CENTER);\n");
	writer.write("        validate.addActionListener(this);\n");

	writer.write("        JPanel boutons = new JPanel(new BorderLayout());\n");

	writer.write("        close.setActionCommand(\"close\");\n");
	writer.write("        close.addActionListener(this);\n");

	writer.write("        this.add(boutons, BorderLayout.SOUTH);\n");


//	writer.write("        while (this._accessor.getManager() == null );\n");


        writer.write("    }\n\n"); 
    }

    /**
     * <p>
     *  This writes out the methods involving properties for the GUI Bean
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param properties <code>List</code> of properties to write out as
     *        member variables
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeGUIPanelMethods(Writer writer, 
                                                  String className,
                                                  List properties,
						  String packageName)

        throws IOException {

	writer.write("    public void startListening() {\n");
	writer.write("        refreshForm();\n");
	writer.write("        refresh();\n");
	writer.write("        _listener = new ");
	writer.write(className);
	writer.write("ListenerImpl( (");
	writer.write(className);
	writer.write("GUI) this);\n");
	writer.write("    }\n\n");
	
	writer.write("    public void actionPerformed(ActionEvent cliquage) {\n\n");

	writer.write("        if (cliquage.getActionCommand().equals(\"close\")) {\n");
	writer.write("            this.setVisible(false);\n");
	writer.write("            this.getParent().repaint();\n");
	writer.write("            this.getParent().remove(this);\n");
	writer.write("        }\n");
	writer.write("        if (cliquage.getActionCommand().equals(\"validate\")) {\n");
	writer.write("            this.validateChanges();\n");
	writer.write("        }\n");
	writer.write("    }\n\n");

	writer.write("    public void validateChanges() {\n\n");

	writer.write("        try {\n");
	for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
	    if ( (! isIgnoredBinding(property.getXMLName(),
                                 property.getXMLNamespaceURI())) &&
		(!property.getXMLName().startsWith("xmlns")) &&
		(!property.getModifier().get(Property.MUTABILITY_FINAL)) &&
		( (property.getModifier().get(Property.SOURCE_ATTLIST)) ||
		(ClassUtils.isJavaPrimitive(property.getJavaType())) )
		)
	    {
		writer.write("            ((");
		writer.write(className);
		writer.write("Bean)_accessor.getManager().getSource()).set");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write("( (String) (Class.forName(stringClass");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(").getMethod(stringGetmethod");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(",null).invoke(visualField");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(", null)));\n");
	    }
	}
	
	writer.write("        } catch (Exception e) {\n");
	writer.write("            e.printStackTrace();\n");
	writer.write("        }\n");
	writer.write("\n        _accessor.getManager().getSource().notifyListeners();\n");
	writer.write("    }\n\n");
	
	writer.write("    public void refresh() {\n\n");
	writer.write("        String parameter =\"\";\n");
	writer.write("        Object[] parameters = {parameter};\n");
	writer.write("        Class[] parametersTypes = {parameter.getClass()};\n");
	writer.write("        try {\n");

	for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
	    if (property.getModifier().get(Property.SOURCE_ATTLIST) ||
		(ClassUtils.isJavaPrimitive(property.getJavaType())) )
	    {
		writer.write("            java.lang.reflect.Array.set(parameters, 0, ((");
		writer.write(className);
		writer.write("Bean)_accessor.getManager().getSource()).get");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write("());\n");
		writer.write("            ");
		writer.write("Class.forName(stringClass");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(").getMethod(stringSetmethod");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(",parametersTypes).invoke(visualField");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(", parameters);\n");
	    }
	}

	writer.write("        } catch (Exception e) {\n");
	writer.write("            e.printStackTrace();\n");
	writer.write("        }\n");
	writer.write("    }\n\n");

	writer.write("    public void refreshForm() {\n");

	try {
	    File modelFile = new File(this.modelsDirectory
				+packageName.substring(packageName.lastIndexOf(".")+1)
				+"_guirepres.xml");
	    if ( modelFile.exists() ) {
//		System.out.println("found model file: "+modelFile.getName());
		org.objectweb.zeusx.descriptor.guirepres.Guirepres representer 
			= org.objectweb.zeusx.descriptor.guirepres.GuirepresUnmarshaller.unmarshal (modelFile);
		java.util.List branches = representer.getBranchList();
		for (java.util.Iterator bi = branches.iterator(); bi.hasNext(); ) {
		    org.objectweb.zeusx.descriptor.guirepres.Branch branch = (org.objectweb.zeusx.descriptor.guirepres.Branch)bi.next();
		    if ( branch.getName().equals(CapitalizationUtils.initialLower(className)) ) {
                	java.util.List attributs = branch.getAttrList();
			for (Iterator i = properties.iterator(); i.hasNext(); ) {
			    Property property = (Property)i.next();
			    if (property.getModifier().get(Property.SOURCE_ATTLIST) ||
				(ClassUtils.isJavaPrimitive(property.getJavaType())) )
			    {
				for (int iattr=0;iattr<attributs.size();iattr++)
				    if ((((org.objectweb.zeusx.descriptor.guirepres.Attr)attributs.get(iattr)).getName()).equals(property.getXMLName())) {
	                        	writer.write("        stringClass");
					writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
					writer.write(" = \"");
					writer.write(((org.objectweb.zeusx.descriptor.guirepres.Attr)attributs.get(iattr)).getRepres());
					writer.write("\";\n");
					writer.write("        stringGetmethod");
					writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
					writer.write(" = \"");
					writer.write(((org.objectweb.zeusx.descriptor.guirepres.Attr)attributs.get(iattr)).getGetmethod());
					writer.write("\";\n");
					writer.write("        stringSetmethod");
					writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
					writer.write(" = \"");
					writer.write(((org.objectweb.zeusx.descriptor.guirepres.Attr)attributs.get(iattr)).getSetmethod());
					writer.write("\";\n");
					writer.write("        stringSetsizemethod");
					writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
					writer.write(" = \"");
				        writer.write(((org.objectweb.zeusx.descriptor.guirepres.Attr)attributs.get(iattr)).getSetsizemethod());
					writer.write("\";\n\n");

	    				writer.write("        try {\n");
					writer.write("            visualField");
					writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	    				writer.write(" = (Component) (Class.forName(stringClass");
					writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
					writer.write(")).newInstance();\n");
					writer.write("        } catch (Exception e) {\n");
					writer.write("            e.printStackTrace();\n");
					writer.write("        }\n\n");
				    }
			    }
			}
		    }
		}
	    }

    	} catch (Exception e) {
            e.printStackTrace();
        }

/*
	writer.write("        try {\n");
	writer.write("            File modelFile = new File(\"");
	writer.write("./models/");
	writer.write(packageName.substring(packageName.lastIndexOf(".")+1));
	writer.write("_guirepres.xml\");\n");
	writer.write("            if ( modelFile.exists() ) {\n");
	writer.write("                Guirepres representer = GuirepresUnmarshaller.unmarshal (modelFile);\n");
	writer.write("                java.util.List branches = representer.getBranchList();\n");
	writer.write("                for (java.util.Iterator bi = branches.iterator(); bi.hasNext(); ) {\n");
	writer.write("                    Branch branch = (Branch)bi.next();\n");
	writer.write("                    if (branch.getName().equals(\"");
	writer.write(CapitalizationUtils.initialLower(className));
	writer.write("\")) {\n");
	writer.write("                        java.util.List attributs = branch.getAttrList();\n\n");
	writer.write("                        try {\n");

	for (Iterator i = properties.iterator(); i.hasNext(); ) {
	    Property property = (Property)i.next();
	    if (property.getModifier().get(Property.SOURCE_ATTLIST) ||
		(ClassUtils.isJavaPrimitive(property.getJavaType())) )
	    {
		writer.write("                            for (int i=0;i<attributs.size();i++)\n");
	        writer.write("                                if ((((Attr)attributs.get(i)).getName()).equals(\"");
	        writer.write(property.getXMLName());
	        writer.write("\")) {\n");
		writer.write("                                    stringClass");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(" = ((Attr)attributs.get(i)).getRepres();\n");
		writer.write("                                    stringGetmethod");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(" = ((Attr)attributs.get(i)).getGetmethod();\n");
		writer.write("                                    stringSetmethod");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(" = ((Attr)attributs.get(i)).getSetmethod();\n");
		writer.write("                                    stringSetsizemethod");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(" = ((Attr)attributs.get(i)).getSetsizemethod();\n");

	        writer.write("                                    visualField");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(" = (Component) (Class.forName(stringClass");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(")).newInstance();\n");
		writer.write("                                }\n\n");
	    }
	}

	writer.write("                        } catch (Exception e) {\n");
	writer.write("                            e.printStackTrace();\n");
	writer.write("                        }\n");

	writer.write("                    }\n");
	writer.write("                }\n");
	writer.write("            }\n\n");
*/		

	writer.write("        try {\n");

	writer.write("            contenu.removeAll();\n");
	writer.write("            java.awt.Dimension parameter = new java.awt.Dimension(200,20);\n");
	writer.write("            Object[] parameters = {parameter};\n");
	writer.write("            Class[] parametersTypes = {parameter.getClass()};\n\n");

	for (Iterator i = properties.iterator(); i.hasNext(); ) {
	    Property property = (Property)i.next();
	    if (property.getModifier().get(Property.SOURCE_ATTLIST) ||
		(ClassUtils.isJavaPrimitive(property.getJavaType())) )
	    {
		writer.write("            JPanel ");
		writer.write(property.getJavaName());
		writer.write("Panel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));\n");
		writer.write("            ");
		writer.write(property.getJavaName());
		writer.write("Panel.add(label");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(");\n");
		
		writer.write("            Class.forName(stringClass");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(").getMethod(stringSetsizemethod");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(",parametersTypes).invoke(visualField");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
		writer.write(", parameters);\n");
	        writer.write("            ");
		writer.write(property.getJavaName());
		writer.write("Panel.add(visualField");
		writer.write(CapitalizationUtils.initialUpper(property.getJavaName()));
	        writer.write(");\n");
		writer.write("            contenu.add(");
		writer.write(property.getJavaName());
		writer.write("Panel);\n\n");
	    }
	}
	
	writer.write("            contenu.repaint();\n");
	writer.write("        } catch (Exception except) {\n");
	writer.write("            except.printStackTrace();\n");
	writer.write("        }\n");
	writer.write("    }\n\n");

	writer.write("    public ");
	writer.write(this.interfacesPackage);
	writer.write(".ExtensionManager getManager() {\n");
	writer.write("        return (_accessor.getManager());\n");
	writer.write("    }\n\n");

//	writer.write("    public ");
//	writer.write(container.getJavaType());
//	writer.write("org.objectweb.ccm.descriptor.GUIManager getGUIManager() {\n");
//	writer.write("        return (_GUIManager);\n");
//	writer.write("    }\n\n");

/*
	writer.write("    public ");
	writer.write(className);
	writer.write("ImplBean getSource() {\n");
	writer.write("        return (_source);\n");
	writer.write("    }\n\n");
*/
    }
    
    /**
     * <p>
     *  This will initialize this instance of <code>Generator</code>,
     *    handling tasks like directory creation.
     * </p>
     *
     * @throws <code>IllegalOutputDirectoryException</code> - when an illegal
     *         output directory has been supplied to this <code>Generator</code>
     */    
    private void initialize() throws IllegalOutputDirectoryException {        
        // Determine the output directory
        if (outputDir == null) {
            outputDir = new File(".");
        }
        
        // Ensure the output directory exists and is a directory
        if ((outputDir.exists()) && (!outputDir.isDirectory())) {
            throw new IllegalOutputDirectoryException(
                outputDir.getAbsolutePath());
        }

        // Create the directory if it does not exist
        if (!outputDir.exists()) {
            outputDir.mkdir();
        }

        initialized = true;
    }        

    /**
     * <p>
     *  This will reset the initialization state of this <code>Generator</code>
     *    instance to <code>false</code>.
     * </p>
     */    
    private void uninitialize() {
        initialized = false;
    }
    
    /**
     * <p>
     *  This will indicate if this <code>Generator</code> instance has been
     *    initialized.
     * </p>
     *
     * @return <code>boolean</code> - whether this instance is initialized.
     */
    private boolean isInitialized() {
        return initialized;
    }
    
    /**
     * <p>
     *  This will determine the directory to use for generation
     *    based on the packages of a <code>{@link Container}</code>.
     * </p>
     *
     * @param container the container to obtain the directory for.
     * @return <code>File</code> - the directory for generation.
     */
    private File getDir(Container container) {
        StringBuffer dirName = new StringBuffer();
        String thisPackage = container.getJavaInterfacePackage();

        for (int i=0, len=thisPackage.length(); i<len; i++) {
            char ch = thisPackage.charAt(i);
            if (ch != '.') {
                dirName.append(ch);
            } else {
                dirName.append(System.getProperty("file.separator"));
            }
        }
        
        File dir = 
            new File(outputDir, dirName.toString());
        
        // Create the directory if it does not exist
        if (!dir.exists()) {
            dir.mkdirs();
        }
        
        return dir;
    }
}
