/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.zeusx;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.BitSet;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.ZeusDefaults;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.BaseBinding;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.util.CapitalizationUtils;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;
import org.enhydra.zeus.util.XMLName;

import org.enhydra.zeus.generator.*;

//import org.objectweb.ccm.descriptor.guirepres.*;

/**
 * <p>
 *  <code>{@link Generator}</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a 
 *    <code>File</code> [directory]).
 * </p><p>
 *  <code>ObjectWebBeansGenerator</code> is the ObjectWeb XmlTool additional
 *    <code>Generator</code> and takes in either a single <code>Binding</code>
 *    or a <code>List</code> of them, and directly generates Java classes.
 * </p>
 *
 * @author Christophe Contreras
 */
public class ObjectWebInterfacesGenerator implements Generator {

    /** A list of bindings to ignore */
    protected List ignoredBindings;
    
    /** This instances's collapsed elements */
    protected List collapsedElements;
    
    /** Base directory for generated files. */
    protected File outputDir;
    
    /** Package where we'll put the interfaces. */
    protected String interfacesPackage;
    
    /** Directory where we'll put the interfaces. */
    protected String targetDir;
    
    /** Keeps track of whether the Generator needs to be reinitialized. */
    protected boolean initialized;

    /**
     * <p>
     *  This will construct a new instance of <code>ObjectWebInterfacesGenerator</code>.
     * </p>
     */
    public ObjectWebInterfacesGenerator(String wantedInterfacesPackage) {
        initialized = false;
        this.interfacesPackage = wantedInterfacesPackage;
	
    }

    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param baseDir the name of the base directory where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(String outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = new File(outputDir);
        uninitialize();
    }
    
    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the <code>File</code> (directory) where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(File outputDir) {
        if (outputDir == null) {
            throw new IllegalArgumentException("A generator cannot have a " +
                "null output directory.");
        }
        
        this.outputDir = outputDir;
        uninitialize();
    }

    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     * @param xmlNamespaceURI the XML namespace URI of the binding to ignore. 
     *        The empty string indicates there is no namespace associated 
     *        with the binding.
     */
    public void setIgnoreBinding(String xmlName, String xmlNamespaceURI) {
        ignoredBindings.add(new XMLName(xmlName, xmlNamespaceURI));
    }
    
    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step 
     *    towards allowing binding schemas. The name of the binding should 
     *    be passed in here, and no property generation for it will occur. This
     *    convenience version supplies an empty string for the namespace URI.
     * </p>
     *
     * @param xmlName the XML local name of the binding to ignore.
     */
    public void setIgnoreBinding(String xmlName) {
        setIgnoreBinding(xmlName, "");
    }
    
    /**
     * This will determine if a particular binding should be ignored. The
     * binding is identified by its XML local name and its XML namespace URI.
     *
     * @param xmlName the XML local name of the binding.
     * @param xmlNamespaceURI the XML namespace URI associated with the binding. 
     *        An empty string indicates that there is no namespace associated 
     *        with the binding.
     *
     * @return true if and only if the binding should be ignored
     */
    protected boolean isIgnoredBinding(String xmlName, String xmlNamespaceURI) {
        if (ignoredBindings != null) {
            for (Iterator i = ignoredBindings.iterator(); i.hasNext(); ) {
                XMLName bindingName = (XMLName)i.next();
                if ((bindingName.getXMLName().equals(xmlName)) &&
                    (bindingName.getNamespaceURI().equals(xmlNamespaceURI))) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * <p>
     *  This is a method that has to be implemented to satisfy the Generator interface
     * </p>
     */
    public void generate (Binding binding)
        throws IOException, ZeusException {
    }

    /**
     * <p>
     *  This will just write in files the interfaces code
     * </p>
     *
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     * @throws <code>ZeusException</code> - when errors in class generation
     *         occur.
     */
    public void generate() 
        throws IOException, ZeusException {
            
        if (!isInitialized()) {
            initialize();
        }
    
        /**
         * Currently, only Containers can be generated (which
         *   includes ContainerProperties), as AtomicProperties
         *   won't result in classes.
         */

	/** the beans classes set */
	/** Generate Bean interface */
	generateBeanInterface();

	/** Generate the Extension interface */
	generateExtensionInterface();
	
	/** Generate the Extension base implementation */
	generateExtensionImplementation();

	/** Generate the ExtensionManager interface */
	generateExtensionManagerInterface();
	    
	/** Generate the GUI interface */
	generateGUIInterface();

    }

    /**
     * <p>
     *  This will generate the Bean interface.
     * </p>
     *
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateBeanInterface() 
        throws IOException {
            
        // Get a new writer
        String filename = 
            new StringBuffer("Bean.java")
                .toString();
        File outputFile = new File(getDir(), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, this.interfacesPackage);
        
        // Write out class header
        writeDeclaration(writer, 
		"interface",
		"Bean",
		"",
		"");

	writer.write("    public ExtensionManager getExtensionManager();\n\n");
	writer.write("    public void marshalBean(java.io.File file) throws java.io.IOException;\n\n");
	writer.write("    public void notifyListeners();\n\n");
	writer.write("    java.util.Vector getAddableChildren();\n\n");

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This will generate the Extension interface.
     * </p>
     *
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateExtensionInterface() 
        throws IOException {
            
        // Get a new writer
        String filename = 
            new StringBuffer("Extension.java")
                .toString();
        File outputFile = new File(getDir(), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, this.interfacesPackage);
        
        // Write out class header
        writeDeclaration(writer, 
		"interface",
		"Extension",
		"",
		"");

	writer.write("    public void setManager (ExtensionManager manager);\n\n");
	writer.write("    public ExtensionManager getManager();\n\n");
	writer.write("    public Object getInstance();\n\n");

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This will generate the Extension implementation.
     * </p>
     *
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateExtensionImplementation() 
        throws IOException {
            
        // Get a new writer
        String filename = 
            new StringBuffer("ExtensionImpl.java")
                .toString();
        File outputFile = new File(getDir(), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, this.interfacesPackage);
        
        // Write out class header
        writeDeclaration(writer, 
		"class",
		"ExtensionImpl",
		"",
		"Extension");

	writer.write("    private ExtensionManager _extManager;\n\n");
	writer.write("    public void setManager(ExtensionManager manager) {\n");
	writer.write("        this._extManager = manager;\n");
	writer.write("    }\n\n");
	writer.write("    public ExtensionManager getManager() {\n");
	writer.write("        return (this._extManager);\n");
	writer.write("    }\n\n");
	writer.write("    public Object getInstance() {\n");
	writer.write("        return(null);\n");
	writer.write("    }\n\n");

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This will generate the ExtensionManager interface.
     * </p>
     *
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateExtensionManagerInterface() 
        throws IOException {
            
        // Get a new writer
        String filename = 
            new StringBuffer("ExtensionManager.java")
                .toString();
        File outputFile = new File(getDir(), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, this.interfacesPackage);
        
        // Write out class header
        writeDeclaration(writer, 
		"interface",
		"ExtensionManager",
		"",
		"");

	writer.write("    public Bean getSource();\n\n");
	writer.write("    public Extension getExtensionByName(String wanted);\n\n");

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This will generate the GUI interface.
     * </p>
     *
     * @throws <code>IOException</code> - when generation errors occur.
     */
    protected void generateGUIInterface() 
        throws IOException {
            
        // Get a new writer
        String filename = 
            new StringBuffer("GUI.java")
                .toString();
        File outputFile = new File(getDir(), filename);
        FileWriter writer = new FileWriter(outputFile);
        
        // Write out the source file header comment
        writer.write(ExtZeusDefaults.GENERATED_CLASS_COMMENT);
        
        // Write the package declaration, if needed
        writePackageDeclaration(writer, this.interfacesPackage);
        
        // Write out class header
        writeDeclaration(writer, 
		"interface",
		"GUI",
		"",
		"");

	writer.write("    public ExtensionManager getManager();\n\n");
	writer.write("    public void validateChanges();\n\n");
	writer.write("    public void refresh();\n\n");

        writer.write("}\n");    
        writer.flush();
        writer.close();
    }

    /**
     * <p>
     *  This writes out the package declaration for this class, if needed.
     * </p>
     *
     * @param writer <code>Writer</code> to perform output to
     * @param packageName the package of this class
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writePackageDeclaration(Writer writer, String packageName) 
        throws IOException {
            
        // Generate a package statement if necessary.
        if (!packageName.equals("")) {
            writer.write("package");
            writer.write(" ");
            writer.write(packageName);
            writer.write(";\n\n");
        }
    }
    
    /**
     * <p>
     *  This writes out the interface bean declaration (the class header).
     * </p>
     *
     * @param writer <code>Writer</code> to output to
     * @param container the container to write the interface for.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    protected void writeDeclaration(Writer writer, 
					     String modifier,
					     String suffix,
					     String extended,
					     String implemented)
        throws IOException {
            
        writer.write("public ");
	writer.write(modifier);
	writer.write(" ");
	writer.write(suffix);
	if (!extended.equals("")) {
	    writer.write(" extends ");
	    writer.write(extended);
	}
	
	if (!implemented.equals("")) {
	    writer.write(" implements ");
	    writer.write(implemented);
	}
	
	writer.write(" {\n\n");
    }
    
    /**
     * <p>
     *  This will initialize this instance of <code>Generator</code>,
     *    handling tasks like directory creation.
     * </p>
     *
     * @throws <code>IllegalOutputDirectoryException</code> - when an illegal
     *         output directory has been supplied to this <code>Generator</code>
     */    
    private void initialize() throws IllegalOutputDirectoryException {        
        // Determine the output directory
        if (outputDir == null) {
            outputDir = new File(".");
        }
        
        // Ensure the output directory exists and is a directory
        if ((outputDir.exists()) && (!outputDir.isDirectory())) {
            throw new IllegalOutputDirectoryException(
                outputDir.getAbsolutePath());
        }

        // Create the directory if it does not exist
        if (!outputDir.exists()) {
            outputDir.mkdir();
        }

        initialized = true;
    }        

    /**
     * <p>
     *  This will reset the initialization state of this <code>Generator</code>
     *    instance to <code>false</code>.
     * </p>
     */    
    private void uninitialize() {
        initialized = false;
    }
    
    /**
     * <p>
     *  This will indicate if this <code>Generator</code> instance has been
     *    initialized.
     * </p>
     *
     * @return <code>boolean</code> - whether this instance is initialized.
     */
    private boolean isInitialized() {
        return initialized;
    }
    
    /**
     * <p>
     *  This will determine the directory to use for generation
     *    based on the packages of a <code>{@link Container}</code>.
     * </p>
     *
     * @param container the container to obtain the directory for.
     * @return <code>File</code> - the directory for generation.
     */
    private File getDir() {
        StringBuffer dirName = new StringBuffer();

        for (int i=0, len=this.interfacesPackage.length(); i<len; i++) {
            char ch = this.interfacesPackage.charAt(i);
            if (ch != '.') {
                dirName.append(ch);
            } else {
                dirName.append(System.getProperty("file.separator"));
            }
        }
        
        File dir = 
            new File(outputDir, dirName.toString());
        
        // Create the directory if it does not exist
        if (!dir.exists()) {
            dir.mkdirs();
        }
        
        return dir;
    }
}
