/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package DiningPhilosophers;

/**
 * This is the implementation of the OMG IDL3 
 * DiningPhilosophers::Philosopher component type.
 *
 * This class inherits from the local CCM_Philosopher interface
 * generated by the OpenCCM's IDL3 to IDL2 mapping generator.
 *
 * This class also implements the Runnable interface to execute
 * the philosopher behavior into a thread.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 *
 * @version 2.0
 *
 */

public class PhilosopherPdaImpl
     extends org.omg.CORBA.LocalObject
  implements CCM_Philosopher, 
       	     java.lang.Runnable,
             org.omg.Components.SessionComponent
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The philosopher name. */
    private String name_;

    /**
     * The philosopher ticks duration.
     * This value is computed during initialization.
     */ 
    private int ticks_duration_ = 0;

    /** The philosopher hungryness counter. */ 
    private int hungryness_;

    /**
     * The counter for hold since how many ticks 
     * the philosopher hold a fork when in hungry state.
     */ 
    private int hungry_counter_;

    /** Tells if the philosopher have got his left fork. */
    private boolean left_fork_;

    /** Tells if the philosopher have got his right fork. */
    private boolean right_fork_;

    /** Contains the philosopher behaviour as a state machine. */
    private java.lang.Thread behaviour_;

    /** The philosopher current status. */
    private PhilosopherState state_;

    /** The philosopher frame */
    private java.awt.Frame frame_;

    /** The philosopher icon button */
    private MyButton button_;

    /** The philosopher CCM context. */
    private CCM_Philosopher_Context the_context_;

    /**
     * Represents duration for EATING, THINKING, HUNGRY, 
     * STARVING and DEAD states.
     */
    private int[] stateDuration = {-1, 3, 10, 40, -1};

    /** Icons used to represent the philosopher state. */
    private java.awt.Image[] stateAsIcon;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * Init a philosopher with an empty name.
     */
    public
    PhilosopherPdaImpl()
    {
        this(null);
    }

    /**
     * The constructor.
     *
     * @param name The philosopher name.
     */
    public
    PhilosopherPdaImpl(String name)
    {
        name_ = name;
        java.util.Random rand = 
            new java.util.Random(System.currentTimeMillis());
        ticks_duration_ = 1000 + rand.nextInt(2001);
        left_fork_ = false;
        right_fork_ = false;
        state_ = null;
        behaviour_ = null;
        hungry_counter_=0;
        hungryness_=0;
        
        // Creates the icon tab. Each icon must be in the jar archive.
        stateAsIcon = new java.awt.Image[5];
        java.net.URL url = PhilosopherPdaImpl.class.getResource("/images/eat.gif");
        stateAsIcon[0] = java.awt.Toolkit.getDefaultToolkit().getImage(url);
        url = PhilosopherPdaImpl.class.getResource("/images/think.gif");
        stateAsIcon[1] = java.awt.Toolkit.getDefaultToolkit().getImage(url);
        url = PhilosopherPdaImpl.class.getResource("/images/hun.gif");
        stateAsIcon[2] = java.awt.Toolkit.getDefaultToolkit().getImage(url);
        url = PhilosopherPdaImpl.class.getResource("/images/hun_right.gif");
        stateAsIcon[3] = java.awt.Toolkit.getDefaultToolkit().getImage(url);
        url = PhilosopherPdaImpl.class.getResource("/images/rip.jpg");
        stateAsIcon[4] = java.awt.Toolkit.getDefaultToolkit().getImage(url);
        
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * The philosopher enter in thinking state.
     * This represents actions to do during one tick.
     *
     * During this state, the philosopher think. When the hungryness 
     * becomes three, the philosopher goes to the hungry state.
     */
    protected void
    inThinkingState()
    {
        if (hungryness_ < stateDuration[PhilosopherState._THINKING])
        {
            setStatus(PhilosopherState.THINKING);
        } else {            
            setStatus(PhilosopherState.HUNGRY);
        }
    }

    /**
     * The philosopher enter in hungry state.
     * This represents actions to do during one tick.
     *
     * <p>
     * During this state, the philosopher would like to eat, 
     * so the philosopher attempts to acquire both forks. 
     * If the philosopher succeeds getting both forks, the 
     * philosopher enters the eating state. If the philosopher 
     * does not succeed getting both forks since hungryness is
     * ten, the philosopher enter the starving state. 
     * </p>
     *
     * <p>
     * The philosopher is not allowed to hold a single fork 
     * for an extended amount of time. If the philosopher hold a 
     * fork while waiting for the other one, the fork held by the 
     * philosopher must be released after five ticks.
     * </p>
     */
    protected void
    inHungryState()
    {
        // trying to get the left fork 
        if (! left_fork_)
        {
            try {
                the_context_.get_connection_left().get();
                left_fork_ = true;
                hungry_counter_=0;                    
            } catch(InUse exc) {
                if (right_fork_ && hungry_counter_ >= 5) {
                    // release the right fork.
                    the_context_.get_connection_right().release();
                    right_fork_=false;
                    hungry_counter_=0;                    
                }
            }
        }
        
        // trying to get the right fork 
        if (! right_fork_)
        {
            try {
                the_context_.get_connection_right().get();
                right_fork_ = true;
                hungry_counter_=0;                    
            } catch(InUse exc) {
                if (left_fork_ && hungry_counter_ >= 5) {
                    // release the left fork.
                    the_context_.get_connection_left().release();
                    left_fork_=false;
                    hungry_counter_=0;                    
                }
            }
        }

        if (left_fork_ && right_fork_) {
            hungry_counter_=0;
            // enter eating state.
            setStatus(PhilosopherState.EATING);                
        } else if (hungryness_ >= stateDuration[PhilosopherState._HUNGRY]) {
            hungry_counter_=0;
            // enter startving state.
            setStatus(PhilosopherState.STARVING);
        } else {
            hungry_counter_++;
            // enter hungry state.
            setStatus(PhilosopherState.HUNGRY);
        } 
    }

    /**
     * The philosopher enter in starving state.
     * This represents actions to do during one tick.
     *
     * <p>
     * During this state, the philosopher acts as in the previous 
     * state but can hold a fork for an extended amount of time 
     * while waiting for the other one.
     * If the philosopher succeeds getting both forks, the 
     * philosopher enters the eating state. If the philosopher 
     * does not succeed getting both forks since hungryness is
     * forty, the philosopher enter the dead state.
     * </p>
     */
    protected void
    inStarvingState()
    {
        // trying to get the left fork 
        if (! left_fork_) 
        {  
            try {
                the_context_.get_connection_left().get();
                left_fork_ = true;
            } catch(InUse exc) {
            }
        }
        
        // trying to get the right fork 
        if (! right_fork_) 
        {
            try {
                the_context_.get_connection_right().get();
                right_fork_ = true;
            } catch(InUse exc) {
            }
        }

        if (left_fork_ && right_fork_) {
            // enter eating state.
            setStatus(PhilosopherState.EATING);
        } else if (hungryness_ < stateDuration[PhilosopherState._STARVING]) {
            // enter starving state.
            setStatus(PhilosopherState.STARVING);            
        } else {
            // enter dead state.
            setStatus(PhilosopherState.DEAD);
        }
    }

    /**
     * The philosopher enter in eating state.
     * This represents actions to do during one tick.
     *
     * <p>
     * During this state, the philosopher holds on to both forks 
     * and spend one-third of the time eating thaht the philosopher 
     * was thinking, hungry or starving. When the philosopher stop 
     * to eat, he drops both forks and enters in thinking state.
     * </p>
     */
    protected void
    inEatingState()
    {
        if (hungryness_ > 0)
        {
            // enter eating state.
            setStatus(PhilosopherState.EATING);
            // eating...
            hungryness_-=3;
        } else {
            hungryness_ = 0;
            // release the left fork.
            if (left_fork_) 
                {
                    the_context_.get_connection_left().release();        
                    left_fork_ = false;
                }
            // release the right fork.
            if (right_fork_) 
                {
                    the_context_.get_connection_right().release(); 
                    right_fork_ = false;
                }
            // enter thinking state.
            setStatus(PhilosopherState.THINKING);
        }
    }

    /**
     * The philosopher enter in dead state.
     * This represents actions to do during one tick.
     *
     * <p>
     * During this state the philosopher releases both forks and 
     * do nothing else. This is no way out of the dead state.
     * </p>
     */
    protected void
    inDeadState()
    {
        // release the left fork.
        if (left_fork_) 
        {
            the_context_.get_connection_left().release();        
            left_fork_ = false;
        }
        // release the right fork.
        if (right_fork_) 
        {
            the_context_.get_connection_right().release(); 
            right_fork_ = false;
        }
        // enter dead state.
        setStatus(PhilosopherState.DEAD);
    }

    /**
     * Process state value and waiting next tick.
     * Resolving the state machine transitions.
     *
     * <p>
     * When calling <code>nextTick</code>, the 
     * philosopher status is processed to enter 
     * in the appropriate state. Then, waiting for 
     * the next tick and callback itself.
     * </p>
     */
    protected void
    nextTick()
    {
        // choose state to go 
        switch(state_.value())
        {
        case PhilosopherState._EATING : 
            {
                inEatingState();
                break;
            }
        case PhilosopherState._THINKING : 
            {
                inThinkingState();
                break;
            }
        case PhilosopherState._HUNGRY : 
            {
                inHungryState();
                break;
            }
        case PhilosopherState._STARVING : 
            {
                inStarvingState();
                break;
            }
        case PhilosopherState._DEAD : 
            {
                inDeadState();
                break;
            }
        }
        
        // waiting next tick.
        try {
            behaviour_.sleep(ticks_duration_);
        } catch(java.lang.InterruptedException exc) {
            exc.printStackTrace();
        }

        // Test if hungryness counter must be incremented.
        switch (state_.value()) 
        {
        case PhilosopherState._THINKING : 
        case PhilosopherState._HUNGRY : 
        case PhilosopherState._STARVING : 
            {
                hungryness_++;
                break;
            }
        }

        // call the next trick
        nextTick();
    }

    /**
     * Change the status and send an event by the info source.
     */
    protected void
    setStatus(PhilosopherState state)
    {
        state_ = state;

        button_.setIcon(stateAsIcon[state_.value()]);
        //        frame_.pack();
        //        frame_.show();

        // Pushes an event to the connected observer.
         the_context_.push_info(new StatusInfoImpl(state_, name_, 
                                                   hungryness_, left_fork_, 
                                                   right_fork_));
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration.
     *
     * <p>
     * Checks philosopher name, ticks duration, forks connections
     * and starts the philosopher behaviour.
     * </p>
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.     
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        // Checks if the configuration is completed.
        if(name_ == null)
            throw new org.omg.Components.InvalidConfiguration();

        // Checks if the configuration is completed.
        if(ticks_duration_ == 0)
            throw new org.omg.Components.InvalidConfiguration();

        // Check if the connection to the left fork is set.
        if(the_context_.get_connection_left() == null)
            throw new org.omg.Components.InvalidConfiguration();

        // Check if the connection to the right fork is set.
        if(the_context_.get_connection_right() == null)
            throw new org.omg.Components.InvalidConfiguration();

        // Creates the Philosopher GUI
        frame_ = new java.awt.Frame(name_ + " GUI");
        java.awt.Panel border = new java.awt.Panel(new java.awt.BorderLayout());
        
        // Creates the icon button
        button_ = new MyButton();
        button_.setIcon(stateAsIcon[PhilosopherState._THINKING]);

        // Setting up a label 
        java.awt.Label label = new java.awt.Label("Hi ! I'm " + name_ + " !!");
        // label.setFont(label.getFont().deriveFont((float)16.0));
 
        // Packing the GUI
        border.add(button_, java.awt.BorderLayout.CENTER);
        border.add(label, java.awt.BorderLayout.SOUTH);
        frame_.add(border);

        // Display the GUI
        frame_.pack();
        frame_.show();

        // Start the behaviour.
        behaviour_ = new java.lang.Thread(this);
        behaviour_.start();
    }

    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Setting the session component context.
     *
     * @param context The session component context.
     */
    public void
    set_session_context(org.omg.Components.SessionContext context)
    throws org.omg.Components.CCMException
    {
        the_context_ = (CCM_Philosopher_Context)context;
    }

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        // Nothing to do currently.
    }				

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove() 
    throws org.omg.Components.CCMException
    {
        // Release the associated frame.
        frame_.dispose();
	frame_ = null;

        // Stop the philosopher behaviour.
        behaviour_.stop();
	behaviour_ = null;
    }				

    // ==================================================================
    //
    // Methods for OMG IDL DiningPhilosophers::Philosopher
    //
    // ==================================================================

    /**
     * The mutator method for the attribute name.
     *
     * @param n The name.
     */
    public void
    name(String n)
    {
        name_ = n;
    }

    /**
     * The accessor method for the attribute name.
     *
     * @return The name.
     */
    public String
    name()
    {
        return name_;
    }

    // ==================================================================
    //
    // Methods for interface java.lang.Runnable
    //
    // ==================================================================

    /**
     * The philosopher behaviour.
     */
    public void
    run()
    {
        // The philosopher starts thinking.
        setStatus(PhilosopherState.THINKING);
        nextTick();
    }
}

/**
 * An internal button GUI for PDA (based on AWT).
 */
class MyButton extends java.awt.Panel 
{

    /** used icons */
    protected java.awt.Image icon_;

    /**
     * Default constructor.
     */
    public MyButton() {
    };

    /**
     * Set the icon to use when check box hasn't been selected.
     */
    public void setIcon(java.awt.Image icon) 
    {
        icon_ = icon;
        repaint();
    }

    /**
     * Overrides java.awt.Component.getPreferredSize 
     * in order to have a nice display of icons.
     */
    public java.awt.Dimension getPreferredSize()
    {
        return new java.awt.Dimension(100, 100);
    }

    /**
     * Overrides the component paint method to draw 
     * icon.
     */
     public void paint(java.awt.Graphics g)
     {
         g.drawImage(icon_, 0, 0, this);
     }
}
