/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Philippe Merle.
Contributor(s): Sylvain Leblanc, Romain Rouvoy________.

====================================================================*/

package org.objectweb.ccm.Components;

/**
 * This class is a basic implementation of the CCMObject interface.
 * It also implements all generic operations for CCM components. 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public abstract class CCMObjectImpl
extends org.omg.CORBA.LocalObject
  implements org.omg.Components.CCMObjectOperations,
           org.omg.Components.SessionContext
{
    // ==================================================================
    // 
    //  Internal state.
    // 
    // ==================================================================

    /**
     ** The default maximum connections for multiple receptacles.
     **/
    private static final int MAX_CONNECTIONS = java.lang.Integer.MAX_VALUE;

    /**
     ** Is the component ready to be used?
     **/
    private boolean ready_;

    /**
     **  Infos related to the facet ports.
     **/
    private org.objectweb.ccm.util.Table facets_;

    /**
     **  Infos related to the receptacle ports.
     **/
    private org.objectweb.ccm.util.Table receptacles_;

    /**
     **  Infos related to the event ports.
     **/
    private org.objectweb.ccm.util.Table consumers_;
    private org.objectweb.ccm.util.Table publishers_;
    private org.objectweb.ccm.util.Table emitters_;

    // ==================================================================
    //
    //  Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public 
    CCMObjectImpl()
    {
        ready_ = false;

        facets_ = new org.objectweb.ccm.util.Table();
        receptacles_ = new org.objectweb.ccm.util.Table();
        consumers_ = new org.objectweb.ccm.util.Table();
        publishers_ = new org.objectweb.ccm.util.Table();
        emitters_ = new org.objectweb.ccm.util.Table();
    }

    // ==================================================================
    //
    //  Internal methods.
    //
    // ==================================================================

    /**
     **  Search something in a Table.
     **
     **  @param key The searched key.
     **  @param table The table where search.
     **
     **  @return The information found.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the key does not refer any information.
     **/
    protected Object
    _search_key(String key,
                org.objectweb.ccm.util.Table table)
    throws org.omg.Components.InvalidName
    {
				// Search the key in the table.
        Object info = table.get(key);
				
				// If not found then throw the appropriate exception.
        if(info == null)
				{
            throw new org.omg.Components.InvalidName();
				}
				
				return info;
    }

    /**
     **  Search a facet information.
     **
     **  @param facet_name The facet port name.
     **
     **  @return The facet information.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a facet port.
     **/
    protected ProvidesInfo
    _search_facet(String facet_name)
    throws org.omg.Components.InvalidName
    {
				// Search the facet information.
        return (ProvidesInfo)_search_key(facet_name, facets_);
    }

    /**
     **  Search a receptacle information.
     **
     **  @param receptacle_name The receptacle port name.
     **
     **  @return The receptable information.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a receptacle port.
     **/
    protected UsesInfo
    _search_receptacle(String receptacle_name)
    throws org.omg.Components.InvalidName
    {
				// Search the receptacle information.
        return (UsesInfo)_search_key(receptacle_name, receptacles_);
    }

    /**
     **  Search a consumer information.
     **
     **  @param consumer_name The consumer port name.
     **
     **  @return The consumer information.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a consumer port.
     **/
    protected ConsumesInfo
    _search_consumer(String consumer_name)
    throws org.omg.Components.InvalidName
    {
				// Search the consumer information.
        return (ConsumesInfo)_search_key(consumer_name, consumers_);
    }

    /**
     **  Search a publisher information.
     **
     **  @param publisher_name The publisher port name.
     **
     **  @return The publisher information.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a publisher port.
     **/
    protected PublishesInfo
    _search_publisher(String publisher_name)
    throws org.omg.Components.InvalidName
    {
				// Search the publisher information.
        return (PublishesInfo)_search_key(publisher_name, publishers_);
    }

    /**
     **  Search an emitter information.
     **
     **  @param emitter_name The emitter port name.
     **
     **  @return The emitter information.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to an emitter port.
     **/
    protected EmitsInfo
    _search_emitter(String emitter_name)
    throws org.omg.Components.InvalidName
    {
				// Search the emitter information.
        return (EmitsInfo)_search_key(emitter_name, emitters_);
    }

    /**
     **  Push an event to a consumer.
     **
     **  @param consumer The consumer.
     **  @param event The event.
     **
     **  Note: It launches a thread for each event.
     **/
    protected void
    _call_push(org.omg.CORBA.Object consumer,
               java.lang.String value_name,
               org.omg.CORBA.Any event)
    {
				// Create a thread to push the event to the consumer.
        ThreadPushEvent pusher = new ThreadPushEvent(consumer, value_name, event);
        pusher.start();
				
				/*
				// Check if the consumer is local.
        if(((org.omg.CORBA.portable.ObjectImpl)consumer)._is_local())
				{
				   // If local then start the new thread which pushes the event. 
				   pusher.start();
				}
        else
        {
				   // If remote the just pushes the event asynchronously.
				   pusher.run();
				}
				*/
    }

    /**
     **
     **/
    protected org.omg.Components.CCMHome
    _home_ref()
    {
        return _component_servant().the_home_servant().the_home_ref();
    }

    /**
     **
     **/
    protected org.omg.Components.PrimaryKeyBase
    _primary_key()
    {
        return _component_servant().the_registration_info();
    }

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.ComponentServant
    _component_servant()
    {
        return _the_component_executor()._component_servant();
    }

    /**
     **
     **/
    protected org.objectweb.ccm.Containers.HomeServant
    _home_servant()
    {
        return _component_servant().the_home_servant();
    }

    /**
     **  Declare a facet port.
     **
     **/
    protected void
    _declare_facet(ProvidesInfo facet)
    {
        org.omg.CORBA.Object ref = _component_servant().get_reference(facet.id());

        facet.facet(ref);
				// Store a facet information.
        facets_.put(facet.name(), facet);
    }

    /**
     **  Declare a simple receptacle port.
     **
     **/
    protected void
    _declare_receptacle(UsesInfo rec)
    {
				// Store the receptacle information.
        receptacles_.put(rec.name(), rec);
    }

    /**
     **  Declare a consumer port.
     **
     **/
    protected void
    _declare_consumer(ConsumesInfo sink)
    {
        org.omg.CORBA.Object ref = _component_servant().get_reference(sink.id());

        sink.consumer(org.omg.Components.EventConsumerBaseHelper.narrow(ref));
				// Insert a descriptor in the consumers table.
        consumers_.put(sink.name(), sink);
    }

    /** 
     **  Declare a publisher port.
     **
     **/
    protected void
    _declare_publisher(PublishesInfo source)
    {
				// Insert a descriptor in the publishers table.
        publishers_.put(source.name(), source);
    }

    /** 
     **  Declare an emitter port.
     **
     **/
    protected void
    _declare_emitter(EmitsInfo source)
    {
				// Insert a descriptor in the emitters table.
        emitters_.put(source.name(), source);
    }

    // ==================================================================
    //
    // Abstract Public methods that must be implemented by subclasses.
    //
    // ==================================================================

    /**
     **
     **/
    public abstract org.objectweb.ccm.Containers.ComponentExecutor
    _the_component_executor();

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /**
     **
     **/
    public void
    _declare_port(PortInfo port)
    {
        switch(port.kind())
        {
            case PortInfo.pk_provides :
                _declare_facet((ProvidesInfo)port);
                break;
            case PortInfo.pk_uses :
                _declare_receptacle((UsesInfo)port);
                break;
            case PortInfo.pk_consumes :
                _declare_consumer((ConsumesInfo)port);
                break;
            case PortInfo.pk_emits :
                _declare_emitter((EmitsInfo)port);
                break;
            case PortInfo.pk_publishes :
                _declare_publisher((PublishesInfo)port);
                break;
            default :
                // should not happen !!!
                break;
        }
    }

    /**
     **  Push an event on a publisher port.
     **
     **  @param publisher_name The publisher name.
     **  @param event The event to push.
     **/
    public void
    _push(String publisher_name,
          String value_name,
          org.omg.CORBA.Any event)
    throws org.omg.Components.InvalidName
    {
        PublishesInfo info = null;
        // Search the publisher descriptor.
        info = _search_publisher(publisher_name);

				// Notify all the consumers.
        for(java.util.Enumeration enume=info.consumers().elements(); enume.hasMoreElements(); )
        {
            _call_push(((org.omg.Components.SubscriberDescription)enume.nextElement()).consumer, value_name, event);
        }
    }

    /**
     **  Push an event on a emitter port.
     **
     **  @param emitter_name The emitter name.
     **  @param event The event to push.
     **/
    public void
    _push_emitter(String emitter_name,
                  String value_name,
                  org.omg.CORBA.Any event)
    throws org.omg.Components.InvalidName
    {
        EmitsInfo info = null;
        // Search the emitter information.
        info = _search_emitter(emitter_name);

				// If there is a consumer then notify it.
        if(info.consumer() != null)
				{
						_call_push(info.consumer(), value_name, event);
        }
    }

    /**
     **
     **/
    public org.omg.Components.Cookie
    _connect(String name,
             org.omg.CORBA.Object connexion)
    throws org.omg.Components.InvalidName,
           org.omg.Components.ExceededConnectionLimit
    {
        try
        {
            return connect(name, connexion);
        }
        catch(org.omg.Components.AlreadyConnected ex)
        {
            // should never happen
        }
        catch(org.omg.Components.InvalidConnection ex)
        {
            // should never happen
        }
        return null;
    }

    /**
     **
     **/
    public void
    _connect_simple_receptacle(String name,
                               org.omg.CORBA.Object connexion)
    throws org.omg.Components.AlreadyConnected,
           org.omg.Components.InvalidName
    {
        try
        {
            connect(name, connexion);
        }
        catch(org.omg.Components.ExceededConnectionLimit ex)
        {
            // should never happen
        }
        catch(org.omg.Components.InvalidConnection ex)
        {
            // should never happen
        }
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    _disconnect(String receptacle_name,
                org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidName
    {
        try
        {
            return disconnect(receptacle_name, ck);
        }
        catch(org.omg.Components.NoConnection ex)
        {
            // should never happen
        }
        catch(org.omg.Components.CookieRequired ex)
        {
            // should never happen
        }
        catch(org.omg.Components.InvalidConnection ex)
        {
            // should never happen
        }
        return null;
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    _disconnect_simple_receptacle(String receptacle_name)
    throws org.omg.Components.NoConnection,
           org.omg.Components.InvalidName
    {
        try
        {
            return disconnect(receptacle_name, null);
        }
        catch(org.omg.Components.InvalidConnection ex)
        {
            // should never happen
        }
        catch(org.omg.Components.CookieRequired ex)
        {
            // should never happen
        }
        return null;
    }

    /**
     **
     **/
    public org.omg.CORBA.Object
    _get_connection_simple_receptacle(String receptacle_name)
    throws org.omg.Components.InvalidName
    {
        UsesInfo info = null;

        // Search the receptacle information.
        info = _search_receptacle(receptacle_name);

				// Return the first connection.
        return ((org.omg.Components.ConnectionDescription)
                (info.connections().elements().nextElement()))
						    .objref;
    }

    /**
     **
     **/
    public org.omg.Components.ConnectionDescription[]
    _get_connections(String receptacle_name)
    throws org.omg.Components.InvalidName
    {
        return get_connections(receptacle_name);
    }

    /**
     **
     **/
    public org.omg.Components.Cookie
    _subscribe(String publisher_name,
               org.omg.Components.EventConsumerBase subscriber)
    throws org.omg.Components.InvalidName,
           org.omg.Components.ExceededConnectionLimit
    {
        try
        {
            return subscribe(publisher_name, subscriber);
        }
        catch(org.omg.Components.InvalidConnection ex)
        {
            // should never happen
        }
        return null;
    }

    /**
     **
     **/
    public org.omg.Components.EventConsumerBase
    _unsubscribe(String publisher_name,
                 org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection
    {
        return unsubscribe(publisher_name, ck);
    }

    /**
     **
     **/
    public org.omg.Components.EventConsumerBase[]
    _get_subscribers(String publisher_name)
    throws org.omg.Components.InvalidName
    {
        PublishesInfo info = _search_publisher(publisher_name);
        org.omg.Components.SubscriberDescription[] connections = null;
        org.omg.Components.EventConsumerBase[] result = null;

        connections = (org.omg.Components.SubscriberDescription[])info.
            consumers().values().toArray(new org.omg.Components.SubscriberDescription[0]);
        result = new org.omg.Components.EventConsumerBase[connections.length];

        for (int i=0;i<result.length;i++)
            result[i] = connections[i].consumer;

        return result;        
    }

    /**
     **
     **/
    public void
    _connect_emitter(String emitter_name,
                     org.omg.Components.EventConsumerBase consumer)
    throws org.omg.Components.InvalidName,
           org.omg.Components.AlreadyConnected
    {
        try
        {
            connect_consumer(emitter_name, consumer);
        }
        catch(org.omg.Components.InvalidConnection ex)
        {
            // should never happen
        }
    }

    /**
     **
     **/
    public org.omg.Components.EventConsumerBase
    _disconnect_emitter(String source_name)
    throws org.omg.Components.InvalidName,
           org.omg.Components.NoConnection
    {
        return disconnect_consumer(source_name);
    }

    /**
     **
     **/
    public org.omg.Components.EventConsumerBase
    _get_emitter(String emitter_name)
    throws org.omg.Components.InvalidName
    {
        EmitsInfo info = _search_emitter(emitter_name);
        return info.consumer();
    }

    /**
     **
     **/
    public org.omg.Components.EventConsumerBase
    _get_consumer(String sink_name)
    throws org.omg.Components.InvalidName
    {
        return get_consumer(sink_name);
    }

    // ==================================================================
    //
    // Methods for the Components::Navigation interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/Navigation/provide_facet:1.0
    //
    /**
     **  Provides the reference of a facet.
     **
     **  @param name The name of the facet.
     **
     **  @return The reference of the facet.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a facet.
     **/
    public org.omg.CORBA.Object
    provide_facet(String name)
    throws org.omg.Components.InvalidName
    {
	// Search the facet information and
	// return the reference to the facet.
        return _search_facet(name).facet();
    }

    //
    // IDL:omg.org/Components/Navigation/get_all_facets:1.0
    //
    /**
     **  Describes all facets.
     **
     **  @return All the facet descriptions.
     **/
    public org.omg.Components.FacetDescription[]
    get_all_facets()
    {
				// Allocate the result.
        org.omg.Components.FacetDescription result[] = new org.omg.Components.FacetDescription[facets_.size()];
				
				// Iterate on all the facet information.
				int idx = 0;
        for(java.util.Enumeration enume = facets_.elements(); enume.hasMoreElements(); )
        {
            ProvidesInfo info = (ProvidesInfo)enume.nextElement();
            result[idx++] = new FacetDescriptionImpl(info.type_id(),
                                                     info.name(),
                                                     info.facet());
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Navigation/get_named_facets:1.0
    //
    /**
     **  Provides a sequence of facets.
     **
     **  @param names A sequence of facet names.
     **
     **  @return The facet descriptions and references.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if a name does not refer to a facet.
     **/
    public org.omg.Components.FacetDescription[]
    get_named_facets(String[] names)
    throws org.omg.Components.InvalidName
    {
				// Allocate the result.
        org.omg.Components.FacetDescription result[] =
            new org.omg.Components.FacetDescription[names.length];

				// Iterate on all the names.
        for(int i=0;i<names.length;i++)
				{
						// Search and store the facet information.
            ProvidesInfo info = _search_facet(names[i]);
            result[i] = new FacetDescriptionImpl(info.type_id(),
                                                 info.name(),
                                                 info.facet());
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Navigation/same_component:1.0
    //
    /**
     **  Test if a reference refers to a facet of the same component.
     **
     **  @param ref The reference to test.
     **
     **  @return True if ref refers to a facet of the same component.
     **/
    public boolean
    same_component(org.omg.CORBA.Object ref)
    {
				// Iterate on all the facet information.
        int idx = 0;
        ProvidesInfo info = null;
				for(java.util.Enumeration enume = facets_.elements();enume.hasMoreElements(); )
        {
            info = (ProvidesInfo)enume.nextElement();
						if (info.facet()._is_equivalent(ref))
								return true;
				}

        return false;
    }

    // ==================================================================
    //
    // Methods for the Components::Receptacles interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/Receptacles/connect:1.0
    //
    /**
     **  Connects to a receptacle.
     **
     **  @param name The name of the receptacle.
     **  @param connection The reference to the connection.
     **
     **  @return The cookie associated to the connection.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a receptacle.
     **  @exception org.omg.Components.InvalidConnection
     **             Thrown if the connection is invalid for the receptacle.
     **  @exception org.omg.Components.AlreadyConnected
     **             Thrown if the receptacle is for single connection and
     **             a connection is already connected.
     **  @exception org.omg.Components.ExceededConnectionLimit
     **             Thrown if there are too many connections.
     **/
    public org.omg.Components.Cookie
    connect(String name,
            org.omg.CORBA.Object connexion)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection,
           org.omg.Components.AlreadyConnected,
           org.omg.Components.ExceededConnectionLimit
    {
				// Search the receptacle information.
        UsesInfo info = _search_receptacle(name);
				
				// Check that the connection supports the receptacle type.
        if (!connexion._is_a(info.type_id()))
				{
            throw new org.omg.Components.InvalidConnection();
				}

        int nb_connections = info.connections().size();
				
				// Check that it is a simple receptacle not already connected.
        if (info.simple() && nb_connections == 1)
				{
            throw new org.omg.Components.AlreadyConnected();
				}

				// Check if the connection limit is exceeded.
        if (!info.simple() && nb_connections >= info.limit())
				{
            throw new org.omg.Components.ExceededConnectionLimit();
				}
				
				// Generate a cookie for this connection.
        CookieImpl ck = CookieImpl.generate();
				
        // create an interceptor for this connection.
        org.objectweb.ccm.Containers.HomeExecutorBase home = _home_servant().the_home_executor();
        org.objectweb.ccm.Containers.Interceptor inter = home._get_interceptor(info.id());
				
        // create the call context for the interceptor and set the needed infos.
        org.objectweb.ccm.Containers.CallContext context =
            new org.objectweb.ccm.Containers.ComponentCallContext(_home_servant(), info.id());
        inter._call_context(context);

        // set the connection as the delegate.
        inter._delegate(home._get_executor_wrapper(connexion, info.id()));

				// Add this connection to the list of connections of this receptacle.
        //        info.connections().put(ck, new ConnectionDescriptionImpl(ck, connexion));
        info.connections().put(ck, new ConnectionDescriptionImpl(ck, (org.omg.CORBA.Object)inter));
				
				// Return the generated cookie.
        return ck;
    }

    //
    // IDL:omg.org/Components/Receptacles/disconnect:1.0
    //
    /**
     **  Disconnects from a receptacle.
     **
     **  @param name The name of the receptacle.
     **  @param ck The cookie associated to the connection.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a receptacle.
     **  @exception org.omg.Components.InvalidConnection
     **             Thrown if the connection is invalid for the receptacle.
     **  @exception org.omg.Components.CookieRequired
     **             Thrown if the cookie is invalid.
     **  @exception org.omg.Components.NoConnection
     **             Thrown if there is no connection.
     **/
    public org.omg.CORBA.Object
    disconnect(String name,
               org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection,
           org.omg.Components.CookieRequired,
           org.omg.Components.NoConnection
    {
				// Search the receptacle information.
        UsesInfo info = _search_receptacle(name);

        org.omg.CORBA.Object result = null;

				// If it is a simple receptacle.
        if(info.simple())
        {
						// Check that a connection is available.
            if(info.connections().size() == 0)
						{
                throw new org.omg.Components.NoConnection();
						}

						// Obtain the connection.
            result = ((org.omg.Components.ConnectionDescription)
                      (info.connections().elements().nextElement())).objref;

						// Clean the connection list.
            info.connections().clear();
        }
        else // Then it is a multiple receptable.
        {
						// Check that the cookie is right.
            if(ck == null)
            {
								throw new org.omg.Components.CookieRequired();
						}

						// Remove and obtain the connection associated to the cookie.
            result = (org.omg.CORBA.Object)info.connections().remove(ck);

						// If no connection then the cookie is wrong.
            if(result == null)
						{
                throw new org.omg.Components.InvalidConnection();
						}
        }

				// Return the connection.
        return result;
    }

    //
    // IDL:omg.org/Components/Receptacles/get_connections:1.0
    //
    /**
     **  Obtains connections for a receptacle.
     **
     **  @param name The name of the receptacle.
     **
     **  @return The description of all connections.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to a receptacle.
     **/
    public org.omg.Components.ConnectionDescription[]
    get_connections(String name)
    throws org.omg.Components.InvalidName
    {
				// Search the receptacle information.
        UsesInfo info = _search_receptacle(name);

        // Allocate the result.
        org.omg.Components.ConnectionDescription[] result =
            new org.omg.Components.ConnectionDescription[info.connections().size()];

				// Iterate on all the connections.
				int idx = 0;
				for(java.util.Enumeration enume = info.connections().elements(); enume.hasMoreElements(); )
            result[idx++] = (org.omg.Components.ConnectionDescription)enume.nextElement();

        return result;
    }

    //
    // IDL:omg.org/Components/Receptacles/get_all_receptacles:1.0
    //
    /**
     **  Obtains all receptacles description.
     **
     **/
    public org.omg.Components.ReceptacleDescription[]
    get_all_receptacles()
    {
        org.objectweb.ccm.Containers.HomeExecutorBase home = _home_servant().the_home_executor();

        // Allocate the result.
        org.omg.Components.ReceptacleDescription[] result =
            new org.omg.Components.ReceptacleDescription[receptacles_.size()];

				// Iterate on all the connections.
				int idx = 0;
				for(java.util.Enumeration enume = receptacles_.elements(); enume.hasMoreElements(); )
				{
            UsesInfo info = (UsesInfo)enume.nextElement();
            result[idx++] = new ReceptacleDescriptionImpl(info.name(),
                                                          info.type_id(),
                                                          !info.simple(),
                                                          (org.omg.Components.ConnectionDescription[])
                                                          info.connections().values().toArray(
																																															new org.omg.Components.ConnectionDescription[0]));
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Receptacles/get_named_receptacles:1.0
    //
    /**
     **  Obtains all receptacles description.
     **
     **/
    public org.omg.Components.ReceptacleDescription[]
    get_named_receptacles(String[] names)
    throws org.omg.Components.InvalidName
    {
				// Allocate the result.
        org.omg.Components.ReceptacleDescription result[] =
            new org.omg.Components.ReceptacleDescription[names.length];

				// Iterate on all the names.
        for(int i=0;i<names.length;i++)
        {
						// Search and store the facet information.
            UsesInfo info = _search_receptacle(names[i]);
            result[i] = new ReceptacleDescriptionImpl(info.name(),
                                                      info.type_id(),
                                                      !info.simple(),
                                                      (org.omg.Components.ConnectionDescription[])
                                                      info.connections().values().toArray(
                                                          new org.omg.Components.ConnectionDescription[0]));
        }

        return result;
    }

    // ==================================================================
    //
    // Methods for the Components::Events interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/Events/get_consumer:1.0
    //
    /**
     **  Obtains an event sink.
     **
     **  @param sink_name The name of the event sink.
     **
     **  @return The reference to the event sink.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to an event sink.
     **/
    public org.omg.Components.EventConsumerBase
    get_consumer(String sink_name)
    throws org.omg.Components.InvalidName
    {
				// Search the consumer information.
        ConsumesInfo info = _search_consumer(sink_name);

				// Return the consumer reference.
        return info.consumer();
    }
		
    //
    // IDL:omg.org/Components/Events/subscribe:1.0
    //
    /**
     **  Subscribes to an event source.
     **
     **  @param publisher_name The name of the event publisher source.
     **  @param subscriber The reference to the subscriber event sink.
     **
     **  @return The cookie associated to the subscription.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to an event
     **             publisher source.
     **/
    public org.omg.Components.Cookie
    subscribe(String publisher_name,
              org.omg.Components.EventConsumerBase subscriber)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection,
           org.omg.Components.ExceededConnectionLimit
    {
				// Search the publisher information.
        PublishesInfo info = _search_publisher(publisher_name);

				// Check that the subscriber supports the event type.
        if (!subscriber._is_a(info.type_id()))
				{
            throw new org.omg.Components.InvalidConnection();
				}
				
        int nb_connections = info.consumers().size();

				// Check if the connection limit is exceeded.
        if (nb_connections >= info.limit())
				{
						throw new org.omg.Components.ExceededConnectionLimit();
				}

				// Generate a cookie for this subscription.
        CookieImpl ck = CookieImpl.generate();

        // create an interceptor for this connection.
        org.objectweb.ccm.Containers.HomeExecutorBase home = _home_servant().the_home_executor();
        org.objectweb.ccm.Containers.Interceptor inter = home._get_interceptor(info.id());

        // create the call context for the interceptor and set the needed infos.
        org.objectweb.ccm.Containers.CallContext context =
            new org.objectweb.ccm.Containers.ComponentCallContext(_home_servant(), info.id());
        inter._call_context(context);

        // set the connection as the delegate.
        inter._delegate(home._get_executor_wrapper(subscriber, info.id()));

				// Add the subscription to the list of this publisher port.
        //        info.consumers().put(ck, new SubscriberDescriptionImpl(ck, subscriber));
        info.consumers().put(ck, new SubscriberDescriptionImpl(ck, (org.omg.Components.EventConsumerBase)inter));

				// Return the subscription cookie.
        return ck;
    }

    //
    // IDL:omg.org/Components/Events/unsubscribe:1.0
    //
    /**
     **  Unsubscribes from an event source.
     **
     **  @param publisher_name The name of the event publisher source.
     **  @param ck The cookie associated to the subscription.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to an event
     **             publisher source.
     **  @exception org.omg.Components.InvalidConnection
     **             Thrown if the connection is invalid for the event
     **             publisher source.
     **/
    public org.omg.Components.EventConsumerBase
    unsubscribe(String publisher_name,
                org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidName,
           org.omg.Components.InvalidConnection
    {
				// Search the publisher information.
        PublishesInfo info = _search_publisher(publisher_name);

				// Remove and obtain the subscriber description associated to the cookie.
        Object sd = info.consumers().remove(ck);

				// Check that the cookie is associated to a consumer.
        if(sd == null)
				{ 
            throw new org.omg.Components.InvalidConnection();
				}

				// Return the consumer.
        return ((org.omg.Components.SubscriberDescription)sd).consumer;
    }

    //
    // IDL:omg.org/Components/Events/connect_consumer:1.0
    //
    /**
     **  Connects to an event emitter source.
     **
     **  @param emitter_name The name of the event emitter source.
     **  @param consumer The reference to the event consumer sink.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to an event
     **             emitter source.
     **  @exception org.omg.Components.AlreadyConnected
     **             Thrown if there is an already connected consumer
     **             to this event emitter source.
     **/
    public void
    connect_consumer(String emitter_name,
                     org.omg.Components.EventConsumerBase consumer)
    throws org.omg.Components.InvalidName,
           org.omg.Components.AlreadyConnected,
           org.omg.Components.InvalidConnection
    {
				// Search the emitter information.
        EmitsInfo info = _search_emitter(emitter_name);

				// Check that the consumer supports the event type.
        if (!consumer._is_a(info.type_id()))
				{
            throw new org.omg.Components.InvalidConnection();
				}

				// Chek that there is not an already connection.
        if(info.consumer() != null)
        {
						throw new org.omg.Components.AlreadyConnected();
				}

        // create an interceptor for this connection.
        org.objectweb.ccm.Containers.HomeExecutorBase home = _home_servant().the_home_executor();
        org.objectweb.ccm.Containers.Interceptor inter = home._get_interceptor(info.id());

        // create the call context for the interceptor and set the needed infos.
        org.objectweb.ccm.Containers.CallContext context =
            new org.objectweb.ccm.Containers.ComponentCallContext(_home_servant(), info.id());
        inter._call_context(context);

        // set the connection as the delegate.
        inter._delegate(home._get_executor_wrapper(consumer, info.id()));

				// Store the consumer.
        //        info.consumer(consumer);
        info.consumer((org.omg.Components.EventConsumerBase)inter);
    }

    //
    // IDL:omg.org/Components/Events/disconnect_consumer:1.0
    //
    /**
     **  Disconnects from an event emitter source.
     **
     **  @param source_name The name of the event emitter source.
     **
     **  @return The reference to the previously connected event sink.
     **
     **  @exception org.omg.Components.InvalidName
     **             Thrown if the name does not refer to an event
     **             emitter source.
     **  @exception org.omg.Components.NoConnection
     **             Thrown if there is no connection.
     **/
    public org.omg.Components.EventConsumerBase
    disconnect_consumer(String source_name)
    throws org.omg.Components.InvalidName,
           org.omg.Components.NoConnection
    {
				// Search the emitter information.
        EmitsInfo info = _search_emitter(source_name);

				// Check that there is a connection.
        if(info.consumer() == null)
        {
						throw new org.omg.Components.NoConnection();
				}

				// Conserve the previous consumer.
				org.omg.Components.EventConsumerBase consumer = info.consumer();

				// Unset the consumer.
        info.consumer(null);

				// Return the previous consumer.
        return info.consumer();
    }

    //
    // IDL:omg.org/Components/Events/get_all_consumers:1.0
    //
    /**
     **  Obtains all event consumers description.
     **
     **/
    public org.omg.Components.ConsumerDescription[]
    get_all_consumers()
    {
        // Allocate the result.
        org.omg.Components.ConsumerDescription[] result =
            new org.omg.Components.ConsumerDescription[consumers_.size()];

				// Iterate on all the connections.
				int idx = 0;
				for(java.util.Enumeration enume = consumers_.elements(); enume.hasMoreElements(); )
        {
            ConsumesInfo info = (ConsumesInfo)enume.nextElement();
            result[idx++] = new ConsumerDescriptionImpl(info.name(), info.type_id(), info.consumer());
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Events/get_named_consumers:1.0
    //
    /**
     **  Obtains named event consumers description.
     **
     **/
    public org.omg.Components.ConsumerDescription[]
    get_named_consumers(String[] names)
    throws org.omg.Components.InvalidName
    {
				// Allocate the result.
        org.omg.Components.ConsumerDescription result[] =
            new org.omg.Components.ConsumerDescription[names.length];
				
				// Iterate on all the names.
        for(int i=0;i<names.length;i++)
				{
						// Search and store the facet information.
            ConsumesInfo info = _search_consumer(names[i]);
            result[i] = new ConsumerDescriptionImpl(info.name(), info.type_id(), info.consumer());
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Events/get_all_emitters:1.0
    //
    /**
     **  Obtains all event emitters description.
     **
     **/
    public org.omg.Components.EmitterDescription[]
    get_all_emitters()
    {
        // Allocate the result.
        org.omg.Components.EmitterDescription[] result =
            new org.omg.Components.EmitterDescription[emitters_.size()];

				// Iterate on all the connections.
				int idx = 0;
				for(java.util.Enumeration enume = emitters_.elements(); enume.hasMoreElements(); )
			  {
            EmitsInfo info = (EmitsInfo)enume.nextElement();
            result[idx++] = new EmitterDescriptionImpl(info.name(), info.type_id(), info.consumer());
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Events/get_named_emitters:1.0
    //
    /**
     **  Obtains named event emitters description.
     **
     **/
    public org.omg.Components.EmitterDescription[]
    get_named_emitters(String[] names)
    throws org.omg.Components.InvalidName
    {
				// Allocate the result.
        org.omg.Components.EmitterDescription result[] =
            new org.omg.Components.EmitterDescription[names.length];

				// Iterate on all the names.
        for(int i=0;i<names.length;i++)
        {
						// Search and store the facet information.
            EmitsInfo info = _search_emitter(names[i]);
            result[i] = new EmitterDescriptionImpl(info.name(), info.type_id(), info.consumer());
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Events/get_all_publishers:1.0
    //
    /**
     **  Obtains all event publishers description.
     **
     **/
    public org.omg.Components.PublisherDescription[]
    get_all_publishers()
    {
        // Allocate the result.
        org.omg.Components.PublisherDescription[] result =
            new org.omg.Components.PublisherDescription[publishers_.size()];

				// Iterate on all the connections.
				int idx = 0;
				for(java.util.Enumeration enume = publishers_.elements(); enume.hasMoreElements(); )
        {
            PublishesInfo info = (PublishesInfo)enume.nextElement();
            result[idx++] = new PublisherDescriptionImpl(info.name(), info.type_id(),
                                                        (org.omg.Components.SubscriberDescription[])
                                                        info.consumers().values().toArray(
                                                            new org.omg.Components.SubscriberDescription[0]));
        }

        return result;
    }

    //
    // IDL:omg.org/Components/Events/get_named_publishers:1.0
    //
    /**
     **  Obtains named event publishers description.
     **
     **/
    public org.omg.Components.PublisherDescription[]
    get_named_publishers(String[] names)
    throws org.omg.Components.InvalidName
    {
				// Allocate the result.
        org.omg.Components.PublisherDescription result[] =
            new org.omg.Components.PublisherDescription[names.length];

				// Iterate on all the names.
        for(int i=0;i<names.length;i++)
        {
						// Search and store the facet information.
            PublishesInfo info = _search_publisher(names[i]);
            result[i] = new PublisherDescriptionImpl(info.name(), info.type_id(),
                                                     (org.omg.Components.SubscriberDescription[])
                                                     info.consumers().values().toArray(
                                                         new org.omg.Components.SubscriberDescription[0]));
        }

        return result;
    }

    // ==================================================================
    //
    // Methods for the Components::CCMObject interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/CCMObject/get_component_def:1.0
    //
    /**
     **  Obtains the component definition into the Interface Repository.
     **
     **  @return The reference to the component definition.
     **/
    public org.omg.CORBA.IRObject
    get_component_def()
    {
	return _home_ref().get_component_def();
    }

    //
    // IDL:omg.org/Components/CCMObject/get_ccm_home:1.0
    //
    /**
     **  Obtains the associated home.
     **
     **  @return The associated home.
     **/
    public org.omg.Components.CCMHome
    get_ccm_home()
    {
        return _home_ref();
    }

    //
    // IDL:omg.org/Components/CCMObject/get_primary_key:1.0
    //
    /**
     **  Obtains the associated primary key.
     **
     **  @return The associated primary key.
     **
     **  @exception org.omg.Components.NoKeyAvailable
     **             Thrown if the component is keyless.
     **/
    public org.omg.Components.PrimaryKeyBase
    get_primary_key()
    throws org.omg.Components.NoKeyAvailable
    {
				// Check that there is an associated primary key.
        if(_primary_key() == null)
        {
						throw new org.omg.Components.NoKeyAvailable();
				}
				
				// Return the associated primary key.
        return _primary_key();
    }

    //
    // IDL:omg.org/Components/CCMObject/configuration_complete:1.0
    //
    /**
     **  Completes the component configuration.
     **
     **  @exception org.omg.Components.InvalidConfiguration
     **             Thrown if the configuration is invalid.     
     **/
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
	// TODO: Must be reviewed when configuration process
	// will be better understood ;-(
	// Currently, this must be defined by component implementors. 
        ready_ = true;
    }

    //
    // IDL:omg.org/Components/CCMObject/remove:1.0
    //
    /**
     **  Removes the component.
     **/
    public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        get_ccm_home().remove_component(_component_servant().the_component_ref());
        _component_servant().remove();
    }

    //
    // IDL:omg.org/Components/CCMObject/get_all_ports:1.0
    //
    /**
     **  Obtain the description of all the component's ports.
     **/
    public org.omg.Components.ComponentPortDescription
    get_all_ports()
    {
        return new ComponentPortDescriptionImpl(get_all_facets(),
                                                get_all_receptacles(),
                                                get_all_consumers(),
                                                get_all_emitters(),
                                                get_all_publishers());
    }

    // ==================================================================
    //
    // Methods for OMG IDL Components::SessionContext
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
    //
    /**
     **  Obtains the associated home.
     **
     **  @return The associated home.
     **/
    public org.omg.Components.CCMHome
    get_CCM_home()
    {
        return _home_ref();
    }

    //
    // IDL:omg.org/Components/SessionContext/get_CCM_object:1.0
    //
    /**
     **  Obtains the associated component reference.
     **
     **  @return The associated component reference.
     **/
    public org.omg.CORBA.Object
    get_CCM_object()
    {
        return _component_servant().the_component_ref();
    }

}

/**
 **  This class launches threads to push events to consumers.
 **  Note: It uses DII to push events asynchronously.
 **/
class ThreadPushEvent
extends Thread
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The event consumer.
     **/
    private org.omg.CORBA.Object consumer;

    /**
     **  The event to push.
     **/
    private org.omg.CORBA.Any event;

    /**
     **
     **/
    private java.lang.String value_name;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param consumer The event consumer.
     **  @param event The event to push.
     **/
    public
    ThreadPushEvent(org.omg.CORBA.Object consumer,
                    java.lang.String value_name,
                    org.omg.CORBA.Any event)
    {
        this.consumer = consumer;
        this.value_name = value_name;
				this.event = event;
    }

    // ==================================================================
    //
    // Methods for the java.lang.Runnable interface.
    //
    // ==================================================================

    /**
     **  Send the DII request asynchronously.
     **/
    public void run()
    {
				// Create a DII request.
        org.omg.CORBA.Request request = consumer._request("push_"+value_name);

				// Set the return type to void.
        request.set_return_type(org.objectweb.ccm.CORBA.TypeCodeUtils.getTC_void());

				// Add one argument ("event", event, ARG_IN)
        request.arguments().add_value("event", event, org.omg.CORBA.ARG_IN.value);

				// Send the request asynchronously.
        request.send_oneway();
    }
}

